(function ($) {
    $(function () {

        function slugify(text) {
            return text
                    .toString()                     // Cast to string
                    .toLowerCase()                  // Convert the string to lowercase letters
                    .normalize('NFD')       // The normalize() method returns the Unicode Normalization Form of a given string.
                    .trim()                         // Remove whitespace from both sides of a string
                    .replace(/\s+/g, '_')           // Replace spaces with -
                    .replace(/[^\w\-]+/g, '')       // Remove all non-word chars
                    .replace(/\-\-+/g, '_');        // Replace multiple - with single -
        }


        $(document).on('click', '#add-custom-meta-field', function () {

            $.confirm({
                type: 'red',
                closeIcon: true,
                useBootstrap: false,
                boxWidth: '50%',
                title: 'Add custom meta field',
                content: 'Name for your custom meta field<br /><input type="text" id="new-custom-meta-field" />',
                buttons: [{
                        text: "Confirm",
                        action: function () {
                            var label = $('#new-custom-meta-field').val();

                            if (!label) {
                                $.alert('Please fill in label for your meta field.');
                                return false;
                            }

                            var slug = slugify(label);

                            if ($('#' + slug).length) {
                                $.alert('It appears that you already have such custom meta field, please change the name.');
                                return false;
                            }

                            var data = {
                                action: 'magicpage_add_custom_meta_field',
                                label: label,
                                term_id: $('input[name="tag_ID"]').val()
                            };


                            $.post(ajaxurl, data, function (response) {
                                if (response) {
                                    $('#custom-fields-table').prepend(response);
                                }
                            });



                        }
                    }, {
                        text: "Cancel",
                        click: function () {

                        }
                    },
                ]
            });




        });


        $(document).on('click', '.delete-editor-custom-field', function () {
            var object = $(this);
            $.confirm({
                type: 'red',
                closeIcon: true,
                useBootstrap: false,
                boxWidth: '50%',
                title: 'Please Confirm',
                content: 'You are about to delete this custom field for all locations.',
                buttons: [{
                        text: "Confirm",
                        action: function () {
                            delete_field(object);
                        }
                    }, {
                        text: "Cancel",
                        click: function () {

                        }
                    },
                ]
            });
        });





        $(document).on('click', '.enable-disable-editor-custom-field', function () {
            var object = $(this);
            if (!object.data('state')) {



                $.confirm({
                    type: 'red',
                    closeIcon: true,
                    useBootstrap: false,
                    boxWidth: '50%',
                    title: 'Please Confirm',
                    content: 'You are about to switch off the editor for this meta field. We will strip the html tags (if any, optional)',
                    buttons: [{
                            text: "Confirm",
                            action: function () {
                                switch_off_editor(object, true);
                            }
                        },
                        {
                            text: "Confirm without html tags strip",
                            action: function () {
                                switch_off_editor(object, false);

                            }
                        },
                        {
                            text: "Cancel",
                            click: function () {

                            }
                        },
                    ]
                });
            } else {
                $.confirm({
                    type: 'red',
                    closeIcon: true,
                    useBootstrap: false,
                    boxWidth: '50%',

                    title: 'Please Confirm',
                    content: 'You are about to switch on the editor for this meta field. \n\
We will need to reload the page in order to display the editor. Confirm without reload if you reload the page manually later',

                    buttons: [{
                            text: "Confirm",
                            action: function () {
                                switch_on_editor(object, true);
                            }
                        },
                        {
                            text: "Confirm without reload",
                            action: function () {
                                switch_on_editor(object, false);
                            }
                        },
                        {
                            text: "Cancel",
                            click: function () {

                            }
                        }
                    ]

                });
            }
        });


        function switch_on_editor(object, reload) {
            var tr = object.parents('.custom-meta-field-td');
            var id = tr.data('id');
            var td = tr.find('.editor-cell');

            var data = {
                action: 'magicpage_editor_turn_on',
                content: td.find('textarea').val(),
                id: id,
                term_id: $('input[name="tag_ID"]').val()
            };

            $.post(ajaxurl, data, function (response) {
                if (response.success && reload) {
                    window.location.reload();
                }
            });
        }


        function switch_off_editor(object, tags_strip) {

            var tr = object.parents('.custom-meta-field-td');
            var id = tr.data('id');
            var td = tr.find('.editor-cell');
            var editor_id = id;


            var data = {
                action: 'magicpage_editor_turn_off',
                content: get_tinymce_content(editor_id),
                id: id,
                term_id: $('input[name="tag_ID"]').val(),
                tags_strip: tags_strip

            };

            $.post(ajaxurl, data, function (response) {
                if (response.success) {
                    td.html(response.data.content);
                    object.addClass('hidden');

                    object.parent().find('.enable-editor').removeClass('hidden');

                }
            });
        }

        function delete_field(object) {

            var tr = object.parents('.custom-meta-field-td');
            var id = tr.data('id');


            var data = {
                action: 'magicpage_custom_meta_delete',
                id: id,
                term_id: $('input[name="tag_ID"]').val()
            };

            $.post(ajaxurl, data, function (response) {
                if (response.success) {
                    tr.remove();
                }
            });
        }



        function get_tinymce_content(editor_id) {

            if (jQuery('#wp-' + editor_id + '-wrap').hasClass('tmce-active') && tinyMCE.get(editor_id)) {
                return tinyMCE.get(editor_id).getContent();
            } else {
                return jQuery('#' + editor_id).val();
            }

            return -1;
        }







    });

})(jQuery);