<?php

/*
  Plugin Name: Magic Page
  Plugin URI: http://www.magicpageplugin.com
  Description: The Worlds Best Mass Page Creator
  Version: 6.7
  Author: Futureproof Profit
 */


if (!defined('ABSPATH')) {
    exit;
}
 
if (!defined('MAGIC_PAGE_VERSION')) {
    define('MAGIC_PAGE_VERSION', '6.7');
}

$magic_page_file = __FILE__;
$magic_page_dir = plugin_dir_path($magic_page_file);
$magic_page_url = plugin_dir_url($magic_page_file);

define('MAGIC_PAGE_FILE', $magic_page_file);
define('MAGIC_PAGE_DIR', $magic_page_dir);
define('MAGIC_PAGE_URL', $magic_page_url);

require_once $magic_page_dir . 'vendor/autoload.php';

require_once $magic_page_dir . 'src/functions.php';
require_once $magic_page_dir . 'src/admin/schema-builder/base.php';
require_once $magic_page_dir . 'src/process-json-ld-schema.php';
require_once $magic_page_dir . 'src/magic-page-query.php';
require_once $magic_page_dir . 'src/footprint-minifier-functions.php';
require_once $magic_page_dir . 'src/api.php';
require_once $magic_page_dir . 'src/admin/metaboxes.php';
require_once $magic_page_dir . 'src/admin/installation.php';
require_once $magic_page_dir . 'src/admin/seo-metabox.php';
require_once $magic_page_dir . 'src/admin/xfields-metabox.php';
require_once $magic_page_dir . 'src/admin/settings.php';
require_once $magic_page_dir . 'src/config.php';
require_once $magic_page_dir . 'src/magic-page-public.php';
require_once $magic_page_dir . 'src/shortcodes-filters.php';
require_once $magic_page_dir . 'src/magic-page-spintax.php';
require_once $magic_page_dir . 'src/admin/spintax-metabox.php';
require_once $magic_page_dir . 'src/admin/location-sets.php';
require_once $magic_page_dir . 'src/cron-functions.php';
require_once $magic_page_dir . 'src/admin/upgrade.php';
require_once $magic_page_dir . 'src/admin/sitemap.php';
require_once $magic_page_dir . 'src/admin/export/base.php';
require_once $magic_page_dir . 'src/admin/import-export.php';
require_once $magic_page_dir . 'src/admin/page-post-list.php';
require_once $magic_page_dir . 'src/admin/dashboard.php';
require_once $magic_page_dir . 'src/magic-page-sitemap.php';

require_once $magic_page_dir . 'src/admin/autosuggest.php';
require_once $magic_page_dir . 'src/autosuggest.php';

 


class MagicPage {

    public function __construct() {
        add_action('init', array($this, 'init'));

        register_activation_hook(__FILE__, array($this, 'magic_page_cron_activation'));
        register_deactivation_hook(__FILE__, array($this, 'magic_page_cron_deactivation'));
    }

    function init() {
        global $magic_page_magicpage, $magic_page_location;

        // clear old hook and activate the new type
        if (!get_option('magic_page_cron_service_scheduled')) {
            wp_clear_scheduled_hook('magic_page_cron_service');
            $this->magic_page_cron_activation();
            update_option('magic_page_cron_service_scheduled', 1);
        }

        register_taxonomy('_magic_page_temp', array('revision'));

        if (!is_magic_page_installed() || !function_exists('magicpage_check_maybe_id')) {
            return false;
        }

        if (!is_admin()) {
            $post_type_name = magic_page_plugin_post_name();
            $magic_page_magicpage['labels']['name'] = $post_type_name;
            $magic_page_magicpage['labels']['singular_name'] = $post_type_name;
        }

        $slug = magic_page_default_post_slug();

        $post = get_posts(
                array(
                    'posts_per_page' => 1,
                    'post_type' => $slug,
                    'post_status' => array('any')
        ));

        // allow just one post
        if ($post) {
            $magic_page_magicpage['capabilities'] = array(
                'create_posts' => false,
            );
            $magic_page_magicpage['map_meta_cap'] = true;
        }

        register_post_type($slug, $magic_page_magicpage);
        register_taxonomy('location', array($slug), $magic_page_location);
    }

    public function magic_page_cron_activation() {
        if (!wp_next_scheduled('magic_page_cron_service')) {
            wp_schedule_event(time(), 'daily', 'magic_page_cron_service');
        }
    }

    public function magic_page_cron_deactivation() {
        delete_option('_magic_page_key_error');
        wp_clear_scheduled_hook('magic_page_cron_service');
    }

}

new MagicPage();