<?php
if (!defined('ABSPATH')) {
    exit;
}

class MagicPageLocationsAutosuggestAdmin {

    public function __construct() {
        add_action('admin_enqueue_scripts', array($this, 'admin_scripts'));
        add_action('admin_menu', array($this, 'autosuggest_settings_page'));
    }

    public function admin_scripts() {
        wp_register_style('select2css', plugins_url('/css/select2.min.css', MAGIC_PAGE_FILE), false, '1.0', 'all');
        wp_enqueue_style('select2css');

        wp_register_script('select2', plugins_url('/js/select2.full.min.js', MAGIC_PAGE_FILE), array('jquery'), '1.0', true);
        wp_enqueue_script('select2');

        wp_enqueue_style('magic-page-tinglecss', plugins_url('/css/tingle.min.css', MAGIC_PAGE_FILE), array(), MAGIC_PAGE_VERSION);
        wp_enqueue_script('magic-page-tinglejs', plugins_url('/js/tingle.min.js', MAGIC_PAGE_FILE), array('jquery'), MAGIC_PAGE_VERSION);

        if (!defined('MAGIC_PAGE_AUTOSUGGEST_ADMIN_PAGE')) {
            define('MAGIC_PAGE_AUTOSUGGEST_ADMIN_PAGE', plugins_url('/js/autosuggest-admin-min.js', MAGIC_PAGE_FILE));
        }

        wp_enqueue_script('magic-page-autosuggest-admin', MAGIC_PAGE_AUTOSUGGEST_ADMIN_PAGE, array('magic-page-tinglejs', 'select2'), MAGIC_PAGE_VERSION);
    }

    public function autosuggest_settings_page() {

        add_submenu_page(
                'edit.php?post_type=magicpage',
                'Location Search',
                'Location Search',
                'manage_options',
                'magicpage-locations-autosuggest',
                array($this, 'add_autosuggest_settings')
        );
    }

    public function add_autosuggest_settings() {
        ?>
        <div class="wrap">
            <?php
            if (isset($_POST['savechanges'])) {

                $options = array(
                    '_magicpage_as_maxresults',
                    '_magicpage_as_key_delay',
                    '_magicpage_as_def_text',
                    '_magicpage_as_empty_text',
                    '_magicpage_as_chars',
                    '_magicpage_as_style_preset',
                );

                $selected_search_fields = [];
                if (isset($_POST['_magicpage_as_input_id'])) {
                    $selected_search_fields = array_map('strip_tags', array_unique($_POST['_magicpage_as_input_id']));
                }
                update_option('_magicpage_as_input_id', $selected_search_fields);

                $selected_categories = [];
                if (isset($_POST['_magicpage_as_categories'])) {
                    $selected_categories = array_map('strip_tags', $_POST['_magicpage_as_categories']);
                }
                update_option('_magicpage_as_categories', $selected_categories);

                foreach ($options as $saveOptions) {
                    if (isset($_POST[$saveOptions])) {
                        update_option($saveOptions, $_POST[$saveOptions]);
                    }
                }
                
                if(isset($_POST['_magicpage_as_hide_search']) && $_POST['_magicpage_as_hide_search']== '1') {
                    update_option('_magicpage_as_hide_search', 'true');
                } else {
                    delete_option('_magicpage_as_hide_search');
                }
                
                ?>

                <div id="message" class="updated fade"><p>Magic Page Location Search Settings updated.</p></div>
                <?php
            }

            $categories = get_option('_magicpage_as_categories', ['post', 'page', 'magicpage']);
            $inputs = get_option('_magicpage_as_input_id', []);
            ?>

            <h2>Magic Page Location Search Settings</h2>
            <div class="postbox">
                <div class="inside">
                    <form action="" method="post">
        <?php wp_nonce_field('update-options'); ?>
                        <table class="form-table">
                            <tr valign="top">
                                <th scope="row">Search Inputs (class & id)</th>
                                <td>
                                    <select style="width:30%; min-width: 150px; max-width:400px"
                                            id="_magicpage_as_input_id" name="_magicpage_as_input_id[]" multiple="multiple">
        <?php
        if ($inputs) {
            foreach ($inputs as $input) {
                echo ('<option selected="selected" value="' . $input . '">' . $input . '</option>');
            }
        }
        ?>
                                    </select>

                                    <span class="button button-secondary" id="add-search-field">Add Search Field via Live Picker</span> <br />
                                    Input manually class or id for the search input you would like to transform or use the Live Picker to select the search field.

                                </td>
                            </tr>
                            <tr valign="top">
                                <th scope="row">Max Results</th>
                                <td><input type="text" value="<?php echo get_option('_magicpage_as_maxresults', '5'); ?>" id="_magicpage_as_maxresults" name="_magicpage_as_maxresults"/>
                                    <br />
                                    Maximum number of suggested results (5 by default). </td>
                            </tr>

                            <tr valign="top">
                                <th scope="row">Post Types</th>
                                <td>
                                    <div style="padding-bottom:10px">
                                        <label><input <?php if (in_array('magicpage', $categories)) { ?>checked="checked"<?php } ?> type="checkbox" value="magicpage"  name="_magicpage_as_categories[]" />Magic Pages</label> &nbsp;
                                        <label><input <?php if (in_array('post', $categories)) { ?>checked="checked"<?php } ?> type="checkbox" value="post" name="_magicpage_as_categories[]" />Posts</label> &nbsp;
                                        <label><input <?php if (in_array('page', $categories)) { ?>checked="checked"<?php } ?> type="checkbox" value="page" name="_magicpage_as_categories[]" />Pages</label> &nbsp;
                                    </div>

                                    Limit the search within the chosen post types  

        <?php if (empty($categories)) {
            ?><div style="color:red; padding-top:10px">Notice: please pick at least one post type</div>
                                        <?php }
                                    ?>

                                </td>
                            </tr>
                            </tr>  
                            <tr valign="top">
                                <th scope="row">Search delay</th>
                                <td><input type="text" value="<?php echo get_option('_magicpage_as_key_delay', '100'); ?>" id="_magicpage_as_key_delay" name="_magicpage_as_key_delay" />
                                    <br />
                                    Number of milliseconds (ms) to delay after a keydown on the input search field and before search is started. Default is 100 ms.</td>
                            </tr>
                            <tr valign="top">
                                <th scope="row">Auto Hide Search Button</th>
                                <td>
                                    <div style="padding-bottom:10px">
                                        <label><input <?php if (get_option('_magicpage_as_hide_search', 'false') == 'true') { ?>checked="checked"<?php } ?> type="checkbox" value="1"  name="_magicpage_as_hide_search" />Auto Hide Search Button</label> &nbsp;
                                    </div>

                                    Attempts to auto hide the search button. For this to work the search field and the search button must be inside a form 


                                </td>
                            </tr>
                            <tr valign="top">
                                <th scope="row">Minimum Characters</th>
                                <td><select name="_magicpage_as_chars" id="_magicpage_as_chars">
                                        <option value="1" <?php echo (get_option('_magicpage_as_chars') == '1' ? ' selected="selected"' : '') ?>>One character</option>
                                        <option value="2" <?php echo (get_option('_magicpage_as_chars', '2') == '2' ? ' selected="selected"' : '') ?>>Two characters</option>
                                        <option value="3" <?php echo (get_option('_magicpage_as_chars') == '3' ? ' selected="selected"' : '') ?>>Three characters</option>
                                        <option value="4" <?php echo (get_option('_magicpage_as_chars') == '4' ? ' selected="selected"' : '') ?>>Four characters</option>
                                        <option value="5" <?php echo (get_option('_magicpage_as_chars') == '5' ? ' selected="selected"' : '') ?>>Five characters</option>
                                    </select><br />Minimum number of characters that must be entered into the Search input field before the search begins.</td>
                            </tr>

                            <tr valign="top">
                                <th scope="row">Default search text</th>
                                <td><input type="text" value="<?php echo get_option('_magicpage_as_def_text', 'Search...'); ?>" id="_magicpage_as_def_text" name="_magicpage_as_def_text" />
                                    <br />Default text to display in the Search input field. </td>
                            </tr>

                            <tr valign="top">
                                <th scope="row">No results text </th>
                                <td><input type="text" value="<?php echo get_option('_magicpage_as_empty_text', 'No Results Found'); ?>" id="_magicpage_as_empty_text" name="_magicpage_as_empty_text" />
                                    <br />
                                    Text which is shown if there aren't any results to display. </td>
                            </tr>


                            <tr valign="top">
                                <th scope="row">Theme preset</th>

        <?php
        $themes = ['base', 'black-tie', 'blitzer', 'cupertino', 'dark-hive', 'dot-luv',
            'eggplant', 'excite-bike', 'flick', 'hot-sneaks', 'humanity', 'le-frog',
            'mint-choc', 'overcast', 'pepper-grinder', 'redmond', 'smoothness',
            'south-street', 'start', 'sunny', 'swanky-purse', 'trontastic', 'ui-darkness', 'ui-lightness',
            'vader',
        ];
        ?>

                                <td><select name="_magicpage_as_style_preset" id="_magicpage_as_style_preset">
                                <?php
                                foreach ($themes as $theme) {
                                    $value = get_option('_magicpage_as_style_preset', 'base');
                                    $label = ucwords(str_replace('-', ' ', $theme));
                                    ?>
                                            <option value="<?php echo($theme); ?>" <?php echo ($value == $theme ? ' selected="selected"' : '') ?>><?php echo($label) ?></option>
                                        <?php } ?>
                                    </select><br />
                                </td>
                            </tr>
                            <tr valign="top">
                                <th scope="row">Example </th>
                                <td>
                                    HTML Code for Search Box  <br />
                                    <pre>
<code style="display:block">&lt;input type="search" id="search-field" /&gt; </code>
                                    </pre>
                               CSS Code to style it     
                                    
                                    <pre><code style="display:block">#search-field {
  width: 100%;
  padding: 12px;
  border: 6px solid #ccc;
  border-radius: 25px;
  resize: vertical;
}</code></pre>            
                                    
                                </td>
                            </tr>

                        </table>
                        <p class="submit"><input class="button button-primary" type="submit" name="savechanges" value="Save Changes" /></p>
                    </form>
                </div>
            </div>
        </div>



        <input data-nonce="<?php echo(esc_attr(wp_create_nonce('autosuggest'))); ?>" id="selectedSearchFields" type="hidden" value='<?php echo(json_encode(get_option('magic_page_search_fields'))); ?>' />


        <div style="display: none" id="load-live-picker">
            <div class="live-picker-box">
                <label>Enter URL
                    <input type="text" class="url-picker" />
                </label>
                <span class="button button-primary load-iframe-url">Load URL</span>
                &nbsp;<span data-url="<?php echo(get_home_url()); ?>" class="load-homepage-url">Click to load homepage</span>
            </div>
        </div>



        <?php
    }

}

new MagicPageLocationsAutosuggestAdmin();
