<?php

if (!defined('ABSPATH')) {
    exit;
}

class MagicPageDashboard {

    private $cacheTime = DAY_IN_SECONDS;
    private $key = 'magicpage_special_offer';

    public function __construct() {
        add_action('admin_head', array($this, 'admin_scripts'));
        add_filter('activity_box_end', array($this, 'view'), 1, 2);
        add_filter('dashboard_glance_items', array($this, 'custom_glance_items'), 1, 2);
        add_action('wp_ajax_magic_page_get_dashboard_data', array($this, 'getData'));
    }

    public function custom_glance_items($items = array()) {
        $magicpage = get_magic_page();

        if ($magicpage) {
            $link = get_edit_post_link($magicpage);
            $items[] = '<a href="' . $link . '">Magic Page: ' . get_the_title($magicpage) . '</a>';
        }

        return $items;
    }

    public function getApiData() {

        global $magicPageApi;

        $data = $magicPageApi->get_offers();

        if ($data && $data['status'] === 'success' && $data['data']) {
            set_transient($this->key, $data['data'], $this->cacheTime);
        }


        return wp_send_json_success($data);
    }

    public function view() {

        $offers = json_encode(get_transient($this->key));

        echo("<div data-offers='" . $offers . "' id='magicpage-discounts'><div id='discounts-left'>Magic Page Discount Coupons <span title='Get Newest Offers' id='refresh-magicpage-offers' class='dashicons dashicons-update-alt'></span></div><div id='discounts-right'></div></div>");
    }

    public function getData() {
        if (current_user_can('administrator')) {

            $offers = $this->getApiData();

            wp_send_json_success($offers);
        }

        wp_send_json_error();
    }

    public function admin_scripts() {
        global $pagenow;

        if ($pagenow && $pagenow === 'index.php') {
            global $magic_page_file;

            wp_enqueue_style('magic-page-dashboard', plugins_url('/css/magic-page-dashboard.css', MAGIC_PAGE_FILE), array(), MAGIC_PAGE_VERSION);
            wp_enqueue_script('magic-page-dashboard', plugins_url('/js/magic-page-dashboard.js', $magic_page_file), array('jquery'), MAGIC_PAGE_VERSION);
        }
    }

}

new MagicPageDashboard();
