<?php
if (!defined('ABSPATH')) {
    exit;
}

class MagicPageExportCompatability {

    public function setup_meta_boxes() {
      if(getMagicPageEditor() === 'gutenberg') {
            add_meta_box('post_submitbox_misc_actions', ' Export to Static & DBB', array($this, 'postActions'), array(magic_page_default_post_slug()), 'side', 'low');
        }
    }

    public function __construct() {
        add_action('admin_init', array($this, 'setup_meta_boxes'));

        add_action('init', array($this, 'add_exporttemplates'), 99);
        add_action('post_submitbox_misc_actions', array($this, 'postActions'), 1);
        add_action('post_submitbox_misc_actions', array($this, 'postActionsExtraInfo'));
        add_action('wp_ajax_get_pdf_export_compatability', array($this, 'getCompatability'));
        add_action('wp_ajax_set_pdf_export_compatability_ok', array($this, 'setCompatabilityOk'));
        add_action('wp_ajax_count_posts_in_group', array($this, 'countPostsInGroup'));

        require_once MAGIC_PAGE_DIR . 'src/admin/export/export-to-static.php';

        if ($this->getIsCompatible()) {
            if (MAGIC_PAGE_DIR) {
                require_once MAGIC_PAGE_DIR . 'src/admin/export/export.php';
            }
        }

        add_action('wp_query', array($this, 'installMagicPageExportTemplates'));

        if (isset($_GET['disableEditor'])) {
            if ($_GET['disableEditor'] == 'true') {
                add_filter('use_block_editor_for_post', '__return_false', 10);
            }
        } else {
            add_action('load-post.php', array($this, 'postLoaded'));
        }
        add_filter('use_block_editor_for_post_type', array($this, 'disableGutenberg'), 10, 2);
        
    }

    protected function countGroupPosts($groupId, $postType = false) {

        global $wpdb;

        $postTypeSql = $postType ? "AND posts.post_type = '$postType' " : '';

        $count = $wpdb->get_row("SELECT COUNT(posts.ID) AS CNT FROM $wpdb->posts posts
                                         LEFT JOIN $wpdb->postmeta postmeta ON ( posts.ID = postmeta.post_id )
                                     WHERE postmeta.meta_key = '_group_id' AND postmeta.meta_value = '$groupId' $postTypeSql");

        return intval($count->CNT);
    }

    public function countPostsInGroup() {

        if (current_user_can('administrator')) {
            $response = $this->processGroups();
            wp_send_json_success($response);
        }

        wp_send_json_error();
    }

    protected function processGroups() {
        $response = array();

        $optiongroups = get_option('_magic_page_groups', array());

        if ($optiongroups) {
            $update = false;

            foreach ($optiongroups as $id => $optiongroup) {
                $posts = $this->countGroupPosts($id, 'post');
                $pages = $this->countGroupPosts($id, 'page');
                $total = $posts + $pages;

                $date = $optiongroup['dateCreated'];
                $date->modify('+1 week');

                // delete unused groups older than a week
                if ($total <= 0 && $date <= new \DateTime()) {

                    $update = true;
                    unset($optiongroups[$id]);
                } else {
                    $response[$id] = array('posts' => $posts, 'pages' => $pages, 'total' => $total);
                }
            }
            if ($update) {
                update_option('_magic_page_groups', $optiongroups);
            }
        }

        return $response;
    }


    public function disableGutenberg($current_status, $post_type) {
        if ($post_type == 'exporttemplate') {
            return false;
        }

        return $current_status;
    }

    public function postLoaded() {
        if (isset($_GET['post'])) {

            if (isset($_GET['action']) && $_GET['action'] === 'trash') {
                return true;
            }

            $post_id = $_GET['post'];
            $post = get_post($post_id);
            if ($post && get_post_meta($post->ID, 'template', true)) {
                $url = html_entity_decode(add_query_arg('disableEditor', 'true', get_edit_post_link($post)));

                wp_redirect($url);
                exit();
            }
        }
    }

    public function postActionsExtraInfo() {
        $id = isset($_GET['post']) && $_GET['post'] ? intval($_GET['post']) : '';

        if ($id) {
            $post = get_post($id);
            $templateId = get_post_meta($post->ID, 'template', true);
            $homepageId = get_post_meta($post->ID, 'homepage', true);

            if ($templateId) {
                $link = get_edit_post_link($templateId);
                $title = get_the_title($templateId);
                ?> <div class="misc-pub-section dashicons-before dashicons-admin-customizer"> Bundled with Template: <a href="<?php echo($link) ?>"><?php echo($title) ?></a> </div><?php
            } elseif ($homepageId) {
                $link = get_edit_post_link($homepageId);
                $title = get_the_title($homepageId);
                ?> <div class="misc-pub-section dashicons-before dashicons-welcome-widgets-menus"> Bundled with Homepage: <a href="<?php echo($link) ?>"><?php echo($title) ?></a> </div><?php
            }
        }
    }

    public function installTemplatesInitial() {
        if (!get_option('_magic_page_installed_templates')) {
            $post = get_magic_page();
            if ($post) {
                $this->installMagicPageExportTemplates();
                update_option('_magic_page_installed_templates', '1.0');
            }
        }
    }

    public function installMagicPageExportTemplates() {

        $templateFolders = glob(MAGIC_PAGE_DIR . 'src/admin/export/templates/*', GLOB_ONLYDIR);
        foreach ($templateFolders as $folder) {
            $exportTemplateSetttings = json_decode(file_get_contents($folder . '/config.json'));

            $installed = get_option('_magic_page_export_themes', array());

            if ($installed && $exportTemplateSetttings && isset($installed[$exportTemplateSetttings->id])) {
                continue;
            }

            // insert homepage as a regular page
            $templateHomepageId = wp_insert_post(array(
                'post_title' => $exportTemplateSetttings->name . ' Homepage',
                'post_content' => file_get_contents($folder . '/homepage.html'),
                'post_author' => get_current_user_id(),
                'post_status' => 'publish',
                'post_type' => 'page',
            ));

            $templatePostId = wp_insert_post(array(
                'post_title' => $exportTemplateSetttings->name,
                'post_content' => file_get_contents($folder . '/base.html'),
                'post_author' => get_current_user_id(),
                'post_status' => 'publish',
                'post_type' => 'exporttemplate'
            ));

            add_post_meta($templatePostId, 'homepage', $templateHomepageId);
            add_post_meta($templateHomepageId, 'template', $templatePostId);

            // insert settings data (inc js, css, img)
            update_post_meta($templatePostId, '_settings', $exportTemplateSetttings);

            // insert non-readonly css & js
            foreach ($exportTemplateSetttings->css as $css) {
                if (!in_array($css, $exportTemplateSetttings->readonly)) {
                    $contents = file_get_contents($folder . '/css/' . $css);
                    update_post_meta($templatePostId, 'exporttemplate_stylesheet_' . $css, $contents);
                }
            }

            foreach ($exportTemplateSetttings->js as $js) {
                if (!in_array($js, $exportTemplateSetttings->readonly)) {
                    $contents = file_get_contents($folder . '/js/' . $js);
                    update_post_meta($templatePostId, 'exporttemplate_javascript_' . $js, $contents);
                }
            }

            $installed['id'] = $exportTemplateSetttings;
            update_option('_magic_page_export_themes', $installed);

            if (file_exists($folder . '/schema.html')) {
                $contents = file_get_contents($folder . '/schema.html');
                update_post_meta($templatePostId, 'schema', $contents);
            }
        }
    }

    public function setCompatabilityOk() {
        if (current_user_can('administrator')) {
            $this->setIsCompatible();

            wp_send_json_success();
        }
        wp_send_json_error();
    }

    public function getIsCompatible() {
        return get_option('_pdf_export_compatible_' . cleanUrl(get_site_url()));
    }

    public function setIsCompatible() {
        return update_option('_pdf_export_compatible_' . cleanUrl(get_site_url()), 1);
    }

    public function getCompatability() {
        if (current_user_can('administrator')) {

            $compatability = array();

            if (is_wp_content_folder_writable()) {
                $compatability['ok']['writable'] = 'Is writable';
            } else {
                $compatability['notok']['writable'] = 'Not writable';
            }

            if (version_compare(PHP_VERSION, '5.4.0') >= 0) {
                $compatability['ok']['php'] = PHP_VERSION;
            } else {
                $compatability['notok']['php'] = 'Not Installed';
            }

            if (extension_loaded('gd') && function_exists('gd_info')) {
                $gd = gd_info();
                $compatability['ok']['gd'] = $gd['GD Version'];
            } else {
                $compatability['notok']['gd'] = 'Not Installed';
            }

            if (extension_loaded('mbstring')) {
                $version = phpversion('mbstring');
                if (!$version) {
                    $version = 'Supported';
                }

                $compatability['ok']['mbstring'] = $version;
            } else {
                $compatability['notok']['mbstring'] = 'Not Installed';
            }

            if (extension_loaded('dom')) {
                $version = phpversion('dom');
                if (!$version) {
                    $version = 'Supported';
                }

                $compatability['ok']['dom'] = $version;
            } else {
                $compatability['notok']['dom'] = 'Not Installed';
            }

            wp_send_json_success($compatability);
        }
        wp_send_json_error();
    }

    protected function getOptionBoxClassStatic($id) {
        $settings = $this->getExportStaticSettings();

        $class = 'collapsed';
        if ($settings && isset($settings['options'][$id])) {
            $class = $settings['options'][$id];
        }

        return $class;
    }

    public function getOptionBoxStatic($id, $title, $content, $data = '') {
        $class = $this->getOptionBoxClassStatic($id);

        $dataType = '';

        if ($data && $data['type']) {
            $dataType = ' data-type="' . $data['type'] . '" ';
            $class .= ' showonly-if-post_type';
        }
        ?>
        <div id="option-box-<?php echo($id) ?>" <?php echo($dataType) ?> data-id="<?php echo($id) ?>" class="box <?php echo($class) ?>">
            <div class="header"><?php
        echo($title);
        $this->collapseExpandButtons();
        ?>
            </div>
            <div class="content">
        <?php echo($content) ?>
            </div>
        </div><?php
    }

    protected function getOptionBoxClass($id) {
        $settings = $this->getSettings();

        $class = 'collapsed';
        if ($settings && isset($settings['options'][$id])) {
            $class = $settings['options'][$id];
        }

        return $class;
    }

    public function getOptionBox($id, $title, $content, $data = '') {
        $class = $this->getOptionBoxClass($id);

        $dataType = '';

        if ($data && $data['type']) {
            $dataType = ' data-type="' . $data['type'] . '" ';
            $class .= ' showonly-if-post_type';
        }
        ?>
        <div id="option-box-<?php echo($id) ?>" <?php echo($dataType) ?> data-id="<?php echo($id) ?>" class="box <?php echo($class) ?>">
            <div class="header"><?php
        echo($title);
        $this->collapseExpandButtons();
        ?>
            </div>
            <div class="content">
        <?php echo($content) ?>
            </div>
        </div><?php
    }

    public function collapseExpandButtons() {
        ?>
        <span class="collapse-expand-icon collapse"><svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 400 400"><path fill="#6b6b6b" d="M394.1 178.1c0-9.8-8-17.7-17.8-17.7H24.8c-9.8 0-17.7 8-17.7 17.7v39c0 9.7 7.9 17.7 17.7 17.7h351.5c9.8 0 17.8-7.9 17.8-17.8v-38.9zM294.4 28.8c0-11.2-9.1-20.3-20.4-20.3h-32.2V8h-82.5v.6h-32.2a20.4 20.4 0 0 0-19.6 25.3c1.4 5.5 5 9 8.8 12.7l29.9 30 34 34.1c9.5 9.5 22.8 17.7 34.5 6h.1L255.2 76l24.6-24.7c6.7-6.4 14.6-12.6 14.6-22.6zm0 343.4c0-9.9-7.6-15.8-13.9-22.2l-25.1-25.2-40.4-40.4h-.1c-11.7-11.7-25-3.6-34.5 6l-34 34-29.9 30c-3.3 3.4-6.8 6.4-8.4 11.1-4.5 13 5.5 27 19.2 27h32.2v.6H242v-.6h32.2a20.4 20.4 0 0 0 20.2-20.3z"/></svg></span>
        <span class="collapse-expand-icon expand"><svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 400 400">
                <path fill="#6b6b6b" d="M281.1 82.938c0-8.5-6.5-13.7-12-19.1l-21.7-21.7-34.8-34.8h-.1c-10.1-10.1-21.5-3.1-29.8 5.1l-29.3 29.3-25.8 25.8c-2.9 2.9-5.8 5.5-7.2 9.6-3.9 11.2 4.7 23.2 16.6 23.2h27.8v158.5c0 13.7-.4 27.5 0 41.3H137c-18.2 0-21.3 20.6-10.5 31.6 6.4 6.5 12.9 12.9 19.3 19.3 14 14.1 28.1 28.1 42.1 42.2h.1c10.1 10 21.6 3.1 29.8-5.1 9.8-9.8 19.6-19.6 29.3-29.4l25.8-25.8c2.9-2.9 5.8-5.5 7.2-9.5 3.9-11.2-4.7-23.2-16.6-23.2h-27.8v-158.5c0-13.7.4-27.5 0-41.2h27.8c9.7-.1 17.6-7.9 17.6-17.6z"/>
            </svg>
        </span><?php
    }

    public function option($value, $setting) {
        $return = 'value="' . $setting . '"';

        if (is_array($value)) {
            if (in_array($setting, $value)) {
                $return .= ' selected="selected"';
            }
        } else {
            if ($value == $setting) {
                $return .= ' selected="selected"';
            }
        }

        return $return;
    }

    private function getSetting($key) {
        $settings = $this->getSettings();

        return isset($settings[$key]) ? $settings[$key] : '';
    }

    private function getExportSetting($key) {
        $settings = $this->getExportSettings();

        return isset($settings[$key]) ? $settings[$key] : '';
    }

    public function postActions() {
           
        if (!current_user_can('administrator')) {
            return false;
        }


        global $post;

        if ($post->post_type != 'magicpage') {
            return false;
        }

        $compatible = $this->getIsCompatible();

        $this->installTemplatesInitial();

        $templates = get_posts(array(
            'post_type' => 'exporttemplate',
            'post_status' => 'any',
        ));

        $settings = $this->getSettings();
        ?>
        <!-- export to static pages -->
        <div class="misc-pub-section misc-pub-post-export ">


            <span id="openExportToStaticPage__NEW">
                <span style="width: 16px;" class="magic-page-icon-export export-to-post-page-icon">
                    <svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px"
                         viewBox="0 0 32 32" enable-background="new 0 0 32 32" xml:space="preserve">
                        <path fill="#82878C" d="M32,8l-9.9-8v6C13.9,6.7,7.5,13.5,7.5,21.8c0,0.1,0,0.2,0,0.3C9,15.3,14.9,10,22.1,9.5v6.3
                              L32,8z"/>
                        <polygon fill="#82878C" points="29.3,16.6 29.3,18.2 29.3,29.3 2.7,29.3 2.7,2.7 18.9,2.7 18.9,0 0,0 0,32 32,32
                                 32,18.2 32,14.6 32,10.7 29.3,12.8 "/>
                    </svg>

                </span> Export to Posts & Pages
            </span>
        </div>


        <!-- export to static pages -->
        <?php
        $exportStaticSettings = $this->getExportStaticSettings();

        $spintaxDisabled = get_option('_magic_page_spintax_support') == 2 ? '<br /><b>Notice</b>: Spintax is disabled for regular posts. You should <b>Enable Spintax Support</b> from settings if you are using spintax. <br /><br />' : '';
        ?>


        <div id="magic-page-export-to-static">
            <div class="magic-page-export-feature">

        <?php echo($spintaxDisabled) ?>

                <div class="save-settings-holder" id="static-export-options">
                    <input type="hidden" id="static-export-settings" value='<?php echo(json_encode($exportStaticSettings)); ?>' />

        <?php
        $optiongroupHtml = '';
        $optiongroups = get_option('_magic_page_groups', array());

        if ($optiongroups) {
            foreach ($optiongroups as $id => $optiongroup) {
                $selected = $optiongroup['id'] == isset($exportStaticSettings['exportGroup']) && $exportStaticSettings['exportGroup'] ? ' selected="selected" ' : '';
                $optiongroupHtml .= "<option $selected value=" . $optiongroup['id'] . ">" . $optiongroup['title'] . "</option>";
            }
        }


        $exportGroupOptions = $exportStaticSettings['exportGroupOptions'];

        $groupClass = $exportGroupOptions ? '' : ' hidden ';

        echo $this->getOptionBoxStatic('export-group', 'Group', ' <div class="option-group">
                                <select style="width:400px" id="export-group"> ' . $optiongroupHtml . '</select>
                            <div id="export-group-existing-count" class="class7">
                            <span class="total">0</span> Total
                            <span class="small">  <span class="posts">0</span> Posts <span class="pages">0</span> Pages</span>

</div>
</div>          <div style="display:none" id="option-append-note">Existing <span class="chosen-output"></span> will not be altered, new <span class="chosen-output"></span> will be appended to already existing in Group <span class="chosen-output-group"></span>.</div>
                <div style="display:none" id="option-overwrite-note"> Existing <span class="chosen-output"></span> in Group <span class="chosen-output-group"></span> will overwritten if you are exporting them again. </div>
                <div style="display:none" id="option-delete-note"> All <span class="chosen-output"></span> in Group <span class="chosen-output-group"></span> will be deleted.</div>


                            <br /><select class="' . $groupClass . '" id="export-group-action">'
                . '<option ' . selected($exportGroupOptions, 'append', false) . ' value="append">Append</option>'
                . '<option ' . selected($exportGroupOptions, 'overwrite', false) . ' value="overwrite">Overwrite existing</option>'
                . ' <option ' . selected($exportGroupOptions, 'delete', false) . '  value="delete">Delete existing</option>
                    </select> <div class="export-mode-info-note"></div>

                           ', array('type' => 'magicpage'));

        $postsSelected = in_array('Posts', $exportStaticSettings['exportTo']) ? ' selected="selected" ' : '';
        $pagesSelected = in_array('Pages', $exportStaticSettings['exportTo']) ? ' selected="selected" ' : '';

        $postPagesSelect = '<div class="box-select2" style="float:left; margin-right:15px">'
                . '<label>Export To</label><select multiple="multiple" style="width:250px" id="export-to-post-pages">'
                . '<option ' . $postsSelected . '>Posts</option>'
                . '<option ' . $pagesSelected . '>Pages</option>'
                . '</select>'
                . '</div>';

        $selectCategoriesSettings = array(
            'hierarchical' => 1,
            'echo' => 0,
            'hide_empty' => 0,
            'name' => 'export-to-post-categories',
            'walker' => new SH_Walker_TaxonomyDropdown()
        );

        $hideCategoryBox = 'style="display:none"';
        $selectCategoriesSettings['selected_values'] = array();
        if (isset($exportStaticSettings['categories']) && $exportStaticSettings['categories']) {
            $selectCategoriesSettings['selected_values'] = $exportStaticSettings['categories'];
            $hideCategoryBox = '';
        }


        $selectCategories = wp_dropdown_categories($selectCategoriesSettings);

        $replace = "<select$1 style='width:400px; ' multiple='multiple'>";
        $selectCategories = preg_replace('#<select([^>]*)>#', $replace, $selectCategories);

        echo $this->getOptionBoxStatic('export-to', 'Export To', ' <div class="option-group">' . $postPagesSelect . '<div ' . $hideCategoryBox . ' class="box-select2"><label>Select Post Categories</label>' . $selectCategories . '</div>
                                                           </div>
                            ', array('type' => 'magicpage'));

        echo $this->getOptionBoxStatic('titles', 'Page Titles', ' <div class="option-group">
                                <div class="option-row">
                                    <input type="text" class="export-titles export-input" />
                                    <span class="delete"></span>
                                </div>
                            </div>
                            <span class="add-more-button">add more</span>', array('type' => 'magicpage'));

        echo $this->getOptionBoxStatic('urls', 'URL Options ', ' <div class="option-group">

                               <label> <input type="checkbox" value="1" id="autogenerate-url" />  Auto-generate based on title </label>
                               <br /><br />

                                <div class="option-row hidden">
                                    <input type="text" class="export-urls export-input" />
                                    <span class="delete"></span>
                                </div>
                            </div>
                            <span class="add-more-button">add custom</span>', array('type' => 'magicpage'));
        ?>

                    <div data-type="magicpage" data-id="location"
                         class="box <?php echo($this->getOptionBoxClassStatic('location')); ?> showonly-if-post_type">
                        <div class="header">Locations <?php
                    $this->collapseExpandButtons();
                    ?>
                        </div>
                        <div class="content">
                            <input placeholder="Search location by name or zip" style="width: 300px; margin-bottom:10px;" type="text" class="magic-page-locs" />
                            <div  class="class7 export-locations-count">
                                <span class="_num"></span>
                                <span class="_num_k"></span>

                                Locations</div>
                    <?php
                    $locations = isset($exportStaticSettings['locations']) ? $exportStaticSettings['locations'] : array();
                    ?> <div class="tagchecklist"> <?php
                    if ($locations) {
                        foreach ($locations as $locationData) {
                            $location = get_term_by('id', $locationData['id'], 'location');

                            if (!$location) {
                                continue;
                            }

                            $zip = get_term_meta($location->term_id, 'zip', true);
                            $region = get_term_meta($location->term_id, 'region', true);
                            $radius = $locationData['radius'];

                            echo('<span class="location-item"><button type="button" class="ntdelbutton"><span class="remove-tag-icon"></span></button> &nbsp; '
                            . '<input class="loc_hidden" type="hidden" value="' . $location->term_id . '" />' . $location->name . ', ' . $region . ',5 ' . $zip . ''
                            . '<input class="radius_field" title="Set Radius" type="text" value="' . $radius . '" /></span> <br />');
                        }
                    }
                    ?></div>
                        </div>
                    </div>

                            <?php
                            echo $this->getOptionBoxStatic('schedule', 'Schedule', ' <div class="option-group">
                    <div class="option-row">
<span class="option-text">
Publish <span class="adjust-link adjust-percent"><span class="publish-percent">100</span>%</span> of the pages <span class="adjust-link adjust-schedule">right now</span>
                   </span>
                        <span class="delete"></span>
                    </div>
                </div>

                <span class="add-more-button">Add Schedule</span>', array('type' => 'magicpage'));

                            $metaFields = get_post_meta($post->ID);

                            $selectedMetas = isset($exportStaticSettings['postmeta']) ? $exportStaticSettings['postmeta'] : array();

                            $removeFields = array(
                                '_edit_last',
                                '_autoinstalled_locations_count',
                                '_all_locations_count',
                                '_edit_lock',
                                '_attached_image_upload_feedback',
                                '_spintax_slug_saved',
                                '_wp_old_slug',
                                '_schema_builder_schema'
                            );

                            $selectOptionsHtml = '';
                            foreach ($metaFields as $id => $_unusedValue) {
                                if (strpos($id, '_magic') === false && !in_array($id, $removeFields)) {
                                    if (in_array($id, $selectedMetas)) {
                                        $selectOptionsHtml .= '<option value="'.$id.'" selected="selected">' . $id . "</option>";
                                    } else {
                                        $selectOptionsHtml .= '<option value="'.$id.'">' . $id . "</option>";
                                    }
                                }
                            }

                            $exportMetaFields = '<div class="box-select2" style="float:left; margin-right:15px">'
                                    . '<label>Export Custom Meta Fields</label><select multiple="multiple" style="width:550px" id="select-export-post-meta-fields">'
                                    . $selectOptionsHtml .
                                    '</select>'
                                    . '</div>';
                            echo $this->getOptionBoxStatic('export-post-meta-fields', 'Export Custom Meta Fields', ' <div class="option-group">' . $exportMetaFields . '<div class="hasLayout">Allows you to export custom meta fields often used by 3rd party plugins such as Elementor.
                     <br />
                   <div class="hidden" id="elementorFound">Elementor plugin was used. We are auto-enabling Elementor meta tags </div>


</div></div>

                            ', array('type' => 'magicpage'));
                            ?>

                </div>

                <div style="margin-bottom:10px" class="hide-while-exporting save-settings-holder">
                    <span class="save-settings">Save Settings</span>

                    <div id="saving-settings-progress" class="sk-fading-circle hidden">
                        <div class="sk-circle1 sk-circle"></div>
                        <div class="sk-circle2 sk-circle"></div>
                        <div class="sk-circle3 sk-circle"></div>
                        <div class="sk-circle4 sk-circle"></div>
                        <div class="sk-circle5 sk-circle"></div>
                        <div class="sk-circle6 sk-circle"></div>
                        <div class="sk-circle7 sk-circle"></div>
                        <div class="sk-circle8 sk-circle"></div>
                        <div class="sk-circle9 sk-circle"></div>
                        <div class="sk-circle10 sk-circle"></div>
                        <div class="sk-circle11 sk-circle"></div>
                        <div class="sk-circle12 sk-circle"></div>
                    </div>

                    <button id="export-to-static-pages" class="button button-primary">Export</button>
                </div>

                <div class="hidden export-progress">
                    <div class="hide-when-done-exporting export-in-progress">
                        <div id="exporting-files">Exporting  <b id="export-number"></b> magic pages to <span class="posts-pages-type-label"></span>. In overwrite mode actual count may be greater.</div>

                    </div>

                    <div style="float:right" class="hide-while-exporting export-done">
                        ALL DONE!</span>
                    </div>


                    <div class="progress-wrap progress hidden">
                        <div class="progress-percent"></div>
                        <div class="progress-bar progress"></div>
                    </div>
                </div>
                <div class="hidden" id="export-to-static-status"> </div>
            </div>


        </div>


        <!-- export to html/pdf -->

        <?php
        $unqiqueContent = $this->getSetting('uniqueContent') == '1' ? 'checked="checked"' : '';
        $metatags = $this->getSetting('generateMetatags') == '1' ? 'checked="checked"' : '';
        $canonical = $this->getSetting('generateCanonical') == '1' ? 'checked="checked"' : '';
        $minifyHtml = $this->getSetting('minifyHtml') == '1' ? 'checked="checked"' : '';
        $minifyCss = $this->getSetting('minifyCss') == '1' ? 'checked="checked"' : '';
        $redirect = $this->getSetting('redirect') == '1' ? 'checked="checked"' : '';

        $rewriteUrlStructure = $this->getSetting('rewriteUrlStructure') == '1' ? 'checked="checked"' : '';
        $installInSubdirectory = $this->getSetting('installInSubdirectory') == '1' ? 'checked="checked"' : '';
        $createSitemap = $this->getSetting('createSitemap') == '1' ? 'checked="checked"' : '';

        $rewriteUrls = $this->getSetting('rewriteUrls') == '1' ? 'checked="checked"' : '';
        $subdirectory = $this->getSetting('subdirectory');
        $url = $this->getSetting('url');
        $homepageId = $this->getSetting('homepage');

        $homepageOption = '';
        if ($homepageId) {
            $homepageOption = '<option selected="selected" value="' . $homepageId . '">' . get_the_title($homepageId) . '</option>';
        }

        $subdirectoryClass = $installInSubdirectory !== '' ? '' : 'hidden';
        //  $createSitemapClass = $createSitemap !== '' ? '' : 'hidden';

        $imageOptions = $this->getSetting('imageOptions');
        $imageFolderName = $this->getSetting('imageFolderName') ? $this->getSetting('imageFolderName') : 'images';

        $stylesheetOptions = $this->getSetting('stylesheetOptions');
        $stylesheetFilename = $this->getSetting('stylesheetFilename') ? $this->getSetting('stylesheetFilename') : 'style';

        $additionalPostPages = $this->getSetting('additionalPostPages');
        $manuallySelectedPostsPages = $this->getSetting('manuallySelectedPostsPages');
        $postPagesOption = '';

        if ($additionalPostPages && $manuallySelectedPostsPages) {
            $postPages = get_posts(array(
                'post_type' => array('page', 'post'),
                'post_status' => 'publish',
                'post__in' => $manuallySelectedPostsPages,
            ));
            if ($postPages) {
                foreach ($postPages as $page) {
                    $postPagesOption .= "<option selected=\"selected\" value=" . $page->ID . "> $page->post_title </option>";
                }
            }
        }
        ?>

        <div class="misc-pub-section curtime misc-pub-curtime">
            <span id="magicPageExport">
                <span class="magic-page-icon-export">

                    <svg xmlns="http://www.w3.org/2000/svg" viewBox="-13 15 32 32">
                        <path fill="none" d="M-14 14h582v402H-14z"/>
                        <g>
                            <path fill="#82878C" d="M13 30c-.2-2.2-2.2-4-4.5-4-1.1 0-2 .4-2.8 1a6.99 6.99 0 0 0-13.6 3.1A4.01 4.01 0 0 0-7 38h6v2h-6a6.02 6.02 0 0 1-3-11.2c.1-4.9 4.1-8.8 9-8.8 3.2 0 6.1 1.7 7.7 4.3.5-.2 1.2-.3 1.8-.3 2.8 0 5.2 1.8 6.1 4.2A6.02 6.02 0 0 1 13 40H7v-2h6c2.2 0 4-1.8 4-4s-1.8-4-4-4zM3 29l5 6H5v8H1v-8h-3l5-6z"/>
                        </g>
                    </svg>
                </span>Dynamic Backlink Builder
            </span>
        </div>





        <div id="magic-page-pdf-export">
            <div class="magic-page-export-feature">
                <div <?php if ($compatible) { ?>class="hidden"<?php } ?>  id="pdf-export-compatability-test">
                    We are testing the compatability with your server.
                    This feature will require: <br /><br />
                    <div class="requirement php"><span class="icon"></span>PHP version 5.4.0 or higher <span class="version-info"></span></div>
                    <div class="requirement dom"><span class="icon"></span>DOM extension <span class="version-info"></span></div>
                    <div class="requirement gd"><span class="icon"></span>GD extension <span class="version-info"></span></div>
                    <div class="requirement mbstring"><span class="icon"></span>MBString extension <span class="version-info"></span></div>

                    <div class="hidden" id="pdf-all-ok">
                        Congratulations! Your server is compatible with the requirements, you can now <br /><br />

                        <button id="pdf-export-success-continue" class="button button-primary button-large">Continue</button>
                    </div>

                    <div class="hidden" id="pdf-not-ok">
                        Your server is not compatible with the requirements, you cannot proceed
                    </div>
                </div>

                <div class="<?php if (!$compatible) { ?>hidden<?php } ?> save-settings-holder" id="pdf-export-options">

                    <input type="hidden" id="pdf-export-settings" value='<?php echo(json_encode($settings)); ?>' />

        <?php
        $unqiqueContent = $this->getSetting('uniqueContent') == '1' ? 'checked="checked"' : '';
        $metatags = $this->getSetting('generateMetatags') == '1' ? 'checked="checked"' : '';
        $canonical = $this->getSetting('generateCanonical') == '1' ? 'checked="checked"' : '';
        $minifyHtml = $this->getSetting('minifyHtml') == '1' ? 'checked="checked"' : '';
        $minifyCss = $this->getSetting('minifyCss') == '1' ? 'checked="checked"' : '';
        $redirect = $this->getSetting('redirect') == '1' ? 'checked="checked"' : '';

        $rewriteUrlStructure = $this->getSetting('rewriteUrlStructure') == '1' ? 'checked="checked"' : '';
        $installInSubdirectory = $this->getSetting('installInSubdirectory') == '1' ? 'checked="checked"' : '';
        $createSitemap = $this->getSetting('createSitemap') == '1' ? 'checked="checked"' : '';

        $rewriteUrls = $this->getSetting('rewriteUrls') == '1' ? 'checked="checked"' : '';
        $subdirectory = $this->getSetting('subdirectory');
        $url = $this->getSetting('url');
        $homepageId = $this->getSetting('homepage');

        $homepageOption = '';
        if ($homepageId) {
            $homepageOption = '<option selected="selected" value="' . $homepageId . '">' . get_the_title($homepageId) . '</option>';
        }

        $subdirectoryClass = $installInSubdirectory !== '' ? '' : 'hidden';
        //  $createSitemapClass = $createSitemap !== '' ? '' : 'hidden';

        $imageOptions = $this->getSetting('imageOptions');
        $imageFolderName = $this->getSetting('imageFolderName') ? $this->getSetting('imageFolderName') : 'images';

        $stylesheetOptions = $this->getSetting('stylesheetOptions');
        $stylesheetFilename = $this->getSetting('stylesheetFilename') ? $this->getSetting('stylesheetFilename') : 'style';

        $additionalPostPages = $this->getSetting('additionalPostPages');
        $manuallySelectedPostsPages = $this->getSetting('manuallySelectedPostsPages');
        $postPagesOption = '';

        if ($additionalPostPages && $manuallySelectedPostsPages) {
            $postPages = get_posts(array(
                'post_type' => array('page', 'post'),
                'post_status' => 'publish',
                'post__in' => $manuallySelectedPostsPages,
            ));
            if ($postPages) {
                foreach ($postPages as $page) {
                    $postPagesOption .= "<option selected=\"selected\" value=" . $page->ID . "> $page->post_title </option>";
                }
            }
        }

        $manualClass = is_array($additionalPostPages) && in_array('select', $additionalPostPages) ? '' : 'hidden';

        echo $this->getOptionBox('exportFormat', 'Output Options', 'Output format: <b class="chosenOutputFormat"></b><br />

                <div id="exportFormat" class="class0">
                    <div data-format="html" class="class1 format">
                       <svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 60 60">
                        <path fill="#2E2E2E" d="M39.2 2v12h12.2z"/>
                        <path fill="#5EA4D3" d="M51.4 15.4H37.8V2.1H10.1v30.4H36l6.6 17.9H10v7.9h41.2l.2-42.9z"/>
                        <path fill="#2E2E2E" d="M35 33.8H8V49h32.6L35 33.8zM15.1 45H14v-2.9h-3V45H9.8v-6.8h1.1v3h3v-3H15l.1 6.8zm6.2-5.8h-2.1v5.9h-1.1v-5.9H16v-.9h5.3v.9zm8 5.8h-1.1v-4.9L26.3 45h-.8l-1.9-5v5h-1.1v-6.8H24l1.9 5.2 1.9-5.2h1.4l.1 6.8zm1.7 0v-6.8h1.1v5.9h3.2v.9H31z"/>
                      </svg>
                    </div>
                    <div data-format="pdf" class="class1 format">
                        <svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 60 60">
                          <path fill="#444541" d="M55.5 16.4L45.4 4.7l-2-2.3H19.6a5.8 5.8 0 0 0-5.8 5.8v7.4h3.6v-6c0-1.8 1.5-3.3 3.2-3.3H41V16c0 3.6 2.9 6.5 6.5 6.5h7.1l-.4 27.8c0 1.8-1.5 3.2-3.2 3.3H20.4c-1.6 0-3-1.6-3-3.6v-2.4h-3.6v3.5c0 3.5 2.4 6.4 5.3 6.4H52c3.2 0 5.8-2.6 5.8-5.8V19.1l-2.3-2.7"/>
                          <path fill="#EA4D3B" d="M39.8 49.5H2.5V13.8h37.4v35.7M6.1 45.9h30.2V17.4H6.1"/>
                          <path fill="none" stroke="#EA4D3B" stroke-linejoin="round" stroke-miterlimit="10" stroke-width=".75" d="M21.6 19.2c.6.3.4.6.3 2.2-.2 1.7-.8 4.7-1.9 7.7-1.1 3-2.8 6-4.3 8.1-1.5 2.2-2.7 3.6-3.7 4.4-1 .8-1.6 1-2.1 1.1-.4.1-.6 0-.8-.2-.1-.2-.2-.6-.1-1s.3-.9 1-1.6c.7-.7 1.8-1.5 3.3-2.2 1.5-.7 3.5-1.4 5.1-1.9 1.6-.5 2.9-.7 4-.9 1.1-.2 2-.3 3-.4a16.37 16.37 0 0 1 5.2.4c.8.2 1.4.4 1.9.7.6.3 1 .8 1.3 1.3.2.5.2 1.1 0 1.5-.2.4-.6.7-1 .8-.5.1-1 .2-1.6 0-.6-.2-1.3-.6-2.1-1.1-.7-.5-1.5-1.2-2.6-2.2a45 45 0 0 1-3.4-3.8c-1.1-1.3-1.9-2.7-2.5-3.8-.6-1.1-.9-2-1.2-2.9-.3-.9-.5-1.8-.5-2.6-.1-.8 0-1.6 0-2.1.1-.6.2-1 .4-1.3.2-.3.5-.4.7-.5.2-.1.3-.1.4-.1.1 0 .2-.1.4 0 .4.2.6.4.8.4"/>
                        </svg>
                    </div>
                </div>

                <div class="class3 html-format-only">
                    <div class="prevent-click"></div>

                    <div class="heading">HTML FORMAT ONLY OPTIONS</div>
                    <div class="class6 checkbox-switch">
                         <label class="class4"><input ' . $installInSubdirectory . ' type="checkbox" value="1" id="install-in-subdirectory" /> Install in Subdirectory</label>
                         <input value="' . $subdirectory . '" class="class5 no-slashes ' . $subdirectoryClass . '" id="install-in-subdirectory-name" type="text" placeholder="Folder Name"  />
                    </div>

                    <div class="class6">
                        <label class="class4"><input ' . $createSitemap . ' id="create-xml-sitemap" type="checkbox" value="1" />  Create XML Sitemap</label>
                    </div>

                   <div id="website-url-box" class="class6 hidden">
                         <input value="' . $url . '" id="website-url" class="class5 no-slashes" type="text" placeholder="Website URL" />
                   </div>

                    <div class="class6">
                        <label class="class4">
                            <input ' . $rewriteUrls . ' id="rewrite-urls" id type="checkbox" value="1" /> Rewrite URLs with htaccess (removes the .html extension from the URLs)
                        </label>
                    </div>

                    <div class="class6">
                        <label class="class4">
                            <input ' . $redirect . ' id="redirect-missing-pages" id type="checkbox" value="1" /> 301 redirect to homepage on missing pages
                        </label>
                    </div>

                </div>

                ');
        ?>

                    <div data-type="magicpage" data-id="location"
                         class="box <?php echo($this->getOptionBoxClass('location')); ?> showonly-if-post_type">
                        <div class="header">Locations <?php
                    $this->collapseExpandButtons();
                    ?>
                        </div>
                        <div class="content">
                            <input placeholder="Search location by name or zip" style="width: 300px; margin-bottom:10px;" type="text" class="magic-page-locs" />
                            <div class="class7 export-locations-count">
                                <span class="_num"></span>
                                <span class="_num_k"></span>

                                Locations</div>
                    <?php
                    $locations = isset($settings['locations']) ? $settings['locations'] : array();
                    ?> <div class="tagchecklist"> <?php
                    if ($locations) {
                        foreach ($locations as $locationData) {
                            $location = get_term_by('id', $locationData['id'], 'location');

                            if (!$location) {
                                continue;
                            }

                            $zip = get_term_meta($location->term_id, 'zip', true);
                            $region = get_term_meta($location->term_id, 'region', true);
                            $radius = $locationData['radius'];

                            echo('<span class="location-item"><button type="button" class="ntdelbutton"><span class="remove-tag-icon"></span></button> &nbsp; '
                            . '<input class="loc_hidden" type="hidden" value="' . $location->term_id . '" />' . $location->name . ', ' . $region . ',5 ' . $zip . ''
                            . '<input class="radius_field" title="Set Radius" type="text" value="' . $radius . '" /></span> <br />');
                        }
                    }
                    ?></div>
                        </div>
                    </div>

        <?php
        $imageFolderClass = $imageOptions == 'export' ? '' : 'hidden';
        $stylesheetFilenameClass = $stylesheetOptions == 'export' ? '' : 'hidden';

        echo $this->getOptionBox('contentOptions', 'Content & Optimization', '<div>

              <div class="class3 html-format-only">
                    <div class="prevent-click"></div>

                                <div class="heading">HTML FORMAT ONLY OPTIONS</div>

                                 <div class="class6">
                                        <label>  Select Optional Homepage (index.html) </label> <br />
                                        <select style="width: 100%" id="homepage-option"> ' . $homepageOption . ' </select>
                                  </div>

                                  <div class="class6">
                                        <label class="class4">Image Options <br />
                                           <select id="image-options">
                                               <option ' . $this->option($imageOptions, 'export') . '>Export the images in /' . $imageFolderName . '/ folder</option>
                                               <option ' . $this->option($imageOptions, '1') . '>Keep original image source url</option>
                                               <option ' . $this->option($imageOptions, 'base64') . '>Embed images in the HTML with Base64</option>
                                               <option ' . $this->option($imageOptions, '') . '>Strip all images</option>
                                           </select>
                                       </label>

                                        <label id="image-folder-name-label" class="class4 ' . $imageFolderClass . '">Image Folder Name  <br />
                                            <input type="text" value="' . $imageFolderName . '" id="image-folder-name" />
                                        </label>
                                  </div>


                                  <div class="class6">
                                        <label class="class4">CSS Stylesheet Option<br />
                                           <select id="stylesheet-options">
                                               <option ' . $this->option($stylesheetOptions, 'export') . '>Export in stylesheet.css </option>
                                               <option ' . $this->option($stylesheetOptions, 'embed') . '>Embed in the HTML head section</option>
                                           </select>
                                       </label>

                                        <label id="stylesheet-file-name-label" class="class4 ' . $stylesheetFilenameClass . '">Filename <br />
                                            <input type="text" value="' . $stylesheetFilename . '" id="stylesheet-file-name" />
                                        </label>
                                  </div>


                                <div class="class6">
                                    <label>
                                        <input ' . $rewriteUrlStructure . ' type="checkbox" value="1" id="rewrite-url-structure" /> Rewrite the URL structure
                                    </label>
                                </div>

                                <div class="class6">
                                    <label>
                                        <input ' . $metatags . ' type="checkbox" value="1" id="generateMetatags" /> Generate Metatags (SEO Settings)
                                    </label>
                                </div>

                                <div class="class6">
                                    <label>
                                        <input ' . $minifyHtml . ' type="checkbox" value="1" id="minifyHtml" /> Minify HTML
                                    </label>
                                </div>

                                <div class="class6">
                                    <label>
                                        <input ' . $minifyCss . ' type="checkbox" value="1" id="minifyCss" /> Minify CSS
                                    </label>
                                </div>

                                <div class="class6">
                                    <label>
                                        <input ' . $canonical . ' type="checkbox" value="1" id="generateCanonical" /> Generate canonical link
                                    </label>

                                    <span class="option-not-available important-note">
                                        option not available
                                    </span>
                                    <br />

                                    <span id="important-note" class="important-note">
                                        Canonical link can be generated if <b>Website URL</b> is filled (see Output Options)
                                    </span>
                                </div>


                </div>
                <br />
                    <div class="class6">
                            <label>
                                <input ' . $unqiqueContent . ' type="checkbox" value="1" id="uniqueContent" /> Create Unique Spintax Shortcode & Random Shortcode Content
                            </label>
                    </div>

                    <div class="class6">
                        <label>Export Additional Posts & Pages </label> <br />
                        <select style="width: 100%" multiple="multiple" id="additional-postpage">
                            <option ' . $this->option($additionalPostPages, '0') . '>No Additional Posts and Pages</option>
                            <option ' . $this->option($additionalPostPages, 'post') . '>Include All Posts</option>
                            <option ' . $this->option($additionalPostPages, 'page') . '>Include All Pages</option>
                            <option ' . $this->option($additionalPostPages, 'select') . '>Manually Select</option>
                        </select>
                    </div>

                    <div id="manually-select-post-page" class="class6 ' . $manualClass . '">
                         <label> Manually Select Posts & Pages </label> <br />
                         <select multiple="multiple" style="width: 100%" id="manually-selected-posts-pages">
                            ' . $postPagesOption . '
                        </select>
                    </div>

                         </div>', array('type' => 'magicpage'));

        echo $this->getOptionBox('titles', 'Page Titles', ' <div class="option-group">
                                <div class="option-row">
                                    <input type="text" class="export-titles export-input" />
                                    <span class="delete"></span>
                                </div>
                            </div>
                            <span class="add-more-button">add more</span>', array('type' => 'magicpage'));

        echo $this->getOptionBox('filenames', 'Filenames', '<div class="option-group">
                                <div class="option-row">
                                    <input type="text" class="export-filenames export-input" />
                                    <span class="delete"></span>
                                </div>
                            </div>
                            <span class="add-more-button">add more</span>');

        $templateHtml = '<select style="width: 100%" id="exportTemplate">';
        foreach ($templates as $template) {
            $templateHtml .= '<option value="' . $template->ID . '">' . $template->post_title . '</option>';
        }

        $templateHtml .= '</select>';

        echo $this->getOptionBox('exportTemplate', 'Template', $templateHtml);

        echo $this->getOptionBox('searchAndReplaceText', 'Search and Replace Text', '<div id="search-replace-text" class="option-group">
                                <div class="option-row">
                                    <div class="half">
                                        <input type="text" placeholder="Search (Example: https://mysite.com)" class="search max-width placeholder" />
                                    </div>
                                    <div class="half">
                                        <input type="text" placeholder="Replace with (Example: https://mysite.net)" class="replace max-width placeholder" />
                                    </div>
                                    <span class="delete"></span>
                                </div>
                            </div>
                            <span class="add-more-button">add more</span>');
        ?>

                </div>

                <div style="margin-bottom:10px" class="hide-while-exporting save-settings-holder <?php if (!$compatible) { ?>hidden<?php } ?>">
                    <span class="save-settings">Save Settings</span>

                    <div id="saving-settings-progress" class="sk-fading-circle hidden">
                        <div class="sk-circle1 sk-circle"></div>
                        <div class="sk-circle2 sk-circle"></div>
                        <div class="sk-circle3 sk-circle"></div>
                        <div class="sk-circle4 sk-circle"></div>
                        <div class="sk-circle5 sk-circle"></div>
                        <div class="sk-circle6 sk-circle"></div>
                        <div class="sk-circle7 sk-circle"></div>
                        <div class="sk-circle8 sk-circle"></div>
                        <div class="sk-circle9 sk-circle"></div>
                        <div class="sk-circle10 sk-circle"></div>
                        <div class="sk-circle11 sk-circle"></div>
                        <div class="sk-circle12 sk-circle"></div>
                    </div>

                    <button id="export-html-pdf" class="button button-primary">Export</button>
                </div>

                <div class="hidden export-progress">
                    <div class="hide-when-done-exporting export-in-progress">
                        <div id="exporting-files">Exporting  <b id="export-number"></b> files to <b class="format chosenOutputFormat"></b> Format</div>
                        <div class="hidden" id="waiting-for-zip">Now creating the Zip Archive</div>
                    </div>

                    <div class="hide-while-exporting export-done">
                        <span id="export-progress-alldone"> ALL DONE!</span>
                        <span id="download-zip" style="height: 25px; line-height:21px; margin-left:10px" class="button primary-button">DOWNLOAD ZIP</span>
                    </div>


                    <div class="progress-wrap progress hidden">
                        <div class="progress-percent"></div>
                        <div class="progress-bar progress"></div>
                    </div>
                </div>
                <div class="hidden" id="export-html-pdf-status"> </div>
            </div>
        </div>

        <?php
    }

    public function getSettings() {
        $settings = get_option('_magic_page_export_settings');

        if (!$settings) {
            $settings = array(
                'rewriteUrls' => "1",
                'uniqueContent' => "1",
                'generateMetatags' => "1",
                'generateCanonical' => null,
                'imageOptions' => 'export',
                'imageFolderName' => 'images',
                'rewriteUrlStructure' => '1',
                'stylesheetOptions' => 'export',
                'stylesheetFilename' => 'style',
                'minifyHtml' => "1",
                'minifyCss' => "1",
                'redirect' => '1',
                'exportFormat' => 'html',
                'filenames' => array(),
                'exportTemplate' => array(),
                'searchReplace' => array(),
                'locations' => array(),
                'additionalPostPages' => '0',
                'options' => array(
                    'exportFormat' => 'expanded',
                    'location' => 'collapsed',
                    'contentOptions' => 'collapsed',
                    'titles' => 'collapsed',
                    'filenames' => 'collapsed',
                    'exportTemplate' => 'collapsed',
                    'searchAndReplaceText' => 'collapsed',
                )
            );

            $magicPage = get_magic_page();

            $url = get_option('_magic_page_url', magic_page_default_post_slug());

            $settings['titles'] = array();
            $settings['titles'][] = get_the_title($magicPage);

            $settings['filenames'] = array();
            $settings['filenames'][] = $url . '-[location]';

            $templates = get_posts(array(
                'post_type' => 'exporttemplate',
                'post_status' => 'any',
            ));

            if ($templates) {
                $settings['exportTemplate'] = $templates[0]->ID;
            }
        }

        return $settings;
    }

    public function getExportStaticSettings() {
        $settings = get_option('_magic_page_export_static_settings');

        if (!$settings) {
            $magicPage = get_magic_page();

            $title = $magicPage ? array($magicPage->post_title) : array();

            $settings = array(
                'exportTo' => array(),
                'autogenerateTitles' => true,
                'categories' => array(),
                'exportGroup' => '',
                'exportGroupOptions' => 'append',
                'exportTo' => array(),
                'locations' => array(),
                'schedules' => array(array('from' => 'now', 'percent' => 100, 'to' => '')),
                'titles' => $title,
                'urls' => array(),
                'postmeta' => array(),
                'options' => array(), // @todo
//                'options' => array(
//                    'exportFormat' => 'expanded',
//                    'location' => 'collapsed',
//                    'contentOptions' => 'collapsed',
//                    'titles' => 'collapsed',
//                    'filenames' => 'collapsed',
//                    'exportTemplate' => 'collapsed',
//                    'searchAndReplaceText' => 'collapsed',
//                )
            );
        }



        return $settings;
    }

    // Register Custom Post Type
    public function add_exporttemplates() {

        $labels = array(
            'name' => _x('Export Templates', 'Post Type General Name', 'text_domain'),
            'singular_name' => _x('Export Template', 'Post Type Singular Name', 'text_domain'),
            'menu_name' => __('Export Templates', 'text_domain'),
            'name_admin_bar' => __('Post Type', 'text_domain'),
            'archives' => __('Item Archives', 'text_domain'),
            'attributes' => __('Item Attributes', 'text_domain'),
            'parent_item_colon' => __('Parent Item:', 'text_domain'),
            'all_items' => __('Export Templates', 'text_domain'),
            'add_new_item' => __('Add New Item', 'text_domain'),
            'add_new' => __('Add New', 'text_domain'),
            'new_item' => __('New Item', 'text_domain'),
            'edit_item' => __('Edit Item', 'text_domain'),
            'update_item' => __('Update Item', 'text_domain'),
            'view_item' => __('View Item', 'text_domain'),
            'view_items' => __('View Items', 'text_domain'),
            'search_items' => __('Search Item', 'text_domain'),
            'not_found' => __('Not found', 'text_domain'),
            'not_found_in_trash' => __('Not found in Trash', 'text_domain'),
            'featured_image' => __('Featured Image', 'text_domain'),
            'set_featured_image' => __('Set featured image', 'text_domain'),
            'remove_featured_image' => __('Remove featured image', 'text_domain'),
            'use_featured_image' => __('Use as featured image', 'text_domain'),
            'insert_into_item' => __('Insert into item', 'text_domain'),
            'uploaded_to_this_item' => __('Uploaded to this item', 'text_domain'),
            'items_list' => __('Items list', 'text_domain'),
            'items_list_navigation' => __('Items list navigation', 'text_domain'),
            'filter_items_list' => __('Filter items list', 'text_domain'),
        );
        $args = array(
            'label' => __('Export Template', 'text_domain'),
            'description' => __('Export Templates', 'text_domain'),
            'labels' => $labels,
            'supports' => array('title', 'editor', 'revisions'),
            'hierarchical' => false,
            'public' => true,
            'show_ui' => true,
            'show_in_menu' => "edit.php?post_type=magicpage",
            'menu_position' => 1,
            'menu_icon' => 'dashicons-admin-appearance',
            'show_in_admin_bar' => false,
            'show_in_nav_menus' => true,
            'can_export' => true,
            'has_archive' => false,
            'exclude_from_search' => true,
            'publicly_queryable' => true,
            'capability_type' => 'page',
            'capabilities' => array(
                'create_posts' => 'do_not_allow',
            ), 'map_meta_cap' => true,
        );
        register_post_type('exporttemplate', $args);
    }

}

new MagicPageExportCompatability();

class SH_Walker_TaxonomyDropdown extends Walker_CategoryDropdown {

    function start_el(&$output, $category, $depth = 0, $args = array(), $id = 0) {

        $pad = str_repeat('&nbsp;', $depth * 2);
        $cat_name = apply_filters('list_cats', $category->name, $category);

        if (!isset($args['value'])) {
            $args['value'] = ( $category->taxonomy != 'category' ? 'slug' : 'id' );
        }

        $value = ($args['value'] == 'slug' ? $category->slug : $category->term_id );

        $output .= "\t<option class=\"level-$depth\" value=\"$value\"";
        if ($args['selected_values'] && in_array($value, $args['selected_values'])) {
            $output .= ' selected="selected"';
        }
        $output .= '>';
        $output .= $pad . $cat_name;
        if ($args['show_count'])
            $output .= '&nbsp;&nbsp;(' . $category->count . ')';

        $output .= "</option>\n";
    }

}
