<?php

if (!defined('ABSPATH')) {
    exit;
}

use JasonGrimes\Paginator;

class MagicPageExportToStatic {

    protected $itemsPerPage = 10;

    public function __construct() {

        add_action('wp_ajax_export_to_static_pages', array($this, 'export'));
        add_action('wp_ajax_export_static_save_settings', array($this, 'saveSettings'));

        add_action('wp_ajax_export_to_static_pages_prepare', array($this, 'prepareExport'));
        add_action('wp_ajax_magicpage_process_export_static', array($this, 'processExport'));
        add_action('admin_init', array($this, 'addSearchAction'));

        add_action('wp_ajax_magicpage_is_elementor_detected', array($this, 'is_elementor_detected'));
        add_action('init', array($this, 'previewElementorExportedPage'));
    }

    public function previewElementorExportedPage() {
        if (isset($_GET['preview-elementor-page'])) {
            if (isset($_GET['nonce']) && wp_verify_nonce($_GET['nonce'], 'elementor-preview')) {
                $this->clearElementorCache();

                $link = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ?
                        "https" : "http") . "://" . $_SERVER['HTTP_HOST'] .
                        $_SERVER['REQUEST_URI'];

                $url = esc_url(remove_query_arg(array('nonce', 'preview-elementor-page'), $link));
                wp_redirect($url);
                exit();
            }
        }
    }

    protected function clearElementorCache() {
        if (did_action('elementor/loaded')) {
            \Elementor\Plugin::instance()->files_manager->clear_cache();
        }
    }

    public function addSearchAction() {


        // verify nonce
        if (isset($_GET['action']) && $_GET['action'] == 'location_search_action' && isset($_GET['q']) && $_GET['q']) {


            $postId = intval($_GET['postId']);

            if (!current_user_can('edit_post', $postId)) {
                return false;
            }

            $taxonomy = 'location';
            $tax = get_taxonomy($taxonomy);

            if (!$tax) {
                wp_die(0);
            }

            $data = array();

            $s = strip_tags(wp_unslash($_GET['q']));

            global $wpdb;

            $rows = $wpdb->get_results(
                    "SELECT DISTINCT t.*, tt.*
            FROM $wpdb->terms AS t
            LEFT JOIN $wpdb->termmeta ON (t.term_id = $wpdb->termmeta.term_id )
            LEFT JOIN $wpdb->term_taxonomy AS tt ON t.term_id = tt.term_id
            WHERE tt.taxonomy IN ('location') AND (
              ( ($wpdb->termmeta.meta_key = 'zip' AND $wpdb->termmeta.meta_value LIKE '%$s%') OR t.name LIKE '%$s%' OR t.term_id LIKE '$s%')
            ) ORDER BY t.name ASC LIMIT 0, 20");

            if ($rows) {
                foreach ($rows as $row) {
                    $data[] = array('id' => $row->term_id, 'text' => $row->name);
                }

                header('Content-Type: application/json');
                echo(json_encode(array('results' => $data)));
                exit();
            }
        }
    }

    public function is_elementor_detected() {
        if (!current_user_can('administrator')) {
            return false;
        }

        $postId = intval($_POST['postId']);

        $found = $this->is_elementor_found($postId);

        $fields = array();

        if ($found) {
            $metas = get_post_meta($found['ID']);

            if ($metas) {
                foreach ($metas as $field => $value) {
                    if (strpos($field, '_elementor_') !== false) {
                        $fields[] = $field;
                    }
                }
            }
        }


        $data = array('found' => $found ? true : false, 'fields' => $fields);

        wp_send_json_success($data);
    }

    protected function get_revisions_elementor($postId) {
        $revision = $this->get_elementor_revisions($postId);

        if ($revision) {
            return $revision;
        }


        return (array) get_post($postId);
    }

    protected function is_elementor_found($postId) {
        $detected = $this->get_elementor_revisions($postId);

        if ($detected) {
            return $detected;
        }

        $testPost = $this->get_elementor_data($postId);
        if ($testPost) {
            $detected = get_post($postId, 'ARRAY_A');
        }
    }

    protected function get_elementor_revisions($postId) {
        $revisions = wp_get_post_revisions($postId);

        if ($revisions) {
            foreach ($revisions as $revision) {
                if ($this->get_elementor_data($revision)) {
                    return (array) $revision;
                }
            }
        }
    }

    protected function get_elementor_data($post = null) {
        if (is_object($post)) {

            if (get_post_meta($post->ID, '_elementor_data', true) || get_post_meta($post->ID, '_elementor_version', true)) {
                return true;
            }
        }
    }

    protected function setPostMeta($postId, $metaKey, $metaValue) {
        global $wpdb;

        $wpdb->query($wpdb->prepare("INSERT INTO {$wpdb->prefix}postmeta
                (post_id, meta_key, meta_value)
                VALUES (%d,%s,%s )
                ON DUPLICATE KEY UPDATE
                meta_value = %s
            ", $postId, $metaKey, $metaValue, $metaValue
        ));
    }

    protected function getPostMeta($postId, $metaKey) {
        global $wpdb;

        $preSql = "SELECT * FROM $wpdb->postmeta WHERE post_id = %d AND meta_key = %s";

        $sql = $wpdb->prepare($preSql, $postId, $metaKey);
        $results = $wpdb->get_results($sql);

        if ($results && isset($results[0])) {
            return $results[0]->meta_value;
        }
    }

    public function processExport() {
        if (!current_user_can('administrator')) {
            return false;
        }

        $batchId = intval($_POST['batchId']);
        $schedules = get_option('_magic_page_schedules_install', array());

        $output = array('batchId' => $batchId, 'done' => false);

        $totalInstalledInBatch = 0;
        $post = get_magic_page(true);

        $isDone = false;

        if (isset($schedules[$batchId])) {
            $batch = $schedules[$batchId];
            $this->settings = $batch['settings'];

            $totalCountInstalled = $batch['countInstalled'];
            $totalCountToInstall = $batch['totalCount'];

            $metadata = array();

            // meta fields
            if (isset($this->settings['postmeta']) && $this->settings['postmeta']) {
                foreach ($this->settings['postmeta'] as $field) {
                    // for regular fields, except elementor handled below
                    if (strpos($field, '_elementor') === false) {
                        $metadata['metafields'][$field] = get_post_meta($post->ID, $field, true);
                    }
                }

                $revision = $this->get_revisions_elementor($post->ID);

                $parentId = wp_get_post_parent_id($revision['ID']);
                $parentPost = get_post($parentId);

                if ($parentPost && $parentPost->ID !== $post->ID) {
                    $postId = $parentPost->ID;
                } else {
                    $postId = $post->ID;
                }
                // for elementor
                foreach ($this->settings['postmeta'] as $field) {
                    if (strpos($field, '_elementor') !== false) {
                        $metadata['elementor_metafields'][$field] = $this->getPostMeta($postId, $field);
                    }
                }
            }



            if (!$batch['status']) {
                if ($totalCountInstalled < $totalCountToInstall) {

                    foreach ($batch['schedules'] as $id => $schedule) {
                        $itemsPerPage = $schedule['itemsPerPage'];
                        $countToInstall = $schedule['countToInstall'];

                        if ($schedule['countInstalled'] < $schedule['countToInstall']) {
                            $query = new MagicPageQuery(false, false);
                            $query->setLocations($batch['locations']);
                            $query->setQueryType('result');
                            $query->setItemsPerPage($itemsPerPage);
                            $query->setOffset($totalCountInstalled);

                            $magicPages = $query->getResults();

                            if ($magicPages) {
                                foreach ($magicPages as $location) {
                                    if ($totalInstalledInBatch < $this->itemsPerPage && $schedule['countInstalled'] < $countToInstall) {

                                        $schedule['countInstalled']++;
                                        $totalInstalledInBatch++;
                                        $totalCountInstalled++;

                                        global $magic_page_term_object_exporting;
                                        $magic_page_term_object_exporting = $location;

                                        add_filter('unique_content_needed', array($this, '__true'));

                                        $output['results'][] = $this->processLocation($location, $post, $schedule, $batchId, $metadata);
                                    } else {
                                        break;
                                    }
                                }
                            }

                            $schedules[$batchId]['schedules'][$id] = $schedule;
                        }
                    }

                    if (!$totalInstalledInBatch) {
                        $isDone = true;
                    }
                } else {
                    $isDone = true;
                }
            } else {
                $isDone = true;
            }

            if ($isDone || $totalCountInstalled == $totalCountToInstall) {
                $output['done'] = true;
                $this->updateScheduleProperty('status', 1, $batchId);

                $this->clearElementorCache();
            }

            update_option('_magic_page_schedules_install', $schedules);
            $this->updateScheduleProperty('countInstalled', $totalCountInstalled, $batchId);

            $output['countInstalled'] = $totalCountInstalled;
            $output['totalCount'] = $totalCountToInstall;

            wp_send_json_success($output);
        }
    }

    protected function updateScheduleProperty($property, $value, $batchId) {
        $schedules = get_option('_magic_page_schedules_install', array());

        if (isset($schedules[$batchId])) {
            $schedules[$batchId][$property] = $value;
            update_option('_magic_page_schedules_install', $schedules);
        }
    }

    protected function getSchedule($batchId) {
        $schedules = get_option('_magic_page_schedules_install', array());

        if (isset($schedules[$batchId])) {
            return $schedules[$batchId];
        }
    }

    public function saveSettings() {
        if (!current_user_can('administrator')) {
            return false;
        }

        $this->processSettings();

        update_option('_magic_page_export_static_settings', $this->settings);

        wp_send_json_success($this->settings);
    }

    protected function calculatePagination() {
        $magicPagesCount = 0;

        if ($this->settings['locations']) {
            $query = new MagicPageQuery();
            $query->setLocations($this->settings['locations']);
            $query->setQueryType('count');
            $magicPagesCount = $query->getResults();
        }

        $magicPagesPaginator = new Paginator($magicPagesCount, $this->itemsPerPage, 1);

        $count = array();

        $count['count'] = intval($magicPagesCount);
        $count['numPages'] = intval($magicPagesPaginator->getNumPages());

        return $count;
    }

    public function prepareExport() {
        if (!current_user_can('administrator')) {
            return false;
        }





        $this->processSettings();

        $groupId = $this->settings['exportGroupId'];
        $pagination = $this->calculatePagination();

        $count = $pagination['count'];

        $totalToBeInstalled = $count;
        $overallInstalled = 0;
        $processSchedule = array();

        if ($this->settings['exportGroupOptions'] === 'delete') {

            global $wpdb;

            $wpdb->query("DELETE p, pm
                FROM $wpdb->posts p
               INNER JOIN $wpdb->postmeta pm
                  ON pm.post_id = p.ID
               WHERE pm.meta_key = '_group_id'
                 AND pm.meta_value = '$groupId'");
        }


        // save header as "group" option
        $head = get_option('_magicpage_headercode');
        if ($head) {
            update_option('_group_headercode_' . $groupId, $head);
        }

        // save footer as "group" option
        $foot = get_option('_magicpage_footercode');
        if ($foot) {
            update_option('_group_footercode_' . $groupId, $foot);
        }

        $post = get_magic_page();

        $output1 = ['autoenabledAdvancedSchema' => false];
        if ($post) {





            $fields = get_magic_page_meta_fields();
            $groupMeta = array();
            foreach ($fields as $field) {
                $val = get_post_meta($post->ID, $field, true);
                $groupMeta[$field] = $val;
            }
            update_option('_group_meta_fields_' . $groupId, $groupMeta);

            $schema = get_magic_page_schema();

            if ($schema) {
                $schemaEnabled = get_post_meta($post->ID, '_magic_page_schema_enabled', true);
                $typeArray = $_POST['typeArray'];

                foreach ($typeArray as $type) {
                    $name = getPostTypeNamedByLabel($type);
                    if ($schemaEnabled === 'advanced' && !checkAdvancedSchemaBuilderEnabled($name)) {
                        enablePostPageAdvancedSchemaBuilder($type, $name);
                        $output1['autoenabledAdvancedSchema'] = true;
                    }
                }



                update_option('_group_schema_' . $groupId . '_schematic_schema', $schema);
            }
        }

        if ($this->settings['schedules']) {
            foreach ($this->settings['schedules'] as $schedule) {
                $ratio = $schedule['percent'] / 100;
                $countToInstall = intval(ceil($count * $ratio));

                $oneRangeCountMax = $countToInstall;

                if ($schedule['from'] >= 0 && $schedule['to'] && $schedule['percent']) {
                    $countToInstallRange = intval(ceil($countToInstall / ($schedule['to'] - $schedule['from'])));

                    if ($countToInstallRange < 1) {


                        $totalToBeInstalled = $totalToBeInstalled - $count;
                        $countToInstall = intval(ceil($count));
                        $schedule['to'] = $schedule['from'] + 1;
                    }


                    $oneRangeCount = 0;

                    for ($start = $schedule['from']; $start < $schedule['to']; $start++) {


                        $totalToBeInstalled = $countToInstallRange - $countToInstall;

                        if ($totalToBeInstalled < 0) {
                            $countToInstall = intval(ceil($countToInstall - absint($totalToBeInstalled)));
                        }

                        $overallInstalled = $overallInstalled + $countToInstall;

                        $date = new DateTime();
                        $date->modify('+' . $start . ' day');
                        $date->format('Y-m-d H:i:s');

                        $paginator = new Paginator($countToInstall, $this->itemsPerPage, 1);

                        // make sure we are not out of range
                        if ($countToInstall < $paginator->getItemsPerPage()) {
                            $itemsPerPage = $countToInstall;
                        } else {
                            $itemsPerPage = $paginator->getItemsPerPage();
                        }

                        // do we have enough for the last batch
                        $oneRangeCount = $oneRangeCount + $countToInstall;
                        if ($oneRangeCount > $oneRangeCountMax) {
                            $countToInstall = $countToInstall - ($oneRangeCount - $oneRangeCountMax);
                            $itemsPerPage = $countToInstall;
                        }

                        if ($countToInstall > 0) {


                            $processSchedule[] = array(
                                'date' => $date,
                                'countToInstall' => $countToInstall,
                                'countInstalled' => 0,
                                'itemsPerPage' => $itemsPerPage,
                            );
                        }
                    }
                } else if (!$schedule['from'] && !$schedule['to'] && $schedule['percent']) {
                    $paginator = new Paginator($countToInstall, $this->itemsPerPage, 1);

                    if ($countToInstall < $paginator->getItemsPerPage()) {
                        $itemsPerPage = $countToInstall;
                    } else {
                        $itemsPerPage = $paginator->getItemsPerPage();
                    }

                    if ($countToInstall > 0) {
                        $processSchedule[] = array(
                            'countToInstall' => $countToInstall,
                            'countInstalled' => 0,
                            'itemsPerPage' => $itemsPerPage,
                        );
                    }
                }
            }

            if ($overallInstalled < $totalToBeInstalled) {
                $index = count($processSchedule) - 1;

                if ($index >= 0) {
                    $addonExtra = $processSchedule[$index];
                    $addonExtra['countToInstall'] = intval($totalToBeInstalled - $overallInstalled);
                }

                $processSchedule[$index] = $addonExtra;
            }
        }

        $time = time();
        $hasSchedule = false;

        if (!empty($processSchedule)) {
            $hasSchedule = true;
            delete_option('_magic_page_schedules_install');

            $oldSchedules = get_option('_magic_page_schedules_install', array());
            $oldSchedules[$time] = array(
                'status' => 0,
                'locations' => $this->settings['locations'],
                'schedules' => $processSchedule,
                'totalCount' => $count,
                'countInstalled' => 0,
                'settings' => $this->settings
            );

            update_option('_magic_page_schedules_install', $oldSchedules);
        }


        $output = array(
            'batchId' => $time,
            'schedules' => $hasSchedule,
            'count' => $count,
            'locations' => $this->settings['locations']
        );
        /// arraay merge todo

        $output = array_merge($output1, $output);

        wp_send_json_success($output);
    }

    public function export() {
        if (!current_user_can('administrator')) {
            return false;
        }


        global $magicPageExport;
        $magicPageExport = true;

        $batchId = isset($this->settings['batchId']) ? ($this->settings['batchId']) : null;
        $page = isset($this->settings['page']) ? intval($this->settings['page']) : 1;
        $count = isset($this->settings['count']) ? intval($this->settings['count']) : null;

        $query = new MagicPageQuery(false, false);
        $query->setLocations($this->settings['locations']);
        $query->setQueryType('result');
        $query->setItemsPerPage($this->itemsPerPage);
        $query->setCurrentPage($page);
        $magicPages = $query->getResults();

        if ($magicPages) {
            foreach ($magicPages as $location) {
                $output['results'][] = $this->process($location, $batchId);
            }
        }

        $paginator = new Paginator($count, $this->itemsPerPage, $page);
        $output['nextPage'] = $paginator->getNextPage();

        wp_send_json_success($output);
    }

    public function processSettings() {

        $post = stripslashes_deep($_POST);

        $settings = array('locations' => '', 'schedules' => '', 'postmeta' => array());

        if (isset($post['titles'])) {
            $settings['titles'] = array_map('cleanup_quotes_slashes', $post['titles']);
        }

        if (isset($post['urls'])) {
            $settings['urls'] = array_map('cleanup_quotes_slashes', $post['urls']);
        }


        if (isset($post['locations'])) {
            $settings['locations'] = $post['locations'];
        }

        if (isset($post['options'])) {
            $settings['options'] = $post['options'];
        }


        if (isset($post['categories'])) {
            $settings['categories'] = $post['categories'];
        }

        if (isset($post['exportTo'])) {
            $settings['exportTo'] = $post['exportTo'];
        }

        if (isset($post['exportGroup'])) {

            $settings['exportGroupId'] = createExportGroup($post['exportGroup']);
        }


        if (isset($post['exportGroupOptions'])) {
            $settings['exportGroupOptions'] = $post['exportGroupOptions'];
        }

        if (isset($post['autogenerateTitles'])) {
            $settings['autogenerateTitles'] = $post['autogenerateTitles'] == 'true' ? true : false;
        }

        if (isset($post['schedules'])) {
            $settings['schedules'] = array();

            foreach ($post['schedules'] as $schedule) {
                $settings['schedules'][] = array_map('intval', $schedule);
            }
        }

        if (isset($post['postmeta'])) {
            $settings['postmeta'] = array_map('strip_tags', $post['postmeta']);
        }



        $this->settings = $settings;
    }

    public function __true() {
        return true;
    }

    protected function getText($text, $location, $autoP = false) {
//        if (!$autoP) {
//            remove_filter('the_content', 'wpautop');
//        }
//
//        //$text = apply_variables($text, $location);
//        //$text = apply_filters('the_content', do_shortcode($text));
//
//        if (!$autoP) {
//            add_filter('the_content', 'wpautop');
//        }

        return $text;
    }

    protected function getParsedText($text, $location) {
        $text = parse_magic_page_if_shortcode($text);
        $text = apply_spintax_filter($text, $location);
        $text = apply_variables($text, $location);
        //$text = apply_filters('the_content', do_shortcode($text));
        return $text;
    }

    protected function fixQuotes($html) {
        return str_replace('&#039;', "'", $html);
    }

    protected function processLocation($location, $_post, $schedule, $batchId, $metadata) {

        $title = getRandomElement($this->settings['titles']);
        $title = $this->getText($title, $location);

        global $post;

        $post = $_post;

        setup_postdata($post);

        $text = $this->getText($post->post_content, $location, true);

        $publishAs = array();
        if (in_array('Posts', $this->settings['exportTo']) && $this->settings['categories']) {
            $publishAs[] = 'post';
        }

        if (in_array('Pages', $this->settings['exportTo'])) {
            $publishAs[] = 'page';
        }

        if (count($publishAs) == 2) {
            $rand_keys = array_rand($publishAs, 1);
            $chosenType = $publishAs[$rand_keys];
        } else if (count($publishAs)) {
            $chosenType = $publishAs[0];
        }



        $posts = array();
        if ($this->settings['exportGroupOptions'] === 'overwrite') {
            $args = array(
                'post_type' => $chosenType,
                'meta_query' => array(
                    'relation' => 'AND',
                    array(
                        'key' => '_group_id',
                        'value' => $this->settings['exportGroupId'],
                        'compare' => '='
                    ),
                    array(
                        'key' => '_location_id',
                        'value' => $location->term_id,
                        'compare' => '='
                    )
                )
            );
            $query = new WP_Query($args);

            $posts = $query->posts;
        }


        $postdata = array(
            'post_title' => wp_strip_all_tags($title),
            'post_content' => $text,
            'post_status' => 'publish',
            'post_author' => 1,
            'post_type' => $chosenType,
            'meta_input' => array(
                '_group_id' => $this->settings['exportGroupId'],
                '_location_id' => $location->term_id,
                '_batch_id' => $batchId,
                '_use_group_settings' => true
            )
        );
//
//        $spintax = get_option('_magic_page_spintax_cache');
//        if($spintax) {
//            $postdata['meta_input']['_magic_page_spintax_cache'] = $spintax;
//        }


        if (isset($metadata['metafields'])) {
            foreach ($metadata['metafields'] as $id => $value) {
                $postdata['meta_input'][$id] = $value;
            }
        }

        if ($this->settings['autogenerateTitles']) {
            $this->settings['urls'][] = true;
        }
 

        
        
        
        
        if (isset($this->settings['urls']) && $this->settings['urls']) {
            $url = getRandomElement($this->settings['urls']);
            
        
            if ($url !== true) {

                $postdata['post_name'] = $this->getParsedText($url, $location);
            } else {
                $postdata['post_name'] = $this->getParsedText($title, $location);
            }
        } else {
            $postdata['post_name'] = $this->getParsedText($title, $location);
        }
        
        
       
        
        



        // now
        if (!isset($schedule['date'])) {
            $date = new DateTime();
        } else {
            $date = $schedule['date'];
            $date->setTime(mt_rand(0, 23), mt_rand(0, 59));
        }


        $postdata['post_date'] = $date->format('Y-m-d H:i:s');

        if ($chosenType == 'post') {
            $rand_keys = array_rand($this->settings['categories'], 1);
            $chosenCategory = $this->settings['categories'][$rand_keys];
            $postdata['post_category'] = array($chosenCategory);
        }


        $postIds = array();
        $output = array();
        $newPosts = array();

        if ($posts) {
            foreach ($posts as $post) {
                $postdata['ID'] = $post->ID;
                $postIds[] = wp_insert_post($postdata);
            }
        } else {
            $id = wp_insert_post($postdata);
            $postIds[] = $id;
            $newPosts[] = $id;
        }


        foreach ($postIds as $postId) {
            if (isset($metadata['elementor_metafields'])) {
                foreach ($metadata['elementor_metafields'] as $field => $value) {
                    $this->setPostMeta($postId, $field, $value);
                }
            }

            $url = get_permalink($postId);

            $post = get_post($postId);
            if ($this->get_elementor_data($post)) {
                $url = add_query_arg(array(
                    'preview-elementor-page' => 'true',
                    'nonce' => esc_attr(wp_create_nonce('elementor-preview'))
                        ), $url);
            }

            $editUrl = get_edit_post_link($postId);
            $locationId = get_post_meta($postId, '_location_id', true);
            $location = get_term_by('id', $locationId, 'location');
            $locationName = $location ? $location->name : '';
            $isNewPost = in_array($postId, $newPosts) ? true : false;

            $output[] = array('url' => $url, 'editUrl' => $editUrl, 'title' => $title, 'location' => $locationName, 'isNewPost' => $isNewPost);
        }


        return $output;
    }

}

new MagicPageExportToStatic();

