<?php

if (!defined('ABSPATH')) {
    exit;
}

use Dompdf\Dompdf;
use Dompdf\Options;
use JasonGrimes\Paginator;
use samdark\sitemap\Sitemap;
use samdark\sitemap\Index;
use TopWebStudio\PhpUtilities\ImageParser;
use TopWebStudio\PhpUtilities\HtmlMinify;

class MagicPageExport {

    protected $itemsPerPage = 50;

    public function __construct() {
        if (isset($_POST['exporting']) && $_POST['exporting'] == '1') {
            remove_action('shutdown', 'wp_ob_end_flush_all', 1);
        }

        add_filter('tiny_mce_before_init', array($this, 'editorStyles'));
        add_action('edit_form_advanced', array($this, 'exportFields'));
        add_action('save_post', array($this, 'savePost'));

        add_action('wp_ajax_export_html_pdf', array($this, 'export'));
        add_action('wp_ajax_export_prepare', array($this, 'prepareExport'));
        add_action('wp_ajax_export_save_settings', array($this, 'saveSettings'));
        add_action('wp_ajax_export_compress_files', array($this, 'compressFiles'));

        add_action('wp_ajax_export_get_posts', array($this, 'getPosts'));
        add_action('the_editor_content', array($this, 'modifyHTML'));
        add_action('enqueue_block_assets', array($this, 'addGutenbergCss'));
    }

    protected function getMagicPageLocationUrl($domainAndMpSlug, $mpSlug, $href) {
        if (strpos($href, $mpSlug) !== false) {
            $locationSlug = str_replace($domainAndMpSlug, '', $href);
            $locationSlug = str_replace($mpSlug, '', $locationSlug);
            $locationSlug = rtrim($locationSlug, '/');

            $folder = '/';
            $extension = '.html';

            if ($this->settings['subdirectory'] && $this->settings['installInSubdirectory']) {
                $folder .= $this->settings['subdirectory'] . '/';
            }

            if ($this->settings['rewriteUrls']) {
                $extension = '';
            }

            if ($locationSlug) {
                $recoveredData = file_get_contents($this->getUploadFolder() . 'data.txt');
                $recoveredArray = unserialize($recoveredData);

                if (isset($recoveredArray[$locationSlug])) {
                    return $folder . $recoveredArray[$locationSlug] . $extension;
                }
            }
        }
    }

    protected function fixUrls($input) {
        $mp = trailingslashit(get_option('_magic_page_url', magic_page_default_post_slug()));
        $domain = trailingslashit(get_site_url());

        // replace magicpage/services/near -> magicpage-services-near
        ///         $mp = trailingslashit(str_replace('', '', $mp));


        $domainAndMpSlug = $domain . $mp;
        $mpSlug = '/' . $mp;

        $pattern = "/(?<=href=(\"|'))[^\"']+(?=(\"|'))/";
        $regexp = "<a\s[^>]*href=(\"??)([^\" >]*?)\\1[^>]*>(.*)<\/a>";

        if (preg_match_all("/$regexp/siU", $input, $matches, PREG_SET_ORDER)) {
            foreach ($matches as $match) {
                $href = $match[2];
                $fulllink = $match[0];
                $slug = $this->getMagicPageLocationUrl($domainAndMpSlug, $mpSlug, $href);

                // mp full url
                if ($slug && strpos($href, $domainAndMpSlug) !== false) {
                    $newlink = preg_replace($pattern, $slug, $fulllink);
                    $input = str_replace($fulllink, $newlink, $input);
                }
            }
        }

        if (preg_match_all("/$regexp/siU", $input, $matches, PREG_SET_ORDER)) {
            foreach ($matches as $match) {

                $href = $match[2];
                $fulllink = $match[0];
                $slug = $this->getMagicPageLocationUrl($domainAndMpSlug, $mpSlug, $href);

                if ($slug && strpos($href, $mpSlug) !== false) { // mp relative url
                    $newlink = preg_replace($pattern, $slug, $fulllink);
                    $input = str_replace($href, $slug, $input);
                }
            }
        }



        if (preg_match_all("/$regexp/siU", $input, $matches, PREG_SET_ORDER)) {
            foreach ($matches as $match) {
                $href = $match[2];
                $fulllink = $match[0];

                if (strpos($href, $domain) !== false) {
                    $newlink = str_replace($domain, '/', $fulllink);

                    $prefix = '';
                    if ($this->settings['subdirectory'] && $this->settings['installInSubdirectory']) {
                        $prefix = '/' . $this->settings['subdirectory'];
                    }

                    // remove trailing slash from url
                    preg_match('/href=(["\'])([^\1]*)\1/i', $newlink, $exactLink);
                    $newlink = str_replace($exactLink[2], $prefix . rtrim($exactLink[2], '/'), $newlink);

                    $input = str_replace($fulllink, $newlink, $input);
                }
            }
        }

        return $input;
    }

    public function compressFiles() {
        if (!current_user_can('administrator')) {
            return false;
        }

        @ini_set('max_execution_time', 300);
        @ini_set('memory_limit', '2048M');

        $this->processSettings();

        $zipFile = "download.zip";
        $filename = $this->getUploadFolder() . $zipFile;

        @unlink($filename);

        $zip = new ZipArchive();

        $installInSub = '';
        $subdir = '';
        $subdir2 = '';

        if ($this->settings['subdirectory'] && $this->settings['installInSubdirectory'] && $this->settings['exportFormat'] == 'html') {
            $installInSub = $this->settings['subdirectory'];
            if ($installInSub) {
                $subdir = $installInSub . '/';
                $subdir2 = '/' . $installInSub . '/';
                $zip->addEmptyDir($installInSub);
            }
        }

        // rewrite url structure
        if ($this->settings['rewriteUrlStructure'] && $this->settings['exportFormat'] == 'html') {
            $files = glob($this->getUploadFolder() . '*.html');
            foreach ($files as $file) {
                if (file_exists($file) && is_file($file)) {
                    $content1 = file_get_contents($file);
                    $content2 = $this->fixUrls($content1);

                    if ($content2 && $content1 !== $content2) {
                        file_put_contents($file, $content2);
                    }
                }
            }
        }

        if ($zip->open($filename, ZipArchive::CREATE) !== TRUE) {
            exit("cannot open <$filename>\n");
        }

        $extension = '.html';

        if ($this->settings['rewriteUrls']) {
            $extension = '';
        }

        // sitemap
        if ($this->sitemapEnabled()) {
            $sitemap = new Sitemap($this->getUploadFolder() . 'sitemaps.xml');
            $sitemap->setMaxUrls(5000);
        }

        $files = glob($this->getUploadFolder() . '*');

        foreach ($files as $file) {
            if (file_exists($file) && is_file($file) && basename($file) !== 'data.txt') {
                $zip->addFile($file, $subdir . basename($file));

                $f = basename($file);
                if ($this->settings['rewriteUrls']) {
                    $f = str_replace('.html', '', $f);
                }

                if ($this->sitemapEnabled()) {
                    $sitemap->addItem($this->getFullUrl() . $f, time(), Sitemap::MONTHLY);
                }

                if ($this->settings['generateCanonical'] && $this->settings['exportFormat'] === 'html') {
                    $canonical = $this->getFullUrl() . $f;
                    if (strpos($canonical, '/index') !== false) {
                        $canonical = str_replace('/index', '', $canonical);
                    }

                    $canonical = str_replace('{CANONICAL}', $canonical, file_get_contents($file));
                    file_put_contents($file, $canonical);
                }
            }
        }

        if ($this->settings['exportFormat'] == 'html') {
            if ($this->sitemapEnabled()) {
                $sitemap->write();

                $staticSitemapUrls = $sitemap->getSitemapUrls($this->getFullUrl());
                $index = new Index($this->getUploadFolder() . '/sitemap.xml');
                foreach ($staticSitemapUrls as $sitemapUrl) {
                    $index->addSitemap($sitemapUrl);
                }
                $index->write();
            }


            // get all xml files.
            $files = glob($this->getUploadFolder() . '*.xml');
            foreach ($files as $file) {
                if (file_exists($file) && is_file($file)) {
                    $zip->addFile($file, $subdir . basename($file));
                }
            }

            // robots.txt
            if ($this->sitemapEnabled()) {
                $string = 'Sitemap: ' . $this->getFullUrl() . 'sitemap.xml';
                $myfile = fopen($this->getUploadFolder() . "robots.txt", "w");
                fwrite($myfile, $string);
                fclose($myfile);
                $zip->addFile($this->getUploadFolder() . "robots.txt", $subdir . 'robots.txt');
            }

            ////////////////

            if ($this->settings['rewriteUrls'] && $this->settings['exportFormat'] === 'html') {
                $string = htaccessCreate($subdir2, $this->settings['redirect']);

                $myfile = fopen($this->getUploadFolder() . "htaccess.txt", "w");
                fwrite($myfile, $string);
                fclose($myfile);

                $zip->addFile($this->getUploadFolder() . "htaccess.txt", $subdir . '.htaccess');
            }


            $createdImageFolder = '';
            // images folder

            $files = glob($this->getImageUploadFolder() . '*');

            if ($files) {
                foreach ($files as $file) {
                    $subdirImg = $this->settings['imageFolderName'];
                    if (!$createdImageFolder) {
                        $zip->addEmptyDir($subdir . $subdirImg);
                        $createdImageFolder = $subdir . $subdirImg . '/';
                    }

                    $zip->addFile($file, $createdImageFolder . basename($file));
                }
            }


            // stylesheet file
            if ($this->settings['stylesheetOptions'] == 'export' && $this->settings['stylesheetFilename']) {
                $stylesheetFile = $this->getUploadFolder() . $this->settings['stylesheetFilename'] . ".css";
                if (file_exists($stylesheetFile)) {
                    $zip->addFile($stylesheetFile, $subdir . $this->settings['stylesheetFilename'] . ".css");
                }
            }


            // javascript
            $settings = get_post_meta($this->settings['exportTemplate'], '_settings', true);
            if ($settings && $settings->js && $settings->js_settings) {

                $jsFilename = $settings->js_settings->filename;
                $jsString = '';

                foreach ($settings->js as $script) {
                    if (in_array($script, $settings->readonly)) {
                        $jsString .= file_get_contents(MAGIC_PAGE_DIR . 'src/admin/export/templates/' . $settings->id . '/js/' . $script);
                    } else {
                        $jsString .= get_post_meta($this->settings['exportTemplate'], 'exporttemplate_javascript_' . $script, true);
                    }
                }

                $zip->addFromString($subdir . $jsFilename, $jsString);
            }

            // template images folder
            if ($settings && $settings->images) {

                $imagesFolder = MAGIC_PAGE_DIR . 'src/admin/export/templates/' . $settings->id . '/' . $settings->images . '/';
                $images = glob($imagesFolder . '*');

                if ($images) {
                    $templateImageFolderCreated = false;
                    $templateImageFolder = $subdir . $settings->images . '/';
                    if ($templateImageFolder !== $createdImageFolder && !$templateImageFolderCreated) {
                        $zip->addEmptyDir($templateImageFolder);
                        $templateImageFolderCreated = true;
                    }

                    foreach ($images as $image) {
                        $zip->addFile($image, $templateImageFolder . basename($image));
                    }
                }
            }

            // settings and config

            $fileUid = sanitize_title_with_dashes(sanitize_title(wp_generate_password())) . $this->getRandomNumber();

            // settings file
            $settingsFile = $this->getUploadFolder() . 'settings-' . $fileUid . ".config";
            $string = serialize($this->settings);

            $myfile = fopen($settingsFile, "w");
            fwrite($myfile, $string);
            fclose($myfile);
            $zip->addFile($settingsFile, $subdir . basename($settingsFile));

            // data file
            $string = file_get_contents($this->getUploadFolder() . 'data.txt');
            $dataFile = $this->getUploadFolder() . 'data-' . $fileUid . ".config";
            $myfile = fopen($dataFile, "w");
            fwrite($myfile, $string);
            fclose($myfile);
            $zip->addFile($dataFile, $subdir . basename($dataFile));
        }


        // all done
        $zip->close();

        $response = content_url('/uploads/' . $this->getUidFolder() . $zipFile);
        wp_send_json_success($response);
    }

    protected function rrmdir($src) {
        $dir = opendir($src);
        while (false !== ( $file = readdir($dir))) {
            if (( $file != '.' ) && ( $file != '..' )) {
                $full = $src . '/' . $file;
                if (is_dir($full)) {
                    $this->rrmdir($full);
                } else {
                    unlink($full);
                }
            }
        }
        closedir($dir);
        rmdir($src);
    }

    public function saveSettings() {
        if (!current_user_can('administrator')) {
            return false;
        }

        $this->processSettings();

        unset($this->settings['postId']);

        update_option('_magic_page_export_settings', $this->settings);

        wp_send_json_success($this->settings);
    }

    public function savePost() {
        global $post;

        if (!current_user_can('administrator')) {
            return false;
        }

        $fields = $this->_getExportFields($post);

        foreach ($fields as $field) {
            if ($field[1]) {
                foreach ($field[1] as $_field) {
                    $postField = str_replace('.', '-', $_field);

                    if (isset($_POST[$postField])) {
                        $content = stripslashes(wp_kses_stripslashes(wp_kses_post($_POST[$postField])));
                        update_post_meta($post->ID, $_field, $content);
                    }
                }
            }
        }
    }

    protected function getExportFieldsQuery($type, $post) {
        if ($post) {
            global $wpdb;
            $id = $post->ID;
            $typeId = $type . '_%';

            return $wpdb->get_results("SELECT meta_key FROM {$wpdb->prefix}postmeta WHERE post_id = $id AND meta_key LIKE 'exporttemplate_$typeId'", ARRAY_N);
        }
    }

    protected function _getExportFields($post, $types = array('stylesheet', 'javascript')) {
        $return = array();

        foreach ($types as $type) {
            $results = $this->getExportFieldsQuery($type, $post);
            if ($results && $results[0]) {
                $return[] = array($type, $results[0]);
            }
        }
        return $return;
    }

    public function exportFields() {
        global $post;

        if ($post && get_post_meta($post->ID, 'homepage') || get_post_meta($post->ID, 'template')) {
            foreach ($this->_getExportFields($post) as $result) {
                foreach ($result[1] as $r) {
                    $res = explode('_', $r);
                    $this->editorTextarea($post->ID, $result[0], end($res));
                }
            }
        }
    }

    protected function getFieldName($field, $id) {
        return 'exporttemplate_' . $field . '_' . str_replace('.', '-', $id);
    }

    protected function getFieldId($field) {
        return 'exporttemplate_' . $field;
    }

    public function editorTextarea($postId, $type, $id) {
        $content = get_post_meta($postId, 'exporttemplate_' . $type . '_' . $id, true);
        echo('<h3 class="description">' . ucfirst($type) . ' (' . $id . ') Editor </h3>');
        echo('<div class="option-wrap"> '
        . '<div class="ace-editor"><textarea name="' . $this->getFieldName($type, $id) . '"  data-type="' . $type . '" class="editor">' . $content . '</textarea></div></div>');
    }

    protected function additonalPagesEnabled() {
        return !empty($this->settings['additionalPostPages']) && $this->settings['additionalPostPages'] != array('0' => '0');
    }

    public function calculatePagination() {
        $magicPagesCount = 0;

        if ($this->settings['locations']) {
            $query = new MagicPageQuery();
            $query->setLocations($this->settings['locations']);
            $query->setQueryType('count');
            $magicPagesCount = $query->getResults();
        }

        $magicPagesPaginator = new Paginator($magicPagesCount, $this->itemsPerPage, 1);

        $count = array();
        $postsPagesCount = 0;
        $homepagePostType = null;

        if ($this->settings['homepage']) {
            $homepage = get_post($this->settings['homepage']);
            if ($homepage && get_post_status($homepage) == 'publish') {
                $homepagePostType = get_post_type($homepage);
                $postsPagesCount++;
            }
        }


        $count['results']['magicpage'] = array(
            'count' => intval($magicPagesCount),
            'numPages' => $magicPagesPaginator->getNumPages()
        );

        if ($this->additonalPagesEnabled()) {
            if (in_array('post', $this->settings['additionalPostPages'])) {
                $post = wp_count_posts('post');
                $postsPagesCount += $post->publish;
            }

            if (in_array('page', $this->settings['additionalPostPages'])) {
                $page = wp_count_posts('page');
                $postsPagesCount += $page->publish;
            }

            if ($homepagePostType == 'post' || $homepagePostType == 'page') {
                $postsPagesCount--;
            }

            // selected
            $args = array(
                'post_status' => array('publish')
            );

            // exclude homepage from common page results
            if ($this->settings['homepage']) {
                $args['post__not_in'] = array($this->settings['homepage']);
            }

            if ($this->settings['additionalPostPages'] == array('select') && $this->settings['manuallySelectedPostsPages']) {
                $args['post__in'] = $this->settings['manuallySelectedPostsPages'];
                $args['post_type'] = array('post', 'page');
            }

            $posts = new WP_Query($args);
            $postsPagesCount += $posts->found_posts;
        }


        $postsPagesPaginator = new Paginator($postsPagesCount, $this->itemsPerPage, 1);

        $count['results']['post_page'] = array(
            'count' => $postsPagesCount,
            'numPages' => $postsPagesPaginator->getNumPages()
        );

        $count['totalCount'] = $postsPagesCount + $magicPagesCount;
        $count['totalPagesCount'] = $magicPagesPaginator->getNumPages() + $postsPagesPaginator->getNumPages();

        return $count;
    }

    public function prepareExport() {
        if (!current_user_can('administrator')) {
            return false;
        }

        $this->rrmdir($this->getUploadFolder());

        $this->processSettings();
        $output = $this->calculatePagination();
        $output['contentUrl'] = content_url('uploads/' . $this->getUidFolder());
        wp_send_json_success($output);
    }

    public function export() {
        if (!current_user_can('administrator')) {
            return false;
        }

        global $magicPageExport;
        $magicPageExport = true;

        $this->processSettings();

        $page = isset($_POST['page']) ? intval($_POST['page']) : 1;
        $type = in_array($_POST['type'], array('post_page', 'magicpage')) ? $_POST['type'] : 'magicpage';
        $typeData = $_POST['typedata'];
        $manuallySelectedPostsPages = isset($_POST['manuallySelectedPostsPages']) ? $_POST['manuallySelectedPostsPages'] : [];

        global $magicPageExportUrl;
        $magicPageExportUrl = $this->getFullUrl();

        $output = array();

        if ($type == 'magicpage') {
            if ($this->settings['locations']) {
                $query = new MagicPageQuery();
                $query->setLocations($this->settings['locations']);
                $query->setQueryType('result');
                $query->setItemsPerPage($this->itemsPerPage);
                $query->setCurrentPage($page);
                $magicPages = $query->getResults();
            } else {
                $magicPages = array();
            }

            if ($magicPages) {
                foreach ($magicPages as $location) {
                    $output['results'][] = $this->process($location);
                }
            }
        }

        // postpages
        if ($type == 'post_page') {
            $defaultLocation = getMagicPageDefaultTerm();

            $args = array();

            // add homepage on 1st iteration
            if ($this->settings['homepage']) {
                if ($page == 1) {
                    $argsHome = array(
                        'post_type' => array('page', 'post'),
                        'post_status' => array('publish'),
                        'post__in' => array($this->settings['homepage']),
                    );

                    if (($hp = get_posts($argsHome))) {
                        $this->settings['postId'] = $hp[0]->ID;
                        $output['results'][] = $this->process($defaultLocation);
                    }
                }
            }

            if ($this->additonalPagesEnabled()) {
                $args = array(
                    'post_status' => array('publish'),
                    'paged' => $page,
                    'posts_per_page' => $this->itemsPerPage
                );

                // exclude homepage from common page results
                if ($this->settings['homepage']) {
                    $args['post__not_in'] = array($this->settings['homepage']);
                }

                $args['post_type'] = $this->settings['additionalPostPages'];

                if ($this->settings['additionalPostPages'] == array('select') && $manuallySelectedPostsPages) {
                    $args['post__in'] = $manuallySelectedPostsPages;
                    $args['post_type'] = array('post', 'page');
                }

                $posts = get_posts($args);
                if ($posts) {
                    foreach ($posts as $post) {
                        $this->settings['postId'] = $post->ID;
                        $output['results'][] = $this->process($defaultLocation);
                    }
                }
            }
        }


        $paginator = new Paginator($typeData['count'], $this->itemsPerPage, $page);
        $output['nextPage'] = $paginator->getNextPage();

        wp_send_json_success($output);
    }

    public function parseJsonLdSchema($html, $uploadFolder) {
        $dom = new DOMDocument();
        @$dom->loadHTML($html);
        $xpath = new DOMXpath($dom);

        $script = $dom->getElementsByTagName('script');
        $script = $xpath->query('//script[@type="application/ld+json"]');

        if ($script && $script->item(0)) {
            $json = json_decode($script->item(0)->nodeValue);
            if (isset($json->logo)) {
                $html = $this->processDirectImages($html, $json->logo, $uploadFolder);
            }

            if (isset($json->image)) {
                $html = $this->processDirectImages($html, $json->image, $uploadFolder);
            }
        }

        return $html;
    }

    public function processDirectImages($html, $imageSrc, $uploadFolder) {
        $file = file_get_contents($imageSrc);
        if ($file) {
            $imageSrcFilename = basename($imageSrc);
            $newFilename = $uploadFolder . $imageSrcFilename;
            file_put_contents($newFilename, $file);
        }

        return str_replace($imageSrc, $this->getFullUrl() . $this->settings['imageFolderName'] . '/' . $imageSrcFilename, $html);
    }

    public function processSettings() {
        $_POST = stripslashes_deep($_POST);

        $settings = array();
        $settings['postId'] = intval($_POST['postId']);
        $settings['uniqueContent'] = boolval($_POST['uniqueContent']);
        $settings['generateMetatags'] = boolval($_POST['generateMetatags']);
        $settings['generateCanonical'] = boolval($_POST['generateCanonical']);
        $settings['minifyHtml'] = boolval($_POST['minifyHtml']);
        $settings['minifyCss'] = boolval($_POST['minifyCss']);

        $settings['exportFormat'] = $_POST['exportFormat'];

        $settings['filenames'] = array_map('cleanup_quotes_slashes', $_POST['filenames']);

        $settings['exportTemplate'] = intval($_POST['exportTemplate']);
        $settings['titles'] = array_map('cleanup_quotes_slashes', $_POST['titles']);

        $searchReplace = isset($_POST['searchReplace']) && $_POST['searchReplace'] ? $_POST['searchReplace'] : array();
        $settings['searchReplace'] = $searchReplace;

        $settings['locations'] = isset($_POST['locations']) ? $_POST['locations'] : array();
        $settings['options'] = $_POST['options'];

        $settings['installInSubdirectory'] = boolval($_POST['installInSubdirectory']);
        $settings['subdirectory'] = $_POST['subdirectory'];
        $settings['createSitemap'] = boolval($_POST['createSitemap']);
        $settings['url'] = ($_POST['url']);
        $settings['rewriteUrls'] = boolval($_POST['rewriteUrls']);
        $settings['redirect'] = boolval($_POST['redirect']);

        $settings['homepage'] = $_POST['homepage'];
        $settings['imageOptions'] = ($_POST['imageOptions']);
        $settings['imageFolderName'] = ($_POST['imageFolderName']);

        $settings['stylesheetOptions'] = ($_POST['stylesheetOptions']);
        $settings['stylesheetFilename'] = ($_POST['stylesheetFilename']);

        $settings['rewriteUrlStructure'] = boolval($_POST['rewriteUrlStructure']);
        $settings['additionalPostPages'] = $_POST['additionalPostPages'];

        if (isset($_POST['manuallySelectedPostsPages'])) {
            if (is_array($_POST['manuallySelectedPostsPages'])) {
                $settings['manuallySelectedPostsPages'] = array_map('intval', $_POST['manuallySelectedPostsPages']);
            } else {
                $settings['manuallySelectedPostsPages'] = intval($_POST['manuallySelectedPostsPages']);
            }
        }

        $this->settings = $settings;
    }

    public function __true() {
        return true;
    }

    protected function getText($text, $location, $autoP = false) {
        if (!$autoP) {
            remove_filter('the_content', 'wpautop');
        }

        $text = apply_variables($text, $location);
        $text = apply_filters('the_content', do_shortcode($text));

        if (!$autoP) {
            add_filter('the_content', 'wpautop');
        }

        return $text;
    }

    protected function getTemplate($templateId) {
        $post = get_post($templateId);

        return $post;
    }

    protected function getFormatFilename($location, $post) {
        if (get_post_type($post) !== 'magicpage') {
            if ($post->ID == $this->settings['homepage'] && $this->settings['exportFormat'] === 'html') {
                $filename = 'index';
            } else {
                $filename = get_post_field('post_name', $post);
            }
        } else {
            if ($this->settings['filenames']) {
                $filename = getRandomElement($this->settings['filenames']);
                $filename = $this->getText($filename, $location);
            } else {
                $filename = $this->getRandomNumber();
            }
        }

        return $filename;
    }

    public function process($location) {
        global $magic_page_term_object_exporting;
        $magic_page_term_object_exporting = $location;

        $return = array();

        $post = get_post($this->settings['postId']);

        if ($this->settings['uniqueContent']) {
            add_filter('unique_content_needed', array($this, '__true'));
        }


        if (strpos($this->settings['exportFormat'], 'pdf') !== false) {
            $data = $this->processLocation('pdf', $location, $post);

            if ($data) {
                $return[] = $data;
            }
        }

        if (strpos($this->settings['exportFormat'], 'html') !== false) {
            $data = $this->processLocation('html', $location, $post);

            if ($data) {
                $return[] = $data;
            }
        }

        return $return;
    }

    protected function fixQuotes($html) {
        return str_replace('&#039;', "'", $html);
    }

    protected function processLocation($format, $location, $p) {
        global $post;

        $post = $p;

        $templatePost = $this->getTemplate($this->settings['exportTemplate']);

        if (!$templatePost) {
            return false;
        }

        // javascript
        $jsString = '';
        $jsPosition = '';
        $settings = get_post_meta($this->settings['exportTemplate'], '_settings', true);
        if ($settings && $settings->js && $settings->js_settings) {
            $jsFilename = $settings->js_settings->filename . '?v=' . time();
            $async = $settings->js_settings->async ? 'async="async" ' : '';
            $jsString = '<script type="text/javascript" ' . $async . 'src="' . $jsFilename . '"></script>';
            $jsPosition = $settings->js_settings->position;
        }

        //css
        $stylesheet = $this->getStylesheetsString($templatePost);

        // minify css
        if ($this->settings['minifyCss']) {
            $stylesheet = minifyCss($stylesheet);
        }

        // for regular posts or if we don't have titles
        if (get_post_type($post) != 'magicpage' || !$this->settings['titles']) {
            $title = get_the_title($post);
        } else {
            $title = getRandomElement($this->settings['titles']);
            $title = $this->getText($title, $location);
        }

        $head = '';

        setup_postdata($post);

        if ($format != 'pdf') {
            if (class_exists('WPSEO_Frontend')) {
                $head .= $this->getMetaDescription($post, $location);
            }

            if ($this->settings['generateMetatags'] && get_post_type($post) === 'magicpage') {
                ob_start();

                createMetatags($post, $location, false);

                $output = ob_get_contents();
                ob_end_clean();

                // get images and replace images path.
                $uploadFolder = $this->getImageUploadFolder();
                $output = $this->parseJsonLdSchema($output, $uploadFolder);

                $head .= $output;
            }

            if ($this->settings['generateCanonical'] && $this->settings['rewriteUrlStructure']) {
                $head .= '<link rel="canonical" href="{CANONICAL}" />';
            }
        }

        if ($stylesheet && $this->settings['stylesheetOptions'] == 'embed' || $format == 'pdf') {
            $head .= '<style type="text/css">' . $stylesheet . ' </style>';
        }
        if ($stylesheet && $this->settings['stylesheetOptions'] == 'export' && $this->settings['stylesheetFilename'] && $format != 'pdf') {
            $stylesheetName = $this->settings['stylesheetFilename'];
            $head .= '<link rel="stylesheet" href="' . $stylesheetName . '.css?v=' . time() . '" type="text/css" media="all" />';

            // create file if not exists $stylesheet
            $file = $this->getUploadFolder() . $stylesheetName . '.css';

            if (!file_exists($file)) {
                $myfile = fopen($this->getUploadFolder() . $stylesheetName . ".css", "w");
                fwrite($myfile, $stylesheet);
                fclose($myfile);
            }
        }

        $text = $this->getText($post->post_content, $location, true);

        $body = $templatePost->post_content;
        $body = str_replace('{title}', $title, $body);
        $body = str_replace('{content}', $text, $body);
        $body = $this->getText($body, $location, false);

        if ($jsString && $jsPosition == 'bottom') {
            $body .= $jsString;
        } elseif ($jsString && $jsPosition == 'top') {
            $head .= $jsString;
        }

        if (get_post_type($post) === 'magicpage' && $format != 'pdf') {
            $head .= $this->fixQuotes(html_entity_decode(get_option('_magicpage_headercode')));
            $body .= $this->fixQuotes(html_entity_decode(get_option('_magicpage_footercode')));
        }


        $schema = get_post_meta($this->settings['exportTemplate'], 'schema', true);

        if (!$schema) {
            $schema = '<!doctype html><html><head><title>%s</title><meta charset="UTF-8" />'
                    . '<meta name="viewport" content="width=device-width, initial-scale=1" />%s</head>'
                    . '<body>%s</body></html>';
            $output = sprintf($schema, $title, $head, $body);
        } else {
            $output = str_replace('{title}', $title, $schema); // start with schema first
            $output = str_replace('{head}', $head, $output);
            $output = str_replace('{body}', $body, $output);
        }

        // images
        if (($this->settings['imageOptions'] == 'export' || $this->settings['imageOptions'] == 'base64') &&
                $this->settings['exportFormat'] === 'html'
        ) {
            $uploadFolder = $this->getImageUploadFolder();
            $embed = $this->settings['imageOptions'] == 'base64' ? true : false;

            $parser = new ImageParser($output, $uploadFolder, $this->settings['imageFolderName'] . '/', $embed);
            $output = $parser->parse();
        } elseif ($this->settings['imageOptions'] == '') {
            $output = preg_replace("/<img[^>]+\>/i", "", $output);
        }

        if ($this->settings['searchReplace']) {
            foreach ($this->settings['searchReplace'] as $searchReplace) {
                $output = str_replace($searchReplace['search'], $searchReplace['replace'], $output);
            }
        }


        // minify
        if ($this->settings['minifyHtml']) {
            $output = new HtmlMinify($output);
        }

        $filename = $this->getFormatFilename($location, $post);

        if ($format === 'pdf') {
            $options = new Options(array(
                'isRemoteEnabled' => true,
                'isHtml5ParserEnabled' => true
            ));

            $dompdf = new Dompdf($options);
            $dompdf->loadHtml($output);
            $dompdf->render();
            $output = $dompdf->output();
        }

        $fileData = $this->getFilename($filename, $format);
        file_put_contents($fileData[0], $output);

        if ($format === 'html' && get_post_type($post) == 'magicpage') {
            if (file_exists($this->getUploadFolder() . 'data.txt')) {
                $recoveredData = file_get_contents($this->getUploadFolder() . 'data.txt');
                $recoveredArray = unserialize($recoveredData);
            } else {
                $recoveredArray = array();
            }

            if (!$recoveredArray || !is_array($recoveredArray)) {
                $recoveredArray = array();
            }

            $recoveredArray[$location->slug] = $fileData[1];
            $serializedData = serialize($recoveredArray);
            file_put_contents($this->getUploadFolder() . 'data.txt', $serializedData);
        }

        return array('filename' => $fileData[1], 'title' => $title, 'filetype' => $fileData[2]);
    }

    protected function getUploadFolder() {
        $folder = WP_CONTENT_DIR . '/uploads/' . $this->getUidFolder();
        $folderCreated = wp_mkdir_p($folder);
        if (!$folderCreated) {
            $data = array('note' => 'The folder ' . $folder . ' could not be created');
            wp_send_json_error($data);
        }

        return $folder;
    }

    protected function getUidFolder() {
        $uploadDir = get_option('_magic_page_uploads_dir');
        if (!$uploadDir) {
            $uploadDir = 'magic-page-uploads-' . $this->getRandomNumber() . '-' . $this->getRandomNumber() . '/';
            update_option('_magic_page_uploads_dir', $uploadDir);
        }
        return $uploadDir;
    }

    protected function getImageUploadFolder() {
        $folder = $this->getUploadFolder() . $this->settings['imageFolderName'] . '/';
        wp_mkdir_p($folder);
        return $folder;
    }

    protected function getFilename($_filename, $format) {
        $filename = sanitize_title_with_dashes(sanitize_title($_filename));

        $file = $this->getUploadFolder() . $filename . '.' . $format;

        if (file_exists($file)) {
            $filename = ($filename . '-' . $this->getRandomNumber());
            $file = $this->getUploadFolder() . $filename . '.' . $format;
            return $this->getFilename($filename, $format);
        } else {
            return array($file, $filename, $format);
        }
    }

    protected function getRandomNumber() {
        return wp_rand(10000, 9999999);
    }

    public function getStylesheetsString($post, $includeReadonly = true) {

        $styles = $this->_getExportFields($post, array('stylesheet'));

        $css = '';

        if ($includeReadonly) {
            $settings = get_post_meta($post->ID, '_settings', true);

            if ($settings && $settings->readonly) {
                foreach ($settings->readonly as $item) {
                    if (strpos($item, 'css') !== false) {
                        $css .= file_get_contents(MAGIC_PAGE_DIR . 'src/admin/export/templates/' . $settings->id . '/css/' . $item);
                    }
                }
            }
        }

        if (isset($styles[0]) && isset($styles[0][1])) {
            foreach ($styles[0][1] as $styleToAdd) {
                $css .= get_post_meta($post->ID, $styleToAdd, true);
            }
        }

        return $css;
    }

    public function editorStyles($mceInit) {
        if (!isset($mceInit['classic_block_editor'])) {
            $post = getExportTemplatePost();

            if ($post) {
                $css = $this->getStylesheetsString($post);

                $dir = wp_upload_dir();
                file_put_contents($dir['basedir'] . '/' . $post->ID . '.css', $css);
                $mceInit['content_css'] = $dir['baseurl'] . '/' . $post->ID . '.css'; // @todo unique name
                $mceInit['importcss_append'] = TRUE;
            }
        }

        return $mceInit;
    }

    public function modifyHTML($html) {


        $post = getExportTemplatePost();

        if ($post) {
            $settings = get_post_meta($post->ID, '_settings', true);
            $url = MAGIC_PAGE_URL . 'src/admin/export/templates/' . $settings->id . '/' . $settings->images . '/';

            if ($settings && $settings->images) {
                foreach (getImageSource($html) as $image) {
                    preg_match('@src=(\"|\')([^(\"|\')]+?)(\"|\')@', $image, $match);
                    if ($match) {
                        $search = $settings->images . '/';

                        if (strpos($match['2'], $search) === 0) {
                            $replaceWith = str_replace($search, $url, $match['2']);
                            $html = str_replace($match['2'], $replaceWith, $html);
                        }
                    }
                }

                return $html;
            }
        }

        return $html;
    }

//    private function getTemplatePost() {
//        global $post;
//
//        if ($post) {
//            $homepageId = get_post_meta($post->ID, 'homepage', true);
//            $templateId = get_post_meta($post->ID, 'template', true);
//
//            if ($templateId || $homepageId) {
//                if ($templateId) {
//                    $post = get_post($templateId);
//                }
//                return $post;
//            }
//        }
//    }

    public function getPosts() {
        if (!current_user_can('administrator')) {
            return false;
        }

        $return = array();
        $search_results = new WP_Query(array(
            's' => $_GET['q'],
            'post_status' => array('publish', 'draft'),
            'posts_per_page' => 50,
            'post_type' => array('post', 'page')
        ));

        if ($search_results->have_posts()) :
            while ($search_results->have_posts()) : $search_results->the_post();
                $title = ( mb_strlen($search_results->post->post_title) > 50 ) ? mb_substr($search_results->post->post_title, 0, 49) . '...' : $search_results->post->post_title;

                $title = $search_results->post->post_status == 'draft' ? $title . ' (Draft)' : $title;

                $return[] = array($search_results->post->ID, $title);
                wp_reset_postdata();
            endwhile;
        endif;
        echo json_encode($return);
        die;
    }

    protected function getFullUrl() {
        $return = $this->settings['url'] . '/';

        if ($this->settings['subdirectory'] && $this->settings['installInSubdirectory']) {
            $return .= $this->settings['subdirectory'] . '/';
        }
        return $return;
    }

    protected function sitemapEnabled() {
        return $this->settings['createSitemap'] && $this->settings['url'] && $this->settings['exportFormat'] === 'html';
    }

    protected function getMetaDescription($post, $location) {
        if (get_post_type($post) === 'magicpage') {
            $metaDescription = get_post_meta($post->ID, '_magic_page_meta_description', true);
        }

        if (!isset($metaDescription) || !$metaDescription) {
            $metaDescription = get_post_meta($post->ID, '_yoast_wpseo_metadesc', true);
        }

        if ($metaDescription) {
            return '<meta property="description" content="' . esc_attr($this->getText($metaDescription, $location)) . '" />';
        }
    }

    function addGutenbergCss() {
        $post = getExportTemplatePost();

        if ($post) {
            $css = $this->getStylesheetsString($post);

            $dir = wp_upload_dir();
            file_put_contents($dir['basedir'] . '/' . $post->ID . '.css', $css);

            wp_enqueue_style($post->ID . '-css', $dir['baseurl'] . '/' . $post->ID . '.css', // @todo unique name,
                    array('wp-edit-blocks')
            );
        }
    }

}

new MagicPageExport();
