<?php

if (!defined('ABSPATH')) {
    exit;
}

use PhpOffice\PhpSpreadsheet\Helper\Html as Html_Helper;
use PhpOffice\PhpSpreadsheet\IOFactory;
use PhpOffice\PhpSpreadsheet\Cell\Coordinate;
use PhpOffice\PhpSpreadsheet\Spreadsheet;
use PhpOffice\PhpSpreadsheet\Writer\Xlsx;
use JasonGrimes\Paginator;

class MagicPageImportExportLocations {

    protected $suggestedImportLimit = 100;
    protected $fields = [
        'term_id',
        'location',
        'lat',
        'lon',
        'zip',
        'region',
        'county',
        'country',
        'major_city',
        'location_slug',
        'population'
    ];

    public function __construct() {
        add_action('admin_menu', array($this, 'importExport_Menu'));

        add_action('admin_init', array($this, 'uploadLocationsStep1'));
        add_action('wp_ajax_magic_page_check_database_exist', array($this, 'checkDatabaseExistAction'));
        add_action('wp_ajax_magic_page_import-locations-step2', array($this, 'import_locations_step2'));

        /////////////////
        // export actions
        /////////////////
        // entire database
        add_action('wp_ajax_magic_page_export_entire_database', array($this, 'export_entire_database'));
        add_action('wp_ajax_magic_page_paginate_entire_database', array($this, 'paginate_entire_database'));

        // specific locations
        add_action('wp_ajax_magic_page_manually_export_locations', array($this, 'manually_export_locations'));
        add_action('wp_ajax_magic_page_export_get_locations', array($this, 'get_locations'));

        // regions
        add_action('wp_ajax_magic_page_export_get_regions', array($this, 'export_get_regions'));
        add_action('wp_ajax_magic_page_paginate_by_regions', array($this, 'paginate_by_regions'));
        add_action('wp_ajax_magic_page_export_by_regions', array($this, 'export_by_regions'));

        // user custom databases
        add_action('wp_ajax_magic_page_paginate_own_databases', array($this, 'paginate_own_databases'));
        add_action('wp_ajax_magic_page_export_by_user_databases', array($this, 'export_by_user_databases'));
        // etc
        add_action('wp_ajax_magic_page_database_info', array($this, 'databaseInfo'));
        add_action('wp_ajax_magic_page_delete_user_database', array($this, 'delete_user_database'));

        add_action('wp_ajax_magic_page_import_data', array($this, 'import_data'));
    }

    public function delete_user_database() {
        if (!$this->permissionsCheck($_POST['nonce'])) {
            wp_send_json_error();
        }

        global $wpdb;

        $databases = get_option('_magic_page_database_imports', array());

        $id = sanitize_key($_POST['id']);
        $databaseOnly = $_POST['database_only'];

        if (isset($databases[$id])) {
            unset($databases[$id]);
            update_option('_magic_page_database_imports', $databases);

            if ($databaseOnly == 'false') {
                $wpdb->query("DELETE t,tt,tm,tr
                                         FROM $wpdb->terms t
                                         LEFT JOIN $wpdb->term_taxonomy tt ON t.term_id = tt.term_id
                                         LEFT JOIN $wpdb->termmeta tm ON t.term_id = tm.term_id
                                         LEFT JOIN $wpdb->term_relationships tr ON t.term_id = tr.term_taxonomy_id
                                         WHERE tt.taxonomy = 'location' AND tm.meta_key = 'user_database_id' AND tm.meta_value = '$id'");
            } else {
                $wpdb->query("DELETE tm
                                         FROM $wpdb->terms t
                                         LEFT JOIN $wpdb->term_taxonomy tt ON t.term_id = tt.term_id
                                         LEFT JOIN $wpdb->termmeta tm ON t.term_id = tm.term_id
                                         WHERE tt.taxonomy = 'location' AND tm.meta_key = 'user_database_id' AND tm.meta_value = '$id'");
            }
        }
        
        magic_page_delete_cache();
        
        wp_send_json_success();
    }

    function load() {
        global $wp_scripts;

        wp_register_style('select2css', plugins_url('/css/select2.min.css', MAGIC_PAGE_FILE), false, '1.0', 'all');
        wp_register_script('select2', plugins_url('/js/select2.full.min.js', MAGIC_PAGE_FILE), array('jquery'), '1.0', true);
        wp_enqueue_style('select2css');
        wp_enqueue_script('select2');

        $ui = $wp_scripts->query('jquery-ui-core');

        wp_enqueue_script('confirm-js', plugins_url('/js/jquery-confirm.js', MAGIC_PAGE_FILE), array('jquery'), MAGIC_PAGE_VERSION);
        wp_enqueue_style('confirm-css', plugins_url('/css/jquery-confirm.min.css', MAGIC_PAGE_FILE), array(), MAGIC_PAGE_VERSION);
        wp_enqueue_script('js-notify', plugins_url('/js/notify.min.js', MAGIC_PAGE_FILE), array(), MAGIC_PAGE_VERSION);

        $protocol = is_ssl() ? 'https' : 'http';
        $url = "$protocol://ajax.googleapis.com/ajax/libs/jqueryui/1.12.1/themes/base/jquery-ui.min.css";
        wp_enqueue_style('jquery-ui-base', $url, false, null);

        wp_register_script('progress-circle',
                plugins_url('/js/progress-circle.js', MAGIC_PAGE_FILE),
                array('jquery'), '1.0', true);

        wp_register_script('jquery.iframe-transport',
                plugins_url('/js/jquery.fileupload/jquery.iframe-transport.js', MAGIC_PAGE_FILE),
                array('jquery'), '1.0', true);

        wp_register_script('jquery.fileupload',
                plugins_url('/js/jquery.fileupload/jquery.fileupload.js', MAGIC_PAGE_FILE),
                array('jquery', 'jquery.iframe-transport', 'jquery-ui-widget'), '1.0', true);

        
        
                if (magicPageCompatabilityModeEnabled()) {
            global $magicpage_cdn_url;

            wp_enqueue_script('magic-page-import-export-locations',
                    $magicpage_cdn_url . MAGIC_PAGE_VERSION . '/import-export.js',
                    array('jquery', 'jquery.fileupload', 'jquery-ui-accordion', 'jquery-ui-autocomplete', 'progress-circle',
                        'select2', 'confirm-js', 'js-notify'),
                    MAGIC_PAGE_VERSION);
        } else {
            
            if(!defined('IMPORT_EXPORT_LOCATIONS_JS')) {
                define('IMPORT_EXPORT_LOCATIONS_JS',
                    plugins_url('/js/import-export/import-export.js', MAGIC_PAGE_FILE));
            }
            
            wp_enqueue_script( 'magic-page-import-export-locations', IMPORT_EXPORT_LOCATIONS_JS,
                    array('jquery', 'jquery.fileupload', 'jquery-ui-accordion', 'jquery-ui-autocomplete', 'progress-circle',
                        'select2', 'confirm-js', 'js-notify'),
                    MAGIC_PAGE_VERSION);
        }

        
                 

        wp_enqueue_style('magic-page-import-export-locations',
                plugins_url('/css/import-export.css', MAGIC_PAGE_FILE), '',
                MAGIC_PAGE_VERSION);
    }

    public function export_by_user_databases() {
        if (!$this->permissionsCheck($_POST['nonce'])) {
            wp_send_json_error();
        }

        $this->increaseResources();

        $data = $_POST['data'];

        $settings = array(
            'taxonomy' => 'location',
            'hide_empty' => false,
            'offset' => $data['currentPage'] * $data['limit'] - $data['limit'],
            'number' => $data['limit'],
            'orderby' => 'name',
            'order' => 'ASC',
        );

        foreach ($data['databases'] as $c) {
            $settings['meta_query']['relation'] = 'OR';
            $settings['meta_query'][] = array(
                array(
                    'key' => 'user_database_id',
                    'value' => $c,
                    'compare' => '='
                )
            );
        }

        $entities = get_terms($settings);

        $response = $this->exportFile($entities, $data, 'export-custom-databases');
        $response['currentPage']++;

        wp_send_json_success($response);
    }

    public function paginate_own_databases() {
        if (!$this->permissionsCheck($_POST['nonce'])) {
            wp_send_json_error();
        }

        $limit = $_POST['limit'] ? intval($_POST['limit']) : 20000;
        $databases = $_POST['databases'];

        global $wpdb;

        $databasesSql = '';
        foreach ($databases as $country) {
            $databasesSql .= ' OR tm.meta_value = "' . $country . '"';
        }
        $databasesSql = ltrim($databasesSql, ' OR ');

        $countQuery = $wpdb->get_results("SELECT COUNT(*) AS CNT
                                         FROM  $wpdb->terms t
                                         LEFT JOIN $wpdb->term_taxonomy tt ON t.term_id = tt.term_id
                                         LEFT JOIN $wpdb->termmeta tm ON t.term_id = tm.term_id
                                         WHERE tt.taxonomy = 'location' AND tm.meta_key = 'user_database_id' AND ($databasesSql)");

        $count = $countQuery[0]->CNT;
        $paginator = new Paginator($count, $limit, 1);

        $response = array(
            'databases' => $databases,
            'limit' => $limit,
            'currentPage' => 1,
            'count' => intval($count),
            'lastPage' => $paginator->getNumPages());

        wp_send_json_success($response);
    }

    public function paginate_by_regions() {
        if (!$this->permissionsCheck($_POST['nonce'])) {
            wp_send_json_error();
        }
        $limit = $_POST['limit'] ? intval($_POST['limit']) : 20000;
        $regions = $_POST['regions'];

        global $wpdb;

        $regionsSql = '';
        foreach ($regions as $region) {
            $regionsSql .= ' OR tm.meta_value = "' . $region . '"';
        }
        $regionsSql = ltrim($regionsSql, ' OR ');

        $countQuery = $wpdb->get_results("SELECT COUNT(*) AS CNT
                                         FROM  $wpdb->terms t
                                         LEFT JOIN $wpdb->term_taxonomy tt ON t.term_id = tt.term_id
                                         LEFT JOIN $wpdb->termmeta tm ON t.term_id = tm.term_id
                                         WHERE tt.taxonomy = 'location' AND tm.meta_key = 'region' AND ($regionsSql)");

        $count = $countQuery[0]->CNT;
        $paginator = new Paginator($count, $limit, 1);

        $response = array(
            'regions' => $regions,
            'limit' => $limit,
            'currentPage' => 1,
            'count' => intval($count),
            'lastPage' => $paginator->getNumPages());

        wp_send_json_success($response);
    }

    public function paginate_entire_database() {
        if (!$this->permissionsCheck($_POST['nonce'])) {
            wp_send_json_error();
        }

        global $wpdb;

        $limit = $_POST['limit'] ? intval($_POST['limit']) : 20000;

        $countQuery = $wpdb->get_results("SELECT COUNT(*) AS CNT
                                         FROM  $wpdb->terms t
                                        LEFT JOIN $wpdb->term_taxonomy tt ON t.term_id = tt.term_id
                                         WHERE tt.taxonomy ='location'");

        $count = $countQuery[0]->CNT;
        $paginator = new Paginator($count, $limit, 1);

        $response = array(
            'limit' => $limit,
            'currentPage' => 1,
            'count' => intval($count),
            'lastPage' => $paginator->getNumPages());

        wp_send_json_success($response);
    }

    public function export_by_regions() {
        if (!$this->permissionsCheck($_POST['nonce'])) {
            wp_send_json_error();
        }

        $this->increaseResources();

        $data = $_POST['data'];

        $settings = array(
            'taxonomy' => 'location',
            'hide_empty' => false,
            'offset' => $data['currentPage'] * $data['limit'] - $data['limit'],
            'number' => $data['limit'],
            'orderby' => 'name',
            'order' => 'ASC',
        );

        foreach ($data['regions'] as $reg) {
            $settings['meta_query']['relation'] = 'OR';
            $settings['meta_query'][] = array(
                array(
                    'key' => 'region',
                    'value' => $reg,
                    'compare' => '='
                )
            );
        }

        $entities = get_terms($settings);

        $response = $this->exportFile($entities, $data, 'export-by-regions');
        $response['currentPage']++;

        wp_send_json_success($response);
    }

    public function export_entire_database() {
        if (!$this->permissionsCheck($_POST['nonce'])) {
            wp_send_json_error();
        }

        $this->increaseResources();

        $data = array_filter($_POST['data'], 'intval');

        $entities = get_terms(array(
            'taxonomy' => 'location',
            'hide_empty' => false,
            'offset' => $data['currentPage'] * $data['limit'] - $data['limit'],
            'number' => $data['limit']
        ));

        $response = $this->exportFile($entities, $data, 'entire-database');
        $response['currentPage']++;

        wp_send_json_success($response);
    }

    public function export_get_countries() {
        if (!$this->permissionsCheck($_GET['nonce'])) {
            wp_send_json_error();
        }

        $results = search_magicpage_countries($_GET['q']);

        $out = array();
        if ($results) {
            foreach ($results as $result) {
                $out[] = array(
                    $result->name
                );
            }
        }

        echo json_encode($out);
        die;
    }

    public function export_get_regions() {
        if (!$this->permissionsCheck($_GET['nonce'])) {
            wp_send_json_error();
        }


        $results = search_magicpage_regions($_GET['q']);

        $out = array();
        if ($results) {
            foreach ($results as $result) {
                $out[] = array(
                    $result->name
                );
            }
        }

        echo json_encode($out);
        die;
    }

    public function manually_export_locations() {
        if (!$this->permissionsCheck($_POST['nonce'])) {
            wp_send_json_error();
        }

        $data = array_filter($_POST['data'], 'intval');

        $data['currentPage'] = 1;

        $entities = get_terms(array(
            'taxonomy' => 'location',
            'hide_empty' => false,
            'include' => $data,
        ));

        $data = $this->exportFile($entities, $data, 'manually-picked-locations');

        wp_send_json_success($data);
    }

    public function get_locations() {
        if (!$this->permissionsCheck($_GET['nonce'])) {
            wp_send_json_error();
        }


        $taxonomy = 'location';
        $tax = get_taxonomy($taxonomy);

        if (!$tax) {
            wp_die(0);
        }

        if (!current_user_can($tax->cap->assign_terms)) {
            wp_die(-1);
        }

        $s = strip_tags(wp_unslash($_GET['q']));

        global $wpdb;

        $results = $wpdb->get_results(
                "SELECT DISTINCT t.*, tt.*
            FROM $wpdb->terms AS t
            LEFT JOIN $wpdb->termmeta ON (t.term_id = $wpdb->termmeta.term_id )
            LEFT JOIN $wpdb->term_taxonomy AS tt ON t.term_id = tt.term_id
            WHERE tt.taxonomy IN ('location') AND (
              ( ($wpdb->termmeta.meta_key = 'zip' AND $wpdb->termmeta.meta_value LIKE '%$s%') OR t.name LIKE '%$s%' OR t.term_id LIKE '$s%')
            ) ORDER BY t.name ASC LIMIT 0, 20");

        $out = array();
        if ($results) {
            foreach ($results as $result) {
                $out[] = array(
                    $result->term_id,
                    $result->name . ' (' . $result->term_id . ')',
                );
            }
        }

        echo json_encode($out);
        die;
    }

    protected function exportFile($entities, $data, $fileid) {
        global $wpdb;

        $spreadsheet = new Spreadsheet();
        $sheet = $spreadsheet->getActiveSheet();

        $sheet->setTitle("Database");

        $sheet->SetCellValue('A1', 'term id');
        $sheet->SetCellValue('B1', 'location name');
        $sheet->SetCellValue('C1', 'latitude');
        $sheet->SetCellValue('D1', 'longitude');
        $sheet->SetCellValue('E1', 'zip');
        $sheet->SetCellValue('F1', 'region');
        $sheet->SetCellValue('G1', 'county');
        $sheet->SetCellValue('H1', 'country');
        $sheet->SetCellValue('I1', 'major city');
        $sheet->SetCellValue('J1', 'location slug');
        $sheet->SetCellValue('K1', 'population');

        $fields = get_location_custom_meta_fields();

        $i = 11;
        $lastLetter = 'K';
        if ($fields) {
            foreach ($fields as $field) {
                $lastLetter = Coordinate::stringFromColumnIndex($i + 1);
                $sheet->SetCellValue($lastLetter . '1', 'meta_' . $field);
                $i++;
            }
        }


        $sheet->getStyle('A1:' . $lastLetter . '1')->getFont()->setBold(true)->setSize(12);
        $sheet->getStyle('A:' . $lastLetter)->getAlignment()->setHorizontal('left');

        $id = 2;
        foreach ($entities as $entity) {

            $i = 11;

            $meta = $wpdb->get_results("SELECT * FROM $wpdb->termmeta WHERE term_id = '$entity->term_id'");

            $sheet->SetCellValue('A' . $id, $entity->term_id);
            $sheet->SetCellValue('B' . $id, $entity->name);
            $sheet->SetCellValue('C' . $id, $this->getTermMetaValue($meta, 'lat'));
            $sheet->SetCellValue('D' . $id, $this->getTermMetaValue($meta, 'lon'));
            $sheet->SetCellValue('E' . $id, $this->getTermMetaValue($meta, 'zip'));
            $sheet->SetCellValue('F' . $id, $this->getTermMetaValue($meta, 'region'));
            $sheet->SetCellValue('G' . $id, $this->getTermMetaValue($meta, 'county'));
            $sheet->SetCellValue('H' . $id, $this->getTermMetaValue($meta, 'country'));
            $sheet->SetCellValue('I' . $id, $this->getTermMetaValue($meta, 'major_city'));

            $sheet->SetCellValue('J' . $id, $entity->slug);
            $sheet->SetCellValue('K' . $id, $this->getTermMetaValue($meta, 'population'));

            if ($fields) {
                foreach ($fields as $field) {
                    $lastLetter = Coordinate::stringFromColumnIndex($i + 1);
                    $value = $this->getTermMetaValue($meta, $field);

                    if (get_option('_magic_page_editor_' . $field) && $value != strip_tags($value)) {
                        $wizard = new Html_Helper();
                        $value = $wizard->toRichTextObject(htmlentities($this->getTermMetaValue($meta, $field)));
                    }

                    $sheet->SetCellValue($lastLetter . $id, $value);

                    $i++;
                }
            }

            $id++;
        }

        $writer = new Xlsx($spreadsheet);

        $uploads = wp_upload_dir();
        $filename = $fileid . '-' . date('d') . '-' . date('m') . '-' . date('Y') . '-' . $data['currentPage'] . '.xlsx';
        $folder = $uploads['basedir'] . $this->getUidFolder();
        wp_mkdir_p($folder);

        $writer->save($folder . $filename);

        $data['file'] = $filename;
        $data['url'] = $uploads['baseurl'] . $this->getUidFolder() . $filename . '?v=' . rand();

        return $data;
    }

    public function checkDatabaseExistAction() {
        if (!$this->permissionsCheck($_POST['nonce'])) {
            wp_send_json_error();
        }

        $label = sanitize_title($_POST['label']);
        $id = sanitize_key($label);

        $databases = get_option('_magic_page_database_imports', array());

        if (isset($databases[$id])) {
            wp_send_json_success(array('label' => $label, 'id' => $id, 'exists' => true));
        } else {
            $database = array('label' => $label, 'id' => $id, 'time' => time());
            $databases[$id] = $database;

            update_option('_magic_page_database_imports', $databases);

            wp_send_json_success($database);
        }
    }

    public function set_term_meta($term_id, $data, $databaseId = null) {



        foreach ($this->fields as $index => $field) {
            if ($field !== 'term_id' && $field !== 'location' && $field !== 'location_slug') {
                update_term_meta($term_id, $field, $data[$index]);
            }
        }

        // keep this when importing!
        if ($databaseId) {
            update_term_meta($term_id, 'user_database_id', $databaseId);
        }
    }

    public function updateLocation($data, $databaseId = null) {
        global $wpdb;

        $term_id = null;

        // update term
        if ($data[0]) {
            $location = get_term_by('term_id', intval($data[0]), 'location');

            if ($location) {
                $term_id = $location->term_id;

                // update the name of the location
                wp_update_term($term_id, 'location', array('name' => $data[1]));

                // update slug
                if ($data[9]) {
                    wp_update_term($term_id, 'location', array('slug' => $data[9]));
                }
            }
        }



        // create new
        if ($term_id === null && $data[1]) {
            $slug = $data[9] ? $data[9] : generate_unique_slug($data[1]);

            $wpdb->insert($wpdb->terms, array(
                'name' => $data[1],
                'slug' => $slug
                    ), array(
                '%s',
                '%s',
            ));

            $term_id = $wpdb->insert_id;

            $wpdb->insert($wpdb->term_taxonomy, array(
                'term_id' => $wpdb->insert_id,
                'taxonomy' => 'location'
                    ), array(
                '%d',
                '%s',
            ));
        }


        if ($term_id) {
            $this->set_term_meta($term_id, $data, $databaseId);
        }
    }

    public function uploadLocationsStep1() {
        if (current_user_can('administrator') && (isset($_POST['nonce']) || isset($_GET['nonce']))) {
            if (isset($_GET['magic-page-import-locations-file']) && wp_verify_nonce($_POST['nonce'], 'magicpage-import-export')) {

                $this->increaseResources();

                $memory_limit = getMaxMemoryLimit();
                $max_execution_time = ini_get('max_execution_time');

                if ((!$max_execution_time || $max_execution_time >= 15) && $memory_limit >= 256) {
                    $suggestedLimit = $this->suggestedImportLimit * 4;
                } if ($max_execution_time > 0 && $max_execution_time <= 10 && $memory_limit <= 128) {
                    $suggestedLimit = $this->suggestedImportLimit;
                } else {
                    $suggestedLimit = $this->suggestedImportLimit * 2;
                }

                $uploads = wp_upload_dir();
                $file = $_FILES['file']['tmp_name'];
                $filename = $_FILES['file']['name'];

                $folder = $uploads['basedir'] . $this->getUidFolder();
                wp_mkdir_p($folder); // 1

                file_put_contents($folder . $filename, file_get_contents($file));

                $type = IOFactory::identify($file);
                $objReader = IOFactory::createReader($type);

                $objPHPExcel = $objReader->load($file);
                $sheet = $objPHPExcel->getSheet(0);

                $highestRow = $sheet->getHighestRow() - 1;
                $paginator = new Paginator($highestRow, $suggestedLimit, 1);

                $response = array(
                    'filename' => $filename,
                    'currentPage' => 1,
                    'itemsPerPage' => $suggestedLimit,
                    'count' => intval($paginator->getTotalItems()),
                    'lastPage' => $paginator->getNumPages());

                wp_send_json_success($response);
            }


            if (isset($_GET['magic-page-import-data-file']) && wp_verify_nonce($_POST['nonce'], 'magicpage-import-export')) {
                $uploads = wp_upload_dir();
                $file = $_FILES['file']['tmp_name'];
                $filename = 'import-data-file.txt';

                $folder = $uploads['basedir'] . $this->getUidFolder();
                wp_mkdir_p($folder); // 1

                file_put_contents($folder . $filename, file_get_contents($file));

                $response = array(
                    'filename' => $filename,
                );
                wp_send_json_success($response);
            }

            if (isset($_GET['magic_page_export_data']) && wp_verify_nonce($_GET['nonce'], 'magicpage-import-export')) {
                $this->export_data();
            }
        }
    }

    public function import_data() {
        if (!$this->permissionsCheck($_POST['nonce'])) {
            wp_send_json_error();
        }

        $options = $_POST['options'];

        $uploads = wp_upload_dir();
        $file = $_FILES['file']['tmp_name'];
        $filename = $_POST['filename'];
        $overwrite = $_POST['overwrite'];

        $folder = $uploads['basedir'] . $this->getUidFolder();

        $contents = file_get_contents($folder . $filename);

        $array = unserialize($contents);

        if (isset($options['locationsets']) && $options['locationsets']) {
            if (isset($array['location-sets']) && !empty($array['location-sets']) && is_array($array['location-sets'])) {
                foreach ($array['location-sets'] as $id => $item) {
                    foreach ($item[3] as $locId => $locTerm) {
                        $maybeLoc = $this->maybe_location($locTerm->name, $locTerm->lat, $locTerm->lon);

                        if (!$maybeLoc) {
                            // remove this location as it's not found in the db
                            unset($array['location-sets'][$id][3][$locId]);
                        } else {
                            // reassign found term_id
                            $array['location-sets'][$id][3][$locId]->term_id = $maybeLoc->term_id;
                        }
                    }
                }

                if (!$overwrite) {
                    $existing = $this->getLocationSets();
                } else {
                    $existing = array();
                }

                $new_arr = array_merge($existing, $array['location-sets']);
                $this->updateLocationSets($new_arr);
            }
        }


        if (isset($options['xfields']) && $options['xfields']) {
            if (isset($array['xfields']) && !empty($array['xfields']) && is_array($array['xfields'])) {
                if (!$overwrite) {
                    $existing = get_option('_magic_page_xfields', array());
                    $new_arr = array_merge($existing, $array['xfields']);
                    update_option('_magic_page_xfields', $new_arr);
                } else {
                    update_option('_magic_page_xfields', $array['xfields']);
                }
            }
        }

        if (isset($options['shortcodes']) && $options['shortcodes']) {
            if (isset($array['spintax-shortcodes']) && !empty($array['spintax-shortcodes']) && is_array($array['spintax-shortcodes'])) {

                if (!$overwrite) {
                    $existing = get_option('_magic_page_spintax_expressions', array());
                    $new_arr = array_merge($existing, $array['spintax-shortcodes']);
                    update_option('_magic_page_spintax_expressions', $new_arr);
                } else {
                    update_option('_magic_page_spintax_expressions', $array['spintax-shortcodes']);
                }
            }
        }

        magic_page_delete_cache();

        wp_send_json_success();
    }

    protected function export_data() {

        if (isset($_GET['xfields'])) {
            $export['xfields'] = get_option('_magic_page_xfields', array());
        }

        if (isset($_GET['shortcodes'])) {
            $export['spintax-shortcodes'] = get_option('_magic_page_spintax_expressions', array());
        }

        if (isset($_GET['locationsets'])) {
            $export['location-sets'] = $this->getLocationSets();
        }

        $filename = "magic-page-data-" . date("Y-m-d") . ".txt";

        $now = gmdate("D, d M Y H:i:s");
        header("Expires: Tue, 03 Jul 2001 06:00:00 GMT");
        header("Cache-Control: max-age=0, no-cache, must-revalidate, proxy-revalidate");
        header("Last-Modified: {$now} GMT");

        header("Content-Type: application/force-download");
        header("Content-Type: application/octet-stream");
        header("Content-Type: application/download");

        header("Content-Disposition: attachment;filename={$filename}");
        header("Content-Transfer-Encoding: binary");

        $fp = fopen("php://output", 'w');
        fwrite($fp, serialize($export));
        fclose($fp);
        die();
    }

    protected function maybe_location($name, $lat, $lon) {
        global $wpdb;

        return $wpdb->get_row($wpdb->prepare("
       SELECT $wpdb->terms.term_id,
       $wpdb->terms.name,
       pm2.meta_value as lat,
       pm3.meta_value as lon

       FROM $wpdb->terms
            LEFT JOIN $wpdb->termmeta AS pm2 ON ($wpdb->terms.term_id = pm2.term_id AND pm2.meta_key = 'lat')
            LEFT JOIN $wpdb->termmeta AS pm3 ON ($wpdb->terms.term_id = pm3.term_id AND pm3.meta_key = 'lon')
       WHERE $wpdb->terms.name = %s AND pm2.meta_value = %s AND pm3.meta_value = %s
         LIMIT 0,1", array($name, $lat, $lon)));
    }

    protected function updateLocationSets($sets) {
        update_option('location_sets', $sets);
    }

    protected function getLocationSets() {
        return get_option('location_sets', array());
    }

    public function import_locations_step2() {
        if (!$this->permissionsCheck($_POST['nonce'])) {
            wp_send_json_error();
        }


        $this->increaseResources();

        $data = $_POST['data'];
        $uploads = wp_upload_dir();
        $folder = $uploads['basedir'] . $this->getUidFolder();
        $file = $data['filename'];
        $databaseId = $_POST['databaseId'];
        $currentPage = intval($data['currentPage']);
        $itemsPerPage = intval($data['itemsPerPage']);

        $type = IOFactory::identify($folder . $file);
        $objReader = IOFactory::createReader($type);

        $objPHPExcel = $objReader->load($folder . $file);
        $sheet = $objPHPExcel->getSheet(0);

        $highestColumnLetter = $sheet->getHighestColumn(0);
        $highestColumnIndex = Coordinate::columnIndexFromString($highestColumnLetter);

        // custom meta
        if ($highestColumnIndex > 11) {
            for ($i = 11; $i <= $highestColumnIndex - 1; $i++) {
                $column = Coordinate::stringFromColumnIndex($i + 1);
                $metaCellValue = str_replace('-', '_', sanitize_title_with_dashes($sheet->getCell($column . '1')->getValue()));

                $this->fields[] = str_replace('meta_', '', $metaCellValue);
            }
        }


        $offset = $currentPage * $itemsPerPage - $itemsPerPage + 2;  // skip header +2
        $end = $offset + $itemsPerPage;

        $data['debug'] = array('offset' => $offset, 'end' => $end);

        for ($row = $offset; $row < $end; $row++) {
            $locationData = $sheet->rangeToArray('A' . $row . ':' . $highestColumnLetter . $row, NULL, TRUE, FALSE);

            $this->updateLocation($locationData[0], $databaseId);
        }

        $currentPage++;

        if ($currentPage > $data['lastPage']) {
            magic_page_delete_cache();
            @unlink($folder . $file);

            $data['databaseId'] = $databaseId;
            $data['done'] = true;
            wp_send_json_success(array('data' => $data));
        }


        $data['currentPage'] = $currentPage;

        wp_send_json_success(array('data' => $data));
    }

    function importExport_Menu() {
        $page = add_submenu_page(
                '',
                'Magic Page Import / Export Locations',
                'Magic Page Import / Export Locations',
                'manage_options',
                'magic-page-import-export',
                array($this, 'template')
        );
        add_action('load-' . $page, array($this, 'load'));
    }

    public function template() {

        if (!current_user_can('manage_options')) {
            return;
        }


        $this->increaseResources();

        $memory_limit = getMaxMemoryLimit();
        $max_execution_time = ini_get('max_execution_time');

        if ((!$max_execution_time || $max_execution_time >= 15) && $memory_limit >= 256) {
            $suggestedLimit = 10000;
        } if ($max_execution_time > 0 && $max_execution_time <= 10 && $memory_limit <= 128) {
            $suggestedLimit = 2000;
        } else {
            $suggestedLimit = 5000;
        }


        $tab = isset($_GET['tab']) ? $_GET['tab'] : 'import-locations';
        
        
            $data = get_option('_magicpage_d_l');
        $_k = null;

        $postEditUrl = null;
        $post = get_magic_page();
        if ($post ) {
            $postEditUrl = admin_url('post.php?post='.$post->ID.'&action=edit');
        }

        if ($data && !is_array($data)) {
            $unserialized = unserialize($data);

            if ($unserialized && isset($unserialized['key']) && !$unserialized['flag']) {
                if (strlen($unserialized['key']) === 20) {
                    $_k = $unserialized['key'];
                }
            }
        }
        
        ?>
   <!-- don't edit this line -->
            <input type="hidden" data-post-edit-url="<?php echo($postEditUrl) ?>" id="_magicpage_d_l" value="<?php echo($_k); ?>" />
            <!-- don't edit this line -->


        <div  id="import-export-databases" class="wrap">
            <?php wp_nonce_field('magicpage-import-export') ?>


            <h1><?php echo esc_html(get_admin_page_title()); ?></h1>

            <nav class="nav-tab-wrapper">
                <a href="?page=magic-page-import-export&tab=import-locations" class="nav-tab <?php if ($tab === 'import-locations'): ?>nav-tab-active<?php endif; ?>">Import Locations</a>
                <a href="?page=magic-page-import-export&tab=export-locations" class="nav-tab <?php if ($tab === 'export-locations'): ?>nav-tab-active<?php endif; ?>">Export Locations</a>
                <a href="?page=magic-page-import-export&tab=import-export-data" class="nav-tab <?php if ($tab === 'import-export-data'): ?>nav-tab-active<?php endif; ?>">Import Export Data</a>

            </nav>

            <div class="tab-content">
                <?php
                switch ($tab) :
                    case 'export-locations':
                        $this->exportTemplate($suggestedLimit);
                        break;
                    case 'import-locations':
                        $this->importTemplate($suggestedLimit);
                        break;
                    case 'import-export-data':
                        $this->importExportDataTemplate();
                        break;
                endswitch;
                ?>
            </div>
        </div>

        <?php
    }

    public function exportTemplate($suggestedLimit) {

        $databases = get_option('_magic_page_database_imports', array());
        ?>

        <div id="export-accordion">
            <h3>Export Entire Database</h3>
            <div id="export-entire-database">

                This will export your entire database with all locations.
                <br /><br />
                <label>Locations limit per file </label><br />
                <input type="number" class="export-limit" value="<?php echo($suggestedLimit) ?>" />

                <?php $this->commonHtml(); ?>

            </div>

            <h3>Manually Pick Locations</h3>
            <div id="manually-export-locations">

                Manually pick the locations you want to export.
                <br /><br />
                <label>Locations</label><br />

                <select style="width:80%" id="export-locations-pick-locations" multiple="multiple"> </select>

                <div class="success-response-data"> </div>

                <div class="progress-circle __hidden infobox"></div>

                <div class="progress-stats __hidden infobox">
                    Total locations to export:  <span class="total-count"></span>
                </div>

                <button class="do-locations-export button">Export</button>


            </div>
            <h3>Manually Pick Regions</h3>
            <div id="manually-export-by-regions">
                Manually pick the regions you want to export.
                <br /><br />
                <label>Regions</label><br />

                <select style="width:80%" id="export-locations-pick-regions" multiple="multiple"> </select>

                <br /><br />
                <label>Locations limit per file </label><br />
                <input type="number" class="export-limit" value="<?php echo($suggestedLimit) ?>" />

                <?php $this->commonHtml(); ?>
            </div>

            <h3>Export Your Own Databases</h3>
            <div id="export-own-databases">

                Pick your databases you want to export
                <br /><br />
                <label>Databases</label><br />

                <select style="width:80%" id="export-locations-pick-databases" multiple="multiple">

                    <?php foreach ($databases as $database) {
                        ?><option value="<?php echo($database['label']) ?>"><?php echo(labelize_magic_page($database['label'])) ?></option> <?php }
                    ?>

                </select>

                <br /><br />
                <label>Locations limit per file </label><br />
                <input type="number" class="export-limit" value="<?php echo($suggestedLimit) ?>" />

                <?php $this->commonHtml(); ?>

            </div>




        </div>



        <?php
    }

    protected function commonHtml() {
        
    
        ?>


         
        <div class="success-response-data"> </div>

        <div class="progress-circle __hidden infobox"></div>

        <div class="progress-stats __hidden infobox">
            Files exported: <span class="current-page"></span> of <span class="pages"></span> <br />
            Total locations to export:  <span class="total-count"></span>
        </div>

        <button class="do-locations-export button">Export</button><?php
    }

    protected function importTemplate() {

        $importUrl = admin_url('plugins.php?page=magic-page-import-export&magic-page-import-locations-file=true');
        $downloadUrl = MAGIC_PAGE_URL . 'src/admin/samples/sample.xlsx?ver=' . rand();
        $databases = get_option('_magic_page_database_imports', array());

      
        ?>


        <div style="float:left; width: 70%;">



            <div class="form-group">
                <label>Label</label><br />
                <input id="import_label" type="text" required="required" maxlength="100" placeholder="Database label" class="form-control dbfield" />
                <em>Example: <b>My Custom Locations</b></em>
            </div>

            <div class="form-group hidden" style="margin:10px 0 0 0" id="country-result">
                Creating new database: <b>  </b>
            </div>

            <br />
            <button class="button button-secondary disabled" id="check-database-exist">Continue</button>


            <div id="upload-file" class="form-group hidden">
                <div class="fade well" id="upload-dropzone-xls-database-dropzone">
                    <div class="label_upload">
                        <div class="padding-1">
                            Upload Database File (XLSX, XLX or CSV format)
                        </div>
                        <div><small>Drag and Drop file or click here to choose file</small></div>

                    </div>
                    <input id="upload-dropzone-xls-database" type="file" name="file"   data-url="<?php echo($importUrl) ?>" />
                    <div class="uploaded"></div>
                </div>
                <div class="divdone"><a target="_blank" class="fs12 fr" href="<?php echo($downloadUrl) ?>">Download Sample Database File</a></div>
                <div class="form-group" id="country-result-info" style="margin:10px 0"> </div>
                <span id="process-database" class="button-secondary button hidden">Process Database</span>


                <div class="progress-circle __hidden infobox"></div>

                <div class="progress-stats __hidden infobox">
                    Step: <span class="current-page"></span> of <span class="pages"></span> <br />
                    Total locations to import:  <span class="total-count"></span>
                </div>
            </div>


            <div id="step2" class="form-group hidden">
                <hr />
                Database file is processed successfully and it has been split into <span id="split-count"></span> parts. <br />
                You can now start the installation process.

                <div class="divdone">
                    <hr />
                    <span id="start-installation" class="btn btn-round btn-fill btn-info">Start Installation</span>
                    <span class="hidden" id="filesLeftDiv"> Files Left <b id="filesLeft"></b> </div>
            </div>

            <div id="step3" class="form-group hidden">
                <hr />
                Database is imported successfully. <br />


                <div class="divdone">
                    <hr />
                    <span id="create-database-release" class="">All Done!</span> <br /><br />

                    <button id="import-reset" class="button button-primary">Upload New Database</span>
                </div>

            </div>

            <div id="step4" class="form-group hidden">
                <hr /><b></b> database releases are done. Task Completed!
            </div>

        </div>

        <div style="float:left;width: calc(30% - 20px); padding-left:20px;">
            <h3> Your Custom Databases </h3>

        <?php if (!$databases) { ?> <div id="no-database-imports">No databases imported yet </div> <?php } ?>

            <div id="user-databases-list">
            <?php
            foreach ($databases as $database) {
                $this->databaseInfo($database);
            }
            ?>
            </div>
        </div>
        <div style="clear:both"></div>

        <?php
    }

    public function importExportDataTemplate() {
        $magicpage = get_magic_page();

        $exportUrl = wp_nonce_url(admin_url('plugins.php?page=magic-page-import-export&magic_page_export_data=1'), 'magicpage-import-export', 'nonce');
        $importUrl = admin_url('plugins.php?page=magic-page-import-export&magic-page-import-data-file=true');
        ?>

        <input type="hidden" value="<?php echo($magicpage->ID) ?>" id="magic-page-id" />
        <br /><br />
        <div class="form-group">
            <style>
                #import-export-databases .import-export-data {
                    padding: 8px 60px;
                    font-size: 17px;
                    border-radius: 11px;
                    margin-right: 10px;
                }
                #export-options {
                    padding: 20px 15px 10px 15px;
                }
            </style>



            <button data-action="import" class="button button-secondary import-export-data">
                Import Data
            </button>

            <button data-action="export" class="button button-secondary import-export-data">
                Export Data
            </button>




            <div class="hidden" id="export-options">
                Choose which data you would like to <span class="action"></span> <br /><br />

                <div><label><input class="export-option" name="xfields" value="xfields" type="checkbox"> xFields</label> </div>
                <div><label><input class="export-option" name="shortcodes" value="shortcodes" type="checkbox"> Spintax shortcodes</label></div>
                <div><label><input class="export-option" name="locationsets" value="locationsets" type="checkbox"> Location sets</label></div>
            </div>


            <button data-href="<?php echo($exportUrl) ?>"  class="export-data button button-secondary hidden">Export</button>



            <div class="form-group upload-data-file-div hidden ">
                <div class="fade well" id="upload-data-file-dropzone">
                    <div class="label_upload">


                        <div class="padding-1">Upload Data File (TXT Format)</div>

                        <div><small>Drag and Drop file or click here to choose file</small></div>
                    </div>
                    <input id="upload-data-file" type="file" name="file" data-url="<?php echo($importUrl) ?>" />
                    <div class="uploaded"></div>
                </div>

            </div>




        </div>

        <?php
    }

    public function databaseInfo($database = null) {
        if (!$database) {
            if (!$this->permissionsCheck($_POST['nonce'])) {
                wp_send_json_error();
            }
            $ajax = true;
            $databases = get_option('_magic_page_database_imports', array());
            $database = $databases[$_POST['databaseId']];
        }

        $count = get_own_databases_count($database['id']);
        ?><div data-id="<?php echo($database['id']) ?>" class="imported-database">
            <span title="Delete Database" class="delete-database"> X </span>
            <b><?php echo( $database['label']) ?></b> <br />
            contains: <?php echo($count); ?> locations <br />
            created: <?php echo(date("d M Y", $database['time']) ); ?>



        </div><?php
        if (isset($ajax)) {
            exit();
        }
    }

    protected function getUidFolder() {
        $uploadDir = get_option('_magic_page_import_export_dir');
        if (!$uploadDir) {
            $uploadDir = 'magic-page-import-export-' . rand() . '-' . rand() . '/';
            update_option('_magic_page_import_export_dir', $uploadDir);
        }
        return '/' . $uploadDir;
    }

    protected function getTermMetaValue($meta, $field) {
        foreach ($meta as $row) {

            if ($row->meta_key === $field) {
                return $row->meta_value;
            }
        }
    }

    public function increaseResources() {
        @ini_set('max_execution_time', 60);
        @ini_set('memory_limit', '1024M');
    }

    protected function permissionsCheck($nonce) {
        if (wp_verify_nonce($nonce, 'magicpage-import-export') && current_user_can('administrator')) {
            return true;
        }
    }

}

new MagicPageImportExportLocations();

