<?php
if (!defined('ABSPATH')) {
    exit;
}

class MagicPageInstallation {

    public $increment_batch = 50;
    public $time = WEEK_IN_SECONDS;

    public function __construct() {
        add_action('admin_init', array($this, 'init'));

        $this->time_start = microtime(true);

        add_action('admin_menu', array($this, 'installation_page'));
        add_action('admin_notices', array($this, 'notice'));
        add_action('admin_head', array($this, 'admin_scripts'));
        add_action('wp_ajax_magic_page_save_api_key', array($this, 'magic_page_save_api_key'));
        add_action('wp_ajax_magic_page_determine_speed', array($this, 'magic_page_determine_speed'));
        add_action('wp_ajax_magic_page_insert_db', array($this, 'magic_page_insert_db'));
        add_action('wp_ajax_magic_page_delete_test_terms', array($this, 'delete_test_terms'));
        add_action('wp_ajax_magic_page_uninstall_region', array($this, 'uninstall_region'));
        add_action('wp_ajax_magic_page_save_region_filter', array($this, 'save_region_filter'));
        add_action('wp_ajax_magic_page_save_population', array($this, 'save_population'));
        add_action('wp_ajax_magic_page_save_database_chosen_country', array($this, 'save_database_chosen_country'));
        add_action('wp_ajax_magic_page_force_database_refresh', array($this, 'forceDatabaseRefresh'));
        add_action('wp_ajax_delete_unused_locations', array($this, 'deleteUnusedLocations'));
    }

    public function init() {
        get_regions_update_info();

        if (isset($_POST['locations_installed']) && current_user_can('administrator')) {
            return $this->locations_installed();
        }
        if (isset($_POST['update_completed']) && current_user_can('administrator')) {
            return $this->update_completed();
        }
    }

    public function magic_page_save_api_key() {
        global $magicPageApi;

        if (current_user_can('administrator')) {
            $magicPageKey = sanitize_text_field($_POST['_magicpage_api_key']);
            $key = sanitize_text_field($_POST['key']);

            $data = $magicPageApi->get_licensed_databases_list($magicPageKey);

            if ($data && $data['status'] == 'success') {
                update_option('_magicpage_d_l', serialize(array('key' => $key, 'flag' => false)));

                update_option('_magic_page_spintax_autorewrite', '0');
                update_option('_magic_page_spintax_support', '1'); // enabled by default
                update_option('_magic_page_autorefresh_content_status', '0');

                delete_option('_magic_page_key_error');
                update_option('_magicpage_installed', '1');
                update_option('_magicpage_api_key', $magicPageKey);
                set_transient('_magic_page_api_licensed_databases_list', $data['data'], $this->time);

                update_option('_magic_page_seo_for_all_pages', '1');

                $response = array('url' => 'admin.php?page=magicpage-installation&action=databases');

                wp_send_json_success($response);
            }

            wp_send_json_error();
        }
    }

    public function notice() {
        if ((isset($_GET['page']) && $_GET['page'] === 'magicpage-installation') or ( is_magic_page_installed())) {
            return false;
        }
        ?>
        <div class="notice notice-success is-dismissible">
            <p>Magic Page is activated. Please continue with the installation of
                <a href="<?php echo admin_url('admin.php?page=magicpage-installation') ?>">Magic Page </a> </p>
        </div>
        <?php
    }

    public function isUpdate() {
        if (isset($_GET['page']) && $_GET['page'] == 'magicpage-installation' && isset($_GET['updates_only'])) {
            return true;
        }
    }

    public function admin_scripts() {


        if (isset($_GET['page']) && in_array($_GET['page'], array('magic-page-updates', 'magicpage-installation'))) {
            wp_register_style('select2css', plugins_url('/css/select2.min.css', MAGIC_PAGE_FILE), false, '1.0', 'all');
            wp_register_script('select2', plugins_url('/js/select2.full.min.js', MAGIC_PAGE_FILE), array('jquery'), '1.0', true);
            wp_enqueue_style('select2css');
            wp_enqueue_script('select2');

            wp_enqueue_style('wp-jquery-ui-dialog');

            wp_enqueue_script('CircularLoader', plugins_url('/js/CircularLoader-v1.3.js', MAGIC_PAGE_FILE), array('jquery'), MAGIC_PAGE_VERSION);
            wp_enqueue_script('magic-page-admin-installation', plugins_url('/js/magic-page-admin-installation.js', MAGIC_PAGE_FILE), array('CircularLoader', 'select2', 'jquery-ui-dialog'), MAGIC_PAGE_VERSION);
      
            wp_enqueue_script('magic-page-delete-unused-locations', plugins_url('/js/delete-unused-locations.js', MAGIC_PAGE_FILE), array(  'jquery'), MAGIC_PAGE_VERSION);
   
        }

        if (!is_inline_editor_page() && $this->shouldLoad()) {
            wp_enqueue_style('magic-page-admin', plugins_url('/css/magic-page-admin.css', MAGIC_PAGE_FILE), array(), MAGIC_PAGE_VERSION);
        }
    }

    function installation_page() {


        if (isset($_GET['action']) && $_GET['action'] === "magic-page-new-key") {
            delete_option('_magicpage_api_key');
            delete_option('_magicpage_installed');

            wp_redirect(admin_url('admin.php?page=magicpage-installation'));
        }

        if (is_magic_page_installed()) {
            if ($this->isUpdate()) {
                add_submenu_page('admin.php?page=magicpage-installation&action=databases', 'Updates', 'Updates', 'administrator', 'magicpage-installation', array($this, 'install_page'));
            } else if (isset($_GET['action']) && $_GET['action'] == 'databases') {
                add_submenu_page('admin.php?page=magicpage-installation&action=databases', 'Location Databases', 'Location Databases', 'administrator', 'magicpage-installation', array($this, 'install_page'));
            }
        } else {
            add_menu_page('Magic Page', 'Magic Page', 'administrator', 'magicpage-installation', array($this, 'install_page'), null, 8);
        }
    }

    public function install_page() {
        // step 1 verify
        if (!get_option('_magicpage_api_key')) {
            $this->verifyApiKeyStep();
        } else {
            // step 2 show map
            $this->showLicensedDatabases();
        }
    }

    public function verifyApiKeyStep() {
        ?>
        <div class="wrap">

            <h2>Magic Page Installation</h2>

            <div id="apikey_box"> <h2> PLEASE VERIFY YOUR MAGIC PAGE API KEY </h2>

                <div class="container">
                    <p> Don't know what is API KEY? Please <a href="http://www.magicpageplugin.com">magicpageplugin.com </a> for more info</p>

                    <input type="text" id="magic_page_key" placeholder="API KEY" />
                    <input id="magic_page_verify_api_key" type="button" value="VERIFY KEY" class="button button-primary" />

                </div>
            </div>

            <div id="api_key_success"></div>
            <div style="display: none" id="api_key_success_proceed">
                You now may <span id="api_key_save" class="button">Proceed</span> with the installation of the plugin.
            </div>
            <div id="api_key_error"></div>
        </div>

        <?php
    }

    public function get_allowed_countries() {
        global $magicPageApi;

        $options = get_transient('_magic_page_api_licensed_databases_list');

        if (!$options || !isset($options[0]) || !is_array($options[0])) {
            $data = $magicPageApi->get_licensed_databases_list(get_option('_magicpage_api_key'));
            if ($data && $data['status'] == 'success') {
                $options = (array) $data['data'];

                set_transient('_magic_page_api_licensed_databases_list', $options, $this->time);
            }
        }

        if ($options) {
            $allowed_countries = array();
            foreach ($options as $option) {
                if (isset($option['country'])) {
                    $allowed_countries[] = $option['country'];
                }
            }
            return array_unique($allowed_countries);
        }

        return array();
    }

    public function get_allowed_regions_by_country($country) {
        global $magicPageApi;

        $options = get_transient('_magic_page_api_licensed_databases_list');

        if (!$options) {
            $data = $magicPageApi->get_licensed_databases_list(get_option('_magicpage_api_key'));
            if ($data['status'] == 'success') {
                $options = $data['data'];

                set_transient('_magic_page_api_licensed_databases_list', $options, $this->time);
            }
        }

        $out = array();

        if ($options) {
            foreach ($options as $option) {
                if (isset($option['country']) && $option['country'] == $country) {
                    $out[] = $option;
                }
            }
        }

        return $out;
    }

    public function select2disabled($allowed_countries, $country) {
        if (!in_array($country[0], $allowed_countries)) {
            echo(' disabled="disabled" ');
        }
    }

    public function showLicensedDatabases() {
        global $magic_page_countries;

        $allowed_countries = $this->get_allowed_countries();

        $chosenCountry = get_option('_magic_page_chosen_country_db_install');
        if (!$chosenCountry && $allowed_countries) {
            $chosenCountry = $allowed_countries[0];
        }
        ?>
        <div class="wrap">
            <h2>Magic Page Databases

                <span id="countries-databases-holder">
                    <select data-chosen-country="<?php echo($chosenCountry) ?>" data-flags-url="<?php echo(MAGIC_PAGE_URL . 'img/4x3/') ?>"  id="countries-databases">
                        <?php foreach ($magic_page_countries as $country) {
                            ?> <option <?php $this->select2disabled($allowed_countries, $country) ?> <?php selected($country[0], $chosenCountry); ?> value="<?php echo($country[0]); ?>"><?php echo($country[1]); ?></option> <?php }
                        ?>
                    </select>
                </span>    
                <span title="Force Database Refesh" id="force-database-refresh" style="width: 26px; height: 26px; display: inline-block; float: right; cursor: pointer; margin: 4px 5px 0 0">
                    <svg xmlns="http://www.w3.org/2000/svg" viewBox="206 -206 512 512">
                        <path d="M457.6-119.8c-22.4 1.8-43.4 19-59.7 48.7-4.3 7.9-8.1 16.3-11.4 25.4h71.1v-74.1zM457.6-37.7h-73.9C375.5-12.6 371 16 370.6 46h87v-83.7zM465.6 46h87c-.4-30-4.9-58.6-13.1-83.7h-73.9V46zM457.6 54h-87c.4 29.4 4.7 57.5 12.6 82.3h74.3V54zM532.3-74.9c4.9 9 9.2 18.8 12.9 29.2H602c-24.2-35.3-61.3-61.1-104.5-70.4 12.9 8.9 24.8 22.8 34.8 41.2zM552.6 54h-87v82.3H540c7.9-24.8 12.2-52.9 12.6-82.3zM465.6-45.7h71.1c-3.3-9-7.1-17.5-11.4-25.4-16.3-29.7-37.3-46.9-59.7-48.7v74.1zM457.6 144.3H386c3.4 9.5 7.4 18.5 12 26.8 16.2 29.7 37.3 46.9 59.7 48.7v-75.5zM321.3-45.7H378c3.7-10.4 8-20.2 12.9-29.2 10-18.3 21.9-32.3 34.9-41.2-43.2 9.3-80.4 35-104.5 70.4zM375.3-37.7h-59.2A168.6 168.6 0 0 0 291.7 46h70.9c.4-29.8 4.8-58.4 12.7-83.7zM425.8 216.1c-13-9-24.9-22.9-34.9-41.2-5.1-9.4-9.6-19.7-13.4-30.6h-57.2c24.1 36 61.7 62.4 105.5 71.8zM315.3 136.3h59.6c-7.7-24.9-11.9-53-12.3-82.3h-70.9c.7 30 9.2 58 23.6 82.3zM465.6 219.8c22.4-1.8 43.4-18.9 59.7-48.7 4.5-8.3 8.5-17.3 12-26.8h-71.6l-.1 75.5zM602.9 144.3h-57.2c-3.8 11-8.3 21.2-13.4 30.6-10 18.3-21.9 32.3-34.9 41.2 43.8-9.4 81.4-35.8 105.5-71.8zM631.5 54h-70.9c-.4 29.2-4.6 57.3-12.3 82.3H608C622.3 112 630.8 84 631.5 54zM607.1-37.7h-59.2c7.9 25.3 12.3 53.9 12.7 83.7h70.9a168 168 0 0 0-24.4-83.7z"/>
                        <path d="M467.8 235a3.85 3.85 0 0 0-4.2-.8c-1.4.6-2.4 2-2.4 3.6V256a204.9 204.9 0 0 1-182.6-112.4c-1-1.9-3.3-2.7-5.2-1.7-1.9 1-2.7 3.3-1.7 5.2 17.9 35.1 45.1 64.6 78.7 85.2 33.4 20.5 71.7 31.3 110.8 31.5V282c0 1.6.9 3 2.4 3.6a3.86 3.86 0 0 0 4.2-.8l22.1-22.1a3.86 3.86 0 0 0 0-5.4L467.8 235zM696.3 44l-21.8-21.8c-1.5-1.5-4-1.5-5.5 0l-22.1 22a3.85 3.85 0 0 0-.8 4.2c.6 1.4 2 2.4 3.6 2.4h18.2c-.3 77.5-43.3 147.4-112.4 182.7a3.86 3.86 0 0 0 1.8 7.3c.6 0 1.2-.1 1.8-.4 35.1-17.9 64.6-45.1 85.2-78.7 20.5-33.4 31.3-71.7 31.5-110.8H694c2.1 0 3.9-1.7 3.9-3.9-.2-1.3-.7-2.3-1.6-3zM456.2-135a3.86 3.86 0 0 0 4.2.8c1.4-.6 2.4-2 2.4-3.6V-156c77.5.3 147.4 43.3 182.6 112.4.7 1.3 2 2.1 3.5 2.1a3.86 3.86 0 0 0 3.5-5.6 213.83 213.83 0 0 0-78.7-85.2 213.72 213.72 0 0 0-110.8-31.5V-182c0-1.6-.9-3-2.4-3.6-1.4-.6-3.1-.3-4.2.8l-22.1 22a3.86 3.86 0 0 0 0 5.4l22 22.4zM277.9 51.6c-.6-1.4-2-2.4-3.6-2.4h-18.2a204.9 204.9 0 0 1 112.4-182.6c1.9-1 2.7-3.3 1.7-5.2-1-1.9-3.3-2.7-5.2-1.7a213.83 213.83 0 0 0-85.2 78.7 213.1 213.1 0 0 0-31.4 110.8h-18.2c-1.6 0-3 .9-3.6 2.4s-.3 3.1.8 4.2l22.1 22.1a3.86 3.86 0 0 0 5.4 0L277 55.8c1.1-1.1 1.5-2.8.9-4.2z"/>
                    </svg>
                </span>

            </h2>
            <div class="postbox">
                <div class="inside">

                    <div id="magic-page-minify-footprint-dialog" class="hidden">
                        <label>Choose only the Counties you want to Install<br />
                            <span id="placeholder-chosen-counties-to-install"></span>
                        </label>

                        <br /> <br />
                        <label>Zip Code must contain <span title="Zip code must contain (default): %zip%; start with zip%; end with %zip">[?]</span><br />
                            <span id="placeholder-chosen-zip-codes"></span>
                        </label>
                        <br /> <br />

                        <label>
                            <input type="checkbox" id="majorCity" />Install Major Locations Only
                        </label> <br /> <br />


                        <label style="float: left; margin-right: 10px">Min. Population <br />
                            <input type="number" id="min-location-population" />
                        </label>

                        <label style="float: left; margin-right: 10px">Max. Population <br />
                            <input type="number" id="max-location-population" />
                        </label>

                        <div class="clear"></div>
                        <br /> 


                        <span id="apply-region-filter" class="button button-primary">Apply Filters</span>
                    </div>

                    <?php
                    if ($allowed_countries) {
                        foreach ($allowed_countries as $country) {
                            if ($country === $chosenCountry) {
                                $file_to_load = MAGIC_PAGE_DIR . 'src/admin/maps/' . $country . '.php';

                                if (file_exists($file_to_load)) {
                                    require_once $file_to_load;
                                }

                                $this->handle_country_data($country);
                            }
                        }
                    } else {
                        $apiKey = get_option('_magicpage_api_key');
                        ?> <br /> Unfortunately we couldn't find any licensed databases with your API KEY (<b><?php echo($apiKey) ?></b>). <br />
                        To go back and enter new valid API key click <a href="<?php echo admin_url('admin.php?page=magicpage-installation&action=magic-page-new-key') ?>">here</a>
                        <br />
                        Please contact support if you believe this is an error.

                        <?php
                    }
                    ?>
                </div>
            </div>
        </div>
        <?php
    }

    public function estimate_total_installation_time($regions, $country) {
        $time = 0;

        foreach ($regions as $region) {
            if (isset($region['locations_count']) && !$this->is_region_installed($region['slug'], $region['country_slug'])) {
                $time += $this->estimate_installation_time($region, true);
            }
        }

        echo $this->formatSeconds($time);
    }

    public function estimate_installation_time($region, $unformated = false, $installed = true) {
        $locations = $region['locations_count'];

        $data = get_transient('_magic_page_api_determined_speed_optimal_working');
        $iterations = $locations / $data['items_per_page'];
        $time = $iterations * ($data['time_taken'] + 1.25); // 1.25 is average ajax api server response

        if ($unformated === true) {
            return $time;
        }

        echo $this->formatSeconds($time, $installed);
    }

    public function count_regions($regions, $country) {
        $out = 0;

        foreach ($regions as $region) {
            if ($region['country_slug'] == $country) {
                $out++;
            }
        }

        return $out;
    }

    public function get_region_names($regions, $country) {
        $out = '';

        foreach ($regions as $region) {
            if ($region['country_slug'] == $country) {
                $out .= $region['name'] . ', ';
            }
        }

        $out = rtrim($out, ', ');

        return $out;
    }

    public function is_update_only() {
        return (isset($_GET['updates_only']) && $_GET['updates_only'] == true);
    }

    public function is_all_data_installed($regions) {
        foreach ($regions as $region) {
            if (isset($region['version'])) {
                $installed = $this->is_region_installed($region['slug'], $region['country']) && !$this->is_update_only();
                $incomplete = $this->is_region_incomplete_installed($region['slug'], $region['country']);

                if (!$installed or $incomplete) {
                    return false;
                }
            }
        }
        return true;
    }

    public function handle_country_data($country) {
        ?>
        <div data-country="<?php echo($country); ?>" class="country">
            <?php
            $regions = $this->get_allowed_regions_by_country($country);
            ?><div class="magic-page-license-info"><?php
            echo( $this->count_regions($regions, $country));
            ?> Regions API License - <?php echo $this->get_region_names($regions, $country); ?></div><?php
            $population = get_option('_magic_page_population_filter');
            $populationNumberMin = 0;
            $populationNumberMax = 0;

            if ($population && isset($population[$country]) && $population[$country]) {
                $populationNumberMin = $population[$country]['minPopulation'];
                $populationNumberMax = $population[$country]['maxPopulation'];
            }

            if ($populationNumberMin) {
                $populationNumberMinClass = '';
            } else {
                $populationNumberMinClass = 'hidden';
            }

            if ($populationNumberMax) {
                $populationNumberMaxClass = '';
            } else {
                $populationNumberMaxClass = 'hidden';
            }
            ?>
            <div class="min-population-filter-row <?php echo($populationNumberMinClass) ?>">
                Minimum population filter: <b class="min-population"><?php echo($populationNumberMin); ?></b>
            </div>
            <div class="max-population-filter-row <?php echo($populationNumberMaxClass) ?>">
                Maximum population filter: <b class="max-population"><?php echo($populationNumberMax); ?></b>
            </div>

            <br /> <br />

            <button style="float:left; margin-right: 15px;" class="filterByPopulation button button-secondary">Filter By Population</button>

            <span style="float:left; margin-right: 15px;" id="install_all_databases" class="button button-primary"> Install All Databases </span>

            <span style="float:left" id="deleteUnusedLocationsDialog" class="button button-secondary"> Delete unused locations </span>

              <?php
                require_once(MAGIC_PAGE_DIR . 'src/admin/views/delete-unused-locations.html');
               ?>
            
        </div>

        <div id="continue-to-settings">
            <form  method="post" action="<?php admin_url('admin.php?page=magicpage-installation') ?>">
                <input type="hidden" name="locations_installed" value="1" />
                <input class="button button-secondary magic-page-continue-button" type="submit" value="Continue to Settings" />
            </form>
        </div>

        <div class="clear"></div>

        <div class="filter-by-population-holder hidden">
            <div> <b> Apply population filter to all regions </b></div>  <br />

            <label class="label"> Minimum population:  <br />
                <input type="number" pattern="\d*" step="1" value="<?php echo($populationNumberMin); ?>" class="minimum-population" />
            </label>

            <label class="label"> Maximum Population:  <br />
                <input type="number" pattern="\d*" step="1" value="<?php echo($populationNumberMax); ?>" class="maximum-population" />
            </label>

            <label class="label">   <br />
                <button class="button button-primary apply-population">Apply</button>
            </label>

        </div>

        <span id="uninstall_all_databases" > Uninstall All Databases </span>

        <br />  <br />  <br />

        <?php
        foreach ($regions as $id => $region) {
            $this->region_template($region, $country);
        }
        // }
        ?><input type="hidden" class="countries_data" value='<?php echo( json_encode($regions)); ?>' />


        <br />  <br />  <br />
        <div class="clear"></div>
        <?php
        if ($this->isUpdate()) {
            ?>

            <form method="post" action="<?php admin_url('edit.php?post_type=magicpage&page=magic-page-updates') ?>">
                <input type="hidden" name="update_completed" value="1" />
                <input style="margin-top:20px; display: none" class="button button-primary magic-page-continue-button" type="submit" value="Continue" />
            </form>
            <div class="clear"></div><?php
        } else {
            ?>


            <form method="post" action="<?php admin_url('admin.php?page=magicpage-installation') ?>">
                <input type="hidden" name="locations_installed" value="1" />
                <input style="margin-top:20px; display: none" class="button button-primary magic-page-continue-button" type="submit" value="Continue" />
            </form>
            <div class="clear"></div>

            <?php
        }
        ?>   <?php
    }

    function isRemainingMemoryLimitBiggerThan($memory = 32) {
        $available = getMaxMemoryLimit(true);
        $used = memory_get_peak_usage(false);
        $reserve = $memory * 1024 * 1024;

        if ($available - $used - $reserve > 0) {
            return true;
        }
    }

    function isRemainingMaxExecutionTimeBiggerThan($seconds = 10) {
        $max_execution_time = ini_get('max_execution_time');
        if (!$max_execution_time) { // max execution time is essentially disabled
            return true;
        }

        $spendMiliseconds = $this->get_time_taken();
        $remainingMiliseconds = $max_execution_time - $spendMiliseconds;
        return ($remainingMiliseconds >= $seconds);
    }

    public function get_time_taken() {
        return microtime(true) - $this->time_start;
    }

    public function insert_term_meta($term_id, $item) {
        global $wpdb;

        $region = $item[5];
        if ($item[5] == 'N/A') { // no region is set so import as blank
            $region = '';
        }

        $sql = '';
        if (isset($item[7]) && $item[7]) {
            $sql = ", ($term_id, 'major_city', '$item[7]')";
        }

        $wpdb->query("INSERT INTO $wpdb->termmeta
                (term_id, meta_key, meta_value) VALUES
            ($term_id, 'zip', '$item[1]'),
             ($term_id, 'lat', '$item[2]'),
                   ($term_id, 'lon', '$item[3]'),
                          ($term_id, 'county', '$item[4]'),
                               ($term_id, 'region', '$region'),
                                   ($term_id, 'country', '$item[6]') $sql
            ");
    }

    public function magic_page_insert_db() {
        if (!current_user_can('administrator')) {
            wp_send_json_error(array('status' => 'Security Error! Are you sure you\'re logged in?'));
        }

        global $wpdb;
        global $magicPageApi;

        $country = $_POST['country'];
        $region = $_POST['region'];
        $update = $_POST['update'];
        $min_version = false;
        $max_version = false;

        $_install_data_new = $this->get_install_data($region, $country);

        if ($update === 'update') {
            $update_data = get_region_update_info($region, $country);
            if ($_install_data_new['version'] != $update_data['version']) {
                $_install_data_new['page'] = 1;
                $min_version = $_install_data_new['version'];
                $max_version = $update_data['version'];
            }
        }


        $items_per_per_page = get_option('magic_page_installation_items_per_page', 50);
        $data = $magicPageApi->browse_database($region, $country, $items_per_per_page, $_install_data_new['page'], $min_version, $max_version);

        if ($data['status'] == 'success' && $data['data']['locations']) {
            global $wpdb;

            foreach ($data['data']['locations'] as $item) {

                $wpdb->insert($wpdb->terms, array(
                    'name' => $item[0],
                    'slug' => generate_unique_slug($item[0])
                        ), array(
                    '%s',
                    '%s',
                ));

                $term_id = $wpdb->insert_id;

                $wpdb->insert($wpdb->term_taxonomy, array(
                    'term_id' => $wpdb->insert_id,
                    'taxonomy' => 'location'
                        ), array(
                    '%d',
                    '%s',
                ));

                $this->insert_term_meta($term_id, $item);
            }


            if (!isset($data['data']) && !isset($data['data']['pagination'])) {
                $this->finalize($country, $region, $data);
            } else {

                if ($data['data']['pagination'] && isset($data['data']['pagination']['next']) && $data['data']['pagination']['next']) {
                    $this->update_data($country, $region, $data, false);

                    $return = array(
                        'next_page' => $data['data']['pagination']['next'],
                        'last_page' => $data['data']['pagination']['pageCount']
                    );

                    wp_send_json_success($return);
                } else {
                    $this->finalize($country, $region, $data);
                }
            }
        }

        wp_send_json_error();
    }

    public function finalize($country, $region, $data) {
        $this->update_data($country, $region, $data, true);
        $return = array('message' => 'all_done');

        magic_page_delete_cache();

        wp_send_json_success($return);
    }

    public function update_data($country, $region, $data, $completed) {
        if (isset($data) && !empty($data)) {
            if (!isset($data['update'])) {
                $next = isset($data['data']['pagination']['next']) && $data['data']['pagination']['next'] ? $data['data']['pagination']['next'] : $data['data']['pagination']['pageCount'];
                $save = array(
                    'version' => $data['data']['version'],
                    'notes' => $data['data']['notes'],
                    'locations_count' => $data['data']['locations_count'],
                    'date' => $data['data']['date'],
                    'page' => $next,
                    'last_page' => $data['data']['pagination']['pageCount'],
                    'completed' => $completed);
            } else {
                // still updating
                if (!$completed) {
                    $save = array(
                        'page' => isset($data['data']['pagination']['next']) ? $data['data']['pagination']['next'] : $data['data']['pagination']['pageCount'],
                        'last_page' => $data['data']['pagination']['pageCount'],
                        'completed' => $completed);
                } else {
                    // when update is completed
                    $save = array(
                        'version' => $data['update']['version'],
                        'notes' => $data['update']['notes'],
                        'locations_count' => $data['update']['locations_count'],
                        'date' => $data['update']['date'],
                        'page' => 1,
                        'last_page' => 1,
                        'completed' => $completed);

                    delete_region_update($country, $region);
                }
            }
        } else {
            $save = array(
                'version' => '',
                'notes' => '',
                'locations_count' => '',
                'date' => '',
                'page' => '',
                'last_page' => '',
                'completed' => ''
            );
        }


        $this->update_install_data($region, $country, $save);
    }

    public function magic_page_determine_speed() {
        if (!current_user_can('administrator')) {
            wp_send_json_error(array('status' => 'Security Error! Are you sure you\'re logged in?'));
        }

        $speed_data = get_transient('_magic_page_api_determined_speed');

        if (!$speed_data or!is_array($speed_data)) {
            $_data = get_transient('_magic_page_api_determined_speed_fetch_data');

            if (!$_data) {
                $_data = array(
                    'items_per_page' => 0,
                    'time_taken_within_limits' => true,
                    'memory_taken_within_limits' => true
                );
            }

            $_data['items_per_page'] = $_data['items_per_page'] + $this->increment_batch;

            if ($_data['items_per_page'] > 200) {
                $_data['items_per_page'] = 200;
            }

            global $magicPageApi;

            $data = $magicPageApi->get_sample_database($_data['items_per_page']);

            if ($data['status'] == 'success') {
                $options = $data['data'];
                if ($data['status'] == 'success' && $data['data']['locations']) {

                    global $wpdb;

                    foreach ($data['data']['locations'] as $item) {
                        $wpdb->insert($wpdb->terms, array(
                            'name' => $item[0],
                            'slug' => generate_unique_slug($item[0])
                                ), array(
                            '%s',
                            '%s',
                        ));

                        $term_id = $wpdb->insert_id;

                        $wpdb->insert($wpdb->term_taxonomy, array(
                            'term_id' => $wpdb->insert_id,
                            'taxonomy' => '_magic_page_temp'
                                ), array(
                            '%d',
                            '%s',
                        ));

                        $this->insert_term_meta($term_id, $item);
                        update_option('magic_page_installation_temp', 1);
                    }
                }
            }

            $time_taken = round($this->get_time_taken(), 1);

            $_data['time_taken_within_limits'] = $this->isRemainingMaxExecutionTimeBiggerThan();
            $_data['memory_taken_within_limits'] = $this->isRemainingMemoryLimitBiggerThan();

            set_transient('_magic_page_api_determined_speed_fetch_data', $_data, $this->time);

            // if used resources are within limit save these as optimal settings
            if ($_data['time_taken_within_limits'] && $_data['memory_taken_within_limits']) {
                // calculate time needed  count of items, time taken for each.
                $_data['time_taken'] = $time_taken;

                set_transient('_magic_page_api_determined_speed_optimal_working', $_data, $this->time);
                update_option('magic_page_installation_items_per_page', $_data['items_per_page']);

                if ($_data['items_per_page'] + $this->increment_batch > 200) {
                    wp_send_json_success(array('done' => true));
                }

                wp_send_json_success();
            }

            // use last known working settings
            else if (!$_data['time_taken_within_limits'] or!$_data['memory_taken_within_limits']) {
                $_data = get_transient('_magic_page_api_determined_speed_optimal_working');

                if (!$_data) {
                    $_data = array(
                        'items_per_page' => 50,
                        'time_taken_within_limits' => true,
                        'memory_taken_within_limits' => true,
                    );
                }

                update_option('magic_page_installation_items_per_page', $_data['items_per_page']); // 4.6

                set_transient('_magic_page_api_determined_speed', true, $this->time);
                set_transient('_magic_page_api_determined_speed_optimal_working', $_data, $this->time);
                wp_send_json_success(array(
                    'done' => true
                ));
            }

            if ($_data['items_per_page'] + $this->increment_batch > 200) {
                $speed_data = set_transient('_magic_page_api_determined_speed', true, $this->time);
                wp_send_json_success(array('done' => true));
            }
        }
    }

    public function delete_terms($taxonomy) {
        global $wpdb;

        $query = 'SELECT t.name, t.term_id
            FROM ' . $wpdb->terms . ' AS t
            INNER JOIN ' . $wpdb->term_taxonomy . ' AS tt
            ON t.term_id = tt.term_id
            WHERE tt.taxonomy = "' . $taxonomy . '" LIMIT 0,500';

        $terms = $wpdb->get_results($query);

        if ($terms) {
            foreach ($terms as $term) {
                wp_delete_term($term->term_id, $taxonomy);
            }
        }
        return count($terms);
    }

    function formatSeconds($secondsLeft, $installed = false) {
        $initial = $secondsLeft;
        $minuteInSeconds = 60;
        $hourInSeconds = $minuteInSeconds * 60;
        $dayInSeconds = $hourInSeconds * 24;

        $days = floor($secondsLeft / $dayInSeconds);
        $secondsLeft = $secondsLeft % $dayInSeconds;

        $hours = floor($secondsLeft / $hourInSeconds);
        $secondsLeft = $secondsLeft % $hourInSeconds;

        $minutes = floor($secondsLeft / $minuteInSeconds);

        $seconds = $secondsLeft % $minuteInSeconds;

        $timeComponents = array();

        if ($days > 0) {
            $timeComponents[] = $days . " day" . ($days > 1 ? "s" : "");
        }

        if ($hours > 0) {
            $timeComponents[] = $hours . " hour" . ($hours > 1 ? "s" : "");
        }

        if ($minutes > 0) {
            $timeComponents[] = $minutes . " minute" . ($minutes > 1 ? "s" : "");
        }

        if ($seconds > 0) {
            $timeComponents[] = $seconds . " second" . ($seconds > 1 ? "s" : "");
        }

        if (count($timeComponents) > 0 or $initial > 0) {
            $formattedTimeRemaining = implode(" ", $timeComponents);
            $formattedTimeRemaining = trim($formattedTimeRemaining);

            if (!$formattedTimeRemaining) {
                $formattedTimeRemaining = " less than a second ";
            }
        } else {
            if ($installed) {
                $formattedTimeRemaining = "All done!";
            } else {
                $formattedTimeRemaining = "N/A";
            }
        }

        return $formattedTimeRemaining;
    }

    public function delete_test_terms() {

        if (!current_user_can('administrator')) {
            wp_send_json_error(array('status' => 'Security Error! Are you sure you\'re logged in?'));
        }

        $this->delete_terms('_magic_page_temp');

        wp_send_json_success();
    }

    public function locations_installed() {
        ob_start();
        magic_page_rewrite_rules();
        flush_rewrite_rules();
        ob_end_clean();

        update_option('_magicpage_installed', 1);

        wp_redirect(admin_url('edit.php?post_type=magicpage&page=magic-page-settings'));
        exit;
    }

    public function update_completed() {
        wp_redirect(admin_url('edit.php?post_type=magicpage&page=magic-page-settings'));
        exit;
    }

    function is_region_installed($region, $country) {
        $data = $this->get_install_data($region, $country);

        if ($data && isset($data['page']) && isset($data['last_page']) && $data['page'] === $data['last_page'] && $data['completed'] == 1) {
            return true;
        }
    }

    function is_region_incomplete_installed($region, $country) {
        $data = $this->get_install_data($region, $country);

        if (isset($data['page']) && $data['page'] > 1 && $data['page'] < $data['last_page']) {
            return true;
        }
    }

    function get_region_complete_percent($region, $country) {
        $data = $this->get_install_data($region, $country);
        return ceil(($data['page'] / $data['last_page']) * 100);
    }

    public function update_install_data($region, $country, $new_data) {
        $magicpage_data = get_option('magic_page_install_data_new', array());

        if ($country && $region) {
            if (!isset($magicpage_data[$country])) {
                $magicpage_data[$country] = array();
            }
            if (!isset($magicpage_data[$country][$region])) {
                $magicpage_data[$country][$region] = array();
            }

            $magicpage_data[$country][$region] = $new_data;

            update_option('magic_page_install_data_new', $magicpage_data);
        }
    }

    public function get_install_data($region, $country) {
        $magicpage_data = get_option('magic_page_install_data_new', array());
        if (!isset($magicpage_data[$country])) {
            $magicpage_data[$country] = array();
        }

        if (!isset($magicpage_data[$country][$region])) {
            $magicpage_data[$country][$region] = array('page' => 1, 'item_index' => 0);
        }

        if (!isset($magicpage_data[$country][$region]['page'])) {
            $magicpage_data[$country][$region]['page'] = 1;
        }
        if (!isset($magicpage_data[$country][$region]['item_index'])) {
            $magicpage_data[$country][$region]['item_index'] = 0;
        }


        return $magicpage_data[$country][$region];
    }

    public function region_has_update($region, $country) {
        $data = get_regions_update_info();

        if (isset($data[$country]) && isset($data[$country][$region])) {
            return true;
        }
    }

    public function get_allowed_region($slug, $country) {
        $regions = $this->get_allowed_regions_by_country($country);
        foreach ($regions as $region) {
            if ($region['slug'] === $slug) {
                return $region;
            }
        }
    }

    public function getItemsPerPage() {
        $speed_data = get_transient('_magic_page_api_determined_speed_optimal_working');
        if ($speed_data && $speed_data['items_per_page'] && $speed_data['time_taken_within_limits'] &&
                $speed_data['memory_taken_within_limits']) {
            return $speed_data['items_per_page'];
        }
        return $this->increment_batch;
    }

    public function uninstall_region() {
        if (!current_user_can('administrator')) {
            wp_send_json_error(array('status' => 'Security Error! Are you sure you\'re logged in?'));
        }

        $region = $_POST['region'];
        $country = $_POST['country'];

        $region_data = $this->get_allowed_region($region, $country);

        $args = array(
            'number' => $this->getItemsPerPage(),
            'hide_empty' => false,
            'meta_query' => array(
                array(
                    'key' => 'region',
                    'value' => $region_data['name'],
                    'compare' => '='
                )
            )
                /// @todo maybe exclude custom db
                /**
                 *  $settings['meta_query']['relation'] = 'OR';
                  $settings['meta_query'][] = array(
                  array(
                  'key' => 'region',
                  'value' => $reg,
                  'compare' => '='
                  )
                  );
                 */
        );

        $args['meta_query']['relation'] = 'AND';
        $args['meta_query'][] = array(
            array(
                'key' => 'region',
                'value' => $region_data['name'],
                'compare' => '='
            )
        );

        $args['meta_query'][] = array(
            array(
                'key' => 'user_database_id',
                'compare' => 'NOT EXISTS'
            )
        );

        // not yet installed
        if (!taxonomy_exists('location')) {
            global $magic_page_location;
            $slug = magic_page_default_post_slug();
            register_taxonomy('location', array($slug), $magic_page_location);
        }

        $terms = get_terms('location', $args);

        $spintax = magic_page_get_spintax_enabled();
        if (count($terms) > 0) {
            foreach ($terms as $term) {
                if ($spintax) {
                    delete_option('_magic_page_spintax_cache_' . $term->term_id);
                }

                wp_delete_term($term->term_id, 'location');
            }
            wp_send_json_success(array('working' => true, 'count' => count($terms)));
        } else {
            // uninstall complete, return default empty value

            $this->update_install_data($region, $country, array());

            magic_page_delete_cache();
            wp_send_json_success(array('done' => true));
        }
    }

    public function region_template($region, $country) {
        if (isset($region['version'])) {
            if ($this->is_update_only() && !$this->region_has_update($region['slug'], $region['country'])) {
                return false;
            }
            if ($region['country_slug'] != $country) {
                return false;
            }

            $data = (array) $region;

            $installLabel = 'Install Database';
            $buttonClass = 'install-database';

            if ($this->is_update_only()) {
                $installLabel = 'Update Database';
                $buttonClass .= ' update-database';
            }

            $installed = $this->is_region_installed($region['slug'], $region['country']) && !$this->is_update_only();

            if ($this->region_has_update($region['slug'], $region['country']) && get_region_update_info($region['slug'], $region['country'])) {
                $data = get_region_update_info($region['slug'], $region['country']);
            }


            $incomplete = $this->is_region_incomplete_installed($region['slug'], $region['country']);

            if (is_array($data['date'])) {
                $dt = new DateTime($data['date']['date']);
            } else {
                $dt = new DateTime($data['date']);
            }

            if ($incomplete) {
                $installLabel = 'Continue Installation';
            }

            $filterData = get_magic_page_region_filter($region['country_slug'], $region['slug']);
            ?><div data-country="<?php echo($region['country_slug']) ?>" data-filter='<?php echo(json_encode($filterData)); ?>' data-counties='<?php echo($region['counties']); ?>'
                 data-info='<?php echo(json_encode($region)); ?>' data-region="<?php echo $region['slug'] ?>" class="magic_page_install_now
                 <?php if ($installed) { ?> completed <?php } ?><?php if ($incomplete) { ?> incomplete <?php } ?>"
                 <?php if ($incomplete) { ?> data-completed="<?php echo $this->get_region_complete_percent($region['slug'], $region['country']); ?>" <?php } ?>>
                <h2>
                    <span style="width:35px; <?php if (!$installed) { ?> display: none <?php } ?>" class="icon-small completed">
                        <svg version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px"
                             viewBox="0 0 100 100" enable-background="new 0 0 100 100" xml:space="preserve">
                            <path fill="#6ece66" d="M49.6,0C22.2,0,0,22.2,0,49.6s22.2,49.6,49.6,49.6s49.6-22.2,49.6-49.6S77.1,0,49.6,0z M49.6,86.1
                                  c-20.1,0-36.4-16.3-36.4-36.4s16.3-36.4,36.4-36.4s36.4,16.3,36.4,36.4S69.8,86.1,49.6,86.1z"/>
                            <path id="XMLID_1_" fill="#38c52c" d="M61.5,35.1L46.3,50.2l-5.3-5.3c-2.6-2.6-6.8-2.6-9.4,0c-2.6,2.6-2.6,6.8,0,9.4l9.9,9.9
                                  c2.6,2.6,6.8,2.6,9.4,0c0,0,0,0,0,0l19.9-19.9c2.6-2.6,2.6-6.8,0-9.4C68.3,32.4,64.1,32.4,61.5,35.1L61.5,35.1L61.5,35.1z"/>
                        </svg>
                    </span>
                    <div id="region-<?php echo $region['slug'] ?>" class="div-progress">

                    </div> <?php echo $region['name'] ?>
                </h2>
                <div class="clear"></div>

                <div class="db_version_1"><b>Database Version:</b> <?php echo $data['version'] ?>
                    <span class="button <?php echo $buttonClass ?> button-primary"><?php echo $installLabel; ?></span>
                    <span class="button uninstallRegion button-secondary" data-country="<?php echo($country) ?>" data-region="<?php echo($region['slug']) ?>">Uninstall Region </span>

                    <b>Released on:</b> <?php echo $dt->format('d/m/Y') ?> <br />
                    <b>Contains:</b>   <?php echo $data['locations_count'] ?> Locations  <?php if (!$this->isEmptyFilter($filterData)) { ?> with filters<?php } ?> <br />
                    <div class="db_version_2">  <?php if ($data['notes'] && $this->is_update_only()) { ?> <b>Release notes:</b> <br /> <?php
                            echo $data['notes'];
                        }
                        ?>


                    </div>
                    <br />


                    <span class="button-apply-filters choose-install-options <?php if (!$this->isEmptyFilter($filterData)) { ?> has-filter <?php } ?>">
                        <?php if (!$this->isEmptyFilter($filterData)) { ?>
                            <span class="filter-data-icon">
                                <svg xmlns="http://www.w3.org/2000/svg" width="18" height="18"><path fill="#3BBCA6" d="M17.8 9.1C17.8 4.3 13.9.3 9 .3S.2 4.2.2 9.1s3.9 8.8 8.8 8.8 8.8-4 8.8-8.8z"/><path fill="#FFF" d="M13.8 6.1c-.2-.5-.6-.4-1.1-.4-.3.1-1.5.4-3.4 2.4-.8.9-1.3 1.6-1.6 2.1-.2-.3-.5-.5-.7-.8-.8-.8-1.7-1.3-1.7-1.3-.4-.2-.9-.1-1.1.2-.2.4-.1.9.3 1.1 0 0 .8.5 1.4 1.1.7.7 1.3 1.6 1.3 1.6.1.2.4.3.6.3h.1c.3-.1.5-.3.6-.6 0 0 .3-.9 1.9-2.6 1.3-1.4 2.2-1.8 2.5-1.9h.1c.1-.1.4-.2.4-.2.4-.2.6-.6.4-1z"/></svg>
                            </span>
                        <?php } ?>
                        Filters
                    </span>

                </div>

                <div class="clear"></div>


            </div>



            <?php
        }
    }

    public function save_region_filter() {
        if (current_user_can('administrator')) {
            $data = get_magic_page_region_filter();

            $country = $_POST['country'];
            $region = $_POST['region'];

            $regionData = array(
                'majorCity' => $_POST['majorCity'],
                'counties' => $_POST['counties'],
                'zip' => $_POST['zip'],
                'minPopulation' => intval($_POST['minPopulation']),
                'maxPopulation' => intval($_POST['maxPopulation'])
            );

            if ($regionData['majorCity'] !== 'false' || $regionData['counties'] !== NULL || $regionData['zip'] !== NULL || $regionData['minPopulation'] !== NULL || $regionData['maxPopulation'] !== NULL
            ) {
                $data[$country][$region] = $regionData;
            } else if (isset($data[$country][$region])) {
                unset($data[$country][$region]);
            }

            update_option('_magic_page_region_filter', $data);

            global $magicPageApi;
            $data = $magicPageApi->get_licensed_databases_list(get_option('_magicpage_api_key'));

            if ($data['status'] == 'success') {
                set_transient('_magic_page_api_licensed_databases_list', $data['data'], $this->time);
            }

            $_region = $this->get_allowed_region($region, $country);

            ob_start();
            $this->region_template($_region, $country);
            $content = ob_get_contents();
            ob_end_clean();

            wp_send_json_success(array('content' => $content));
        }

        wp_send_json_error();
    }

    public function isEmptyFilter($filter) {
        if (empty($filter)) {
            return true;
        }

        if ($filter && isset($filter['majorCity']) && $filter['majorCity'] == 'false') {
            unset($filter['majorCity']);

            if (!array_filter($filter)) {
                return true;
            }
        }

        return false;
    }

    public function save_database_chosen_country() {
        if (current_user_can('administrator')) {
            update_option('_magic_page_chosen_country_db_install', $_POST['chosenCountry']);

            wp_send_json_success();
        } else {
            wp_send_json_error();
        }
    }

    public function save_population() {
        if (current_user_can('administrator')) {
            $data = get_magic_page_region_filter();

            $country = $_POST['country'];
            $regions = $_POST['regions'];
            $minPopulation = intval($_POST['minPopulation']);
            $maxPopulation = intval($_POST['maxPopulation']);

            $emptyData = array(
                'minPopulation' => 0, 'maxPopulation' => 0, 'zip' => '', 'counties' => '', 'majorCity' => false
            );

            foreach ($regions as $region) {
                if ($minPopulation || $maxPopulation) {
                    if (!isset($data[$country]) || !isset($data[$country][$region])) {
                        $data[$country][$region] = $emptyData;
                    }

                    $data[$country][$region]['minPopulation'] = $minPopulation;
                    $data[$country][$region]['maxPopulation'] = $maxPopulation;
                } else {
                    if (!$data[$country][$region]['zip'] && !$data[$country][$region]['counties'] && $data[$country][$region]['majorCity'] == false) {
                        unset($data[$country][$region]);
                    }
                }
            }

            update_option('_magic_page_region_filter', $data);

            $populationFilter = get_option('_magic_page_population_filter', array());
            $populationFilter[$country] = array('minPopulation' => $minPopulation, 'maxPopulation' => $maxPopulation);

            update_option('_magic_page_population_filter', $populationFilter);

            global $magicPageApi;

            $data = $magicPageApi->get_licensed_databases_list(get_option('_magicpage_api_key'));
            if ($data['status'] == 'success') {
                set_transient('_magic_page_api_licensed_databases_list', $data['status'], $this->time);
            }

            $content = array();

            foreach ($regions as $region) {
                ob_start();
                $_region = $this->get_allowed_region($region, $country);
                $this->region_template($_region, $country);
                $content[$region] = ob_get_contents();
                ob_end_clean();
            }


            wp_send_json_success(array('content' => $content));
        }

        wp_send_json_error();
    }

    public function forceDatabaseRefresh() {
        if (current_user_can('administrator')) {
            magic_page_fetch_new_api_data();
            wp_send_json_success();
        }
        wp_send_json_error();
    }

    private function shouldLoad() {
        global $pagenow;

        if ($pagenow == 'admin.php' && isset($_GET['page']) && $_GET['page'] === 'magicpage-installation') {
            return true;
        }

        return false;
    }

    public function deleteUnusedLocations() {
        if (current_user_can('administrator')) {

            $data = $_POST['data'];
            $offset = intval($data['offset']);
            $databaseData = $data['databaseData'] != null ? $data['databaseData'] : [];

            $magicpage = get_magic_page();
            $locs = wp_get_object_terms($magicpage->ID, 'location');
            $locationObjects = [];

            if ($locs) {
                foreach ($locs as $loc) {
                    $distance = get_term_meta($loc->term_id, 'distance', true);
                    $locations[] = ['id' => $loc->term_id, 'radius' => $distance];
                    $locationObjects[] = $loc;
                }
            }


            $query1 = new MagicPageQuery();
            $query1->setLocations($locations);
            $query1->setExcludeTerms($locationObjects);
            $query1->setFilter2(true);
            $query1->setAutoinstalled(magic_page_is_autoinstalled_enabled());
            $query1->setQueryType('result');

            if ($data['count']) {
                $query1->setItemsPerPage(20);
                $query1->setOffset(0);
            }

            $query1->setIsStaticPage(true);
            $query1->setPostType(['post', 'page']);

            $locations = $query1->getResults();

            if (!$data['count']) {
                $count = count($locations);
            } else {
                $count = intval($data['count']);

                if ($locations) {
                    foreach ($locations as $location) {

                        $country = get_term_meta($location->term_id, 'country', true);
                        if ($country) {
                            if (!isset($databaseData[$country])) {
                                $databaseData[$country] = [];
                            }

                            $region = get_term_meta($location->term_id, 'region', true);
                            if ($region) {
                                if (!in_array($region, $databaseData[$country])) {
                                    $databaseData[$country][] = $region;
                                }
                            }
                        }


                        if (wp_delete_term($location->term_id, 'location')) {
                            $offset++;
                        }
                    }
                }
            }

            $response = [
                'count' => $count,
                'offset' => $offset,
                'databaseData' => $databaseData
            ];

            if ($offset >= $count) {
                $response['done'] = true;

                global$magicPageApi;
                $databaseApi = $magicPageApi->get_licensed_databases_list(get_option('_magicpage_api_key'));

                foreach ($databaseData as $country => $regions) {
                    foreach ($regions as $region) {
                        foreach ($databaseApi['data'] as $regionApiDatabase) {
                            if ($regionApiDatabase['country_name'] == $country && $regionApiDatabase['name'] == $region) {
                                $this->update_install_data($regionApiDatabase['slug'], $regionApiDatabase['country_slug'], array());
                            }
                        }
                    }
                }
            }

            magic_page_delete_cache();

            wp_send_json_success($response);
        }
    }

}

new MagicPageInstallation();
