<?php
if (!defined('ABSPATH')) {
    exit;
}

class MagicPageLocationSets {

    public function __construct() {
        add_action('admin_init', array($this, 'admin_init'));
        add_action('wp_ajax_save_location_set', array($this, 'save_location_set'));
        add_action('wp_ajax_reload_location_sets_table', array($this, 'reload_location_sets_table'));
        add_action('wp_ajax_delete_location_set', array($this, 'delete_location_set'));
        add_action('wp_ajax_get_locations_in_location_set', array($this, 'get_locations_in_location_set'));
    }

    public function get_locations_in_location_set() {
        if (current_user_can('edit_post', intval($_POST['post_id']))) {
            $id = $_POST['id'];
            $sets = $this->getLocationSets();

            if (isset($sets[$id])) {
                return wp_send_json_success(array('locations' => $sets[$id][3]));
            }
        }
    }

    public function reload_location_sets_table() {
        if (current_user_can('edit_post', intval($_POST['post_id']))) {
            $this->table();
            exit();
        }
    }

    public function delete_location_set() {
        if (current_user_can('edit_post', intval($_POST['post_id']))) {
            $id = $_POST['id'];

            $array = $this->getLocationSets();
            if ($id && isset($array[$id])) {
                unset($array[$id]);
                $this->updateLocationSets($array);
            }

            wp_send_json_success();
        }
    }

    public function admin_init() {
        if (isset($_GET['export_location_sets'])) {
            $this->export_location_sets();
        }

        if (isset($_GET['import-location-sets']) && isset($_FILES['files'])) {
            $this->import_location_sets();
        }

        add_meta_box('location_sets_box', 'Magic Page Location Sets', array($this, 'box'), magic_page_default_post_slug(), 'normal', 'low');
    }

    public function getLocationSets() {
        return get_option('location_sets', array());
    }

    public function updateLocationSets($sets) {
        update_option('location_sets', $sets);
    }

    public function generateLocationSet($id, $radius, $limit) {
        $location = get_term_by('id', $id, 'location');

        if (!$location) {
            return false;
        }

        return get_nearby_location_set($location, $radius, $limit);
    }

    public function table() {
        ?>

        <div id="magic-page-location-sets-dialog" class="hidden" style="max-width:800px"> </div>

        <table width="100%" cellpadding="0" cellspacing="0" class="table location-sets-table">
        <?php if ($this->getLocationSets()) { ?>
                <tr>
                    <th>Label</th>
                    <th>Location</th>
                    <th>Radius from Location</th>
                    <th>Locations Count</th>
                    <th>Locations</th>
                    <th>Delete Set</th>
                </tr>

            <?php
            $i = 0;

            foreach ($this->getLocationSets() as $label => $location) {
                $term = get_term_by('id', $location[0], 'location');

                $name = isset($term->name) ? $term->name : 'N/A';
                $countCount = count($location[3]);
                $locationJson = $countCount <= 50 ? json_encode($location[3]) : json_encode(array());
                ?>
                    <tr data-id="<?php echo $label ?>" <?php if ($i >= 5) { ?> style="display: none" <?php } ?>>
                        <td>
                            <div class="input-group ">
                                <input class="form-control input-monospace input-sm js-zeroclipboard-target js-url-field " id="location_set_id_<?php echo $i ?>" type="text" value="%location-set-<?php echo($label); ?>%">
                                <span class="input-group-button">
                                    <div class="input-group-button">
                                        <button aria-label="Copy to clipboard"  data-clipboard-target="#location_set_id_<?php echo $i ?>" class="js-zeroclipboard btn btn-sm zeroclipboard-button tooltipped tooltipped-s" data-copied-hint="Copied!" type="button"><svg aria-hidden="true" class="octicon octicon-clippy" height="16" version="1.1" viewBox="0 0 14 16" width="14"><path fill-rule="evenodd" d="M2 13h4v1H2v-1zm5-6H2v1h5V7zm2 3V8l-3 3 3 3v-2h5v-2H9zM4.5 9H2v1h2.5V9zM2 12h2.5v-1H2v1zm9 1h1v2c-.02.28-.11.52-.3.7-.19.18-.42.28-.7.3H1c-.55 0-1-.45-1-1V4c0-.55.45-1 1-1h3c0-1.11.89-2 2-2 1.11 0 2 .89 2 2h3c.55 0 1 .45 1 1v5h-1V6H1v9h10v-2zM2 5h8c0-.55-.45-1-1-1H8c-.55 0-1-.45-1-1s-.45-1-1-1-1 .45-1 1-.45 1-1 1H3c-.55 0-1 .45-1 1z"></path></svg></button>
                                    </div>
                                </span>
                            </div>
                        </td>
                        <td><?php echo($name); ?></td>
                        <td><?php echo($location[1]); ?></td>
                        <td><?php echo($countCount); ?></td>
                        <td><span data-locations='<?php echo($locationJson); ?>' class="view-locations">View Locations</span></td>
                        <td> <span class="delete-location-set"> Delete </span></td>
                    </tr>

                <?php
                $i++;
            }

            if ($i > 5) {
                ?><tr>
                        <td class="text-center" colspan="6"> <span class="button-large button-secondary" id="show-all">Show All</span> </td>
                    </tr>
                    <?php
                }
            }
            ?>
        </table> <?php
        }

        public function box() {

            global $post;
            $this->table();
            ?>

        <div style="display: none" class="location-sets">
            <div class="location-set big-field">
                Label <br />
                <input class="field label" type="text" />
            </div>

            <div class="location-set big-field">
                Choose Location  <br />
                <input class="field location" type="text" />
                <input class="field location_id" type="hidden" />
            </div>

            <div class="location-set">
                Radius from Location <br />
                <input class="field radius" type="number" />
            </div>

            <div class="location-set">
                Locations Count <br />
                <input class="field limit" type="number" />
            </div>

            <span id="magic-page-save-location-set"> Save Location Set</span>
            <span id="magic-page-cancel-location-set"> Cancel </span>
        </div>


        <input id="magic-page-new-location-set" type="button" class="button button-primary button-large" value="New Location Set" />

        <a href="?export_location_sets=true&post_id=<?php echo($post->ID) ?>" class="button button-secondary button-large" target="_blank">  Export </a>

        <span class="btn btn-success fileinput-button button button-secondary">
            <i class="glyphicon glyphicon-plus"></i>
            <span>Import</span>
            <!-- The file input field used as target for the file upload widget -->
            <input id="magic-page-fileupload" type="file" name="files[]" multiple="" />
        </span>

        <?php
    }

    public function save_location_set() {
        if (current_user_can('edit_post', intval($_POST['post_id']))) {
            $label = sanitize_title_with_dashes($_POST['label']);
            $id = intval($_POST['location_id']);
            $radius = intval($_POST['radius']);
            $limit = intval($_POST['limit']);

            $sets = $this->getLocationSets();

            if (isset($sets[$label])) {
                wp_send_json_error(array('status' => 'This Label already exists'));
            }

            if (!($label)) {
                wp_send_json_error(array('status' => 'Please fill in label field'));
            }

            if (!($id)) {
                wp_send_json_error(array('status' => 'Please fill in location field'));
            }

            if (!($radius)) {
                wp_send_json_error(array('status' => 'Please fill in radius field'));
            }

            if (!($limit)) {
                wp_send_json_error(array('status' => 'Please fill in limit field'));
            }


            $set = $this->generateLocationSet($id, $radius, $limit);

            if ($set === false) {
                wp_send_json_error(array('status' => 'This Location cannot be found, please fill in other Location'));
            }

            if (empty($set)) {
                wp_send_json_error(array('status' => 'No Locations have been found with the given parameters, please try different settings'));
            }

            $sets[$label] = array($id, $radius, $limit, $set);

            $this->updateLocationSets($sets);

            wp_send_json_success();
        }
    }

    public function export_location_sets() {
        if (current_user_can('edit_post', intval($_GET['post_id']))) {
            $locationSets = $this->getLocationSets();

            $filename = "magic-page-location-sets-" . date("Y-m-d") . ".txt";

            $now = gmdate("D, d M Y H:i:s");
            header("Expires: Tue, 03 Jul 2001 06:00:00 GMT");
            header("Cache-Control: max-age=0, no-cache, must-revalidate, proxy-revalidate");
            header("Last-Modified: {$now} GMT");

            header("Content-Type: application/force-download");
            header("Content-Type: application/octet-stream");
            header("Content-Type: application/download");

            header("Content-Disposition: attachment;filename={$filename}");
            header("Content-Transfer-Encoding: binary");

            $fp = fopen("php://output", 'w');
            fwrite($fp, serialize($locationSets));
            fclose($fp);
        }
        die();
    }

    public function import_location_sets() {
        if (current_user_can('edit_post', intval($_GET['post_id']))) {

            if (isset($_FILES['files']['tmp_name'][0])) {
                $contents = file_get_contents($_FILES['files']['tmp_name'][0]);

                $array = unserialize($contents);

                if (is_array($array) && !empty($array)) {

                    foreach ($array as $id => $item) {
                        foreach ($item[3] as $locId => $locTerm) {
                            $maybeLoc = $this->maybe_location($locTerm->name, $locTerm->lat, $locTerm->lon);

                            if (!$maybeLoc) {
                                // remove this location as it's not found in the db
                                unset($array[$id][3][$locId]);
                            } else {
                                // reassign found term_id
                                $array[$id][3][$locId]->term_id = $maybeLoc->term_id;
                            }
                        }
                    }

                    $existing = $this->getLocationSets();
                    $new_arr = array_merge($existing, $array);
                    $this->updateLocationSets($new_arr);

                    wp_send_json_success();
                }
            }
        }
    }

    function maybe_location($name, $lat, $lon) {
        global $wpdb;

        return $wpdb->get_row($wpdb->prepare("
       SELECT $wpdb->terms.term_id,
       $wpdb->terms.name,
       pm2.meta_value as lat,
       pm3.meta_value as lon

       FROM $wpdb->terms
            LEFT JOIN $wpdb->termmeta AS pm2 ON ($wpdb->terms.term_id = pm2.term_id AND pm2.meta_key = 'lat')
            LEFT JOIN $wpdb->termmeta AS pm3 ON ($wpdb->terms.term_id = pm3.term_id AND pm3.meta_key = 'lon')
       WHERE $wpdb->terms.name = %s AND pm2.meta_value = %s AND pm3.meta_value = %s
         LIMIT 0,1", array($name, $lat, $lon)));
    }

}

new MagicPageLocationSets();
