<?php

if (!defined('ABSPATH')) {
    exit;
}

class MagicPagePostList {

    public function __construct() {
        add_filter('manage_edit-page_columns', array($this, 'addColumns'));
        add_filter('manage_edit-post_columns', array($this, 'addColumns'));
        add_action('manage_page_posts_custom_column', array($this, 'manageColumns'), 10, 2);
        add_action('manage_post_posts_custom_column', array($this, 'manageColumns'), 10, 2);
        add_action('admin_init', array($this, 'filter'));
        add_filter('views_edit-page', array($this, 'views'));
        add_filter('views_edit-post', array($this, 'views'));
    }

    public function filter() {
        global $pagenow, $typenow;
        if ('edit.php' == $pagenow && in_array($typenow, array('page', 'post'))) {
            add_action('parse_query', array($this, 'set_viewable_posts_by_cat'));
        }
    }

    public function addColumns($cols) {
        global $current_screen;

        $thumbCol = array();

        if (checkPostsWithMetaExist('_location_id', $current_screen->post_type)) {
            $thumbCol['magic-page-location'] = 'Location';
        }

        if (checkPostsWithMetaExist('_group_id', $current_screen->post_type)) {
            $thumbCol['magic-page-export-group'] = 'Group';
        }

        return $this->arrayPushAfter($cols, $thumbCol, 'title');
    }

    public function manageColumns($column, $postId) {

        switch ($column) {
            case 'magic-page-location':
                $locationId = get_post_meta($postId, '_location_id', true);

                if ($locationId) {
                    $location = get_term($locationId, 'location');

                    if ($location) {
                        $locationName = true;
                        echo($location->name) . ' (' . $locationId . ')';
                    }
                }


                if (!isset($locationName)) {
                    $post = get_post($postId);
                    $location = getMagicPageDefaultTerm(true);

                    if ($location) {
                        $locationName = true;
                        echo($location->name) . ' (Default)';
                    }
                }



                break;
            case 'magic-page-export-group':
                $exportGroupId = get_post_meta($postId, '_group_id', true);
                if ($exportGroupId) {
                    $groups = get_option('_magic_page_groups');

                    if ($groups && isset($groups[$exportGroupId])) {
                        $url = esc_url(add_query_arg('group', $exportGroupId));
                        echo '<a href="' . $url . '">' . $groups[$exportGroupId]['title'] . '</a>';
                    }
                }

                break;
        }
    }

    public function set_viewable_posts_by_cat($query) {

        if (isset($_GET['group']) && $_GET['group']) {
            $meta_query = array(
                array(
                    'key' => '_group_id',
                    'value' => strip_tags($_GET['group']),
                    'compare' => '='
                )
            );

            $query->set('meta_query', $meta_query);
        }


        return $query;
    }

    public function views($views) {
        global $current_screen;

        $types = get_post_types(array(
            'public' => true,
                ), 'objects');

        $title = '';
        foreach ($types as $id => $type) {
            if ($id === $current_screen->post_type) {
                $title = $type->labels->all_items;
            }
        }


        if (isset($_GET['group'])) {
            $exportGroupId = strip_tags($_GET['group']);
            $groups = get_option('_magic_page_groups');

            if (isset($groups[$exportGroupId])) {
                $url = esc_url(remove_query_arg('group'));

                echo '<p>Viewing ' . $title . ' in <b>' . $groups[$exportGroupId]['title'] . '</b> magic page group <a href="' . $url . '">[x]</a></p> ';
            }
        }




        return $views;
    }

    protected function arrayPushAfter($src, $in, $pos) {
        if (is_int($pos))
            $r = array_merge(array_slice($src, 0, $pos + 1), $in, array_slice($src, $pos + 1));
        else {
            foreach ($src as $k => $v) {
                $r[$k] = $v;
                if ($k == $pos)
                    $r = array_merge($r, $in);
            }
        }
        return $r;
    }

}

new MagicPagePostList();
