<?php
if (!defined('ABSPATH')) {
    exit;
}

class MagicPageSchemaBuilderView {

    public function __construct() {
        add_action('admin_enqueue_scripts', array($this, 'admin_scripts'));
        add_action('admin_footer', array($this, 'view'));
    }

    public function admin_scripts() {


        if (isAdvancedSchemaCompatable() && $this->shouldLoad()) {

            wp_enqueue_script('ace', plugins_url('/js/ace/ace.js', MAGIC_PAGE_FILE), array('jquery'), MAGIC_PAGE_VERSION);
            wp_enqueue_script('d3', plugins_url('/js/d3.v3.min.js', MAGIC_PAGE_FILE), array(), MAGIC_PAGE_VERSION);
            wp_enqueue_script('d3-tip', plugins_url('/js/d3-tip.js', MAGIC_PAGE_FILE), array('d3'), MAGIC_PAGE_VERSION);
            wp_enqueue_script('jspagination', plugins_url('/js/pagination.min.js', MAGIC_PAGE_FILE), array('jquery'), MAGIC_PAGE_VERSION);

            wp_enqueue_style('wp-jquery-ui-dialog');

            wp_enqueue_style('schema-builder-font', plugins_url('/font/schemabuilder.css', MAGIC_PAGE_FILE), array(), MAGIC_PAGE_VERSION);
            wp_enqueue_style('schema-builder-pagination', plugins_url('/css/schema-builder-pagination.css', MAGIC_PAGE_FILE), array(), MAGIC_PAGE_VERSION);
            wp_enqueue_style('schema-builder-style', plugins_url('/css/schema-builder-style.css', MAGIC_PAGE_FILE), array(), MAGIC_PAGE_VERSION);

            wp_enqueue_script('jsrender', plugins_url('/js/jsrender.min.js', MAGIC_PAGE_FILE), array(), MAGIC_PAGE_VERSION);
            wp_enqueue_script('confirm-js', plugins_url('/js/jquery-confirm.js', MAGIC_PAGE_FILE), array('jquery'), MAGIC_PAGE_VERSION);

            wp_register_style('select2css', plugins_url('/css/select2.min.css', MAGIC_PAGE_FILE), false, '1.0', 'all');
            wp_register_script('select2', plugins_url('/js/select2.full.min.js', MAGIC_PAGE_FILE), array('jquery'), '1.0', true);
            wp_enqueue_style('select2css');
            wp_enqueue_script('select2');

            wp_enqueue_script('jquery-ui-tabs');

            if (magicPageCompatabilityModeEnabled()) {
                global $magicpage_cdn_url;

                if (!defined('MAGIC_PAGE_SCHEMA_BUILDER')) {
                    define('MAGIC_PAGE_SCHEMA_BUILDER', $magicpage_cdn_url . MAGIC_PAGE_VERSION . '/schema-builder.js');
                }
            } else {
                if (!defined('MAGIC_PAGE_SCHEMA_BUILDER')) {
                    define('MAGIC_PAGE_SCHEMA_BUILDER', plugins_url('/js/schema-builder.js', MAGIC_PAGE_FILE));
                }
            }


            wp_enqueue_script('schema-builder', MAGIC_PAGE_SCHEMA_BUILDER, array('ace', 'd3-tip', 'jquery-ui-tabs', 'jspagination', 'jsrender', 'confirm-js', 'jquery-ui-tabs', 'select2'), MAGIC_PAGE_VERSION);
        }
    }

    private function shouldLoad() {
        global $pagenow, $typenow;

        if (empty($typenow) && !empty($_GET['post'])) {
            $post = get_post($_GET['post']);

            if ($post) {
                $typenow = $post->post_type;
            }
        }

        if (($pagenow == 'post-new.php' or $pagenow == 'post.php') && $typenow == magic_page_default_post_slug()) {
            return true;
        }

        return false;
    }

    public function view() {
        if (!isAdvancedSchemaCompatable()) {
            return false;
        }

        if (!$this->shouldLoad()) {
            return false;
        }
        ?>

        <script id="schema-catalog-list" type="text/x-jsrender">

            {{if agreement }}   {{else}} 
            <div id="notice-schema-agreement">Every schema in the catalog is validated and approved by us but use with caution at your own risk.  <b style="cursor: pointer;" id="schema-agreement-ok">[x]</b></div>
            {{/if}}


            <div id="schema-catalog-list">
            {{:catalog}}
            </div>	

        </script> 
        <script id="save-schema-form-catalog" type="text/x-jsrender">

            <div {{if name && email && password}}  {{else}}  style="display:none"  {{/if}}  id="save-schema-form-catalog-template">


            <span {{if name }}  {{else}}  style="display:none"  {{/if}} id="api-catalog-logged-in-bar"> 
            <b id="api-catalog-name-b">{{:name}}</b> you are logged in
            </span>
            <br />

            <b> Schema Information </b><br />

            <div class="field-row"> <input autocomplete="off" id="new-schema-title" type="text" placeholder="Schema Title" />  </div>
            <div class="field-row"> <input autocomplete="off" id="new-schema-description" type="text" placeholder="Schema Description" />  </div>
            <div class="field-row"> <input autocomplete="off" id="new-schema-tags" type="text" placeholder="Schema Tags, ex: tag1, tag2" /> </div>
            <button id="save-schema-catalog" class="ui-button ui-corner-all ui-widget">Submit</button>
            <button id="cancel-save-schema" class="ui-button ui-corner-all ui-widget">Cancel</button>
            </div>


            <div id="tabs-schema-profile">
            <ul>
            <li id="tab-login"><a href="#tabs-schema-profile-1">Login</a></li>
            <li id="tab-profile"><a href="#tabs-schema-profile-2">Profile</a></li>
            <li id="tab-register"><a href="#tabs-schema-profile-3">Register</a></li>
            </ul>

            <div id="tabs-schema-profile-1">
            Login if you already have an account with our Schema Catalog API <br /><br />
            <input value="{{:email}}" class="account-information regular-text" id="api-login-email" type="text" placeholder="Your Email" /><br />
            <input value="{{:password}}" class="account-information regular-text" id="api-login-password" type="password" placeholder="Your Password" />
            <br /> <br />

            <button id="login-schema-builder-api" class="button button-primary button-large">Login</button>
            </div> 

            <div id="tabs-schema-profile-2">
            <div class="field-row">
            <input type="text" placeholder="Name" class="input" id="api-account-name" />
            </div>
            <div class="field-row">
            <input type="text" placeholder="Email" class="input" id="api-account-email" />
            </div>       
            <div class="field-row">
            <input type="password" placeholder="Password" class="input" id="api-account-password" />
            </div>   
            <div class="field-row">
            <input type="text" placeholder="About" id="api-account-content" />
            </div>         

            <button id="save-api-profile" class="ui-button ui-corner-all ui-widget">Save</button>
            <button id="cancel-save-api-profile" class="ui-button ui-corner-all ui-widget">Close</button>

            <span id="api-catalog-logout">Logout</span>
            </div>

            <div id="tabs-schema-profile-3">
            Create new account <br />
            <div class="field-row">

            <input class="account-information" id="api-account-name" type="text" placeholder="Your Name or Nickname" />
            <input class="account-information" id="api-account-email" type="text" placeholder="Your Email" />
            <input style="margin-right:0" class="account-information" id="api-account-password" type="password" placeholder="Your Password" />
            <br /> <br />  
            <i><small>All fields are required. Your email will remain private</small></i><br /><br />
            <button class="button button-primary button-large" id="register-api-account">Register</button>
            <br /> 
            </div>

            </div>
            </div>
            </div>            


        </script>

        <script id="save-schema-form-own" type="text/x-jsrender">

            <b> Schema Information </b><br />
            <div class="field-row"> <input id="new-schema-title" type="text" placeholder="Schema Title" />  </div>
            <div class="field-row"> <input id="new-schema-description" type="text" placeholder="Schema Description" />  </div>
            <div class="field-row"> <input id="new-schema-tags" type="text" placeholder="Schema Tags, ex: tag1, tag2" /> </div>

            <button id="save-schema-own" class="ui-button ui-corner-all ui-widget">Submit</button>
            <button id="cancel-save-schema" class="ui-button ui-corner-all ui-widget">Cancel</button>
        </script>

        <script id="schema-parser-output" type="text/x-jsrender">
            <div class="schema-parser-output">
            {{if grouptitle}}<div class="grouptitle">{{:grouptitle}}</div>{{/if}}
            <div data-mode="3" class="parsed-schema-item">
            <div class="parsed-schema-title">
            <span title="Import Schema" class="json-builder-button">
            <i class="icon-plus"></i>
            </span>
            <span class="schema-type">{{:title}}</span>
            </div>

            <div class="schema-option-group">
            <span data-type="{{:type}}" data-id="{{:id}}" title="Schema Visualizer" class="json-builder-button visualize">
            <i class="icon-visualizer"></i> Visualizer
            </span>

            <span data-type="{{:type}}" data-id="{{:id}}" title="Schema Preview" class="json-builder-button preview">
            <i class="icon-view"></i> Preview
            </span>
            </div>

            <div style="display:none" class="schema-exists-options">
            <b> {{:title}} </b> Schema already exists.
            We will <span class="fully-replace-existing-schema" style="display:none">FULLY OVERWRITE EXISTING SCHEMA</span>
            <span class="schema-join">
            <span class="schema-join-mode">JOIN </span>
            the 2 schemas - we will prefer <span class="schema-keep-existing">KEEPING YOURS</span>
            </span>
            </div>

            </div>
            </div>
        </script>


        <script id="schema-row-template" type="text/x-jsrender">

            <div class="schema-row-template {{if schema.pending==true}}schema-pending{{/if}}" data-id="{{:schema.id}}" data-schema-index="{{:schemaIndex}}">
            <div class="title">
            {{if schema.pending==true}} <span title="Pending Approval" class="pending"><i class="icon-pending"></i> </span>{{/if}} {{:schema.title}}
            </div>
            <div class="description">
            {{:schema.description}}
            </div>
            <div class="tags">
            {{:~tagssplit(schema.tags) }}
            </div>

            <div class="actions">
            <span class="visualize-schema" title="Visualize Schema"><i class="icon-visualizer"></i> Visualizer</span>
            <span class="preview-schema" title="Preview Schema"><i class="icon-view"></i> Preview</span>
            <span class="use-schema" title="Import Schema"><i class="icon-save"></i> Import</span>
            <span class="date"><i class="icon-date"></i> {{:schema.date}} </span>


            {{if schema.author_name}}
            <span class="filter-by-author" data-author-id="{{:schema.author_id}}"><i class="icon-user"></i> by {{:schema.author_name}}</span>
            {{else}}
            <span class="delete" title="Delete Schema"> <i class="icon-delete"> </i> Delete</span>
            {{/if}}
            </div>
            </div>
        </script>

        <script id="table-view-td-template" type="text/x-jsrender">
            <tr class="table-tr level-{{:schemaLevel}} {{if css }}{{:css}}{{/if}}"><td class="json-value-td ">{{:id}}</td><td>{{:tddata}}</td></tr>
        </script>
        <script id="table-view-td2-template" type="text/x-jsrender">
            <tr class="table-tr level-{{:schemaLevel}}"><td class="json-value-td" colspan="2">{{:id}}</td></tr>
        </script>

        <script id="table-view-td-int-template" type="text/x-jsrender">
            <tr class="table-tr level-{{:schemaLevel}}"><td colspan="2">  {{:tddata}}</td></tr>
        </script>


        <script id="rule-template" type="text/x-jsrender">
            <div id="builder-basic_rule_{{:id}}" class="rule-container">
            <div class="rule-header">
            <div class="btn-group pull-right rule-actions">
            <span data-delete="rule" title="Delete Node" class="json-builder-button delete-button">
            <i class="icon-delete"></i> Delete
            </span>
            </div>
            </div>

            <div class="rule-filter-container">
            <input type="text" class="form-control id" placeholder="id" />
            </div>

            <div class="rule-filter-container value">
            <input type="text" class="form-control value" placeholder="value" />
            </div>
            </div>

            </div>
        </script>
        <script id="rule-group" type="text/x-jsrender">
            <div id="builder-basic_group_{{:id}}" class="rules-group-container {{if collapse =="true" }}collapsed{{/if}}">
            <div class="rules-group-header">
            <div class="rule-filter-container">
            <input type="text" placeholder="Key" class="form-control id" />
            <span class="json-builder-button">
            <i title="Collapse Group" class="icon-collapse"></i>
            <i title="Expand Group" class="icon-expand"></i>
            </span>
            </div>
            <div class="btn-group pull-right group-actions">

            <span data-add="rule" title="Add Node" class="json-builder-button btn-add-property">
            <i class="icon-plus"></i> Add Node
            </span>

            <span data-add="group" title="Add Group Node" class="json-builder-button btn-add-property">
            <i class="icon-plus"></i> Add Group Node
            </span>

            <span title="Duplicate Group Node" class="json-builder-button duplicate-button">
            <i class="icon-clone"></i> Duplicate
            </span>

            <span data-delete="group" title="Delete Group Node" class="json-builder-button delete-button">
            <i class="icon-delete"></i> Delete
            </span>

            </div>
            </div>
            <div class="rules-group-body">
            <div class="collapsed-block"></div>
            <div class="rules-list">

            </div>

            </div>
            </div>
        </script>


        <script id="render-filter-display" type="text/x-jsrender">
            {{if settings.search || settings.filterAuthorName || settings.filterAuthorContent}}
            {{if settings.search && settings.search != '-1'}}
            <span data-search="true" class="filter-item">
            search: <span class="filter-value">{{:settings.search}}</span>  <span class="remove-filter">[x]</span>
            </span>
            {{/if}}

            {{if settings.filterAuthorName}}
            <span data-author="true" class="filter-item">
            by author: <span class="filter-value">{{:settings.filterAuthorName}}</span> <span class="remove-filter">[x]</span>
            </span>
            {{/if}}

            {{if settings.filterAuthorContent}}
            <div id="schema-author-about">About {{:settings.filterAuthorName}}: {{:settings.filterAuthorContent}} </div>
            {{/if}}


            {{/if}}
        </script>


        <div style="display: none" id="schema-templates-dialog" title="Schema Templates">
            <div class="row-actions-templates">
                <span data-mode="catalog" title="Browse the Schema Catalog" class="schema-browser-button">Schema Catalog </span>
                <span data-mode="own" title="Browse your Private Schemas" class="schema-browser-button">Your Schemas </span>

                <div style="float:right">
                    <input autocomplete="off" placeholder="Search..." id="search-schema-templates" type="search" /> <i id="do-search-schema-templates" class="icon-search"></i>
                </div>

                <div style="float:right">
                    <span id="import-export-private-schemas" title="Import / Export your schemas" data-type="save" class="private-schema-button schema-button"><i class="icon-save"></i> Import/Export </span>
                    <span id="show-private-schemas" title="Save current Schema as a Template" data-type="save" class="private-schema-button schema-button"><i class="icon-save"></i> Save Current Schema </span>
                    <span title="Share current Schema in the Catalog" data-type="share" class="share-current-schema schema-button"><i class="icon-upload"></i> Share Schema </span>
                </div>
            </div>

            <div id="schema-used-filters"></div>

            <div id="schema-list"></div>
            <div id="schema-catalog-pagination"></div>
            <div id="save-schema-form"></div>
            <div id="import-schema-step"></div>

            <span style="display: none" id="do-import-schema3" title="Import the Schemas in the Schema Builder" class="ui-button ui-corner-all ui-widget">
                <i class="icon-save"></i> Import the Schemas
            </span>

            <span style="display: none" id="cancel-import-schema3" title="Cancel" class="ui-button ui-corner-all ui-widget">
                <i class="icon-cancel"></i> Cancel
            </span>

        </div>


        <div style="display: none" id="visualizer-schema-dialog" title="Schema Visualizer">
            <div id="schema-graph"></div>
        </div>

        <div style="display: none" id="import-export-private-schemas-dialog" title="Import / Export Schemas">

            Note:  Importing Schema file will delete your currently saved schemas.<br />

            Pick an action: <br /><br />

            <span class="btn btn-success fileinput-button button button-secondary">
                <span>Import Schema</span>
                <input id="magic-page-fileupload-schemas" type="file" name="files[]" multiple="">
            </span>

            &nbsp;

            <a href="?magic-page-export-private-schemas" class="button button-secondary button-large" target="_blank">  Export Schema </a>   



        </div>



        <div style="display: none" id="schema-options-dialog" title="Schema Builder Options">

            <div class="row5">
                <div class="row5">Scraper Mode </div>
                <label>
                    <input <?php echo($this->get_json_builder_option_checked('json_format')) ?> id="json_format" type="checkbox" value="jsonld" type="checkbox" /> JSON-LD Format
                </label>
                <label>
                    <input <?php echo($this->get_json_builder_option_checked('microdata_format')) ?> id="microdata_format"  type="checkbox" value="1" type="checkbox" /> MICRODATA Format
                </label>
            </div>

            <div class="row5">
                <label>
                    <input <?php echo($this->get_json_builder_option_checked('confirmation_deleting_node')) ?> id="confirmation_deleting_node" type="checkbox" value="1" /> Require confirmation before deleting a Node
                </label>
            </div>

            <div class="row5">
                <label>
                    <input <?php echo($this->get_json_builder_option_checked('auto_collapse_group_nodes')) ?> id="auto_collapse_group_nodes" type="checkbox" value="1" /> Auto collapse Group Nodes
                </label>
            </div>

            <div class="row5">
                <label>
                    <input <?php echo($this->get_json_builder_option_checked('highlight_nodes')) ?> id="highlight_nodes" type="checkbox" value="1" /> Highlight Nodes on mouseover
                </label>
            </div>
            <div class="row5">
                <label>
                    <input <?php echo($this->get_json_builder_option_checked('enable_php_functions')) ?> id="enable_php_functions" type="checkbox" value="1" /> Enable PHP functions
                </label>
            </div>
        </div>

        <div style="display: none" id="import-schema-dialog" title="Import Schema Into the Schema Builder">
            <div id="schema-import-tabs">
                <ul>
                    <li id="webpage-scrape-tab"><a href="#schema-import-tabs-2">SCRAPE FROM A WEBPAGE</a></li>
                    <li id="paste-json-ld-code-tab"><a href="#schema-import-tabs-1">JSON-LD CODE</a></li>
                </ul>
                <div id="schema-import-tabs-1">
                    <div id="temp1"> <div class="row5">Paste JSON format Schema</div>
                        <div id="paste-schema"></div>
                        <br /> <br />
                        <span id="do-import-schema4" title="Process JSON Schema" class="ui-button ui-corner-all ui-widget">
                            <i class="icon-save"></i> Process JSON Schema</span>
                    </div>
                    <div id="temp2"></div>

                    <br /><br />

                    <span style="display: none" id="do-import-schema5" title="Import the Schemas in the Schema Builder" class="ui-button ui-corner-all ui-widget">
                        <i class="icon-save"></i> Import the Schema
                    </span>

                </div>
                <div id="schema-import-tabs-2">
                    <div class="row5">Paste URL to scrape Schema <div style="display:none" id="loading-scrape-schema"></div></div>
                    <div id="webpage-scrap-wrap">
                        <i class="icon-internet"></i>
                        <input type="url" id="webpage-scrape-input" placeholder="Enter URL" />
                        <span id="do-import-schema" title="Scrape schemas from this web page" class="ui-button ui-corner-all ui-widget"> <i class="icon-download"></i> Scrape</span>
                    </div>

                    <div id="schema-parser-results">

                    </div>

                    <br /><br />

                    <span style="display: none" id="do-import-schema2" title="Import the Schemas in the Schema Builder" class="ui-button ui-corner-all ui-widget">
                        <i class="icon-save"></i> Import the Schemas</span>

                </div>
            </div>
        </div>

        <div style="display: none" id="help-schema-dialog" title="Help">
            <br />
            <div class="row5"> 
                <b> Tip 1</b>: You can call PHP functions inside the Schema Builder (<b>Enable PHP functions</b> from Options must be checked.), 
                Use this feature with caution! Don't execute functions which you're not sure what they do or your site might be affected. 

                <br /> <br /> Example 1: Returns your site URL <pre> &lt;?php echo(get_site_url()); ?&gt;  </pre> 
                <br /> Example 2: Returns your site URL with suffix <pre> &lt;?php echo(get_site_url()).'#suffix'; ?&gt;  </pre> 
                <br /> Example 3: Returns admin email <pre> &lt;?php echo(get_option('admin_email')); ?&gt;  </pre>

                <hr> <br /> 

                <b> Tip 2</b>: You can use Magic Page Shortcodes: <br /> 
                <br /> Example 1: Cities Shortcode: <br />
                <img src="<?php echo(MAGIC_PAGE_URL) . 'img/img-examples/cities.png' ?>"  />
                <br /> <br /> Example 2: Location Shortcode: <br />
                <img src="<?php echo(MAGIC_PAGE_URL) . 'img/img-examples/location.png' ?>"  />

            </div>   
        </div>

        <div style="display: none" id="preview-schema-dialog" title="Preview Schema">
            <div id="preview-schema-dialog-parent">
                <div id="leftCol">
                    <h3>Code</h3>
                    <div id="json-code-preview"></div>
                </div>
                <div id="rightCol">
                    <h3 style="padding-left:10px">Overview</h3>
                    <table id="table-schema-preview"></table>
                </div>
            </div>
        </div>

        <div data-options='<?php $this->get_json_ld_builder_options() ?>' id="json-ld-builder" class="query-builder hidden">
            <div style="padding:0px 15px;">
                <a id="closeSchemaBuilder" style="float: right; margin-right:10px" href="#"><i class="icon-cancel"></i> Close</a>
                <h3>JSON-LD Schema Builder</h3>

                <div style="margin-bottom:10px; width: 100%;">
                    <span title="Import Schema" id="import-schema" class="json-builder-button">
                        <i class="icon-upload"></i> Import
                    </span>

                    <span title="Schema Templates" id="schema-templates" class="json-builder-button">
                        <i class="icon-templates"></i> Templates
                    </span>

                    <span title="Schema Options" id="schema-options" class="json-builder-button">
                        <i class="icon-options"></i> Options
                    </span>

                    <span title="Create New Schema" id="create-new-schema" class="json-builder-button">
                        <i class="icon-plus"></i> New Schema
                    </span>

                    <span title="Help" id="help-schema" class="json-builder-button">
                        <i class="icon-help"></i> Help
                    </span>

                    <span title="Save Schema" id="save-current-schema" class="json-builder-button">
                        <i class="icon-save"></i> Save Schema
                    </span>

                    <div style="display:none" id="loading-save-schema-label">Saving...</div>
                </div>

                <div id="schema-builder-container">
                    <div id="rules-group-container-1" class="rules-group-container">
                        <div class="rules-group-header">
                            <div class="btn-group pull-left group-actions">


                                <span title="Minimize Schema" class="json-builder-button minimize-schema">
                                    <i class="icon-resize-small"> </i> Minimize
                                </span>


                                <span title="Maximize Schema" class="json-builder-button maximize-schema">
                                    <i class="icon-resize-small"> </i> Maximize
                                </span>


                                <span title="Schema Visualizer" class="json-builder-button visualize-schema">
                                    <i class="icon-visualizer"></i> Visualizer
                                </span>

                                <span title="Schema Preview" class="json-builder-button preview-schema">
                                    <i class="icon-view"></i> Preview
                                </span>

                                <span title="Delete Schema" class="json-builder-button delete-schema">
                                    <i class="icon-delete"> </i> Delete
                                </span>
                            </div>

                            <span class="schema-label"></span>

                        </div>
                        <div class="rules-group-body">
                            <div class="rules-list">

                            </div>

                            <div>
                                <span data-add="rule" title="Add Node" class="json-builder-button btn-add-property">
                                    <i class="icon-plus"></i> Add Node
                                </span>
                                <span data-add="group" title="Add Group Node" class="json-builder-button btn-add-property">
                                    <i class="icon-plus"></i> Add Group Node
                                </span>
                            </div>

                        </div>
                    </div></div>
            </div></div>






        <div id="export-to-static-page-holder" class="query-builder hidden">

            <div style="padding:0px 15px;">


                <a id="closeExportToStaticPage" style="float: right; margin-right:10px" href="#"><i class="icon-cancel"></i> Close</a>
                <h3>Export To Posts & Pages</h3>


                <div class="rules-group-container">
                    <div class="rule-container">
                        <div style="overflow: hidden; margin-bottom: 15px;">


                            <b>URL OPTIONS</b> 

                            <span style="float: right">
                                <i class="icon-plus"></i> Help <!--Possible shortcodes: [location], [county], spintax: {spintax|is|possible}</icon> -->
                            </span>

                        </div> 

                        <div class="rule-row">
                            <div class="variations-left">
                                <label>Url</label>
                                <input type="text" class="form-control value-field" placeholder="magicpage/service/[location]" />
                            </div>  
                            <div class="variations-right">
                                Priority <span class="priority-percent">100</span> %
                                <div class="priority-percent-slider"></div> 
                            </div> 
                            <div class="variations-delete">
                                <a href="#"><i class="icon-cancel"></i> Delete</a>
                            </div>
                        </div>



                    </div>


                    <span id="add-url-option" title="Add Url Option" class="json-builder-button ">
                        <i class="icon-plus"></i> Add Url Option
                    </span>




                </div>


            </div>

        </div>
        <?php
    }

    function get_json_builder_option_checked($option) {
        $options = $this->get_json_ld_builder_options(false);

        if (isset($options[$option]) && $options[$option] === 'true') {
            return (' checked="checked" ');
        }
    }

    function get_json_ld_builder_options($jsonResponse = true) {
        $chosen = get_schema_builder_options();

        if (!$jsonResponse) {
            return $chosen;
        }

        echo json_encode($chosen);
    }

}

$schemaBuilderView = new MagicPageSchemaBuilderView();
