<?php
if (!defined('ABSPATH')) {
    exit;
}

use YusufKandemir\MicrodataParser\Microdata;
use Symfony\Component\Validator\Validation;
use Symfony\Component\Validator\Constraints\NotBlank;
use Symfony\Component\Validator\Constraints as Assert;
use Topwebstudio\Parsers\JsonLdParser;
use Topwebstudio\Parsers\MicrodataParser;
use Topwebstudio\Parsers\SchemaConverter;
use JasonGrimes\Paginator;

class SchemaBuilder {

    public function __construct() {


        add_action('init', array($this, 'init'));

        add_action('wp_ajax_magic_page_schema_builder_load_schema', array($this, 'load_schema')); // ok
        add_action('wp_ajax_magic_page_schema_builder_save_schema', array($this, 'save_schema'));
        add_action('wp_ajax_magic_page_schema_builder_save_options', array($this, 'save_options'));

        add_action('wp_ajax_magic_page_schema_builder_api_register', array($this, 'builder_api_register'));
        add_action('wp_ajax_magic_page_schema_builder_api_login', array($this, 'builder_api_login'));
        add_action('wp_ajax_magic_page_schema_builder_api_save_profile', array($this, 'api_save_profile'));

        add_action('wp_ajax_magic_page_schema_builder_save_options_catalog', array($this, 'save_options_catalog'));
        add_action('wp_ajax_magic_page_schema_builder_save_template_catalog', array($this, 'save_template_catalog'));
        add_action('wp_ajax_magic_page_schema_builder_save_template', array($this, 'save_template'));
        add_action('wp_ajax_magic_page_schema_builder_delete_schema', array($this, 'delete_schema'));
        add_action('wp_ajax_magic_page_schema_builder_load_data', array($this, 'load_data'));
        add_action('wp_ajax_magic_page_schema_builder_load_schema_feeds', array($this, 'load_schema_feeds'));
        add_action('wp_ajax_magic_page_schema_builder_save_settings', array($this, 'save_settings'));
        add_action('wp_ajax_magic_page_schema_builder_parser', array($this, 'parser'));
        add_action('wp_ajax_magic_page_schema_builder_load_schema_templates', array($this, 'load_schema_templates'));

        add_action('admin_notices', array($this, 'noticeFiles'));
    }

    public function canEdit() {
        $postId = isset($_POST['post_id']) ? intval($_POST['post_id']) : false;

        if ($postId && current_user_can('edit_post', $postId)) {
            return true;
        }
    }

    public function noticeFiles() {
        $compatability = magicPageCompatabilityModeEnabled();
        $file1 = MAGIC_PAGE_DIR . 'js/magic-page-admin-page.js';
        $file2 = MAGIC_PAGE_DIR . 'js/schema-builder.js';

        if (strlen(MAGIC_PAGE_VERSION) >= 11) {
            return;
        }

        if (!$compatability && (!file_exists($file1) || !file_exists($file2))) {
            update_option('_magicpage_compatability_mode', '1');
            ?>


            <div class="notice notice-success is-dismissible">
                <p> An important javascript file(s) are missing from magic-page folder.  <br />
                    Some hosts may recognize the code obfuscation we use as a virus and delete these files, don't panic they are not using proven tools but merely attempt to 
                    find obfuscation which we use. <br />
                    In this case you can load the file(s) directly from our server, we have enabled this option for you now since these files are missing (see Compatability under Footprint Minifier). <br />

                </p>
            </div> <?php
        }
    }

    public function init() {
        $user = wp_get_current_user();
        
        if (array_intersect(array('editor', 'administrator', 'author'), $user->roles)) {

            if (isset($_GET['magic-page-export-private-schemas'])) {
                $this->exportPrivateSchemas();
                exit();
            }

            if (isset($_GET['import-magic-page-schemas'])) {

                $this->importPrivateSchemas();
                exit();
            }
        }
    }

    public function importPrivateSchemas() {

        if (isset($_FILES['files']['tmp_name'][0])) {
            $contents = file_get_contents($_FILES['files']['tmp_name'][0]);

            if ($contents) {
                $array = json_decode($contents, true);

                if (is_array($array) && !empty($array)) {
                    $this->save_schema_builder_templates($array);
                    wp_send_json_success();
                }
            }
        }
    }

    private function exportPrivateSchemas() {
        $data = $this->get_schema_builder_templates();

        $filename = "magic-page-json-schemas-" . date("Y-m-d") . ".json";

        $now = gmdate("D, d M Y H:i:s");
        header("Expires: Tue, 03 Jul 2001 06:00:00 GMT");
        header("Cache-Control: max-age=0, no-cache, must-revalidate, proxy-revalidate");
        header("Last-Modified: {$now} GMT");

        header("Content-Type: application/force-download");
        header("Content-Type: application/octet-stream");
        header("Content-Type: application/download");

        header("Content-Disposition: attachment;filename={$filename}");
        header("Content-Transfer-Encoding: binary");

        $fp = fopen("php://output", 'w');
        fwrite($fp, json_encode($data));
        fclose($fp);
        die();
    }

    public function converter($schema, $term) {
        $converter = new SchemaConverter($schema);
        $converter->setFunctions(array(
            'apply_all_filters' => array($term),
            'invoke_php_function' => array())
        );

        return $converter->getOutput();
    }

    public function load_schema() {
        if ($this->canEdit()) {
            $templates = get_magic_page_schema();

            header('Content-Type: application/json');
            echo json_encode($templates);
        }
        exit();
    }

    public function save_schema() {
        if ($this->canEdit()) {
            if (!empty($_POST) && isset($_POST['schemaElementsArray'])) {
                $schema = $this->getSchemaResponse($_POST['schemaElementsArray']);

                $this->save_post_schema($schema);
            }
        }

        exit('OK');
    }

    public function save_options() {
        if ($this->canEdit()) {
            if ($_POST['options']) {
                $options = get_schema_builder_options();
                foreach ($_POST['options'] as $option => $value) {
                    $options[$option] = $value;
                }
            }

            $this->save_schema_builder_options($options);
        }

        exit('OK');
    }

    public function save_options_catalog() {
        if ($this->canEdit()) {
            $_options = $_POST['options'];

            $options = get_schema_builder_options();

            if ($_options) {
                foreach ($_options as $option => $value) {
                    $options[$option] = $value;
                }
            }

            $this->save_schema_builder_options($options);

            header('Content-Type: application/json');
            echo json_encode();
        }
        exit();
    }

    public function save_template_catalog() {
        if ($this->canEdit()) {
            $data = array(
                'title' => $_POST['title'],
                'description' => $_POST['description'],
                'tags' => $_POST['tags'],
                'schema' => $this->getSchemaResponse($_POST['schema']),
                'email' => $_POST['email'],
                'uid' => $_POST['uid'],
            );

            global $magicPageApi;

            $response = $magicPageApi->schema_submit($data);

            echo json_encode($response);
        }
        exit();
    }

    public function save_template() {
        if ($this->canEdit()) {
            $title = $_POST['title'];
            $id = strtolower(trim(preg_replace('/[^A-Za-z0-9-]+/', '-', $title))); // sluggify function
            $description = $_POST['description'];
            $tags = $_POST['tags'];
            $schema = $this->getSchemaResponse($_POST['schema']);
            $datetime = new \DateTime();

            $templates = $this->get_schema_builder_templates();

            if (!$title || !$description) {
                header('Content-Type: application/json');
                echo json_encode(array('error' => 'true', 'message' => 'Title and Description are mandatory fields'));
                exit();
            }

            if ($templates && isset($templates[$id])) {
                header('Content-Type: application/json');
                echo json_encode(array('error' => 'true', 'message' => 'Schema with the same title already exists'));
                exit();
            }

            $templates = array_reverse($templates);

            $templates[$id] = array(
                'id' => $id,
                'title' => $title,
                'description' => $description,
                'tags' => $tags,
                'schema' => $schema,
                'date' => $datetime->format('Y-m-d')
            );

            $templates = array_reverse($templates);

            $this->save_schema_builder_templates($templates);
        }

        exit('OK');
    }

    public function delete_schema() {
        if ($this->canEdit()) {
            $id = $_POST['id'];

            $templates = $this->get_schema_builder_templates();

            if ($templates && isset($templates[$id])) {
                unset($templates[$id]);

                $this->save_schema_builder_templates($templates);

                exit('OK');
            }
        }
    }

    public function load_data() {
        if ($this->canEdit()) {
            $output = array();

            $data = isset($_POST['schemaElementsArray']) ? $_POST['schemaElementsArray'] : [];
            $schemas = $this->getSchemaResponse($data);

            if ($schemas) {
                if (!isset($_POST['flag'])) {
                    foreach ($schemas as $id => $schema) {
                        $converter = new SchemaConverter($schema);
                        $output[$id] = $converter->getOutput();
                    }
                } else {

                    $converter = new SchemaConverter($schemas);
                    $output = $converter->getOutput();
                }
            }

            header('Content-Type: application/json');

            echo json_encode($output, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE);
        }
        exit();
    }

    public function load_schema_feeds() {
        
        
        if ($this->canEdit()) {
            header('Content-Type: application/json');

            global $magicPageApi;

            $response = $magicPageApi->schema_browse_database();
            echo(json_encode($response));
        }
        exit();
    }

    public function save_settings() {
        if ($this->canEdit()) {
            $settings = $_POST['settings'];

            $saved = get_schema_builder_options();

            if ($settings) {
                foreach ($settings as $setting => $value) {
                    $saved[$setting] = $value;
                }
            }

            $this->save_schema_builder_options($saved);

            echo('OK');
        }
        exit();
    }

    public function parser() {
        if ($this->canEdit()) {
            $url = trim($_POST['url']);
            $options = $_POST['options'];

            $validator = Validation::createValidator();
            $violations = $validator->validate($url, array(
                new NotBlank(),
                new Assert\Url(array('checkDNS' => Assert\Url::CHECK_DNS_TYPE_ANY))
            ));

            if (0 !== count($violations)) {
                $output = array('error' => 'true', 'message' => '');
                foreach ($violations as $violation) {
                    $output['message'] .= $violation->getMessage() . ' ';
                }

                header('Content-Type: application/json');
                echo json_encode($output);
                exit();
            }

            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, $url);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 15);
            curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
            curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
            $content = curl_exec($ch);

            if (curl_errno($ch)) {
                header('Content-Type: application/json');
                echo(json_encode(array('message' => curl_error($ch), 'error' => 'true')));
                exit();
            }

            curl_close($ch);

            if (!$content) {
                header('Content-Type: application/json');
                echo(json_encode(array('message' => 'This page is not responding or has no content', 'error' => 'true')));
                exit();
            }

            $result = array('microdata' => array(), 'jsonld' => array());

            if ($options['microdata_format'] == 'true') {
                $microdata = Microdata::fromHTML($content)->toArray();

                $microdataParser = new MicrodataParser($microdata);
                $result['microdata'] = $microdataParser->getParsedData();
            }

            if ($options['json_format'] == 'true') {
                $jsonld = new JsonLdParser($content);
                $result['jsonld'] = $jsonld->getParsedData();
            }

            header('Content-Type: application/json');
            echo(json_encode(['output' => $result, 'error' => 'false'], JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE));
        }

        exit();
    }

    public function load_schema_templates() {
        if ($this->canEdit()) {
            $itemsPerPage = 5;
            $options = get_schema_builder_options();
            $array = $this->get_schema_builder_templates();

            if (isset($options['search']) && $options['search'] && !empty($array)) {
                $found = array();
                foreach ($array as $id => $items) {
                    if (stripos(serialize($items), $options['search']) !== false) {
                        $found[$id] = $items;
                    }
                }

                $array = $found;
            }

            $currentPage = isset($options['page']) && $options['page'] ? $options['page'] : 1;
            $paginator = new Paginator(count($array), $itemsPerPage, $currentPage);

            $pageRange = array();

            if ($paginator->getPages()) {
                foreach ($paginator->getPages() as $page) {
                    $pageRange[] = $page['num'];
                }
            }

            $pagination = array(
                'totalCount' => count($array),
                'numItemsPerPage' => $itemsPerPage,
                'current' => intval($currentPage),
                'pageRange' => $pageRange
            );

            $start = intval($currentPage) * $itemsPerPage - $itemsPerPage;

            $schemas = array_slice($array, $start, $itemsPerPage);

            header('Content-Type: application/json');
            echo json_encode(array(
                'schemas' => $schemas,
                'pagination' => $pagination,
                'parameters' => $options
            ));
        }
        exit();
    }

    private function save_post_schema($data) {
        $post = get_magic_page();
        if ($post) {
            update_post_meta($post->ID, '_schema_builder_schema', stripslashes_deep($data));
        }
    }

    private function save_schema_builder_options($options) {
        update_option('schema_builder_options', $options);
    }

    private function get_schema_builder_templates() {
        return get_option('schema_builder_templates', array());
    }

    private function save_schema_builder_templates($templates) {
        update_option('schema_builder_templates', stripslashes_deep($templates));
    }

    public function builder_api_register() {
        if ($this->canEdit()) {
            header('Content-Type: application/json');

            global $magicPageApi;

            $data = $_POST['data'];

            if (!$data['email'] || !$data['password'] || !$data['name']) {
                header('Content-Type: application/json');
                echo json_encode(array('error' => 'true', 'info' => 'You must provide name, email and password fields.'));
                exit();
            }

            $data['password'] = md5($data['password']);

            $response = $magicPageApi->schema_api_register($data);
            echo(json_encode($response));
        }
        exit();
    }

    public function builder_api_login() {
        if ($this->canEdit()) {
            header('Content-Type: application/json');

            global $magicPageApi;

            $data = $_POST['data'];

            if (!$data['email'] || !$data['password']) {
                header('Content-Type: application/json');
                echo json_encode(array('error' => 'true', 'info' => 'You must provide email and password fields.'));
                exit();
            }

            $data['password'] = md5($data['password']);

            $response = $magicPageApi->schema_api_login($data);
            echo(json_encode($response));
        }
        exit();
    }

    public function api_save_profile() {
        if ($this->canEdit()) {
            header('Content-Type: application/json');

            global $magicPageApi;

            $data = $_POST['data'];

            if (!$data['name'] || !$data['email']) {
                header('Content-Type: application/json');
                echo json_encode(array('error' => 'true', 'info' => 'You must provide email and name '));
                exit();
            }

            if (isset($data['password'])) {
                $data['password'] = md5($data['password']);
            }


            $response = $magicPageApi->schema_api_save_profile($data);
            echo(json_encode($response));
        }
        exit();
    }

    protected function getSchemaResponse($res = null) {

        if ($res) {

            $response = stripslashes_deep($res);

            if (gettype($response) === 'string') {
                return json_decode($response, true);
            }

            if (gettype($response) === 'array') {
                return $response;
            }
        }
    }

}

$advancedSchemaBuilder = new SchemaBuilder();
