<?php
if (!defined('ABSPATH')) {
    exit;
}

use JasonGrimes\Paginator;

class MagicPageSeoMetabox {

    public function __construct() {
        add_action('add_meta_boxes', array($this, 'setupMetaBoxes'));
        add_action('save_post', array($this, 'saveMetabox'));
        add_action('wp_ajax_magicpage_locations_search', array($this, 'locationSearch'));
        add_action('wp_ajax_magicpage_get_available_locations', array($this, 'getAvailableLocations'));
        add_action('admin_init', array($this, 'previewMagicPageUrl'));
        add_action('wp_ajax_magicpage_update_locations_count', array($this, 'locationsCount'));
        add_action('wp_ajax_magicpage_keep_available_locations', array($this, 'installAutolocations'));
        add_action('wp_ajax_magicpage_get_autoinstall_data', array($this, 'getAutoinstallStats'));
        add_action('wp_ajax_magicpage_delete_autoinstalled_locations', array($this, 'deleteAutoinstalledLocations'));

        add_action('wp_ajax_magicpage_update_post_use_group_settings', array($this, 'updatePostUseGroupSettings'));
        add_action('wp_ajax_magicpage_check_post_custom_settings', array($this, 'checkPostCustomSettings'));
        add_action('wp_ajax_magicpage_check_post_custom_settings_group_enabled', array($this, 'checkPostCustomSettingsGroupEnabled'));
        add_action('wp_ajax_magicpage_schema_builder_plugin_enable_support', array($this, 'schemaBuilderPluginEnableSupport'));
    }

    public function previewMagicPageUrl() {
        if (isset($_GET['magicpage-preview-url']) && $_GET['magicpage-preview-url']) {
            global $wp_rewrite;

            if (isset($_GET['_token']) && $_GET['_token'] && wp_verify_nonce($_GET['_token'], 'magicpage')) {
                $term = get_term(intval($_GET['magicpage-preview-url']), 'location');

                if ($term && $wp_rewrite && isset($wp_rewrite->extra_permastructs['magicpage'])) {
                    $path = str_replace('%magicpage%', $term->slug, $wp_rewrite->extra_permastructs['magicpage']['struct']);
                    $url = get_home_url('', $path);

                    wp_redirect($url);
                    exit;
                }
            }
        }
    }

    public function setupMetaBoxes() {
        if (current_user_can('administrator') || current_user_can('editor')) {
            if (get_option('_magic_page_seo_for_all_pages') === '1' || !is_post_page()) {
                add_meta_box('seo_box', 'Magic Page SEO Settings', array($this, 'seobox'), array(magic_page_default_post_slug(), 'post', 'page'), 'normal', 'low');
            }

            add_meta_box('location', 'Location', array($this, 'location'), magic_page_default_post_slug(), 'side', 'high');
        }
    }

    public function selected($value1, $value2) {
        if ($value1 == $value2)
            echo 'selected="selected" ';
    }

    public function businessTypeOption($option, $selected) {
        $v = preg_replace('/\s+/', '', $option);
        ?> <option <?php $this->selected($v, $selected); ?> value="<?php echo($v) ?>"><?php echo($option); ?></option><?php
    }

    public function location($post) {
        $defaultLocationId = get_post_meta($post->ID, '_magic_page_default_location', true);
        $locationName = 'Choose Default';

        if ($defaultLocationId) {
            $terms = wp_get_post_terms($post->ID, 'location');

            if ($terms) {
                foreach ($terms as $term) {
                    if ($term->term_id == $defaultLocationId) {
                        $defaultLocation = get_term_by('id', $defaultLocationId, 'location');
                        $locationName = $defaultLocation->name;
                    }
                }
            }
        }
        ?>

        <input style="width: 95%; margin-top:10px;" type="text" class="magic-page-locs" />

        <p class="howto">Search Locations by name or Zip code.

            Default location: <span id="defaultLocation"><?php echo($locationName); ?></span>


            <input id="magic_page_default_location" type="hidden" name="_magic_page_default_location" value="<?php echo($defaultLocationId) ?>" />
        </p>

        <?php
        $locations = wp_get_object_terms($post->ID, 'location');
        $locationsWithDistance = 0;
        ?> <div class="tagchecklist"> <?php
        if ($locations) {

            foreach ($locations as $location) {
                $zip = get_term_meta($location->term_id, 'zip', true);
                $region = get_term_meta($location->term_id, 'region', true);
                $distance = get_term_meta($location->term_id, 'distance', true);

                if ($distance) {
                    $locationsWithDistance++;
                }


                echo('<span class="location-item"><button type="button" class="ntdelbutton"><span class="remove-tag-icon"></span></button> &nbsp; '
                . '<input class="loc_hidden" type="hidden" name="_locations[]" value="' . $location->term_id . '" />' . $location->name . ', ' . $region . ', ' . $zip . ''
                . '<input class="radius_field" title="Set Radius" type="text" name="distance[' . $location->term_id . ']" value="' . $distance . '" /></span> <br />');
            }
        }
        ?></div>


        <div <?php if ($locationsWithDistance <= 5) { ?> style="display:none" <?php } ?> id="recommended-max-locations"> It's recommended to use  up to 5 locations with radius for maximum site performance</div>

        <div style="display:none" id="locations-count-installed-dialog">
            <div class="loader"><div class="dot dot1"></div><div class="dot dot2"></div><div class="dot dot3"></div><div class="dot dot4"></div></div>
        </div>


        <div style="display:none" id="delayed-locations-dialog-info">
            <div id="autolocations-chart-placeholder">
                <div id="autolocations-chart"></div>

                <div class="rightdata">
                    <div data-days="7" class="days-option selected">Last 7 days</div>
                    <div data-days="14" class="days-option">Last 14 days</div>
                    <div data-days="30" class="days-option">Last 30 days</div>
                    <div> Avg. Rate: <span class="averageCount"></span> per day</div>
                </div>
                <span id="deleteAutoinstallLocations">Delete Auto installed Locations</span>
            </div>
        </div>

        <div id="delayed-locations-dialog">
            <div class="opt-notice hidden" id="delayed-locations-optin-notice">
                <span class="dsds">

                    <svg version="1.1"  xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px"
                         viewBox="0 0 120 120" enable-background="new 0 0 120 120" xml:space="preserve">

                        <g transform="translate(-359.000000, -119.000000)">
                            <g  transform="translate(215.000000, 119.000000)">
                            </g>
                            <path  fill="#6fcc8a" d="M370.3,169.8
                                   c4.4-24.2,25.7-42.6,51.2-42.6c28.7,0,52,23.3,52,52s-23.3,52-52,52c-20.2,0-37.7-11.5-46.3-28.4h5.4c8.2,14.1,23.5,23.6,41,23.6
                                   c26.1,0,47.3-21.2,47.3-47.3c0-26.1-21.2-47.3-47.3-47.3c-22.9,0-42,16.2-46.4,37.8H370.3z M419.2,180.9l24.6,14.5l2.4-4.1
                                   l-22.3-13.3v-31.8h-4.7V180.9z M371.9,184l11.8-14.2H360L371.9,184z"/>
                        </g>
                    </svg>
                </span>

                <h3>You're turning on auto install locations option</h3>

                <div class="row">Choose what happens with the locations which were already installed. <br /></div>
                <div class="row"><b>Option 1</b>: Discard all installed locations and start from 0 (all previously available pages will become 404).  <br /></div>
                <div class="row"><b>Option 2:</b> Keep all available locations up to this point and use auto install for the next locations you pick <b>(recommended for established sites)</b>.  <br />
                    <br /></div>
                <span id="discardAvaiableLocations" class="button-locations-dialog"> Choose Option 1 </span>
                <span id="keepAvaiableLocations" class="button-locations-dialog"> Choose Option 2 </span>

                <span class="loadingHourGlass hidden">
                    <svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 100 100" preserveAspectRatio="xMidYMid" class="lds-hourglass" style="background:0 0"><defs><clipPath id="a"><path d="M0 9.547h100V50H0z"><animate attributeName="y" calcMode="spline" values="0;50;0;0;0" keyTimes="0;0.4;0.5;0.9;1" dur="3.4" keySplines="0.3 0 1 0.7;0.3 0 1 0.7;0.3 0 1 0.7;0.3 0 1 0.7" begin="0s" repeatCount="indefinite"/><animate attributeName="height" calcMode="spline" values="50;0;0;50;50" keyTimes="0;0.4;0.5;0.9;1" dur="3.4" keySplines="0.3 0 1 0.7;0.3 0 1 0.7;0.3 0 1 0.7;0.3 0 1 0.7" begin="0s" repeatCount="indefinite"/></path><path d="M0 90.453h100V100H0z"><animate attributeName="y" calcMode="spline" values="100;50;50;50;50" keyTimes="0;0.4;0.5;0.9;1" dur="3.4" keySplines="0.3 0 1 0.7;0.3 0 1 0.7;0.3 0 1 0.7;0.3 0 1 0.7" begin="0s" repeatCount="indefinite"/><animate attributeName="height" calcMode="spline" values="0;50;50;0;0" keyTimes="0;0.4;0.5;0.9;1" dur="3.4" keySplines="0.3 0 1 0.7;0.3 0 1 0.7;0.3 0 1 0.7;0.3 0 1 0.7" begin="0s" repeatCount="indefinite"/></path></clipPath></defs><g transform="matrix(.9 0 0 .9 5 5)"><animateTransform attributeName="transform" type="rotate" calcMode="linear" values="0 50 50;0 50 50;180 50 50;180 50 50;360 50 50" keyTimes="0;0.4;0.5;0.9;1" dur="3.4s" begin="0s" repeatCount="indefinite"/><path d="M54.864 50c0-1.291.689-2.412 1.671-2.729 9.624-3.107 17.154-12.911 19.347-25.296.681-3.844-1.698-7.475-4.791-7.475H28.908c-3.093 0-5.472 3.631-4.791 7.475 2.194 12.385 9.723 22.189 19.347 25.296.982.317 1.671 1.438 1.671 2.729 0 1.291-.689 2.412-1.671 2.729-9.624 3.107-17.153 12.911-19.347 25.296-.681 3.844 1.698 7.475 4.791 7.475h42.184c3.093 0 5.472-3.631 4.791-7.475-2.194-12.385-9.723-22.189-19.347-25.296-.983-.317-1.672-1.438-1.672-2.729z" clip-path="url(#a)" fill="#5bc0de"/><path d="M81 81.5h-2.724l.091-.578c.178-1.122.17-2.243-.022-3.333-2.332-13.169-10.242-23.556-20.642-27.106l-.339-.116v-.715l.339-.135c10.399-3.552 18.31-13.938 20.642-27.107.192-1.089.2-2.211.022-3.333l-.091-.577H81c2.481 0 4.5-2.019 4.5-4.5S83.481 9.5 81 9.5H19c-2.481 0-4.5 2.019-4.5 4.5s2.019 4.5 4.5 4.5h2.724l-.092.578a10.045 10.045 0 0 0 .023 3.333c2.333 13.168 10.242 23.555 20.642 27.107l.338.116v.715l-.338.135c-10.4 3.551-18.31 13.938-20.642 27.106a10.045 10.045 0 0 0-.023 3.333l.092.578H19c-2.481 0-4.5 2.019-4.5 4.5s2.019 4.5 4.5 4.5h62c2.481 0 4.5-2.019 4.5-4.5S83.481 81.5 81 81.5zm-7.86-.309l-.128.309H26.988l-.128-.309a4.868 4.868 0 0 1-.28-2.729c2.014-11.375 8.944-20.542 17.654-23.354 2.035-.658 3.402-2.711 3.402-5.108 0-2.398-1.368-4.451-3.403-5.108-8.71-2.812-15.639-11.979-17.653-23.353a4.865 4.865 0 0 1 .281-2.731l.128-.308h46.024l.128.308c.244.589.492 1.541.281 2.731-2.015 11.375-8.944 20.541-17.654 23.353-2.035.658-3.402 2.71-3.402 5.108 0 2.397 1.368 4.45 3.403 5.108 8.71 2.812 15.64 11.979 17.653 23.354a4.876 4.876 0 0 1-.282 2.729z" fill="#337ab7"/></g></svg>
                </span>
            </div>

            <div class="opt-notice hidden" id="delayed-locations-optout-notice">
                <span class="dsds">
                    <svg version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px"
                         viewBox="0 0 120 120" enable-background="new 0 0 120 120" xml:space="preserve">

                        <path fill="#6fcc8a" d="M55.3,60.9h-0.2c-1,0-1.7-0.6-2.3-1.3L39.5,41.3l5.2-3.8l11.1,15.3l34.2-34.9
                              C84.5,11.7,75,4.6,59.7,4.6C37.4,4.6,19.3,23,19.3,45.3c0,18.3,12.8,34.7,30.3,39.3l3.4,9.2c-9.5,0.6-25.6,2.9-25.6,10.9
                              c0,10.9,29,11.3,32.4,11.3c3.4,0,32.4-0.4,32.4-11.3c0-8.2-16.2-10.3-25.8-10.9l3.2-9c17.9-4.4,30.7-20.8,30.7-39.3
                              c0-7.6-2.5-15.5-6.7-22.1L57.6,60C57.1,60.6,56.1,60.9,55.3,60.9z M85.5,104.6c-1.9,1.9-11.3,4.8-25.8,4.8
                              c-14.3,0-23.7-2.9-25.8-4.8c1.7-1.7,9.5-4.2,21.4-4.6l2.1,5.5c0.4,1,1.1,1.5,2.3,1.5l0,0c1,0,1.9-0.6,2.3-1.5l1.9-5.5
                              C76,100.4,83.8,102.9,85.5,104.6z"/>
                        <path fill="#6fcc8a" d="M96.6,10.9l-6.7,6.9c0.2,0.2,0.4,0.6,0.8,0.8c1.1,1.3,2.3,2.9,3.2,4.4l7.4-7.6L96.6,10.9z"/>
                    </svg>
                </span>
                <h3>You're turning off auto install locations option</h3>
                All database locations within the chosen radius will become immediately available. <br /> <br /> <br />
                <span class="button-locations-dialog" id="close-delayed-locations-dialog">OK</span>
            </div>
        </div>


        <?php
        ///
        $count1 = getAutoinstalledLocationsCount($post->ID);
        $count2 = get_post_meta($post->ID, '_all_locations_count', true);
        ///

        if (!$count2) {

            $count2 = "0";
        }

        $autodelayDays = get_option('_magic_page_autoinstall_days');
        $defaultValue = $autodelayDays;
        if (!$autodelayDays) {
            $autodelayDays = '30';
        }
        ?>


        <div class="hidden autoloadLocationsIcon">
            <svg version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px"
                 viewBox="-47 49 24 24" enable-background="new -47 49 24 24" xml:space="preserve">
                <path fill="#f1f1f1" d="M-35,49c-4.2,0-8,3.4-8,7.6c0,4.2,3.5,9.2,8,16.4c4.5-7.2,8-12.2,8-16.4C-27,52.4-30.8,49-35,49z M-35,60c-1.7,0-3-1.3-3-3
                      s1.3-3,3-3s3,1.3,3,3C-32,58.7-33.3,60-35,60z"/>
            </svg>
        </div>



        <div id="delayed-locations-box">
            <!-- autodelay on -->
            <div class="autodelay-on-info <?php if (!magic_page_is_autoinstalled_enabled()) { ?> hidden <?php } ?>">
                <span data-installed-only="true" class="view-locations-dialog-show locations-count-installed">
                    <span class="_num"><?php echo(friendly_shorten_number($count1, 'numbers')); ?></span>
                    <span class="_num_k"><?php echo(friendly_shorten_number($count1, 'shorthand')); ?></span>
                </span> locations

                <span id="out-of---text">
                    out of
                    <span class="view-locations-dialog-show locations-count-available">
                        <span class="_num"><?php echo(friendly_shorten_number($count2, 'numbers')); ?></span>
                        <span class="_num_k"><?php echo(friendly_shorten_number($count2, 'shorthand')); ?></span>
                    </span>
                </span>
            </div>

            <!-- autodelay off -->
            <div class="autodelay-off-info <?php if (magic_page_is_autoinstalled_enabled()) { ?> hidden <?php } ?>">
                <span  class="view-locations-dialog-show locations-count-installed">
                    <span class="_num"><?php echo(friendly_shorten_number($count2, 'numbers')); ?></span>
                    <span class="_num_k"><?php echo(friendly_shorten_number($count2, 'shorthand')); ?></span>
                </span> locations
            </div>


            <div class="notes">
                We recommend using auto install locations option which will gradually auto build up locations count.
            </div>
            <div class="option-box">

                <?php ?>

                <input name="_magic_page_autoinstall_days" type="hidden" value="<?php echo($autodelayDays) ?>" id="delayed-locations-days-input" />
                <input <?php if (magic_page_is_autoinstalled_enabled()) { ?> checked="checked" <?php } ?> id="auto-delay-option" name="_magic_page_autoinstall_locations" value="1" type="checkbox" />
                <span id="label-for-auto-delay-option" >
                    Auto install locations
                    <span <?php if (!magic_page_is_autoinstalled_enabled()) { ?> class="hidden" <?php } ?> id="delayed-locations-for-notice">
                        for    <span id="delayed-locations-days"><?php echo($autodelayDays) ?>

                        </span>   days</span></span>
                <span title="Auto-Install Information" class="autodelay-on-info delayed-locations-info-dialog-icon <?php if (!magic_page_is_autoinstalled_enabled()) { ?> hidden <?php } ?>">
                    <svg width="12" height="12" xmlns="http://www.w3.org/2000/svg" viewBox="-43 45 32 32">
                        <path fill="#fdfdfd" d="M-27 46c-8.3 0-15 6.7-15 15s6.7 15 15 15 15-6.7 15-15-6.7-15-15-15zm2 22a2 2 0 0 1-2 2 2 2 0 0 1-2-2v-7c0-1.1.9-2 2-2a2 2 0 0 1 2 2v7zm-2-10.8c-1.2 0-2.2-1-2.2-2.2s1-2.2 2.2-2.2c1.2 0 2.2 1 2.2 2.2s-1 2.2-2.2 2.2z"/>
                    </svg>
                </span>
            </div>

            <div style="display: none;" id="auto-delay-options-menu">
                <div data-days="30" class="line fixed-option">30 days</div>
                <div data-days="60" class="line fixed-option">60 days</div>
                <div data-days="90" class="line fixed-option">90 days</div>
                <div class="line custom">Custom</div>
            </div>

            <div style="display: none;" id="auto-delay-custom-options-menu">
                <label>Custom days:<br />
                    <input value="<?php echo($defaultValue) ?>" id="custom-days-count" type="number" min="1" max="999" step="1" /></label>
                <span id="save-custom-days-count">Save</span>
            </div>

        </div>

        <?php
    }

    public function seobox($post) {
        $isPostPage = in_array($post->post_type, array('post', 'page')) ? true : false;
        $groupId = get_post_meta($post->ID, '_group_id', true);
        $group = getExportGroupById($groupId);
        $useGroupSettings = get_post_meta($post->ID, '_use_group_settings', true);

        if ($isPostPage) {
            if ($group && $useGroupSettings) {
                $title = $group['title'];
            } else {
                $title = '';
            }
            ?>
            <div class="group-settings-class">

                <div class="group-note requires-group <?php if (!$title) { ?>hidden <?php } ?>">
                    <span data-type="false" style="float:right" class="enable-disable-group-link button button-secondary align-right">Enable Custom Post Settings</span>
                    Managed by Group Settings (<b class="chosen-group-name"><?php echo($title) ?></b>). <br />
                    Updating the fields will update the settings for all Posts & Pages with the same Group. <br />
                    <b class="chosen-group-name"><?php echo($title) ?></b> SEO Settings
                </div>
                <div class="group-note group-not-required <?php if ($title) { ?>hidden <?php } ?>">
                    <span data-type="true" style="float:right" class="enable-disable-group-link button button-secondary align-right">Disable Custom Settings</span>
                    Custom Settings Enabled. The SEO settings will be applied for the current page only.
                </div>
            </div>
            <?php
        }


        $meta_title = htmlspecialchars(get_post_meta($post->ID, '_magic_page_meta_title', true));
        $meta_description = htmlspecialchars(get_post_meta($post->ID, '_magic_page_meta_description', true));
        $meta_keywords = htmlspecialchars(get_post_meta($post->ID, '_magic_page_meta_keywords', true));

        $seo_title = htmlspecialchars(get_post_meta($post->ID, '_magic_page_seo_title', true));
        $_magic_page_twitter_site = htmlspecialchars(get_post_meta($post->ID, '_magic_page_twitter_site', true));
        $_magic_page_twitter_creator = htmlspecialchars(get_post_meta($post->ID, '_magic_page_twitter_creator', true));
        $seo_description = htmlspecialchars(get_post_meta($post->ID, '_magic_page_seo_description', true));

        $canonical = get_magic_page_schema_data($post->ID, '_magic_page_meta_generate_canonical_link');

        // json-ld schema
        // prepulate default when upgrading

        $schema_enabled = get_magic_page_schema_data($post->ID, '_magic_page_schema_enabled');
        $schema_type = get_magic_page_schema_data($post->ID, '_magic_page_schema_business_type');

        //var_dump($schema_enabled); exit();
        // no longer used
        // $schema_settings_minimized = get_magic_page_schema_data($post->ID, '_magic_page_schema_settings_minimized');

        $schema_price_range = htmlspecialchars(get_post_meta($post->ID, '_magic_page_price_range', true)); // pre 4.8
        $schema_telephone = htmlspecialchars(get_post_meta($post->ID, '_magic_page_telephone', true)); // pre 4.8
        $schema_email = htmlspecialchars(get_post_meta($post->ID, '_magic_page_email', true)); // pre 4.8

        $schema_business_name = htmlspecialchars(get_magic_page_schema_data($post->ID, '_magic_page_schema_business_name'));
        $schema_id = htmlspecialchars(get_magic_page_schema_data($post->ID, '_magic_page_schema_id'));
        $schema_address = htmlspecialchars(get_post_meta($post->ID, '_magic_page_schema_address', true));
        $schema_city = htmlspecialchars(get_magic_page_schema_data($post->ID, '_magic_page_schema_city'));
        $schema_state = htmlspecialchars(get_magic_page_schema_data($post->ID, '_magic_page_schema_state'));
        $schema_zip = htmlspecialchars(get_magic_page_schema_data($post->ID, '_magic_page_schema_zip'));
        $schema_country = htmlspecialchars(get_magic_page_schema_data($post->ID, '_magic_page_schema_country'));
        $schema_url = htmlspecialchars(get_magic_page_schema_data($post->ID, '_magic_page_schema_url'));
        $schema_logo = htmlspecialchars(get_magic_page_schema_data($post->ID, '_magic_page_schema_logo'));
        $schema_image_url = htmlspecialchars(get_magic_page_schema_data($post->ID, '_magic_page_schema_image_url'));
        $schema_description = htmlspecialchars(get_post_meta($post->ID, '_magic_page_schema_description', true));
        $schema_map_url = htmlspecialchars(get_magic_page_schema_data($post->ID, '_magic_page_schema_map_url'));
        $schema_latitude = get_magic_page_schema_data($post->ID, '_magic_page_schema_latitude');
        $schema_longitude = get_magic_page_schema_data($post->ID, '_magic_page_schema_longitude');
        $schema_hour_ranges = get_post_meta($post->ID, '_magic_page_schema_hour_ranges', true);
        $social_media_links = htmlspecialchars(get_post_meta($post->ID, '_magic_page_schema_social_media_links', true));

        $pageData = [];

        $types = get_post_types(array(
            'public' => true,
                ), 'objects');

        foreach ($types as $id => $type) {
            if ($id === $post->post_type) {
                $pageData = ['id' => $type->name, 'label' => $type->label];
            }
        }
        ?>

        <table class="form-table magic_page_seo_table">

            <tr class="form-field">
                <th scope="row" valign="top"><label> Meta Title </label></th>
                <td>
                    <input type="text" value="<?php echo $meta_title ?>" name="_magic_page_meta_title" />
                </td>
            </tr>
            <tr class="form-field">
                <th scope="row" valign="top"><label> Meta Description </label></th>
                <td>
                    <input type="text" value="<?php echo $meta_description ?>" name="_magic_page_meta_description" />
                </td>
            </tr>
            <tr class="form-field">
                <th scope="row" valign="top"><label> Meta Keywords </label></th>
                <td>
                    <input type="text" value="<?php echo $meta_keywords ?>" name="_magic_page_meta_keywords" />
                </td>
            </tr>

            <tr class="form-field">
                <th style="padding-bottom:15px" scope="row" valign="top"><label> Canonical Link </label></th>
                <td style="padding-bottom:15px">
                    <input value="1" type="checkbox" <?php if ($canonical) { ?>checked="checked"<?php } ?> name="_magic_page_meta_generate_canonical_link" /> Generate Canonical Link
                </td>
            </tr>

            <tr class="form-field border-top">
                <th scope="row" valign="top"><label> Social Title (og:title, twitter:title) </label></th>
                <td>
                    <input type="text" value="<?php echo $seo_title ?>" name="_magic_page_seo_title" />
                </td>
            </tr>
            <tr class="form-field">
                <th scope="row" valign="top"><label> Social Description (og:description, twitter:description) </label></th>
                <td>
                    <input type="text" value="<?php echo $seo_description ?>" name="_magic_page_seo_description" />
                </td>
            </tr>

            <tr class="form-field">
                <th scope="row" valign="top"><label> Twitter Site (twitter:site)  </label></th>
                <td>
                    <input type="text" value="<?php echo $_magic_page_twitter_site ?>" name="_magic_page_twitter_site" />
                </td>
            </tr>

            <tr class="form-field">
                <th style="padding-bottom:15px" scope="row" valign="top"><label>Twitter Creator (twitter:creator) </label></th>
                <td style="padding-bottom:15px">
                    <input type="text" value="<?php echo $_magic_page_twitter_creator ?>" name="_magic_page_twitter_creator" />
                </td>
            </tr>

            <tr class="form-field  border-top">
                <th>
                    <label> JSON-LD Schema</label>
                </th>
                <td>



                    <?php if (defined('WPSCHEMA_VERSION')) { ?>   <input data-type='<?php echo(json_encode($pageData)) ?>' type="hidden" id="schema-plugin-installed" /> <?php } ?>
                    <span id="json-schema-builder-mode-holder">
                        <select name="_magic_page_schema_enabled" style="width: 200px" id="json-schema-builder-mode">
                            <option <?php if ($schema_enabled == 'disabled') { ?> selected="selected" <?php } ?> value="disabled">Disabled</option>
                            <option <?php if ($schema_enabled == 'simple') { ?> selected="selected" <?php } ?> value="simple">Simple Schema Builder</option>

                            <?php
                            if (isAdvancedSchemaCompatable()) {
                                ?>
                                <option <?php if ($schema_enabled == 'advanced') { ?> selected="selected" <?php } ?> value="advanced">Advanced Schema Builder</option> <?php
                            } else {
                                ?>   <option disabled="disabled">Not Compatible - Advanced Schema Builder</option> <?php
                            }
                            ?>

                        </select>
                    </span>

                    <span style="float:right">


                        <span id="openSchemaBuilder"style="float: right" class="button button-secondary <?php if ($schema_enabled == 'disabled') { ?> hidden <?php } ?>">Open Schema Builder</span>
                        <span id="closeSimpleSchemaBuilder"  style="float: right" class="hidden button button-secondary">Close Schema Builder</span>
                    </span>
                </td>
            </tr>

            <tr class="form-field jsonld-td  hidden">


                <th scope="row" valign="top"> <br />
                    <label> Choose Type: </label>
                </th>

                <td>
                    <i> Note: Simple Schema Builder cannot use group settings. </i> <br /><br />
                    <select name="_magic_page_schema_business_type">
                        <?php
                        $options = array(
                            'Local Business', 'Animal Shelter',
                            array('Automotive' => array('Automotive Business', 'Auto Body Shop', 'Auto Dealer', 'Auto Parts Store', 'Auto Rental',
                                    'Auto Repair', 'Auto Wash', 'Gas Station', 'Motorcycle Dealer', 'Motorcycle Repair')),
                            'Child Care', 'Dry Cleaning Or Laundry',
                            array('Emergency' => array('Emergency Service', 'Fire Station', 'Hospital', 'Police Station')),
                            'Employment Agency',
                            array('Entertainment' => array('Entertainment Business', 'Adult Entertainment', 'Amusement Park', 'Art Gallery', 'Casino', 'Comedy Club',
                                    'Movie Theater', 'Night Club')),
                            array('Financial' => array('Accounting Service', 'Automated Teller', 'Bank Or Credit Union', 'Insurance Agency')),
                            array('Food' => array('Food Establishment', 'Bakery', 'Bar Or Pub', 'Brewery', 'Cafe Or Coffee Shop', 'Fast Food Restaurant', 'Ice Cream Shop',
                                    'Restaurant', 'Winery',
                                )),
                            'Government Office', 'Post Office',
                            array('Health And Beauty' => array('Health And Beauty Business', 'Beauty Salon', 'Day Spa', 'Hair Salon', 'Health Club', 'Nail Salon', 'Tattoo Parlor')),
                            array('Home And Construction' => array('Home And Construction Business', 'Electrician', 'General Contractor', 'HVAC Business', 'House Painter',
                                    'Locksmith', 'Moving Company', 'Plumber', 'Roofing Contractor'
                                )),
                            'Internet Cafe', 'Library',
                            array('Lodging' => array('Lodging Business', 'Bed And Breakfast', 'Hostel', 'Hotel', 'Motel')),
                            array('Medical/Dental' => array('Dentist', 'Diagnostic Lab', 'Hospital', 'Medical Clinic', 'Optician', 'Pharmacy', 'Physician', 'Veterinary Care')),
                            array('Professional Services' => array('Professional Service', 'Accounting Service', 'Attorney', 'Dentist', 'Electrician', 'General Contractor',
                                    'House Painter', 'Locksmith', 'Notary', 'Plumber', 'Roofing Contractor'
                                )),
                            'Radio Station', 'Real Estate Agent', 'Recycling Center', 'Self Storage', 'Shopping Center',
                            array('Sports Activities' => array('Sports Activity Location', 'Bowling Alley', 'Exercise Gym', 'Golf Course', 'Health Club',
                                    'Public Swimming Pool', 'Ski Resort', 'Sports Club', 'Stadium Or Arena', 'Tennis Complex'
                                )),
                            array('Stores' => array('Store', 'Auto Parts Store', 'Bike Store', 'Book Store', 'Clothing Store', 'Computer Store',
                                    'Convenience Store', 'Department Store', 'Electronics Store', 'Florist', 'Furniture Store', 'Garden Store',
                                    'Grocery Store', 'Hardware Store', 'Hobby Shop', 'Home Goods Store', 'Jewelry Store', 'Liquor Store', 'Mens Clothing Store',
                                    'Mobile Phone Store', 'Movie Rental Store', 'Music Store', 'Office Equipment Store', 'Outlet Store', 'Pawn Shop', 'Pet Store',
                                    'Shoe Store', 'Sporting Goods Store', 'Tire Shop', 'Toy Store', 'Wholesale Store'
                                )),
                            'Television Station', 'Tourist Information Center', 'Travel Agency'
                        );

                        foreach ($options as $id => $option) {
                            if (is_array($option)) {
                                foreach ($option as $id => $suboptions) {
                                    ?> <optgroup label="<?php echo($id) ?>"><?php
                                        foreach ($suboptions as $suboption) {
                                            $this->businessTypeOption($suboption, $schema_type);
                                        }
                                        ?> </optgroup> <?php
                                }
                            } else {
                                $this->businessTypeOption($option, $schema_type);
                            }
                        }
                        ?>
                    </select>
                </td>
            </tr>

            <tr class="form-field jsonld-td  hidden">
                <th scope="row" valign="top">
                    <label> Business Name </label>
                </th>
                <td>
                    <input type="text" value="<?php echo $schema_business_name ?>" name="_magic_page_schema_business_name" placeholder="[site_title]" />
                </td>
            </tr>

            <tr class="form-field jsonld-td  hidden">
                <th scope="row" valign="top">@id </th>
                <td>  <input type="text" value="<?php echo $schema_id ?>" name="_magic_page_schema_id" placeholder="[site_url]" /> </td>
            </tr>

            <tr class="form-field jsonld-td  hidden">
                <th scope="row" valign="top">Аddress</th>
                <td> <input type="text" value="<?php echo $schema_address ?>" name="_magic_page_schema_address" placeholder="Address" /> </td>
            </tr>

            <tr class="form-field jsonld-td  hidden">
                <th scope="row" valign="top">City</th>
                <td> <input type="text" value="<?php echo $schema_city ?>" name="_magic_page_schema_city" placeholder="[location]" /> </td>
            </tr>

            <tr class="form-field jsonld-td  hidden">
                <th scope="row" valign="top">State</th>
                <td> <input type="text" value="<?php echo $schema_state ?>" name="_magic_page_schema_state" placeholder="[region]" /> </td>
            </tr>

            <tr class="form-field jsonld-td  hidden">
                <th scope="row" valign="top">Zip</th>
                <td> <input type="text" value="<?php echo $schema_zip ?>" name="_magic_page_schema_zip" placeholder="[zip]" /> </td>
            </tr>

            <tr class="form-field jsonld-td  hidden">
                <th scope="row" valign="top">Country</th>
                <td> <input type="text" value="<?php echo $schema_country ?>" name="_magic_page_schema_country" placeholder="[country_iso]" /> </td>
            </tr>

            <tr class="form-field jsonld-td  hidden">
                <th scope="row" valign="top">Phone</th>
                <td> <input type="text" value="<?php echo $schema_telephone ?>" name="_magic_page_telephone" placeholder="Phone" /> </td>
            </tr>

            <tr class="form-field jsonld-td  hidden">
                <th scope="row" valign="top">Email</th>
                <td> <input type="text" value="<?php echo $schema_email ?>" name="_magic_page_email" placeholder="Email" /> </td>
            </tr>

            <tr class="form-field jsonld-td  hidden">
                <th scope="row" valign="top">Url</th>
                <td> <input type="text" value="<?php echo $schema_url ?>" name="_magic_page_schema_url" placeholder="[site_url]" /> </td>
            </tr>

            <tr class="form-field jsonld-td  hidden">
                <th scope="row" valign="top">Logo</th>
                <td> <input type="text" value="<?php echo $schema_logo ?>" name="_magic_page_schema_logo" placeholder="[seo_image_url]" /> </td>
            </tr>

            <tr class="form-field jsonld-td  hidden">
                <th scope="row" valign="top">Image Url</th>
                <td> <input type="text" value="<?php echo $schema_image_url ?>" name="_magic_page_schema_image_url" placeholder="[seo_image_url]" /> </td>
            </tr>

            <tr class="form-field jsonld-td  hidden">
                <th scope="row" valign="top">Price Range</th>
                <td> <input value="<?php echo $schema_price_range ?>" type="text"  name="_magic_page_price_range" placeholder="price range e.g. $50 - 100" /> </td>
            </tr>


            <tr class="form-field jsonld-td  hidden">
                <th scope="row" valign="top">Description</th>
                <td>
                    <textarea name="_magic_page_schema_description" placeholder="Description"><?php echo $schema_description ?></textarea>
                </td>
            </tr>

            <tr class="form-field jsonld-td  hidden">
                <th scope="row" valign="top">Map Url</th>
                <td> <input type="text" value="<?php echo $schema_map_url ?>" name="_magic_page_schema_map_url" placeholder="https://www.google.com/maps/place/[location]/@[latitude],[longitude]" /> </td>
            </tr>

            <tr class="form-field jsonld-td  hidden">
                <th scope="row" valign="top">Latitude</th>
                <td> <input type="text" value="<?php echo $schema_latitude ?>" name="_magic_page_schema_latitude" placeholder="[latitude]" /> </td>
            </tr>

            <tr class="form-field jsonld-td  hidden">
                <th scope="row" valign="top">Longitude</th>
                <td> <input type="text" value="<?php echo $schema_longitude ?>" name="_magic_page_schema_longitude" placeholder="[longitude]" /> </td>
            </tr>

            <tr class="form-field jsonld-td  hidden">
                <th scope="row" valign="top">Opening Hours</th>
                <td>
                    <select name="_magic_page_schema_hour_ranges" id="hour-ranges" style="display:block;">
                        <option <?php $this->selected($schema_hour_ranges, '0') ?> value="0">Opening hour ranges</option>
                        <option <?php $this->selected($schema_hour_ranges, '1') ?> value="1">1 range of opening hours</option>
                        <option <?php $this->selected($schema_hour_ranges, '2') ?> value="2">2 ranges of opening hours</option>
                        <option <?php $this->selected($schema_hour_ranges, '3') ?> value="3">3 ranges of opening hours</option>
                        <option <?php $this->selected($schema_hour_ranges, '4') ?> value="4">4 ranges of opening hours</option>
                        <option <?php $this->selected($schema_hour_ranges, '5') ?> value="5">5 ranges of opening hours</option>
                        <option <?php $this->selected($schema_hour_ranges, '6') ?> value="6">6 ranges of opening hours</option>
                        <option <?php $this->selected($schema_hour_ranges, '7') ?> value="7">7 ranges of opening hours</option>
                    </select>

                    <?php
                    for ($i = 1; $i <= 7; $i++) {
                        $range = get_post_meta($post->ID, '_magic_page_schema_range_' . $i, true);

                        if ($range) {
                            $open = get_post_meta($post->ID, '_magic_page_schema_open_' . $i, true);
                            $close = get_post_meta($post->ID, '_magic_page_schema_close_' . $i, true);
                            ?>

                            <div class="daysdiv days<?php echo($i); ?>">
                                <label><input <?php echo($this->checkedArray('Monday', $range)); ?> type="checkbox" class="Mon" name="_magic_page_schema_range_<?php echo($i); ?>[]" value="Monday">Mon</label>
                                <label><input <?php echo($this->checkedArray('Tuesday', $range)); ?> type="checkbox" class="Tue" name="_magic_page_schema_range_<?php echo($i); ?>[]" value="Tuesday">Tue</label>
                                <label><input <?php echo($this->checkedArray('Wednesday', $range)); ?> type="checkbox" class="wed" name="_magic_page_schema_range_<?php echo($i); ?>[]" value="Wednesday">Wed</label>
                                <label><input <?php echo($this->checkedArray('Thursday', $range)); ?> type="checkbox" class="Thu" name="_magic_page_schema_range_<?php echo($i); ?>[]" value="Thursday">Thu</label>
                                <label><input <?php echo($this->checkedArray('Friday', $range)); ?> type="checkbox" class="Fri" name="_magic_page_schema_range_<?php echo($i); ?>[]" value="Friday">Fri</label>
                                <label><input <?php echo($this->checkedArray('Saturday', $range)); ?> type="checkbox" class="Sat" name="_magic_page_schema_range_<?php echo($i); ?>[]" value="Saturday">Sat</label>
                                <label><input <?php echo($this->checkedArray('Sunday', $range)); ?> type="checkbox" class="Sun" name="_magic_page_schema_range_<?php echo($i); ?>[]" value="Sunday">Sun</label>
                            </div>

                            <div class="hourdiv hour1">
                                <label>Open: <input type="text" class="openhour openclose" name="_magic_page_schema_open_<?php echo($i); ?>"  size="20" value="<?php echo($open); ?>" placeholder="Opening Time"></label>
                                <label>Close: <input type="text"  class=" openclose"  name="_magic_page_schema_close_<?php echo($i); ?>" size="20" value="<?php echo($close); ?>" placeholder="Closing Time"></label>
                            </div>

                            <?php
                        }
                    }
                    ?>
                </td>
            </tr>



            <tr class="form-field jsonld-td  hidden">
                <th scope="row" valign="top">Social media links</th>
                <td>
                    <select name="_magic_page_schema_social_media_links" id="sameas" style="display:block;">
                        <option <?php $this->selected($social_media_links, '0') ?> value="0">Social media links</option>
                        <option <?php $this->selected($social_media_links, '1') ?> value="1">1 social media link</option>
                        <option <?php $this->selected($social_media_links, '2') ?> value="2">2 social media links</option>
                        <option <?php $this->selected($social_media_links, '3') ?> value="3">3 social media links</option>
                        <option <?php $this->selected($social_media_links, '4') ?> value="4">4 social media links</option>
                        <option <?php $this->selected($social_media_links, '5') ?> value="5">5 social media links</option>
                    </select>

                    <div id="sameas-links">
                        <?php
                        for ($i = 0; $i <= 5; $i++) {
                            $socialLink = '_magic_page_schema_social_media_links_' . $i;

                            $link = get_post_meta($post->ID, $socialLink, true);

                            if ($link) {
                                ?> <input value='<?php echo($link); ?>' class="social-media-link" type="text" name="_magic_page_schema_social_media_links_<?php echo($i); ?>" placeholder="Social Media URL" /><?php
                            }
                        }
                        ?>
                    </div>

                </td>
            </tr>

        </table>



        <?php
    }

    public function checkedArray($day, $range) {

        if ((is_array($range) && in_array($day, $range)) || (!is_array($range) && $range == $day)) {
            return ' checked="checked" ';
        }
    }

    function saveMetabox() {
        global $post;

        if (!$post) {
            return false;
        }

        if (!current_user_can('edit_post', $post->ID)) {
            return false;
        }

        if (isset($_POST['action']) && $_POST['action'] === 'elementor_ajax') {
            return false;
        }

        if (empty($_POST)) {
            return false;
        }


        $groupSettings = get_post_meta($post->ID, '_use_group_settings', true);
        $exportGroupId = isset($_POST['post_page_export_group']) ? $_POST['post_page_export_group'] : '';

        if (!isset($_POST['_is_magic_page'])) {
            // post/page with group settings
            if ($groupSettings && $exportGroupId) {
                $arrayData = array();
                $fields = get_magic_page_meta_fields();

                foreach ($fields as $field) {
                    // and not array type
                    if (isset($_POST[$field])) {
                        if (!in_array($field, getGroupArrayMetaFields())) {
                            $arrayData[$field] = cleanup_quotes_slashes($_POST[$field]);
                        } elseif (isset($_POST[$field][0])) {
                            $arrayData[$field] = cleanup_quotes_slashes($_POST[$field][0]);
                        }
                    }
                }

                update_option('_group_meta_fields_' . $exportGroupId, $arrayData);
            } else {
                // for post/pages with custom settings
                $this->saveFields($post);
            }
        } else {
            // for magic page only
            $autoinstall = (isset($_POST['_magic_page_autoinstall_locations']) && $_POST['_magic_page_autoinstall_locations'] == 1) ? 1 : 0;
            $autoinstallDays = (isset($_POST['_magic_page_autoinstall_days'])) ? intval($_POST['_magic_page_autoinstall_days']) : '30';

            $this->saveFields($post);

            update_option('_magic_page_autoinstall_locations', $autoinstall);
            update_option('_magic_page_autoinstall_days', $autoinstallDays);

            $locations = isset($_POST['_locations']) ? $_POST['_locations'] : array();

            if ($locations) {
                $savedLocations = get_post_meta($post->ID, '_magicpage_chosen_locations', true);
                $locations = array_map('intval', $locations);
                $_Locations = array();

                foreach ($locations as $location) {
                    $distance = isset($_POST['distance'][$location]) && $_POST['distance'][$location] ? intval($_POST['distance'][$location]) : '0';
                    $_Locations[$location] = $distance;
                }

                ksort($_Locations);

                if (!$savedLocations || $savedLocations !== $_Locations) {
                    update_post_meta($post->ID, '_magicpage_chosen_locations', $_Locations);
                    wp_set_object_terms($post->ID, $locations, 'location');

                    foreach ($_Locations as $id => $distance) {
                        update_term_meta($id, 'distance', $distance);
                    }

                    magic_page_update_locations_count($post);
                }
            } else {
                wp_delete_object_term_relationships($post->ID, 'location');
                update_post_meta($post->ID, '_autoinstalled_locations_count', 0);
                update_post_meta($post->ID, '_all_locations_count', 0);
                update_post_meta($post->ID, '_magicpage_chosen_locations', array());
            }
        }
    }

    protected function saveFields($post) {
        $fields = get_magic_page_meta_fields();

        foreach ($fields as $field) {
            if (isset($_POST[$field])) {
                update_post_meta($post->ID, $field, cleanup_quotes_slashes($_POST[$field]));
            } else {
                delete_post_meta($post->ID, $field);
            }
        }
    }

    public function locationSearch() {
        $postId = intval($_GET['postId']);

        if (!current_user_can('edit_post', $postId)) {
            wp_send_json_error(array('status' => 'Security Error! Are you sure you\'re logged in?'));
        }

        $taxonomy = 'location';
        $tax = get_taxonomy($taxonomy);

        if (!$tax) {
            wp_die(0);
        }

        if (!current_user_can($tax->cap->assign_terms)) {
            wp_die(-1);
        }

        $s = strip_tags(wp_unslash($_GET['q']));

        global $wpdb;

        $results = $wpdb->get_results(
                "SELECT DISTINCT t.*, tt.*
            FROM $wpdb->terms AS t
            LEFT JOIN $wpdb->termmeta ON (t.term_id = $wpdb->termmeta.term_id )
            LEFT JOIN $wpdb->term_taxonomy AS tt ON t.term_id = tt.term_id
            WHERE tt.taxonomy IN ('location') AND (
              ( ($wpdb->termmeta.meta_key = 'zip' AND $wpdb->termmeta.meta_value LIKE '%$s%') OR t.name LIKE '%$s%' )
            ) ORDER BY t.name ASC LIMIT 0, 20");

        $out = array();
        if ($results) {
            foreach ($results as $result) {
                $out[] = array(
                    'id' => $result->term_id,
                    'name' => $result->name,
                    'zip' => get_term_meta($result->term_id, 'zip', true),
                    'region' => get_term_meta($result->term_id, 'region', true),
                    'distance' => get_option('_magic_page_default_radius', 1000)
                );
            }
        }

        wp_send_json_success($out);
    }

    public function getAvailableLocations() {

        $postId = intval($_POST['postId']);

        if (!current_user_can('edit_post', $postId)) {
            wp_send_json_error(array('status' => 'Security Error! Are you sure you\'re logged in?'));
        }

        $currentPage = isset($_POST['page']) ? intval($_POST['page']) : 1;
        $regions = isset($_POST['regions']) ? $_POST['regions'] : '';
        $search = isset($_POST['search']) ? trim(strip_tags($_POST['search'])) : '';
        $itemsPerPage = isset($_POST['itemsPerPage']) ? intval($_POST['itemsPerPage']) : 15;
        $autoinstalled = isset($_POST['autoinstalledLocationsOnly']) && $_POST['autoinstalledLocationsOnly'] ? true : false;
        $locations = isset($_POST['locations']) && $_POST['locations'] ? $_POST['locations'] : array();
        $orderby = $_POST['orderby'];
        $orderbyType = $_POST['orderbyType'];
        $showInstalledColumn = isset($_POST['showInstalledColumn']) ? true : false;

        if ($itemsPerPage > 100 || $itemsPerPage <= 15) {
            $itemsPerPage = 15;
        }

        $query1 = new MagicPageQuery();
        $query1->setLocations($locations);
        $query1->setSearch($search);
        $query1->setRegions($regions);
        $query1->setAutoinstalled($autoinstalled);
        $query1->setQueryType('result');
        $query1->setOrderby($orderby);
        $query1->setOrderbyType($orderbyType);
        $query1->setItemsPerPage($itemsPerPage);
        $query1->setCurrentPage($currentPage);
        $results = $query1->getResults();

        $query2 = new MagicPageQuery(false, false);
        $query2->setLocations($locations);
        $query2->setSearch($search);
        $query2->setRegions($regions);
        $query2->setAutoinstalled($autoinstalled);
        $query2->setQueryType('count');
        $totalItems = $query2->getResults();

        $urlPattern = '(:num)';

        $paginator = new Paginator($totalItems, $itemsPerPage, $currentPage, $urlPattern);
        $maxPages = $paginator->getNumPages();

        $info = "Viewing " . $totalItems . " locations page $currentPage of $maxPages";

        $realArray = array();

        if ($regions) {
            $realArray = array_unique(array_map('trim', $regions));
            $stringInReadable = rtrim(implode(", ", $realArray), ', ');
            $info .= " in these regions: $stringInReadable";
        }

        if ($search) {
            $info .= " with search term: $search";
        }

        $nonce = wp_create_nonce('magicpage');

        $dateFormat = get_option('date_format');

        $out = array(
            'search' => $search,
            'pagination' => $paginator->toHtml(),
            'regions' => get_magicpage_regions(),
            'chosenRegions' => $realArray,
            'info' => $info,
            'previewUrl' => admin_url("index.php?_token=$nonce&magicpage-preview-url="),
            'locationEditAdminUrl' => admin_url("term.php?taxonomy=location&tag_ID={ID}&post_type=magicpage"),
            'filters' => ($regions || $search) ? 'Clear filters' : '',
            'itemsPerPage' => $itemsPerPage,
            'installedOnly' => ($autoinstalled || $showInstalledColumn),
            'hasZip' => false
        );

        if ($results) {
            $ls = array();
            $_locations = array();
            $_locationsIds = array();

            foreach ($locations as $_location) {
                $_locations[] = get_term_by('id', $_location['id'], 'location');
                $_locationsIds[] = $_location['id'];
            }


            //   print_r($results); exit();

            foreach ($results as $result) {
                $l = array(
                    'id' => $result->term_id,
                    'name' => $result->name,
                    'zip' => $result->zip,
                    'lat' => get_term_meta($result->term_id, 'lat', true),
                    'lon' => get_term_meta($result->term_id, 'lon', true),
                    'region' => get_term_meta($result->term_id, 'region', true),
                    'distance' => get_distance2($result, $_locationsIds, $_locations)
                );

                if ($autoinstalled || $showInstalledColumn) {
                    $date = get_term_meta($result->term_id, 'autoinstalled', true);

                    if ($date) {
                        $l['date'] = date_i18n($dateFormat, $date);
                    } else {

                        if (in_array($result->term_id, $_locationsIds)) {
                            $l['date'] = "Selected";
                        } else {
                            $l['date'] = "Not Installed";
                        }
                    }
                }

                if ($result->zip) {
                    $out['hasZip'] = true;
                }

                $ls[] = $l;
            }

            $out['locations'] = $ls;
        }


        // print_r($ls); exit();

        wp_send_json_success($out);
    }

    public function locationsCount() {
        if (!current_user_can('administrator')) {
            wp_send_json_error(array('status' => 'Security Error! Are you sure you\'re logged in?'));
        }

        $locations = isset($_POST['locations']) && $_POST['locations'] ? $_POST['locations'] : array();

        $out = array();
        $out['count'] = array();

        $count1 = getAutoinstalledCount($locations);
        $count2 = getTotalCount($locations);

        $out['count'][] = array(
            'element' => '.autodelay-on-info .locations-count-installed',
            'num' => friendly_shorten_number($count1, 'numbers'),
            'shorthand' => friendly_shorten_number($count1, 'shorthand')
        );

        $out['count'][] = array(
            'element' => '.autodelay-on-info .locations-count-available',
            'num' => friendly_shorten_number($count2, 'numbers'),
            'shorthand' => friendly_shorten_number($count2, 'shorthand')
        );

        $out['count'][] = array(
            'element' => '.autodelay-off-info .locations-count-installed',
            'num' => friendly_shorten_number($count2, 'numbers'),
            'shorthand' => friendly_shorten_number($count2, 'shorthand')
        );

        $out['totalCount'] = $count2;

        wp_send_json_success($out);
    }

    public function installAutolocations() {
        $post = get_magic_page();
        $out = array('completed' => true);

        if ($post && current_user_can('edit_post', $post->ID)) {
            $locations = wp_get_post_terms($post->ID, 'location');
            $limit = (isset($_POST['limit']) && $_POST['limit']) ? intval($_POST['limit']) : 500;

            if ($locations) {
                $query = new MagicPageQuery(false, false);
                $query->setLocations($locations);
                $query->setExcludeAutoinstalled(true);
                $query->setQueryType('result');
                $query->setStart(0);
                $query->setItemsPerPage($limit);
                $results = $query->getResults();

                if ($results) {
                    foreach ($results as $result) {
                        update_term_meta($result->term_id, 'autoinstalled', time());
                    }
                    $out['completed'] = false;
                }
            } else {
                wp_send_json_error(array('status' => 'To install auto locations you must pick locations and save the post.'));
            }

            if ($out['completed'] === true) {
                magic_page_update_locations_count($post);
            }

            $count = getAutoinstalledCount($locations, false);

            $out['status'] = 'All available locations have saved';
            $out['num'] = friendly_shorten_number($count, 'numbers');
            $out['shorthand'] = friendly_shorten_number($count, 'shorthand');
        } else {
            $out['postMissing'] = true;
        }

        wp_send_json_success($out);
    }

    public function updatePostUseGroupSettings() {
        $postId = intval($_POST['id']);

        if (!current_user_can('edit_post', $postId)) {
            wp_send_json_error(array('status' => 'Security Error! Are you sure you\'re logged in?'));
        }


        $isEnabled = $_POST['type'] === 'false' ? false : true;

        update_post_meta($postId, '_use_group_settings', $isEnabled);

        $response = array();

        // group is enabled
        if ($isEnabled) {
            $groupId = get_post_page_group_id($postId);
            $metadata = get_option('_group_meta_fields_' . $groupId);
            $response['hasMetadata'] = $this->arrayValueExists($metadata);
            $response['metadata'] = $metadata;
            $response['groupEnabled'] = true;
        } else {
            $fields = get_magic_page_meta_fields();
            $metadata = array();
            foreach ($fields as $field) {
                $metadata[$field] = get_post_meta($postId, $field, true);
            }

            $response['hasMetadata'] = $this->arrayValueExists($metadata);
            $response['metadata'] = $metadata;
        }


        wp_send_json_success($response);
    }

    function checkPostCustomSettingsGroupEnabled() {
        $postId = intval($_POST['id']);

        if (!current_user_can('edit_post', $postId)) {
            wp_send_json_error(array('status' => 'Security Error! Are you sure you\'re logged in?'));
        }


        $groupId = strip_tags($_POST['groupId']) ? strip_tags($_POST['groupId']) : get_post_page_group_id($postId);

        $metadata = get_option('_group_meta_fields_' . $groupId);
        $response['hasMetadata'] = $this->arrayValueExists($metadata);
        $response['metadata'] = $metadata;

        wp_send_json_success($response);
    }

    function checkPostCustomSettings() {
        if (!current_user_can('administrator')) {
            wp_send_json_error(array('status' => 'Security Error! Are you sure you\'re logged in?'));
        }

        $postId = intval($_POST['id']);

        update_post_meta($postId, '_use_group_settings', false);

        $fields = get_magic_page_meta_fields();
        $metadata = array();
        foreach ($fields as $field) {
            $metadata[$field] = get_post_meta($postId, $field, true);
        }

        update_post_meta($postId, '_use_group_settings', true);

        $response = array();
        $response['hasMetadata'] = $this->arrayValueExists($metadata);
        $response['metadata'] = $metadata;

        wp_send_json_success($response);
    }

    protected function arrayValueExists($array) {
        if ($array) {
            foreach ($array as $item) {
                if ($item) {
                    return true;
                }
            }
        }
        return false;
    }

    public function getAutoinstallStats() {
        $postId = intval($_POST['postId']);

        if (!current_user_can('edit_post', $postId)) {
            wp_send_json_error(array('status' => 'Security Error! Are you sure you\'re logged in?'));
        }

        global $wpdb;

        $days = isset($_POST['days']) && $_POST['days'] ? intval($_POST['days']) : 7;
        $now = current_time('timestamp');
        $out = array();

        $i = 0;
        $totalCount = 0;
        $max = 0;

        do {
            $time2 = strtotime('-' . $i . ' day', $now);
            $time1 = strtotime('-' . ($i + 1) . ' day', $now);

            $count = $wpdb->get_var("SELECT COUNT(*) AS CNT FROM $wpdb->termmeta WHERE meta_key = 'autoinstalled' "
                    . "AND meta_value < '$time2' AND meta_value > '$time1'");

            $out['array'][] = array('x' => gmdate("Y-m-d", $time2), 'y' => $count);

            if ($max < $count) {
                $max = $count;
            }

            $totalCount += $count;

            $i++;
        } while ($i <= $days);

        $out['avg'] = ceil($totalCount / $days);
        $out['max'] = $max;

        wp_send_json_success($out);
    }

    public function deleteAutoinstalledLocations() {
        $postId = intval($_POST['postId']);

        if (!current_user_can('edit_post', $postId)) {
            wp_send_json_error(array('status' => 'Security Error! Are you sure you\'re logged in?'));
        }

        global $wpdb;

        $wpdb->query("DELETE FROM $wpdb->termmeta WHERE meta_key = 'autoinstalled'");
        $wpdb->query("DELETE FROM $wpdb->postmeta WHERE meta_key = '_autoinstalled_locations_count'");
        $wpdb->query("DELETE FROM $wpdb->options WHERE option_name LIKE '%_magicpage_cache_query_%'");

        wp_send_json_success();
    }

    public function schemaBuilderPluginEnableSupport() {
        $postId = intval($_POST['postId']);

        if (current_user_can('edit_post', $postId)) {

            $groupId = $_POST['groupId'];
            $groupSettings = $_POST['groupSettings'];
            $typeData = $_POST['type'];

            if ($groupId && $groupSettings) {
                $options = get_option('_group_meta_fields_' . $groupId);
                $options['_magic_page_schema_enabled'] = 'advanced';

                update_option('_group_meta_fields_' . $groupId, $options);
            } else {
                update_post_meta($postId, '_magic_page_schema_enabled', 'advanced');
            }

            enablePostPageAdvancedSchemaBuilder($typeData['id'], $typeData['label']);

            wp_send_json_success();
        }
    }

}

new MagicPageSeoMetabox();
