<?php
if (!defined('ABSPATH')) {
    exit;
}

class MagicPageAdminSettings {

    public function __construct() {

        register_activation_hook(MAGIC_PAGE_FILE, array($this, 'onPluginActivate'));
        register_deactivation_hook(MAGIC_PAGE_FILE, array($this, 'onPluginDeactivate'));

        // for location
        add_action('location_edit_form_fields', array($this, 'location_edit_meta_field'), 10, 2);
        add_action('edited_location', array($this, 'save_location_custom_meta'), 10, 2);
        add_action('create_location', array($this, 'save_location_custom_meta'), 10, 2);

        // admin
        add_action('admin_menu', array($this, 'menu'), 5);
        add_action('admin_init', array($this, 'admin'));

        // cache busting when location or magicpage is edited
        add_action('edited_location', 'magic_page_delete_cache');
        add_action('save_post_magicpage', 'magic_page_delete_cache');
        add_action('admin_init', array($this, 'checkForUpdates'));
        add_action('admin_notices', array($this, 'notice'));

        if (!is_inline_editor_page()) {
            add_action('admin_enqueue_scripts', array($this, 'admin_scripts'));
            add_action('admin_enqueue_scripts', array($this, 'admin_files'), 99);

            add_action('admin_enqueue_scripts', array($this, 'admin_scripts_post_page'), 99);
        }

        add_action('wp_ajax_delete_spintax_cache', array($this, 'delete_spintax_cache'));
        add_action('wp_ajax_magicpage_api_key_hidden', array($this, 'magicpage_api_key_hidden'));
        add_action('wp_ajax_magic_page_hidden_unhide', array($this, 'hidden_unhide'));
        add_action('wp_ajax_magicpage_reset_license_data', array($this, 'magicpage_reset_license_data'));
        add_action('wp_ajax_magicpage_compatability_mode', array($this, 'magicpage_compatability_mode'));

        add_action('admin_notices', array($this, 'noticeFiles'));
        add_action('admin_enqueue_scripts', array($this, 'admin_scripts_in_terms_page'));

        add_action('wp_ajax_magicpage_enable_disable_editor_custom_field', array($this, 'enable_disable_editor_custom_field'));
        add_action('wp_ajax_magicpage_editor_turn_off', array($this, 'editor_turn_off'));
        add_action('wp_ajax_magicpage_editor_turn_on', array($this, 'magicpage_editor_turn_on'));

        add_action('wp_ajax_magicpage_custom_meta_delete', array($this, 'custom_meta_delete'));
        add_action('wp_ajax_magicpage_add_custom_meta_field', array($this, 'add_custom_meta_field'));

        add_action('wp_loaded', array($this, 'process_form'));

        add_action('wp_ajax_magicpage_get_themes', array($this, 'get_themes'));

        add_action('wp_ajax_magicpage_download_theme', array($this, 'download_theme'));
    }

    function process_form() {
        if (isset($_POST['savechanges']) && current_user_can('administrator')) {

            if (isset($_POST['_magicpage_api_key'])) {
                $existingKey = get_option('_magicpage_api_key');
                $key = sanitize_text_field($_POST['_magicpage_api_key']);
                $error = get_option('_magic_page_key_error');

                if ($existingKey !== $key || $error) {
                    update_option('_magicpage_api_key', $key);
                    magic_page_fetch_new_api_data();

                    wp_redirect('edit.php?post_type=magicpage&page=magic-page-settings');
                    exit();
                }
            }
        }
    }

    public function add_custom_meta_field() {
        if (current_user_can('administrator')) {

            $term_id = intval($_POST['term_id']);
            $id = str_replace('-', '_', sanitize_title_with_dashes($_POST['label']));

            if (!get_term_meta($term_id, $id)) {
                update_term_meta($term_id, $id, '');
            }
            
            magic_page_delete_cache();

            echo($this->tr($id, $_POST['label'], '', false, ''));
        }
        exit();
    }

    public function custom_meta_delete() {
        if (current_user_can('administrator')) {
            $id = $_POST['id'];

            global $wpdb;
            $wpdb->query("DELETE FROM $wpdb->termmeta WHERE meta_key = '$id'"); // @todo

             magic_page_delete_cache();
            
            wp_send_json_success();
        }
       wp_send_json_error();
    }

    public function editor_turn_off() {
        if (current_user_can('administrator')) {
            $id = $_POST['id'];
            $term_id = intval($_POST['term_id']);
            $content = $_POST['strip_tags'] ? strip_tags($_POST['content']) : $_POST['content'];

            update_term_meta($term_id, $id, $content);

            $response = array('content' => $this->getTextarea($id, $content));

            update_option('_magic_page_editor_' . $id, 0);

            wp_send_json_success($response);
        }

        exit();
    }

    public function magicpage_editor_turn_on() {
        if (current_user_can('administrator')) {
            $id = $_POST['id'];
            $term_id = intval($_POST['term_id']);
            $content = strip_tags($_POST['content']);

            update_term_meta($term_id, $id, $content);
            update_option('_magic_page_editor_' . $id, 1);

            wp_send_json_success();
        }

        exit();
    }

    public function admin_scripts_in_terms_page() {

        global $pagenow;

        if ($pagenow === 'term.php' && isset($_GET['taxonomy']) && $_GET['taxonomy'] === 'location') {


            wp_enqueue_script('confirm-js', plugins_url('/js/jquery-confirm.js', MAGIC_PAGE_FILE), array('jquery'), MAGIC_PAGE_VERSION);

            wp_enqueue_style('confirm-css', plugins_url('/css/jquery-confirm.min.css', MAGIC_PAGE_FILE), array(), MAGIC_PAGE_VERSION);

            wp_enqueue_script('magic-page-term-page', plugins_url('/js/magic-page-term-page.js', MAGIC_PAGE_FILE), array('jquery', 'confirm-js'), MAGIC_PAGE_VERSION);

            wp_enqueue_style('editor-css-forced', includes_url('/css/editor.min.css'), array(), MAGIC_PAGE_VERSION);
        }
    }

    public function noticeFiles() {

        $compatability = magicPageCompatabilityModeEnabled();
        $file1 = MAGIC_PAGE_DIR . 'js/magic-page-admin-page.js';
        $file2 = MAGIC_PAGE_DIR . 'js/schema-builder.js';

        if (isMagicPageDebug()) {
            return;
        }

        // if compatability is OFF and either of the protected files is missing turn compatability ON and show notification
        if (!$compatability && (!file_exists($file1) || !file_exists($file2))) {
            update_option('_magicpage_compatability_mode', '1');
            ?>


            <div class="notice notice-success is-dismissible">
                <p> An important javascript file(s) are missing from magic-page folder.  <br />
                    Some hosts may recognize the code obfuscation we use as a virus and delete these files, don't panic they are not using proven tools but merely attempt to
                    find obfuscation which we use. <br />
                    In this case you can load the file(s) directly from our server, we have enabled this option for you now since these files are missing (see Compatability under Footprint Minifier). <br />

                </p>
            </div> <?php
        }
    }

    public function notice() {
        if (get_option('_magic_page_key_error') && is_magic_page_installed()) {
            ?> <div id="message" class="error fade"><p>Magic Page Notice: <?php echo get_option('_magic_page_key_error') ?></p></div><?php
            delete_option('_magicpage_d_l');
        }
    }

    public function checkForUpdates() {
        if (current_user_can('administrator') && isset($_GET['check_updates']) && $_GET['check_updates'] == 'true') {

            delete_transient('_magic_page_api_licensed_databases_list');

            get_regions_update_info();

            wp_redirect('admin.php?page=magicpage-installation&action=databases');
        }
    }

    public function onPluginActivate() {
        if (!get_option('_magic_page_outside_radius_redirect')) {
            update_option('_magic_page_outside_radius_redirect', get_home_url());
        }

        if (!get_option('_magic_page_default_radius')) {
            update_option('_magic_page_default_radius', 1000);
        }

        if (!get_option('_magic_page_url')) {
            update_option('_magic_page_url', magic_page_default_post_slug());
        }

        ob_start();

        magic_page_rewrite_rules();

        flush_rewrite_rules();

        ob_end_clean();

        magic_page_cron_activate();
    }

    public function onPluginDeactivate() {
        delete_option('_magicpage_installed');
        magic_page_cron_deactivate();
    }

    public function admin() {


        if (current_user_can('administrator') && !defined('DOING_AJAX')) {
            global $wp_rewrite;

            if (!get_option('_magic_page_permalinks_activated') && !$wp_rewrite->using_permalinks()) {
                $pretty_permalinks = $this->maybe_enable_permalinks();

                if ($pretty_permalinks) {
                    magic_page_rewrite_rules(true);
                }

                update_option('_magic_page_permalinks_activated', 1);
            } else {
                if (!$wp_rewrite->using_permalinks()) {
                    $class = 'notice notice-error';
                    $message = 'Magic Page requires enabled permalink structure. Go to <a href="' . admin_url('options-permalink.php') . '">Settings -> Permalinks</a> and enable <b>Post Name</b> or <b>Custom Structure</b>.'
                            . ' <br />Make sure you have mod_rewrite enabled on server and your .htaccess file is writable.<br />'
                            . 'You can refer to the official <a href="https://codex.wordpress.org/Using_Permalinks"> WordPress Using Permalinks Documentation</a> for further information.'
                            . '';

                    printf('<div class="%1$s"><p>%2$s</p></div>', $class, $message);
                }
            }
        }
    }

    public function admin_scripts_post_page() {

        global $pagenow;

        if ($pagenow === 'post.php') {


            wp_register_style('select2css', plugins_url('/css/select2.min.css', MAGIC_PAGE_FILE), false, '1.0', 'all');
            wp_register_script('select2', plugins_url('/js/select2.full.min.js', MAGIC_PAGE_FILE), array('jquery'), '1.0', true);

            wp_enqueue_style('select2css');
            wp_enqueue_script('select2');
        }
    }

    public function admin_scripts() {


        if (isset($_GET['page']) && $_GET['page'] === 'mpp-themes') {

            wp_enqueue_script('magic-page-themes', plugins_url('/js/magic-page-admin-themes.js', MAGIC_PAGE_FILE), array('jquery'), MAGIC_PAGE_VERSION);
            wp_enqueue_style('magic-page-admin-themes', plugins_url('/css/magic-page-admin-themes.css', MAGIC_PAGE_FILE), array(), MAGIC_PAGE_VERSION);
        }



        if (isset($_GET['page']) && $_GET['page'] === 'magic-page-settings') {
            wp_enqueue_style('wp-jquery-ui-dialog');

            wp_enqueue_script('jquery-validate-en', plugins_url('/js/jquery.validationEngine-en.js', MAGIC_PAGE_FILE), array('jquery', 'jquery-ui-dialog'), MAGIC_PAGE_VERSION);
            wp_enqueue_script('jquery-validate', plugins_url('/js/jquery.validationEngine.js', MAGIC_PAGE_FILE), array('jquery-validate-en'), MAGIC_PAGE_VERSION);
            wp_enqueue_script('magic-page-admin-settings', plugins_url('/js/magic-page-admin-settings.js', MAGIC_PAGE_FILE), array('jquery-validate'), MAGIC_PAGE_VERSION);
            wp_enqueue_style('validationEngine.jquery', plugins_url('/css/validationEngine.jquery.css', MAGIC_PAGE_FILE), array(), MAGIC_PAGE_VERSION);
            wp_enqueue_style('magic-page-admin', plugins_url('/css/magic-page-admin.css', MAGIC_PAGE_FILE), array(), MAGIC_PAGE_VERSION);
            wp_enqueue_script('magic-page-delete-unused-locations', plugins_url('/js/delete-unused-locations.js', MAGIC_PAGE_FILE), array('jquery'), MAGIC_PAGE_VERSION);
        }

        if (shouldLoadCheck()) {
            wp_enqueue_style('jquery-ui.min', plugins_url('/css/ui/jquery-ui.min.css', MAGIC_PAGE_FILE), array(), MAGIC_PAGE_VERSION);

            wp_enqueue_script('jquery-ui-accordion');

            wp_enqueue_script('ace', plugins_url('/js/ace/ace.js', MAGIC_PAGE_FILE), array('jquery'), MAGIC_PAGE_VERSION);
            wp_enqueue_script('ace-textarea', plugins_url('/js/ace/ace-textarea.js', MAGIC_PAGE_FILE), array('ace'), MAGIC_PAGE_VERSION);
            wp_enqueue_script('ace-editor-loader', plugins_url('/js/ace-editor-loader.js', MAGIC_PAGE_FILE), array('ace-textarea'), MAGIC_PAGE_VERSION);

            wp_enqueue_script('jquery.fileupload', plugins_url('/js/jquery.fileupload.js', MAGIC_PAGE_FILE), array('jquery', 'jquery-ui-widget'), MAGIC_PAGE_VERSION);
            wp_enqueue_style('wp-jquery-ui-dialog');
            wp_enqueue_script('jquery-validate-en', plugins_url('/js/jquery.validationEngine-en.js', MAGIC_PAGE_FILE), array('jquery'), MAGIC_PAGE_VERSION);
            wp_enqueue_script('jquery-validate', plugins_url('/js/jquery.validationEngine.js', MAGIC_PAGE_FILE), array('jquery-validate-en'), MAGIC_PAGE_VERSION);
            wp_enqueue_script('clipboard', plugins_url('/js/clipboard.min.js', MAGIC_PAGE_FILE), array(), MAGIC_PAGE_VERSION);
            wp_enqueue_script('jsrender', plugins_url('/js/jsrender.min.js', MAGIC_PAGE_FILE), array(), MAGIC_PAGE_VERSION);
            wp_enqueue_script('js-notify', plugins_url('/js/notify.min.js', MAGIC_PAGE_FILE), array(), MAGIC_PAGE_VERSION);

            // charts
            wp_enqueue_script('xcharts', plugins_url('/js/charts/xcharts.min.js', MAGIC_PAGE_FILE), array(), MAGIC_PAGE_VERSION);
            wp_enqueue_script('sugar', plugins_url('/js/charts/sugar.min.js', MAGIC_PAGE_FILE), array(), MAGIC_PAGE_VERSION);
            wp_enqueue_script('d3', plugins_url('/js/d3.v3.min.js', MAGIC_PAGE_FILE), array(), MAGIC_PAGE_VERSION);

            wp_enqueue_style('xcharts', plugins_url('/css/xcharts.min.css', MAGIC_PAGE_FILE), array(), MAGIC_PAGE_VERSION);

            // confirm
            wp_enqueue_script('confirm-js', plugins_url('/js/jquery-confirm.js', MAGIC_PAGE_FILE), array('jquery-validate'), MAGIC_PAGE_VERSION);
            wp_enqueue_style('confirm-css', plugins_url('/css/jquery-confirm.min.css', MAGIC_PAGE_FILE), array(), MAGIC_PAGE_VERSION);

            wp_enqueue_script('custom-underscorejs', plugins_url('/js/custom-underscorejs.js', MAGIC_PAGE_FILE), array(), MAGIC_PAGE_VERSION);
            wp_enqueue_script('jquery.animateNumber', plugins_url('/js/jquery.animateNumber.js', MAGIC_PAGE_FILE), array('jquery'), MAGIC_PAGE_VERSION);

            if (magicPageCompatabilityModeEnabled()) {
                global $magicpage_cdn_url;

                if (!defined('MAGIC_PAGE_ADMIN_PAGE')) {
                    define('MAGIC_PAGE_ADMIN_PAGE', $magicpage_cdn_url . MAGIC_PAGE_VERSION . '/magic-page-admin-page.js');
                }
            } else {
                if (!defined('MAGIC_PAGE_ADMIN_PAGE')) {
                    define('MAGIC_PAGE_ADMIN_PAGE', plugins_url('/js/magic-page-admin-page.js', MAGIC_PAGE_FILE));
                }
            }


            wp_register_style('select2css', plugins_url('/css/select2.min.css', MAGIC_PAGE_FILE), false, '1.0', 'all');
            wp_register_script('select2', plugins_url('/js/select2.full.min.js', MAGIC_PAGE_FILE), array('jquery'), '1.0', true);
            wp_enqueue_style('select2css');
            wp_enqueue_script('select2');

            wp_enqueue_script('magic-page-admin-page', MAGIC_PAGE_ADMIN_PAGE, array('jquery', 'jquery.animateNumber', 'custom-underscorejs', 'select2', 'xcharts', 'd3', 'sugar', 'confirm-js', 'js-notify', 'jsrender', 'jquery.fileupload', 'jquery-ui-dialog', 'jquery-validate', 'jquery-ui-autocomplete', 'jquery-ui-slider', 'clipboard', 'jquery-ui-accordion'), MAGIC_PAGE_VERSION);
            wp_enqueue_style('magic-page-admin', plugins_url('/css/magic-page-admin.css', MAGIC_PAGE_FILE), array(), MAGIC_PAGE_VERSION);
            wp_enqueue_style('magic-page-hints', plugins_url('/css/hint.min.css', MAGIC_PAGE_FILE), array(), MAGIC_PAGE_VERSION);

            wp_enqueue_style('jquery-ui-slider');
        }
    }

    function location_edit_meta_field($term) {
        global $magic_page_location_text_fields;

        foreach ($magic_page_location_text_fields as $field) {
            $value = get_term_meta($term->term_id, $field['id'], true);

            if ($field['id'] == 'major_city') {
                ?>
                <tr class="form-field">
                    <th scope="row" valign="top"><label for="term_meta_<?php echo $field['id'] ?>"> <?php echo $field['label'] ?>  </label></th>
                    <td>
                        <input type="checkbox" name="<?php echo $field['id'] ?>" id="<?php echo $field['id'] ?>" value="true"
                <?php if ($value) { ?>checked="checked" <?php } ?>  />
                        <p class="description"> <?php echo $field['description'] ?> </p>
                    </td>
                </tr>
                <?php
            } else {
                ?>
                <tr class="form-field">
                    <th scope="row" valign="top"><label for="<?php echo $field['id'] ?>"> <?php echo $field['label'] ?>  </label></th>
                    <td>
                        <input type="text" name="<?php echo $field['id'] ?>" id="meta_<?php echo $field['id'] ?>" value="<?php echo $value; ?>">
                        <p class="description"> <?php echo $field['description'] ?> </p>
                    </td>
                </tr>
                <?php
            }
        }

        $custom_fields = get_location_custom_meta_fields();
        ?>


        <tr class="form-field">
            <td style="background: #fff; border: 1px solid #ccc;" colspan="2">
                <h3 style="margin-top:0px; float: left;"> Custom Meta Fields </h3>

                <button id="add-custom-meta-field" type="button" style="float:right" class="button" ><span></span> Add New Custom Field</button>

                <table id="custom-fields-table" width="100%">
        <?php
        foreach ($custom_fields as $field) {

            $label = labelize_magic_page($field);
            $content = get_term_meta($term->term_id, $field, true);

            $editorEnabled = get_option('_magic_page_editor_' . $field);

            $this->tr($field, $label, $content, $editorEnabled, $value);
        }
        ?>

                </table>
            </td>
        </tr>
        <?php
    }

    public function tr($field, $label, $content, $editorEnabled, $value) {
        ?>

        <tr data-id="<?php echo($field) ?>" class="form-field custom-meta-field-td">
            <th scope="row" valign="top">
                <label for="<?php echo $field ?>">
        <?php echo $label ?>
                    [meta_<?php echo $field ?>]
                </label>


                <div style="padding-top:8px;">
                    <span data-state="1" class="button enable-editor enable-disable-editor-custom-field <?php if ($editorEnabled) { ?>hidden<?php } ?> ">Enable Editor</span>
                    <span data-state="0" class="button disable-editor enable-disable-editor-custom-field <?php if (!$editorEnabled) { ?>hidden<?php } ?> ">Disable Editor</span>
                    <span class="button delete-editor-custom-field">Delete </span>

                </div>
            </th>
            <td class="editor-cell">
        <?php
        if (!$editorEnabled) {
            echo($this->getTextarea($field, $content));
        } else {
            $settings = array('textarea_rows' => 5);
            wp_editor($content, $field, $settings);
        }
        ?>


            </td>


        </tr>
        <tr><td colspan="2"> <hr /></td></tr><?php
    }

    public function getTextarea($field, $content) {
        return '<textarea rows="3" cols="10" name="' . $field . '" id="' . $field . '">' . $content . '</textarea>';
    }

    public function save_location_custom_meta($term_id) {
        if (current_user_can('administrator')) {

            if (isset($_POST['major_city']) && $_POST['major_city'] === "true") {
                update_term_meta($term_id, 'major_city', '1');
            } else {
                delete_term_meta($term_id, 'major_city');
            }

            global $magic_page_location_text_fields;

            foreach ($magic_page_location_text_fields as $field) {
                if ($field['id'] != 'major_city') {

                    if (isset($_POST[$field['id']])) {
                        update_term_meta($term_id, $field['id'], sanitize_text_field($_POST[$field['id']]));
                    } else {
                        delete_term_meta($term_id, $field['id']);
                    }
                }
            }



            $customFileds = get_location_custom_meta_fields();

            if ($customFileds) {
                foreach ($customFileds as $field) {
                    $content = wp_kses_post($_POST[$field]);
                    update_term_meta($term_id, $field, $content);
                }
            }
        }
    }

    function menu() {
        if (is_magic_page_installed()) {
            add_submenu_page('edit.php?post_type=magicpage', 'Settings', 'Settings', 'administrator', 'magic-page-settings', array($this, 'magic_settings'));
            add_submenu_page('edit.php?post_type=magicpage', 'Templates', 'Templates', 'administrator', 'mpp-themes', array($this, 'mpp_themes'));
        

        }
    }
    
   

    function mpp_themes() {
        ?>


        <div id="themes-mpp" class="wrap">
            <h1 class="wp-heading-inline">
                Templates
                <span style="display:none" class="title-count theme-count"> </span>

                <a id="help-link-video" href="https://www.youtube.com/watch?v=ymRx_u-i434&ab_channel=KeithBestVideos" target="_blank">Help</a>

            </h1>

            <div id="mpp-theme-browser" class="theme-browser">
                Loading...
            </div>
        </div>
        <?php
    }

    function magic_settings() {
        if (isset($_POST['savechanges']) && current_user_can('administrator')) {
            check_admin_referer('update-magicpage-settings');

            $options = array(
                '_magicpage_api_key',
                '_magic_page_default_radius',
                '_magic_page_google_maps_api_key',
                '_magic_page_outside_radius_redirect',
                '_magic_page_cron_support',
                '_magic_page_spintax_support',
                '_magic_page_spintax_autorewrite',
                '_magic_page_spintax_autorewrite_interval',
                '_magic_page_spintax_autorewrite_coverage',
                '_magic_page_spintax_autorewrite_amount',
                '_magic_page_elementor_fix',
                '_magic_page_editor_style',
                '_magic_page_cache'
            );

            if (isset($_POST['_magicpage_api_key'])) {
                magic_page_fetch_new_api_data();
            }

            if (isset($_POST['_magic_page_url']) && get_option('_magic_page_url') !== $_POST['_magic_page_url']) {
                // remove trailing slash and empty spaces
                $_magic_page_url = trim(untrailingslashit($_POST['_magic_page_url']));
                update_option('_magic_page_url', $_magic_page_url);

                magic_page_rewrite_rules();
            }

            foreach ($options as $saveOptions) {
                if (isset($_POST[$saveOptions])) {
                    update_option($saveOptions, sanitize_text_field($_POST[$saveOptions]));
                }
            }

            if (isset($_POST['_magic_page_seo_for_all_pages'])) {
                update_option('_magic_page_seo_for_all_pages', '1');
            } else {
                update_option('_magic_page_seo_for_all_pages', '0');
            }

            if (isset($_POST['_magic_page_xml_sitemap'])) {
                update_option('_magic_page_xml_sitemap', '1');
            } else {
                update_option('_magic_page_xml_sitemap', '0');
            }

            if (isset($_POST['_magic_page_xml_sitemap_options'])) {
                update_option('_magic_page_xml_sitemap_options', $_POST['_magic_page_xml_sitemap_options']);
            } else {
                update_option('_magic_page_xml_sitemap_options', '');
            }
            ?>

            <div id="message" class="updated fade"><p>Settings successfully updated.</p></div>
            <?php
            magic_page_delete_cache();
        }

        if (!in_array(get_option("_magic_page_seo_for_all_pages"), array('0', '1'))) {
            update_option("_magic_page_seo_for_all_pages", "1");
        }


        if (!isset($_POST['savechanges']) && isset($_GET['installed']) && $_GET['installed'] == 'true') {
            $notice = get_option('magic_page_installation_complete_notification');

            if (!$notice) {
                ?>
                <div id="message" class="updated fade">
                    <p>Congratulations! Magic Page successfully installed. <br />
                        Don't forget to set your Google Maps API Key in the settings below.</p>

                    <p>More options will be made available after you create your Magic Page </p>

                </div><?php
                update_option('magic_page_installation_complete_notification', 1);
            }
            ?>

            <?php
        }
        ?>
        <div class="wrap">
            <h2>Magic Page Settings</h2>
            <div class="postbox">
                <div class="inside">
                    <form novalidate="novalidate" id="magic-page-settings" action="<?php admin_url('edit.php?post_type=magicpage&page=magic-page-setting') ?>" method="post">
        <?php wp_nonce_field('update-magicpage-settings') ?>

                        <div class="alignleft" id="magic_page">
                            <table class="form-table">

        <?php if (magicpage_hidden_by_check()) { ?>
                                    <tr valign="top">
                                        <th scope="row">Magic Page API Key</th>
                                        <td><input id="_magicpage_api_key" data-errormessage-value-missing="Please enter your Magic Page API Key" required="required" class="regular-text validate[required]"
            <?php if (!is_magicpage_api_key_hidden()) { ?> type="text" <?php } else { ?> type="password" <?php } ?>
                                                   value="<?php echo get_option('_magicpage_api_key'); ?>" name="_magicpage_api_key"/>

                                            <span data-keydata="1" class="key-actions key-action-hide <?php if (is_magicpage_api_key_hidden()) { ?>hidden <?php } ?>">hide</span>
                                            <span data-keydata="0" class="key-actions key-action-show <?php if (!is_magicpage_api_key_hidden()) { ?>hidden <?php } ?>">show</span>

                                            <br /> Your Magic Page API Key

                                            <b   <?php if (!magicpage_api_key_hidden_by()) { ?> class="hidden" <?php } ?>  id="hidden-for-all">is <span title="Click to unhide for all users" id="hidden-unhide">hidden</span> for all other users</b>

                                        </td>
                                    </tr>

        <?php } ?>

                                <tr valign="top">
                                    <th scope="row">Redirect URL</th>
                                    <td><input data-errormessage-value-missing="Please enter redirect URL" class="regular-text validate[required,custom[custom_url]]]" type="text"  value="<?php echo get_option('_magic_page_outside_radius_redirect', get_home_url()); ?>" name="_magic_page_outside_radius_redirect"/>
                                        <br /> Redirect to this URL if current location is outside chosen radius
                                    </td>
                                </tr>

                                <tr valign="top">
                                    <th scope="row">Magic Page Editor Type</th>
                                    <td>

                                        <select id="magic_page_editor_style" name="_magic_page_editor_style" />
                                <option value="classic" <?php selected('classic', getMagicPageEditor()); ?>>Classic Editor</option>
                                <option value="gutenberg" <?php selected('gutenberg', getMagicPageEditor()); ?>>Gutenberg Editor</option>
                                </select>

                                <br />  If you like to use GenerateBlocks plugin for example choose Gutenberg Editor
                                </td>
                                </tr>

                                <tr valign="top">
                                    <th scope="row">Magic Page Cache</th>
                                    <td>
                                        <?php 
                                        
                                            $cache = getMagicPageCacheEnabled();
                                            $cachePluginInstalled = isThirdPartyCachePluginEnabled();
                                            
                                            $disabled = '';
                                            $cachePluginNote = '';
                                            if($cachePluginInstalled) {
                                                $cache = false;
                                                $disabled = ' disabled="disabled" ';
                                                $cachePluginNote = 'Cache plugin has been installed, we have disabled Magic Page Cache.';
                                            }
                                            
                                            
                                             
                                        
                                        ?>

                                        <select id="magic_page_cache" name="_magic_page_cache" />
                                <option <?php echo($disabled) ?> value="1" <?php selected(true, $cache); ?>>Enabled</option>
                                <option <?php echo($disabled) ?> value="0" <?php selected(false, $cache); ?>>Disabled</option>
                                </select>

                                <br />  Cache speeds up your site but you must turn it off when using 3rd party database cache plugin. 
                                <?php echo($cachePluginNote) ?>
                                </td>
                                </tr>
                                
                                
                                <tr valign="top">
                                    <th scope="row">XML Sitemap</th>
                                    <td>

                                        <input id="magicPageXmlSitemap" value="1" <?php if (get_option('_magic_page_xml_sitemap')) { ?>checked="checked" <?php } ?>   type="checkbox" name="_magic_page_xml_sitemap" />
                                        Enable
                                        <br />  Create XML Sitemap.

                                        <div style="padding-top: 10px;"><?php echo(getMagicPageSitemapOptionsHtml()); ?></div>


                                        <br /> <div <?php if (!get_option('_magic_page_xml_sitemap')) { ?>style="display:none" <?php } ?> class="sitemapEnabled"> Sitemap URL: <?php echo(get_site_url(null, '/sitemap/sitemap.xml')) ?></div>
                                    </td>
                                </tr>





                                <tr valign="top">
                                    <th scope="row">Default Radius</th>
                                    <td><input class="validate[required,custom[integer],min[1],max[10000]"  type="number" min="0" step="1" value="<?php echo get_option('_magic_page_default_radius', 1000); ?>" name="_magic_page_default_radius"/>
                                        <br /> Choose Default Radius. If you provide custom radius, the default radius will be overwritten.
                                    </td>
                                </tr>
                                <tr valign="top">
                                    <th scope="row">URL Structure</th>
                                    <td><input id="url-structure" data-errormessage-value-missing="Please enter rewrite base for URLs" class="regular-text validate[required,custom[slug]]"  required="required" type="text" value="<?php echo get_option('_magic_page_url', magic_page_default_post_slug()); ?>" name="_magic_page_url"/>
                                        <br /> Examples: magicpage, magicpage/service, magicpage/services/near
                                    </td>
                                </tr>
                                <tr valign="top">
                                    <th scope="row">Google Maps API Key</th>
                                    <td><input  class="regular-text" type="text" value="<?php echo get_option('_magic_page_google_maps_api_key'); ?>" name="_magic_page_google_maps_api_key"/>
                                        <br /> Your <a target="_blank" href="https://developers.google.com/maps/documentation/javascript/get-api-key">Google Maps API</a> Key
                                    </td>
                                </tr>

                                <tr valign="top">
                                    <th scope="row">SEO Settings for Regular Posts & Pages</th>
                                    <td><input value="1" <?php if (get_option('_magic_page_seo_for_all_pages')) { ?>checked="checked" <?php } ?>   type="checkbox" name="_magic_page_seo_for_all_pages" />
                                        Enable<br />     Magic Page SEO Settings metabox for all Posts & Pages
                                    </td>
                                </tr>




        <?php if (get_magicpage_shortid()) {
            ?>

                                    <tr valign="top">
                                        <th scope="row">Cron Service Support </th>
                                        <td>
                                            <select id="magic_page_cron_support" name="_magic_page_cron_support" />
                                    <option value="basic" <?php selected('basic', get_option('_magic_page_cron_support')); ?>>Basic</option>
                                    <option value="advanced" <?php selected('advanced', get_option('_magic_page_cron_support')); ?>>Advanced</option>
                                    </select>
                                    <br /> Used by Content Auto-refresh option.  <br />For better experience "Advanced" mode is highly recommended.
                                    </td>

                                    </tr>
                                    <tr <?php if (get_option('_magic_page_cron_support') !== "advanced") { ?>style="display:none"<?php } ?> id="advanced-cron-info">
                                        <th scope="row">Advanced Cron Info </th>
                                        <td>
                                            <div>
                                                Please create Cron Job (via cPanel or your hosting panel) to run on every 24 hours using this URL: <br/>

            <?php echo get_magicpage_cronurl(); ?>
                                            </div>
                                        </td>
                                    </tr>


        <?php } ?>




                                <tr valign="top">
                                    <th scope="row">Spintax Support</th>
                                    <td>
                                        <select id="_magic_page_spintax_support" name="_magic_page_spintax_support" />
                                <option value="1" <?php selected('1', get_option('_magic_page_spintax_support')); ?>>Enabled</option>
                                <option value="0" <?php selected('0', get_option('_magic_page_spintax_support')); ?>>Disabled</option>
                                <option value="2" <?php selected('2', get_option('_magic_page_spintax_support')); ?>>Disabled for Regular Post Types</option>
                                </select>
                                <br /> Disable Spintax Support if you have 3rd party plugins in the page editor introducing { } brackets <br /><br />

                                <span id="deleteSpintaxCache">Delete Spintax Cache</span> <div style="display: none" class="loader"></div>

                                </td>

                                </tr>

                                <tr <?php if (get_option('_magic_page_spintax_support') != '1') { ?>style="display:none"<?php } ?>  class="magic_page_spintax_support" valign="top">
                                    <th scope="row">Spintax Auto-Rewrite for regular Posts</th>
                                    <td>
                                        <select id="_magic_page_spintax_autorewrite" name="_magic_page_spintax_autorewrite" />
                                <option value="1" <?php selected('1', get_option('_magic_page_spintax_autorewrite')); ?>>Enabled</option>
                                <option value="0" <?php selected('0', get_option('_magic_page_spintax_autorewrite')); ?>>Disabled</option>
                                </select>

                                <br /> Auto-rewrite spintax enabled Posts on given interval using cron.
                                </td>
                                </tr>

                                <tr <?php if (get_option('_magic_page_spintax_support') != '1' or get_option('_magic_page_spintax_autorewrite') == "0") { ?>style="display:none"<?php } ?> class="magic_page_spintax_support magic_page_spintax_autorewrite_options" valign="top">
                                    <th scope="row">Auto-Rewrite Options for Post & Pages </th>
                                    <td>
        <?php
        $interval = get_option('_magic_page_spintax_autorewrite_interval');
        $coverage = get_option('_magic_page_spintax_autorewrite_coverage');
        $amount = get_option('_magic_page_spintax_autorewrite_amount');
        ?>
                                        <label class="fl-mr">Interval<br />
                                            <select name="_magic_page_spintax_autorewrite_interval" id="autorewrite-interval">
                                                <option value="7" <?php selected('7', $interval) ?>>1 Week</option>
                                                <option value="14" <?php selected('14', $interval) ?>>2 Weeks</option>
                                                <option value="21" <?php selected('21', $interval) ?>>3 Weeks</option>
                                                <option value="30" <?php selected('30', $interval) ?>>1 Month</option>
                                                <option value="60" <?php selected('60', $interval) ?>>2 Months</option>
                                                <option value="90" <?php selected('90', $interval) ?>>3 Months</option>
                                            </select>
                                        </label>
                                        <label class="fl-mr">Page coverage<br />
                                            <select name="_magic_page_spintax_autorewrite_coverage" id="autorewrite-page-coverage">
        <?php
        $i = 0;

        do {
            $i += 10;
            ?><option value="<?php echo($i) ?>" <?php selected($i, $coverage) ?>><?php echo($i) ?>%</option><?php
                                                } while ($i < 100);
                                                ?>
                                            </select>
                                        </label>
                                        <label class="fl-mr">Posts amount<br />
                                            <select name="_magic_page_spintax_autorewrite_amount" id="autorewrite-posts-amount">
        <?php
        $i = 0;

        do {
            $i += 10;
            ?><option value="<?php echo($i) ?>" <?php selected($i, $amount) ?>><?php echo($i) ?>%</option><?php
                                                } while ($i < 100);
                                                ?>
                                            </select>

                                        </label>

                                        <div class="clear"></div>
                                        <br />
                                        Rewrite every <b id="interval"></b> <b id="coverage"><?php echo $coverage ?>%</b> of the spintax content on <b id="amount"><?php echo $amount ?>%</b> of the Posts
                                    </td>
                                </tr>



        <?php if (isElementorInstalled()) { ?>

                                    <tr  valign="top">
                                        <th scope="row">Elementor plugin fix</th>
                                        <td>
                                            <select name="_magic_page_elementor_fix" />
                                    <option value="1" <?php selected('1', get_option('_magic_page_elementor_fix', '1')); ?>>Enabled</option>
                                    <option value="0" <?php selected('0', get_option('_magic_page_elementor_fix')); ?>>Disabled</option>
                                    </select>

                                    <br /> Fix for Elementor plugin so the plugin won't break square brackets [].
                                    </td>
                                    </tr>

        <?php } ?>




                            </table>
                        </div>
                        <div class="alignright magic-page-info-box ">
                            <h3> Magic Page </h3>
                            <div><em>Version <?php echo MAGIC_PAGE_VERSION ?></em></div>

                            <h3>Databases</h3>

                            <a href="<?php echo admin_url('admin.php?page=magicpage-installation&action=databases') ?>">
                                <span id="databasesIcon">
                                    <svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 100 100">
                                    <path fill="#0073b2" d="M51.9 44.8c8.1 0 15.7-.7 22.8-2.2s12.6-3.6 16.7-6.5v8.7c0 2.4-1.8 4.6-5.3 6.6s-8.3 3.6-14.4 4.8-12.7 1.7-19.8 1.7-13.7-.6-19.8-1.8-10.9-2.8-14.4-4.8-5.3-4.2-5.3-6.6V36c4.1 2.9 9.6 5.1 16.7 6.5s14.6 2.3 22.8 2.3zm0 39.4c8.1 0 15.7-.7 22.8-2.2s12.6-3.6 16.7-6.5v8.7c0 2.4-1.8 4.6-5.3 6.6s-8.3 3.6-14.4 4.8-12.7 1.8-19.8 1.8-13.7-.6-19.8-1.8-10.9-2.8-14.4-4.8-5.3-4.2-5.3-6.6v-8.7c4.1 2.9 9.6 5.1 16.7 6.5s14.6 2.2 22.8 2.2zm0-19.7c8.1 0 15.7-.7 22.8-2.2s12.6-3.6 16.7-6.5v8.7c0 2.4-1.8 4.6-5.3 6.6s-8.3 3.6-14.4 4.8-12.7 1.8-19.8 1.8-13.7-.6-19.8-1.8-10.9-2.8-14.4-4.8-5.3-4.2-5.3-6.6v-8.7c4.1 2.9 9.6 5.1 16.7 6.5s14.6 2.2 22.8 2.2zm0-59.2c7.1 0 13.7.6 19.8 1.8S82.5 9.8 86 11.9s5.3 4.2 5.3 6.6V25c0 2.4-1.8 4.6-5.3 6.6s-8.3 3.6-14.4 4.8-12.6 1.8-19.7 1.8-13.7-.6-19.8-1.8-10.9-2.8-14.4-4.8-5.3-4.2-5.3-6.6v-6.6c0-2.4 1.8-4.6 5.3-6.6S26 8.2 32.1 7.1s12.6-1.8 19.8-1.8z"/>
                                    </svg>
                            </a>

                            </span>

                            <a href="<?php echo admin_url('admin.php?page=magicpage-installation&action=databases') ?>">Magic Page Databases</a>

        <?php
        $data = get_regions_update_info();

        $error = isset($data['status']) && $data['status'] == 'error';

        if ($data && !$error) {
            foreach ($data as $country => $regions) {
                if ($regions && is_array($regions)) {
                    ?>
                                        <div class="note">Database Updates are Available  </div>
                                        <?php
                                        foreach ($regions as $region) {
                                            $dt = new DateTime($region['date']);
                                            ?><h4><?php echo $region['name'] ?></h4>
                                            <div class="when">Version <b><?php echo $region['version'] ?></b>  released on <?php echo $dt->format('d/m/Y') ?></div>
                                            <?php
                                            if ($region['notes']) {
                                                ?><div class="notes">Notes: <?php echo $region['notes'] ?></div> <?php
                                            }
                                        }
                                        ?>
                                        <a href="<?php echo admin_url('admin.php?page=magicpage-installation&updates_only=true') ?>">Update Databases</a>
                                        <?php
                                    } else {
                                        ?> <div class="note-no-updates"> Locations Database is up to date</div>
                                        <a href="<?php echo admin_url('edit.php?post_type=magicpage&page=magic-page-settings&check_updates=true') ?>">Check for Updates</a>
                                        <?php
                                    }
                                }
                            } else {
                                ?> <div class="note-no-updates"> Locations Database is up to date</div>
                                <a href="<?php echo admin_url('edit.php?post_type=magicpage&page=magic-page-settings&check_updates=true') ?>">Check for Updates</a>

                                &nbsp;<a id="deleteUnusedLocationsDialog" href="#">Delete unused locations</a>



            <?php
            require_once(MAGIC_PAGE_DIR . 'src/admin/views/delete-unused-locations.html');
        }


        require_once(MAGIC_PAGE_DIR . 'src/admin/footprint-minifier.php');
        ?>


                            <h3> Import - Export </h3>
                            <span style="float:left; min-width:66px; margin-right: 5px; width:50px;">


                                <svg   xmlns="http://www.w3.org/2000/svg" viewBox="108 -74.5 236.3 169.5">
                                <defs/>
                                <path fill="#D3D3D3" d="M185.93-68.99l1.7 127.87c0 3 3.4 5.84 7.5 6.24l140.2 13.71c5 .49 9-1.78 9-5.19l-.9-141.17c0-3.4-4.1-6.16-9.1-6.16l-141-.73c-4.2-.08-7.5 2.35-7.4 5.43"/>
                                <path fill="#F9F9F9" d="M217.23-37.02l112.9-.73 1.2 103.68-116.3-9.73 2.2-93.22z"/>
                                <path fill="#A9A9A9" d="M193.93-54.95l.8 109.12 24.3 2.03-.6-110.74-24.5-.41"/>
                                <path fill="#A9A9A9" d="M193.73-37.5l.8-17.7 137.5 1.63-1 19.71-137.3-3.65"/>
                                <g fill="none" stroke="#636363" stroke-width="2">
                                <path d="M869.5 153.1l137.9 2M1007 179.1l-138-4.5M1006.9 197.9l-137.8-2.2M1006.5 220.1l-136.5-4M1008 242.6l-137-7.5M1006.7 264.6l-136.2-11.5M968.6 281l-99.1-10.5M1007.4 302.2l-137.9-14.6M966 154.6l.5 144.5M928 153.1l-.5 140M894.5 153.6v136.5M870.5 288.1l-.8-135.5M1007 153.9V302" transform="matrix(1 0 0 .8113 -675.57 -179.16)"/>
                                </g>
                                <path fill="none" d="M223.47-22.83v-6.85l44.45 20.86-44.45 20.86V4.88c-12.12 0-33.42.9-33.42 6.86 0 8.34 22.04 12.81 22.04 12.81v5.96s-49.59 1.5-49.59-29.2c.37-24.14 42.97-25.03 60.97-24.14z"/>
                                <path fill="#0073b2" d="M223.47-22.83v-6.85l44.45 20.86-44.45 20.86V4.88c-12.12 0-33.42.9-33.42 6.86 0 8.34 22.04 12.81 22.04 12.81v5.96s-49.59 1.5-49.59-29.2c.37-24.14 42.97-25.03 60.97-24.14z"/>
                                <path fill="none" d="M285.91 33.5H168.38v-95.37H285.9z"/>
                                <g>
                                <path fill="#FFF" d="M172.4 27.6h46.2c9.9 0 17.9 4.33 17.9 9.68V78.2c0 5.35-8 9.68-17.9 9.68h-46.2c-9.9 0-17.9-4.33-17.9-9.68V37.28c0-5.3 8-9.68 17.9-9.68z"/>
                                <path fill="none" d="M195.05 76.1c-16.58 0-33.6-2.37-41.46-7.33-.43.48-.43 1.19-.43 1.66v9.2c0 8.04 21.81 11.81 41.89 11.81s41.89-3.77 41.89-11.8v-9.21c0-.71 0-1.18-.44-1.66-7.42 4.96-24.43 7.33-41.45 7.33zm0-17.72c-16.58 0-33.6-2.36-41.46-7.32-.43.48-.43 1.19-.43 1.66v9.2c0 8.04 21.81 11.81 41.89 11.81s41.89-3.77 41.89-11.8v-9.21c0-.71 0-1.18-.44-1.66-7.42 4.96-24.43 7.32-41.45 7.32zm.43-35.18c-20.07 0-41.89 3.78-41.89 11.8v9.22c0 8.03 21.82 11.8 41.9 11.8s41.88-3.77 41.88-11.8V35c0-8.03-21.81-11.81-41.89-11.81zm0 18.9c-18.32 0-33.16-3.08-33.16-7.1 0-4 14.84-7.08 33.16-7.08s33.17 3.07 33.17 7.09c0 4.01-14.84 7.08-33.17 7.08z"/>
                                <path fill="#0073b2" d="M195.05 76.1c-16.58 0-33.6-2.37-41.46-7.33-.43.48-.43 1.19-.43 1.66v9.2c0 8.04 21.81 11.81 41.89 11.81s41.89-3.77 41.89-11.8v-9.21c0-.71 0-1.18-.44-1.66-7.42 4.96-24.43 7.33-41.45 7.33zm0-17.72c-16.58 0-33.6-2.36-41.46-7.32-.43.48-.43 1.19-.43 1.66v9.2c0 8.04 21.81 11.81 41.89 11.81s41.89-3.77 41.89-11.8v-9.21c0-.71 0-1.18-.44-1.66-7.42 4.96-24.43 7.32-41.45 7.32zm.43-35.18c-20.07 0-41.89 3.78-41.89 11.8v9.22c0 8.03 21.82 11.8 41.9 11.8s41.88-3.77 41.88-11.8V35c0-8.03-21.81-11.81-41.89-11.81zm0 18.9c-18.32 0-33.16-3.08-33.16-7.1 0-4 14.84-7.08 33.16-7.08s33.17 3.07 33.17 7.09c0 4.01-14.84 7.08-33.17 7.08z"/>
                                <path fill="none" d="M110.4 11.4h139.63v75.56H110.39z"/>
                                </g>
                                <path fill="#FFF" d="M299.3 35.43l22.8-.5 9.5 7.7-.3 22.6-33.9-2 1.9-27.8z"/>
                                <g>
                                <path fill="none" stroke="#A9CCDB" stroke-miterlimit="10" d="M1387-698.8h512" transform="matrix(.07787 0 0 .06317 187.4 108.62)"/>
                                <path fill="#696969" d="M322.82 34.15L290.2 32.3l-.3 30.93 42.87 3.28V42.24l-9.96-8.09zm7.47 30.32L292.91 61l.5-26.9 29.4 2.08v4.63l7.48 1.43v22.24zM319.21 42h-5.27l-3.73 4.46-3.73-4.46h-5.26l6.31 7.66-7.12 8.52h10.2v-2.9h-2.35l1.95-2.37 4.34 5.27H320l-7.12-8.52 6.32-7.66z"/>
                                </g>
                                </svg>

                            </span>

                            You can import & export data (locations databases) using CSV/XLSX or XLS files as well as xfields, location sets, spintax data. <br /> <br />
                            <a style="margin-top:10px; float: left;" class="button button-secondary" href="<?php echo(admin_url('plugins.php?page=magic-page-import-export')) ?>">Import - Export</a>
                            <br />
                            <div style="clear: both"></div>

                            <h3> Locations Search </h3>
                            <span style="float:left; min-width:66px; margin-right: 5px; width:50px;">

                                <svg fill="#0073b2" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 91 91"><path d="M35.4 36.4c-3.9-.7-7.5-1.8-10.9-3.3l-1.8-.9c-2.7 4-4.5 8.8-4.8 13.9h16.3c.1-3.3.5-6.6 1.2-9.7zM24.8 29.5l1.1.5c3.1 1.4 6.6 2.4 10.3 3.1 1.2-4.3 3-8.3 5.3-11.6l.8-1.2c-6.9.8-13.1 4.2-17.5 9.2zM24.6 62.7c3.3-1.5 7-2.6 10.8-3.2-.7-3.2-1.1-6.5-1.2-9.9H17.9c.3 5.2 2.1 10 4.9 14l1.8-.9zM47.7 20.2l.9 1.4c2.3 3.3 4 7.3 5.3 11.6 4.1-.7 8-1.9 11.4-3.5l.8-.4a27.4 27.4 0 00-18.4-9.1zM24.9 66.3c4.4 4.9 10.5 8.3 17.4 9.1l-.3-.5c-2.5-3.4-4.4-7.6-5.8-12.2-3.6.6-7.1 1.6-10.2 3l-1.1.6zM45 73.3l.4-.5c2.2-3 3.9-6.6 5.1-10.7-1.7-.2-3.5-.3-5.2-.3-1.9 0-3.8.1-5.6.3 1.2 4 2.9 7.7 5.1 10.6l.2.6zM45 22.2l-.8 1.2c-2 2.9-3.5 6.3-4.7 10.2 1.9.2 3.8.3 5.7.3 1.8 0 3.6-.1 5.3-.3a35.4 35.4 0 00-4.7-10.2l-.8-1.2zM66.1 66.7l-.9-.6c-3.4-1.6-7.3-2.8-11.4-3.5A38.3 38.3 0 0148 74.8l-.5.6c7.4-.4 14.1-3.6 18.6-8.7zM62.1 39.1c1.9 1.9 3.1 4.4 3.4 7h7.8c-.3-5.3-2.1-10.1-5-14.2l-1.5.8c-3.1 1.5-6.5 2.6-10.1 3.3 2.1.6 3.9 1.6 5.4 3.1zM45.2 58.6c2.1 0 4.1.1 6.1.3-2.2-.5-4.2-1.5-5.8-3.2-1.7-1.7-2.8-3.8-3.2-6.2h-4.7c.1 3.3.5 6.4 1.1 9.4 2.2-.2 4.3-.3 6.5-.3zM48 37.2c-.9 0-1.9.1-2.8.1-2.2 0-4.4-.1-6.5-.4-.6 3-1 6.1-1.1 9.3h4.6c.3-2.7 1.4-5.1 3.4-7 .7-.9 1.5-1.5 2.4-2zM66.9 63.2l.8.5-6.9-6.9c-1.8 1.3-3.9 2.1-6.1 2.3 0 .1 0 .2-.1.3 4.3.7 8.4 2 12.1 3.7l.2.1zM65.4 49.5c-.3 1.8-1.1 3.4-2.1 4.9l6.7 6.7c1.6-3.1 3.1-6.8 3.4-11.6h-8zM75.1 66.3l-5.2-5.2c-.2.4-.4.7-.6 1.1-.3.6-.7 1.2-1.1 1.8l-.5-.3 5 5 2.4-2.4z"/></svg>

                            </span>

                            Transform any searchbox into locations search box with autosuggest feature <br /> <br />

                            <div style="clear: both"></div>
                            <a style="margin-top:10px; float: left;" class="button button-secondary" href="<?php echo(admin_url('edit.php?post_type=magicpage&page=magicpage-locations-autosuggest')) ?>">Locations Search</a>
                            <br />
                            <div style="clear: both"></div>


                            <h3>Compatability</h3>
                            <p>


                                <span style="float:left; min-width:66px; margin-right: 5px; width:50px;">

                                    <svg version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px"
                                         viewBox="15 -15 50 50" fill="#0073b2" enable-background="new 15 -15 50 50" xml:space="preserve">
                                    <path d="M62.2,6.2l-5.9-1c-0.3-1.2-0.8-2.3-1.4-3.4l3.5-4.9c0.3-0.4,0.2-0.9-0.1-1.3l-3.9-3.9c-0.3-0.3-0.9-0.4-1.3-0.1l-4.8,3.5
                                          c-1.1-0.6-2.2-1.1-3.4-1.4l-1-5.9c-0.1-0.5-0.5-0.8-1-0.8h-5.5c-0.5,0-0.9,0.4-1,0.8l-1,5.9c-1.2,0.3-2.4,0.8-3.4,1.4L27-8.3
                                          c-0.4-0.3-0.9-0.2-1.3,0.1l-3.9,3.9c-0.3,0.3-0.4,0.9-0.1,1.3l3.4,4.9C24.5,2.9,24,4,23.7,5.3l-5.8,1c-0.5,0.1-0.8,0.5-0.8,1v5.5
                                          c0,0.5,0.3,0.9,0.8,1l5.8,1c0.3,1.2,0.8,2.4,1.4,3.5L21.7,23c-0.3,0.4-0.2,0.9,0.1,1.3l3.9,3.9c0.3,0.3,0.9,0.4,1.3,0.1l4.9-3.4
                                          c1.1,0.6,2.2,1.1,3.4,1.4l1,5.9c0.1,0.5,0.5,0.8,1,0.8h5.5c0.5,0,0.9-0.3,1-0.8l1-5.9c1.2-0.4,2.3-0.8,3.4-1.4l4.9,3.4
                                          c0.4,0.3,0.9,0.2,1.3-0.1l3.9-3.9c0.3-0.3,0.4-0.9,0.1-1.3l-3.5-4.9c0.6-1.1,1.1-2.2,1.4-3.4l5.9-1c0.5-0.1,0.8-0.5,0.8-1V7.2
                                          C63,6.7,62.6,6.3,62.2,6.2z M40,17c-3.9,0-7-3.1-7-7s3.1-7,7-7s7,3.1,7,7S43.9,17,40,17z"/>
                                    </svg>


                                </span>


                                Some hosts may recognize the obfuscation we use on some files as a threat.
                                To overcome this and still be able to use the plugin turn the compatability mode on.

                                <br />

                            <div style="width:100%; overflow: hidden">


        <?php if (magicPageCompatabilityModeEnabled()) {
            ?>
                                    <span style="float:left; margin-top:5px;" data-mode="0"
                                          class="button button-secondary compatabiltyMode ">
                                        Turn Off
                                    </span>

                                    <span style="float:left; margin-top:5px;" data-mode="1"
                                          class="compatabiltyMode button button-secondary hidden">
                                        Turn On
                                    </span>

        <?php } else {
            ?>
                                    <span style="float:left; margin-top:5px;" data-mode="0"
                                          class="button button-secondary compatabiltyMode hidden ">
                                        Turn Off
                                    </span>

                                    <span style="float:left; margin-top:5px;" data-mode="1"
                                          class="compatabiltyMode button button-secondary">
                                        Turn On
                                    </span>

            <?php
        }
        ?>


                            </div>

                            <div class="clear"></div>
                            </p>

                            <!--
                                                        <h3>Reset License</h3>
                                                        <p>



                                                            <span style="min-width: 57px;   float: left;  margin-right: 10px;">
                                                                <svg fill="#0073b2" version="1.1"xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px"
                                                                     viewBox="13 -10.6 46.7 41.6" enable-background="new 13 -10.6 46.7 41.6" xml:space="preserve">
                                                                <path d="M35,30.9c-7.2-0.7-13.7-5.2-17-11.5c-0.6-1.2-1.7-4-1.7-4.3c0-0.1,1.4-0.1,3.1-0.1h3.1l0.6,1.4C25,20.8,29,24.3,34,25.5
                                                                      c1.8,0.5,5.9,0.5,7.8,0c1.9-0.5,4.5-1.7,6-2.8c2.4-1.9,4.7-5.2,5.6-8.3c0.5-1.9,0.6-5.1,0.3-7.1c-0.6-3.3-2-5.7-4.5-8.2
                                                                      c-3.1-3-6.3-4.3-10.5-4.6c-2-0.1-2.8,0-4.4,0.4C29.3-3.8,24.8-0.1,23,4.5l-0.4,1h-6.3l0.1-0.6c0.3-1.2,1.7-4.1,2.6-5.6
                                                                      c1.3-2,4-4.7,5.9-6.1c12.2-8.2,28.9-3,33.7,10.4C63,16.1,54,29.3,40,30.8C37.7,31.2,37.3,31.2,35,30.9z"/>
                                                                <path d="M13.3,14.2h12.5v2.5H13.3V14.2z M13,4.1l13,0.2l-6.7,10L13,4.1L13,4.1z"/>
                                                                </svg>
                                                            </span>


                                                            License data is per domain basis (each domain uses unique license data).
                                                            If you use a migration plugin use this feature to reset your license data after migration. This will help you avoid getting white screens or unlicensed domain errors.
                                                            <br />
                                                            <span id="resetLicenseData" style="float:left; margin-top: 10px;" class="button button-secondary">Reset License</span>
                                                        </p>
                            -->


                        </div>
                        <div class="clear"></div>
                        <p class="submit"><input class="button button-primary" type="submit" name="savechanges" value="Save Settings" /></p>
                    </form>
                </div>
            </div>
        </div>


        <?php
    }

    public function admin_files() {
        global $pagenow;

        $pages = magic_page_get_page_req();
        if (($pagenow == 'post-new.php' or $pagenow == 'post.php') && shouldLoadCheck()) {
            if (!wp_script_is($pages[0], $pages[1])) {
                exit();
            }
        }
    }

    public function maybe_enable_permalinks() {
        global $wp_rewrite;

        if (get_option('permalink_structure')) {
            return true;
        }

        $wp_rewrite->set_permalink_structure('/%postname%/');
        $wp_rewrite->flush_rules(true);

        $test_url = '';

        $posts = get_posts(array(
            'posts_per_page' => 1,
            'post_type' => array('post', 'page'),
            'post_status' => 'publish'
        ));

        if ($posts) {
            $test_url = get_permalink($posts[0]->ID);
        }

        $response = wp_remote_get($test_url, array('timeout' => 5));
        $x_pingback_header = wp_remote_retrieve_header($response, 'x-pingback');
        $pretty_permalinks = $x_pingback_header && $x_pingback_header === get_bloginfo('pingback_url');

        if ($pretty_permalinks) {
            return true;
        }

        return false;
    }

    public function delete_spintax_cache() {
        if (current_user_can('administrator')) {
            global $wpdb;

            $wpdb->query("DELETE FROM $wpdb->postmeta WHERE meta_key = '_magic_page_spintax_cache'");

            $wpdb->query("DELETE FROM $wpdb->options WHERE option_name LIKE '%_magic_page_spintax_cache_%'");

            wp_send_json_success();
        }
    }

    public function magicpage_api_key_hidden() {
        if (current_user_can('administrator')) {

            if (magicpage_hidden_by_check()) {
                $action = $_POST['keydata'];
                update_option('_magicpage_api_key_hidden_by', get_current_user_id());

                update_option('_magicpage_api_key_hidden', $action);
            }

            wp_send_json_success();
        }
    }

    public function hidden_unhide() {
        if (current_user_can('administrator')) {
            delete_option('_magicpage_api_key_hidden');
            delete_option('_magicpage_api_key_hidden_by');

            wp_send_json_success();
        }

        wp_send_json_error();
    }

    public function magicpage_reset_license_data() {
        if (current_user_can('administrator')) {
            delete_option('_magicpage_d_l');
            wp_send_json_success();
        }
    }

    public function magicpage_compatability_mode() {
        if (current_user_can('administrator')) {
            update_option('_magicpage_compatability_mode', $_POST['mode']);
            wp_send_json_success(array('status' => magicPageCompatabilityModeEnabled()));
        }
        wp_send_json_error();
    }

    function enable_disable_editor_custom_field() {
        if (current_user_can('administrator')) {
            $state = intval($_POST['state']);
            $id = sanitize_text_field($_POST['id']);

            update_option('_magic_page_editor_' . $id, $state);
            wp_send_json_success();
        }
    }

    function get_themes() {
        if (current_user_can('administrator')) {

            delete_transient('_magic_page_themes_data');

            $data = get_transient('_magic_page_themes_data');

            if (!$data) {
                global $magicPageApi;

                $response = $magicPageApi->themes_get_list();

                if ($response['status'] === 'success') {
                    $data = $response['data'];
                    set_transient('_magic_page_themes_data', $data, MINUTE_IN_SECONDS * 15);
                }
            }

            $prefix = $data['url'] . $data['image_folder'];

            $count = 0;
            $html = '';
            if ($data['themes']) {
                $count = count($data['themes']);
                foreach ($data['themes'] as $theme) {
                    $html .= $this->itemTemplate($prefix, $theme);
                }
            }

            $response = array('html' => $html, 'count' => $count);

            wp_send_json_success($response);
        }
    }

    function download_theme() {


        if (current_user_can('administrator')) {
            $id = intval($_POST['id']);

            global $magicPageApi;

            $response = $magicPageApi->theme_download_url($id);

            if ($response) {
                wp_send_json_success($response);
            }
        }
    }

    public function itemTemplate($prefix, $theme) {
        ob_start();
        ?>
        <div class="theme">
            <div class="theme-screenshot">
                <img data-src="<?php echo($prefix . $theme['image']) ?>" alt="">
            </div>
            <div class="theme-id-container">
                <h2 class="theme-name">
        <?php echo($theme['title']) ?>
                </h2>
                <div class="theme-actions">
                    <span class="download-mpp-theme-button button button-primary customize load-customize hide-if-no-customize" data-id="<?php echo($theme['id']) ?>">Download</span>
                </div>
            </div>
        </div><?php
        $output = ob_get_contents();
        ob_end_clean();
        return $output;
    }

}

new MagicPageAdminSettings();
