<?php

if (!defined('ABSPATH')) {
    exit;
}

class MagicPageSitemap {

    private $key = '';
    private $content = '';
    private $post = '';

    public function __construct() {

        $this->key = sanitize_title(magic_page_plugin_post_name());

        add_action('init', array($this, 'register_sitemap'), 1);
        add_action('init', array($this, 'init_sitemap'), 1);

        add_filter('wpseo_sitemap_index', array($this, 'add_sitemap_custom_items'), 1);
        add_filter('wpseo_sitemap_exclude_post_type', array($this, 'exclude_post_type'), 10, 2);
        add_filter('wpseo_sitemap_exclude_taxonomy', array($this, 'exclude_taxonomy'), 1, 2);
    }

    function generate_sitemap() {
        global $wpseo_sitemaps;

        $this->post = get_magic_page();
        if ($this->post) {
            $locations = wp_get_post_terms($this->post->ID, 'location');

            $query = new MagicPageQuery();
            $query->setLocations($locations);
            $query->setAutoinstalled(magic_page_is_autoinstalled_enabled());
            $query->setQueryType('result');
            $results = $query->getResults();

            if ($results) {
                $renderer = new WPSEO_Sitemaps_Renderer();
                foreach ($results as $result) {
                    $link = get_term_link(intval($result->term_id), 'location');

                    if (!is_object($link)) {
                        $url = array(
                            'loc' => $link,
                            'mod' => $this->getDate(),
                            'chf' => 'daily',
                            'pri' => 1
                        );
                        $this->content .= $renderer->sitemap_url($url);
                    }
                }
            }
        }

        if (empty($this->content)) {
            $wpseo_sitemaps->bad_sitemap = true;
            return;
        }

        $sitemap = '<urlset xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" ';
        $sitemap .= 'xsi:schemaLocation="http://www.sitemaps.org/schemas/sitemap/0.9 http://www.sitemaps.org/schemas/sitemap/0.9/sitemap.xsd" ';
        $sitemap .= 'xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">' . "\n";
        $sitemap .= $this->content . '</urlset>';

        $wpseo_sitemaps->set_sitemap($sitemap);
    }

    function register_sitemap() {
        global $wpseo_sitemaps;

        if ($wpseo_sitemaps) {
            $wpseo_sitemaps->register_sitemap($this->key, array($this, 'generate_sitemap'));
        }
    }

    public function getDate() {

        if ($this->post) {
            $timezone = new WPSEO_Date_Helper();
            return $timezone->format($this->post->post_modified_gmt);
        }
    }

    function add_sitemap_custom_items() {
        $string = '';
        $post = get_magic_page();
        $locations = wp_get_post_terms($post->ID, 'location');

        if ($post && $locations) {
            $string .= '<sitemap>' . "\n";
            $string .= '<loc>' . site_url() . '/' . $this->key . '-sitemap.xml</loc>' . "\n";
            $string .= '<lastmod>' . htmlspecialchars($this->getDate()) . '</lastmod>' . "\n";
            $string .= '</sitemap>' . "\n";
        }

        return $string;
    }

    function init_sitemap() {
        add_action("wpseo_do_sitemap_" . $this->key, array($this, 'generate_sitemap'));
    }

    function exclude_post_type($value, $post_type) {
        $post_type_to_exclude = array('magicpage', 'exporttemplate');
        if (in_array($post_type, $post_type_to_exclude))
            return true;
    }

    public function exclude_taxonomy($unused, $taxonomy) {
        if ($taxonomy == 'location') {
            return true;
        }
    }

}

new MagicPageSitemap();


