<?php
if (!defined('ABSPATH')) {
    exit;
}

class MagicPageSpintaxMetabox {

    public function __construct() {

        if (magic_page_get_spintax_enabled('mp')) {
            add_action('admin_init', array($this, 'setup_meta_boxes'));
            add_action('save_post', array($this, 'save_metabox'));
        }
    }

    public function setup_meta_boxes() {
        if (current_user_can('administrator') || current_user_can('editor')) {
            add_meta_box('spintax_box', 'Magic Page Spintax Shortcodes', array($this, 'box'), array(magic_page_default_post_slug(), 'post', 'page'), 'normal', 'low');
        }
    }

    public function box() {
        $spintax_expressions = get_option('_magic_page_spintax_expressions', array());
        ?>

        <div id="magic-page-spintax-table">
            <?php
            if (!empty($spintax_expressions)) {
                foreach ($spintax_expressions as $id => $spintaxData) {
                    ?>

                    <div id="<?php echo($id); ?>" data-cancel="true" class="viewmode spintaxItem">
                        <div class="spintaxValueLabel">
                            Label <br>
                                <div class="input-group ">
                                    <input id="spintax_<?php echo $id; ?>" class="spintaxLabel form-control input-monospace input-sm js-zeroclipboard-target js-url-field" type="text" value="{spintax_<?php echo $spintaxData['label']; ?>}" readonly="readonly" name="spintax_expressions[<?php echo $id ?>][label]">
                                        <span class="input-group-button">
                                            <div class="input-group-button">
                                                <button data-clipboard-target="#spintax_<?php echo $id; ?>"  aria-label="Copy to clipboard" class="js-zeroclipboard btn btn-sm zeroclipboard-button tooltipped tooltipped-s" data-copied-hint="Copied!" type="button"><svg aria-hidden="true" class="octicon octicon-clippy" height="16" version="1.1" viewBox="0 0 14 16" width="14"><path fill-rule="evenodd" d="M2 13h4v1H2v-1zm5-6H2v1h5V7zm2 3V8l-3 3 3 3v-2h5v-2H9zM4.5 9H2v1h2.5V9zM2 12h2.5v-1H2v1zm9 1h1v2c-.02.28-.11.52-.3.7-.19.18-.42.28-.7.3H1c-.55 0-1-.45-1-1V4c0-.55.45-1 1-1h3c0-1.11.89-2 2-2 1.11 0 2 .89 2 2h3c.55 0 1 .45 1 1v5h-1V6H1v9h10v-2zM2 5h8c0-.55-.45-1-1-1H8c-.55 0-1-.45-1-1s-.45-1-1-1-1 .45-1 1-.45 1-1 1H3c-.55 0-1 .45-1 1z"></path></svg></button>
                                            </div>
                                        </span>
                                </div>
                        </div>

                        <div class="spintaxValueActions">
                            Actions <br />
                            <span class="actions">
                                <input type="button" class="button button-primary button-large saveSpintax" value="Save" />
                                <input type="button" class="button button-secondary button-large cancelSpintax" value="Cancel" />
                                <input type="button" class="button button-secondary button-large editSpintax" value="Edit" />
                                <input type="button" class="button button-secondary button-large deleteSpintax" value="Delete" />
                            </span>
                        </div>

                        <div class="customFieldValues">
                            <?php
                            if ($spintaxData) {


                                if (isset($spintaxData['options']) and $spintaxData['options']) {
                                    foreach ($spintaxData['options'] as $uid => $option) {
                                        if ($this->isHTML($option)) {
                                            $dataAttr = ' data-editor="true" ';
                                        } else {
                                            $dataAttr = ' data-editor="false" ';
                                        }
                                        $name = 'spintax_expressions[' . $id . '][options][' . $uid . ']';
                                        ?>
                                        <div data-uid="<?php echo($uid); ?>" <?php echo($dataAttr); ?> class="customRow">

                                            <div class="equalFields">
                                                Option
                                                <?php
                                                if ($this->isHTML($option)) {

                                                    $settings = array(
                                                        'quicktags' => true,
                                                        'wpautop' => false,
                                                        'editor_class' => 'spintaxValue',
                                                        'textarea_name' => $name
                                                    );

                                                    if (strpos($option, '<iframe') !== false) {

                                                        $settings['tinymce'] = false;
                                                        $settings['editor_class'] = 'spintaxValue iframeEditor';
                                                    }





                                                    wp_editor($option, 'spintax-editor-' . $id . '-' . $uid, $settings);
                                                } else {
                                                    ?><input id="<?php echo($uid); ?>" name="<?php echo($name); ?>" class="spintaxValue" value="<?php echo(htmlspecialchars($option)); ?>" type="text" /><?php
                                                }
                                                ?>
                                            </div>
                                            <div class="spintaxValueActionsSmall" style="display:none">

                                                <span title="Delete Item"  class="deleteItemSpintax"><svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px"
                                                                                                          viewBox="-250 252 48 48" style="enable-background:new -250 252 48 48;" xml:space="preserve">
                                                        <path d="M-209,300h-34v-41h34V300z M-241,298h30v-37h-30V298z"/>
                                                        <path d="M-215,261h-22v-8h22V261z M-235,259h18v-4h-18V259z M-234,293c-0.6,0-1-0.4-1-1v-25c0-0.6,0.4-1,1-1c0.6,0,1,0.4,1,1v25
                                                              C-233,292.6-233.4,293-234,293z M-226,293c-0.6,0-1-0.4-1-1v-25c0-0.6,0.4-1,1-1s1,0.4,1,1v25C-225,292.6-225.4,293-226,293z
                                                              M-218,293c-0.6,0-1-0.4-1-1v-25c0-0.6,0.4-1,1-1s1,0.4,1,1v25C-217,292.6-217.4,293-218,293z"/>
                                                        <path d="M-250,259h48v2h-48V259z"/>
                                                    </svg></span>
                                                <span title="Switch Editor Mode" class="switchEditor">
                                                    <svg xmlns="http://www.w3.org/2000/svg" viewBox="-262 264 24 24">
                                                        <path d="M-244.1 266.2c.7-.7 1.4 0 1.4 0l3.5 3.5s.7.7 0 1.4L-254 286l-6 1.1 1.1-6 14.8-14.9zm-2.2 3.5l3.5 3.5-11.2 11.4-3.5-3.5 11.2-11.4zm-11.8 12.2l-.7 3.9 3.9-.7-3.2-3.2zm14.7-15l3.5 3.5-2.1 2.1-3.5-3.5 2.1-2.1z"/>
                                                    </svg>
                                                </span>
                                            </div>
                                        </div>
                            <?php
                        }
                    }
                    ?>
                                <div style="display:none" class="optional spintaxCustomValueSample">

                                    <div class="equalFields">
                                        Option
                                        <input type="text" class="spintaxValue" value=''  />
                                    </div>
                                    <div class="spintaxValueActionsSmall" style="display:none">

                                        <span title="Delete Item"   class="deleteItemSpintax"><svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px"
                                                                                                   viewBox="-250 252 48 48" style="enable-background:new -250 252 48 48;" xml:space="preserve">
                                                <path d="M-209,300h-34v-41h34V300z M-241,298h30v-37h-30V298z"/>
                                                <path d="M-215,261h-22v-8h22V261z M-235,259h18v-4h-18V259z M-234,293c-0.6,0-1-0.4-1-1v-25c0-0.6,0.4-1,1-1c0.6,0,1,0.4,1,1v25
                                                      C-233,292.6-233.4,293-234,293z M-226,293c-0.6,0-1-0.4-1-1v-25c0-0.6,0.4-1,1-1s1,0.4,1,1v25C-225,292.6-225.4,293-226,293z
                                                      M-218,293c-0.6,0-1-0.4-1-1v-25c0-0.6,0.4-1,1-1s1,0.4,1,1v25C-217,292.6-217.4,293-218,293z"/>
                                                <path d="M-250,259h48v2h-48V259z"/>
                                            </svg></span>

                                        <span title="Switch Editor Mode" class="switchEditor"> <svg xmlns="http://www.w3.org/2000/svg" viewBox="-262 264 24 24">
                                                <path d="M-244.1 266.2c.7-.7 1.4 0 1.4 0l3.5 3.5s.7.7 0 1.4L-254 286l-6 1.1 1.1-6 14.8-14.9zm-2.2 3.5l3.5 3.5-11.2 11.4-3.5-3.5 11.2-11.4zm-11.8 12.2l-.7 3.9 3.9-.7-3.2-3.2zm14.7-15l3.5 3.5-2.1 2.1-3.5-3.5 2.1-2.1z"/>
                                            </svg>
                                        </span> 

                                    </div>
                                </div>

                                <span class="addCustomValueSpintax"> Add New Option</span>
                            </div>
                        </div>
                    <?php
                }
            }
        }
        ?>


            <!-- sample -->
            <div style="display:none" id="magic-page-spintax-form-sample">
                <div class="spintaxValueLabel">
                    Label <br />
                    <div class="input-group ">
                        <input id="" class="spintaxLabel form-control input-monospace input-sm js-zeroclipboard-target js-url-field" id="shortcode_location" type="text" value="" />
                        <span class="input-group-button">
                            <div class="input-group-button">
                                <button data-clipboard-target="" aria-label="Copy to clipboard" class="js-zeroclipboard btn btn-sm zeroclipboard-button tooltipped tooltipped-s" data-copied-hint="Copied!" type="button"><svg aria-hidden="true" class="octicon octicon-clippy" height="16" version="1.1" viewBox="0 0 14 16" width="14"><path fill-rule="evenodd" d="M2 13h4v1H2v-1zm5-6H2v1h5V7zm2 3V8l-3 3 3 3v-2h5v-2H9zM4.5 9H2v1h2.5V9zM2 12h2.5v-1H2v1zm9 1h1v2c-.02.28-.11.52-.3.7-.19.18-.42.28-.7.3H1c-.55 0-1-.45-1-1V4c0-.55.45-1 1-1h3c0-1.11.89-2 2-2 1.11 0 2 .89 2 2h3c.55 0 1 .45 1 1v5h-1V6H1v9h10v-2zM2 5h8c0-.55-.45-1-1-1H8c-.55 0-1-.45-1-1s-.45-1-1-1-1 .45-1 1-.45 1-1 1H3c-.55 0-1 .45-1 1z"></path></svg></button>
                            </div>
                        </span>
                    </div>
                </div>

                <div class="spintaxValueActions">
                    <br />
                    <span class="actions">
                        <input type="button" class="button button-primary button-large saveSpintax" value="Save" />
                        <input type="button" class="button button-secondary button-large cancelSpintax" value="Cancel" />
                        <input type="button" class="button button-secondary button-large editSpintax" value="Edit" />
                        <input type="button" class="button button-secondary button-large deleteSpintax" value="Delete" />
                    </span>
                </div>
                <div class="customFieldValues">
                    <div style="display:none" class="optional spintaxCustomValueSample">

                        <div class="equalFields">
                            Option
                            <input type="text" class="spintaxValue" />
                        </div>
                        <div class="spintaxValueActionsSmall" style="display:none">
                            <span title="Delete Item"   class="deleteItemSpintax"><svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px"
                                                                                       viewBox="-250 252 48 48" style="enable-background:new -250 252 48 48;" xml:space="preserve">
                                    <path d="M-209,300h-34v-41h34V300z M-241,298h30v-37h-30V298z"/>
                                    <path d="M-215,261h-22v-8h22V261z M-235,259h18v-4h-18V259z M-234,293c-0.6,0-1-0.4-1-1v-25c0-0.6,0.4-1,1-1c0.6,0,1,0.4,1,1v25
                                          C-233,292.6-233.4,293-234,293z M-226,293c-0.6,0-1-0.4-1-1v-25c0-0.6,0.4-1,1-1s1,0.4,1,1v25C-225,292.6-225.4,293-226,293z
                                          M-218,293c-0.6,0-1-0.4-1-1v-25c0-0.6,0.4-1,1-1s1,0.4,1,1v25C-217,292.6-217.4,293-218,293z"/>
                                    <path d="M-250,259h48v2h-48V259z"/>
                                </svg></span>
                            <span title="Switch Editor Mode" class="switchEditor"> <svg xmlns="http://www.w3.org/2000/svg" viewBox="-262 264 24 24">
                                    <path d="M-244.1 266.2c.7-.7 1.4 0 1.4 0l3.5 3.5s.7.7 0 1.4L-254 286l-6 1.1 1.1-6 14.8-14.9zm-2.2 3.5l3.5 3.5-11.2 11.4-3.5-3.5 11.2-11.4zm-11.8 12.2l-.7 3.9 3.9-.7-3.2-3.2zm14.7-15l3.5 3.5-2.1 2.1-3.5-3.5 2.1-2.1z"/>
                                </svg></span>
                        </div>
                    </div>
                    <span class="addCustomValueSpintax"> Add New Option</span>
                </div>
            </div>
            <!-- sample -->
        </div>

        <br />

        <input type="button" class="button button-primary button-large addSpintax" value="New Spintax Shortcode" />

        <input type="hidden" value="1" name="magic_page_spintax" />
        <?php
    }

    function save_metabox() {
        if (!current_user_can('administrator')) {
            return false;
        }

        $spintax_expressions = array();

        if (isset($_POST['magic_page_spintax']) && current_user_can('administrator')) {
            $original_spintax_expressions = get_option('_magic_page_spintax_expressions', array());

            if (!empty($_POST['spintax_expressions'])) {
                foreach ($_POST['spintax_expressions'] as $id => $data) {
                    $label = str_replace('{spintax_', '', $data['label']);
                    $label = trim(str_replace('}', '', $label));

                    $spintax_expressions[$id]['label'] = $label;

                    if (isset($data['options']) and $data['options']) {
                        foreach ($data['options'] as $optionId => $option) {
                            $spintax_expressions[$id]['options'][$optionId] = trim(cleanup_quotes_slashes($option));
                        }
                    }
                }
            }

            update_option('_magic_page_spintax_expressions', $spintax_expressions, 'no');

            // clear cache @todo
            //$this->deleteUnusedExpressionHashes($original_spintax_expressions, $spintax_expressions);
//            $spintax = new MagicPageSpintax();
//            $spintax->deleteUnusedHashCache($post);
        }
    }

    public function deleteUnusedExpressionHashes($original_spintax_expressions, $spintax_expressions) {
        $deletedKeys = array();

        $originalKeys = array_keys($original_spintax_expressions);
        foreach ($originalKeys as $originalKey) {
            if (!isset($spintax_expressions[$originalKey])) {
                $deletedKeys[] = $originalKey;
            }
        }

        if ($deletedKeys) {
            global $wpdb;

            $results = $wpdb->get_results("SELECT * FROM $wpdb->options WHERE option_name LIKE '%_magic_page_spintax_cache%'");

            foreach ($results as $result) {
                $values = unserialize($result->option_value);

                foreach ($deletedKeys as $deletedKey) {
                    if (isset($values[$deletedKey])) {
                        unset($values[$deletedKey]);
                    }
                }

                if ($values) {
                    $wpdb->update(
                            $wpdb->options, array('option_value' => serialize($values)), array('option_id' => $result->option_id), array('%s'), array('%d')
                    );
                } else {
                    $wpdb->delete($wpdb->options, array('option_id' => $result->option_id));
                }
            }
        }
    }

    function isHTML($string) {
        return preg_match("/\/[a-z]*>/i", $string) != 0;
    }

}

new MagicPageSpintaxMetabox();
