<?php

class MagicPageUpgrade {

    public function __construct() {

        if (!is_admin() || !is_magic_page_installed()) {
            return false;
        }

        add_action('admin_notices', array($this, 'admin_notices'));

        $this->mpUpgrade59Notices();

        add_action('admin_init', array($this, 'versionUpgrade'));

        global $magicPageApi;

        if ($magicPageApi) {
            Puc_v4_Factory::buildUpdateChecker(
                    $magicPageApi->get_plugin_update(), MAGIC_PAGE_FILE, 'magic-page'
            );

            add_filter('puc_request_update_result-magic-page', array($this, 'filterUpdateResult'));
            add_filter('puc_request_info_result-magic-page', array($this, 'resultFilter'));
        }
    }

    public function admin_notices() {

        $duplicate = get_option('_magic_page_admin_notice_duplicate_mpp_bug', false);

        if ($duplicate) {
            $termId = checkDupicateMagicPageAndLocationSlug(); //refresh again

            if ($termId) {
                $term = get_term_by('term_id', $termId, 'location');

                if ($term) {
                    $termUrl = get_edit_term_link($term);
                    $termName = $term->name;
                    $slug = $term->slug;

                    $text = '<div class="notice minify-error-notice notice-error"><p> Magic Page Notice: You have location with the same slug as the magic page itself (%s). '
                            . 'This will likely cause problems it is recomended that you change either the location slug <a target="_blank" href="%s">%s</a> or your magic page permalink.</p></div>';

                    echo sprintf($text, $slug, $termUrl, $termName);
                }
            }
        }




        if (defined('WPSCHEMA_VERSION') && version_compare(WPSCHEMA_VERSION, '1.05', '<=')) {
            $shown = get_option('_magic_page_admin_notice_wpschema_upgrade', 0);
            if ($shown < 5) {
                echo('<div class="notice minify-error-notice notice-error"><p> Magic Page Notice: Please upgrade your WP Schema Plugin to the latest version. Your currently installed WP Schema Plugin version is outdated and does not fully support the latest features.</p></div>');

                $shown++;
                update_option('_magic_page_admin_notice_wpschema_upgrade', $shown);
            }
        }
    }

    public function filterUpdateResult($update) {
        $update->download_url = add_query_arg(array('api_key' => get_option('_magicpage_api_key')), $update->download_url);
        return $update;
    }

    public function resultFilter($res) {

        if ($res && isset($res->status)) {

            if (isset($res->error_data)) {
                update_option('_' . getMpPluginSlug() . '_d_l', $res->error_data);
            }

            if ($res->status === 'error') {
                update_option('_magic_page_key_error', $res->error_note);
            } else {
                delete_option('_magic_page_key_error');
            }
        }

        return $res;
    }

    public function versionUpgrade() {
        $currentVersion = get_option('_magicpage_version');

        if (!$currentVersion or $currentVersion !== MAGIC_PAGE_VERSION) {
            update_option('_magicpage_version', MAGIC_PAGE_VERSION);
            if ($currentVersion == '5.7' && MAGIC_PAGE_VERSION == '5.8') {
                $this->performJvzooPksUpgrade();
            }
        }
    }

    protected function mpUpgrade59Notices() {
        $notice1Id = '_magic_page_spintax_support_auto_enabled_notice';
        $notice2Id = '_magic_page_seo_for_all_pages_enabled_notice';

        if (get_option('_magic_page_spintax_support') != '1' && get_option($notice1Id) != 'disabled') {
            update_option('_magic_page_spintax_support', '1');

            $args = array(
                'notice' => 'Auto enabled Spintax support for Posts & Pages. If you like you can turn it off from Settings page',
                'id' => $notice1Id,
                'value' => 'disabled');
            add_action('admin_notices', function () use ($args) {
                mpp_admin_notice($args);
            });
        }


        if (get_option('_magic_page_seo_for_all_pages') != '1' && get_option($notice2Id) != 'disabled') {
            update_option('_magic_page_seo_for_all_pages', '1');

            $args = array(
                'notice' => 'Auto Enabled SEO Settings for Posts & Pages. If you like you can turn it off from Settings page',
                'id' => $notice2Id,
                'value' => 'disabled');
            add_action('admin_notices', function () use ($args) {
                mpp_admin_notice($args);
            });
        }
    }

    protected function performJvzooPksUpgrade() {
        $magicPageShortIdExisting = get_magicpage_shortid();

        if ($magicPageShortIdExisting) {
            update_option('_magicpage_old_short_id', $magicPageShortIdExisting);
        }

        delete_option('magic_page_installation_items_per_page');
        delete_option('magic_page_install_data_new');
        delete_option('magic_page_installation_temp');

        $pksKey = get_option('_magicpage_pks_verified_key');

        delete_option('_magicpage_pks_verified_key');
        update_option('_magicpage_api_key', $pksKey);

        delete_option('_magicpage_d_l');
        delete_option('external_updates-magic-page');
        delete_option('_magic_page_api_licensed_databases_list');
        delete_option('_magic_page_key_error');
    }

}

new MagicPageUpgrade();

