<?php

if (!defined('ABSPATH')) {
    exit;
}

class MagicPageApi {

    protected $url = 'https://mpapi.co.uk/api/';

    public function __construct() {
        add_action('wp_ajax_magic_page_verify_api_key', array($this, 'verify_api_key'));
        add_action('wp_ajax_magicpage_update_license_data', array($this, 'update_license_data'));
    }

    /**
     * verify api key
     */
    public function verify_api_key() {
        if (current_user_can('administrator')) {
            $_key = isset($_POST['_magicpage_api_key']) ? $_POST['_magicpage_api_key'] : get_option('_magicpage_api_key');

            $params = $this->post_params(array('api_key' => $_key));
            $_response = wp_remote_post($this->get_api_post_url('validate_api_key'), $params);

            $response = $this->getApiResponse($_response);

            wp_send_json_success($response);
        }

        wp_send_json_error();
    }

    /**
     * get licensed databases list
     */
    public function get_licensed_databases_list($key = null) {
        if (isset($_POST['key']) && !$key) {
            $key = $_POST['key'];
        }

        $array = array(
            'api_key' => $key,
            'filter' => serialize(get_magic_page_region_filter())
        );

        $params = $this->post_params($array);
        $response = wp_remote_post($this->get_api_post_url('get_licensed_databases_list'), $params);

        return $this->getApiResponse($response);
    }

    /**
     *  browse database
     */
    public function browse_database($region, $country, $count_per_page = 100, $page = 1, $version_min = false, $version_max = false) {
        $array = array(
            'api_key' => get_option('_magicpage_api_key'),
            'region' => $region,
            'country' => $country,
            'count_per_page' => $count_per_page,
            'page' => $page,
            'filter' => serialize(get_magic_page_region_filter())
        );

        if ($version_min) {
            $array['version_min'] = $version_min;
        }

        if ($version_max) {
            $array['version_max'] = $version_max;
        }

        $params = $this->post_params($array);
        $response = wp_remote_post($this->get_api_post_url('browse_database'), $params);

        return $this->getApiResponse($response);
    }

    /**
     * get database updates
     */
    public function get_database_updates($regions) {
        $array = array(
            'api_key' => get_option('_magicpage_api_key'),
            'regions' => $regions,
        );

        $params = $this->post_params($array);
        $response = wp_remote_post($this->get_api_post_url('database_updates'), $params);

        return $this->getApiResponse($response);
    }

    /** update license data - mp ajax */
    public function update_license_data() {
        if (current_user_can('administrator')) {
            //check_ajax_referer('schematic-nonce');

            $params = $this->post_params(array('api_key' => get_option('_magicpage_api_key')));
            $_response = wp_remote_post($this->get_api_post_url('update_license_data'), $params);

            $response = $this->getApiResponse($_response);

            if (isset($response['key_data'])) {
                $response['key'] = unserialize($response['key_data'])['key'];
                unset($response['key_data']);

                wp_send_json_success($response);
            }
        }

        wp_send_json_error();
    }

    /** re verify api key - settings * */
    public function re_verify_api_key() {
        if (current_user_can('administrator')) {

            $_key = isset($_POST['_magicpage_api_key']) ? $_POST['_magicpage_api_key'] : get_option('_magicpage_api_key');

            $params = $this->post_params(array('api_key' => $_key));
            $response = wp_remote_post($this->get_api_post_url('validate_api_key'), $params);


            if ($response && isset($response['body'])) {
                return json_decode($response['body'], true);
            }
        }
    }

    /** get plugin update -$_GET * */
    public function get_plugin_update() {
        return $this->get_api_url('get_latest_plugin_version', array(
                    'api_key' => get_option('_magicpage_api_key'),
        ));
    }

    /** minify footprint * */
    public function minify_footprint($newplugin_folder) {

        $settings = get_magic_page_googlemap_parameters();

        $settings['url'] = MAGIC_PAGE_URL . basename(MAGIC_PAGE_FILE);
        $settings['oldfolder'] = basename(MAGIC_PAGE_DIR);
        $settings['newfolder'] = $newplugin_folder;

        unset($settings['js_fullurl']);
        unset($settings['js_fullfolder']);

        $array = array(
            'api_key' => get_option('_magicpage_api_key'),
            'settings' => $settings,
        );

        $params = $this->post_params($array);
        $response = wp_remote_post($this->get_api_post_url('minify_footprint'), $params);

        return $this->getApiResponse($response);
    }

    /** generic * */
    public function getUrl() {

        if (defined('MAGIC_PAGE_API_URL') && strpos(MAGIC_PAGE_API_URL, 'mpapi.co.uk') !== false) {
            return MAGIC_PAGE_API_URL;
        }

        return $this->url;
    }

    public function get_api_url($action, $array) {
        MAGIC_PAGE_FILE;

        $version = get_option('_magicpage_version', MAGIC_PAGE_VERSION);
        $license = @unserialize(get_option('_magicpage_d_l'));
        $licenseId = isset($license['key']) ? $license['key'] : '';


        $params = array(
            'license_id' => $licenseId,
            'domain' => get_site_url(),
            'version' => $version,
            'name' => 'Magic Page Plugin',
            'path' => MAGIC_PAGE_URL . basename(MAGIC_PAGE_FILE),
        );

        if (is_magic_page_footprint_minifed()) {
            $params['plugin_folder'] = get_option('_magic_page_plugin_folder');
        }

        $arr = array_merge($array, $params);
        $url = add_query_arg($arr, $this->getUrl() . $action);

        return $url;
    }

    public function post_params($array) {

        $version = get_option('_magicpage_version', MAGIC_PAGE_VERSION);
        $dl = get_option('_magicpage_d_l');

        if (!is_array($dl)) {
            $license = unserialize(get_option('_magicpage_d_l'));
        }

        $licenseId = isset($license['key']) ? $license['key'] : '';

        $params = array(
            'version' => $version,
            'domain' => get_site_url(),
            'license_id' => $licenseId,
            'version' => get_option('_magicpage_version'),
            'name' => get_magic_page_name(),
            'path' => MAGIC_PAGE_URL . basename(MAGIC_PAGE_FILE));

        if (is_magic_page_footprint_minifed()) {
            $params['plugin_folder'] = get_option('_magic_page_plugin_folder');
        }

        return array(
            'method' => 'POST',
            'timeout' => 30,
            'body' => array_merge($array, $params)
        );
    }

    public function get_api_post_url($action) {
        return $this->getUrl() . $action;
    }

    /* Schema  */

    public function schema_api_login($data) {
        $array = array(
            'data' => $data,
            'api_key' => get_option('_magicpage_api_key'),
        );

        $params = $this->post_params($array);
        $response = wp_remote_post($this->get_api_post_url('schema/api-login'), $params);

        return $this->getApiResponse($response);
    }

    public function schema_api_register($data) {
        $array = array(
            'data' => $data,
            'api_key' => get_option('_magicpage_api_key'),
        );

        $params = $this->post_params($array);
        $response = wp_remote_post($this->get_api_post_url('schema/api-register'), $params);

        return $this->getApiResponse($response);
    }

    public function schema_api_save_profile($data) {
        $array = array(
            'data' => $data,
            'api_key' => get_option('_magicpage_api_key'),
        );

        $params = $this->post_params($array);
        $response = wp_remote_post($this->get_api_post_url('schema/api-save-profile'), $params);

        return $this->getApiResponse($response);
    }

    public function schema_submit($data) {
        $array = array(
            'data' => serialize($data),
            'api_key' => get_option('_magicpage_api_key'),
        );

        $params = $this->post_params($array);
        $response = wp_remote_post($this->get_api_post_url('schema/submit'), $params);

        return $this->getApiResponse($response);
    }

    public function schema_browse_database() {
        $array = array(
            'data' => serialize(get_schema_builder_options()),
            'api_key' => get_option('_magicpage_api_key'),
        );

        $params = $this->post_params($array);
        $response = wp_remote_post($this->get_api_post_url('schema/browse'), $params);

        return $this->getApiResponse($response);
    }

    public function themes_get_list() {
        $array = array(
            'api_key' => get_option('_magicpage_api_key'),
        );

        $params = $this->post_params($array);
        $response = wp_remote_post($this->get_api_post_url('themes/get-list'), $params);

        return $this->getApiResponse($response);
    }

    public function theme_download_url($id) {
        return $this->get_api_url('themes/download/' . $id, array(
                    'api_key' => get_option('_magicpage_api_key'),
        ));
    }

    public function get_offers() {
        $array = array(
            'api_key' => get_option('_magicpage_api_key'),
        );

        $params = $this->post_params($array);
        $response = wp_remote_post($this->get_api_post_url('get_offers'), $params);

        return $this->getApiResponse($response);
    }

    /** common fn * */
    protected function getApiResponse($response) {
        if ($response && is_array($response) && isset($response['body'])) {
            $response = json_decode($response['body'], true);

            if ($response && is_array($response) && isset($response['status'])) {
                if ($response['status']) {

                    if ($response['status'] === 'error') {
                        update_option('_' . getMpPluginSlug() . '_d_l', $response['error_data']);
                        update_option('_magic_page_key_error', $response['error_note']);
                    } else {
                        delete_option('_magic_page_key_error');

                        if (isset($response['key_data'])) {
                            update_option('_' . getMpPluginSlug() . '_d_l', $response['key_data']);
                        }
                    }
                }

                return $response;
            }
        }
    }

}

$magicPageApi = new MagicPageApi();
