<?php

if (!defined('ABSPATH')) {
    exit;
}

class MagicPageLocationsAutosuggest {

    public function __construct() {
        add_action('init', array($this, 'init'));
    }

    public function init() {
        if ($this->loadAutosuggestScript()) {

            add_action('wp_enqueue_scripts', array($this, 'enqueueScripts'));

            if (!defined('MAGIC_PAGE_AUTOSUGGEST_PUBLIC')) {
                $id = $this->getId();
                $settings = get_option($id);
            } else {
                $settings = ['action' => 'temp_script_action'];
            }


            if ($settings && isset($settings['action'])) {
                add_action('wp_ajax_' . $settings['action'], array($this, 'locationSearch'));
                add_action('wp_ajax_nopriv_' . $settings['action'], array($this, 'locationSearch'));
            }
        }
    }

    function locationSearch() {
        $s = trim(esc_sql($_GET['q']));

        $out = array();

        $magicpage = get_magic_page();

        $locs = wp_get_object_terms($magicpage->ID, 'location');
        $locIds = [];

        if ($locs) {
            foreach ($locs as $loc) {
                $distance = get_term_meta($loc->term_id, 'distance', true);
                $locIds[] = ['id' => $loc->term_id, 'radius' => $distance];
            }
        }

        $categories = get_option('_magicpage_as_categories', ['post', 'page', 'magicpage']);
        $excludeLocations = [];

        if (in_array('magicpage', $categories)) {
            $query1 = new MagicPageQuery();
            $query1->setLocations($locIds);
            $query1->setSearch($s);
            $query1->setAutoinstalled(magic_page_is_autoinstalled_enabled());
            $query1->setQueryType('result');
            $query1->setOrderby('name');
            $query1->setOrderbyType('asc');

            $query1->setItemsPerPage(10);
            $query1->setCurrentPage(1);
            $magicPages = $query1->getResults();

            if ($magicPages) {
                global $wp_rewrite;
                foreach ($magicPages as $magicpage) {
                    $path = str_replace('%magicpage%', $magicpage->slug, $wp_rewrite->extra_permastructs['magicpage']['struct']);
                    $url = get_home_url('', $path);

                    $excludeLocations[] = $magicpage;

                    $out[] = array(
                        'name' => $magicpage->name,
                        'url' => $url
                    );
                }
            }
        }

        if (in_array('post', $categories) || in_array('page', $categories)) {
            if (in_array('magicpage', $categories)) {
                foreach (array_keys($categories, 'magicpage') as $key) {
                    unset($categories[$key]);
                }
            }
 
            $query = new MagicPageQuery();
            $query->setFilter1(true);
            $query->setExcludeTerms($excludeLocations);
            $query->setIsStaticPage(true);
            $query->setPostType($categories);
            $query->setSearch($s);
            $query->setOrderby('name');
            $query->setOrderbyType('asc');
            $query->setCurrentPage(1);
            $query->setItemsPerPage(10);

            $results = $query->getResults();

            if ($results) {
                foreach ($results as $result) {
                    $post = get_post($result->ID);

                    $out[] = array(
                        'name' => $result->name,
                        'url' => get_permalink($post)
                    );
                }
            }
        }

        usort($out, array($this, 'compareByName'));
     
        $limit = get_option('_magicpage_as_maxresults', '5');

        wp_send_json_success(array_slice($out, 0, $limit));
    }

    public function compareByName($a, $b) {
        return strcmp($a["name"], $b["name"]);
    }

    public function getId() {
        $url = str_replace('.', '_', cleanUrl(get_site_url()));
        return '_magicpage_autosuggest_assets_' . $url;
    }

    public function enqueueScripts() {
        $fields = $this->loadAutosuggestScript();

        $preset = get_option('_magicpage_as_style_preset', 'base');
        wp_enqueue_style('jquery-ui-css', 'https://cdnjs.cloudflare.com/ajax/libs/jqueryui/1.12.1/jquery-ui.min.css', array(), MAGIC_PAGE_VERSION);
        wp_enqueue_style('jquery-ui-' . $preset, 'https://cdnjs.cloudflare.com/ajax/libs/jqueryui/1.12.1/themes/' . $preset . '/jquery-ui.min.css', array('jquery-ui-css'), MAGIC_PAGE_VERSION);

        if (!defined('MAGIC_PAGE_AUTOSUGGEST_PUBLIC')) {
            $id = $this->getId();
            if (!get_option($id)) {
                $settings = [
                    'id1' => generateRandomString(6, 10),
                    'id2' => generateRandomString(6, 10),
                    'action' => generateRandomString(4, 10) . '_' . generateRandomString(2, 5),
                    'filename' => generateRandomString(6, 12),
                ];

                global $magic_page_dir;
                $fileContents = file_get_contents($magic_page_dir . '/js/autosuggest-settings-min.js');
                $fileContents = str_replace('frontend_ajax_object', $settings['id2'], $fileContents);

                $file = WP_CONTENT_DIR . '/uploads/' . $settings['filename'] . '.js';

                file_put_contents($file, $fileContents);

                update_option($id, $settings);
            } else {
                $settings = get_option($id);
            }
            $contentUrl = content_url() . '/uploads/' . $settings['filename'] . '.js';
        } else {
            $settings = ['id1' => 'id1', 'id2' => 'frontend_ajax_object', 'action' => 'temp_script_action'];
            $contentUrl = MAGIC_PAGE_AUTOSUGGEST_PUBLIC;
        }

        wp_register_script($settings['id1'], $contentUrl, array('jquery-ui-core', 'jquery', 'jquery-ui-autocomplete'), MAGIC_PAGE_VERSION);

        wp_enqueue_script($settings['id1']);

        $array = array(
            'data' => json_encode($fields),
            'url' => admin_url('admin-ajax.php'),
            'action' => $settings['action'],
            'placeholder' => get_option('_magicpage_as_def_text', 'Search...'),
            'minLength' => get_option('_magicpage_as_chars', 2),
            'noresults' => get_option('_magicpage_as_empty_text', 'No Results Found'),
            'delay' => get_option('_magicpage_as_key_delay', 100)
        );

        if (get_option('_magicpage_as_hide_search', 'false') == 'true') {
            $array['search'] = true;
        }

        wp_localize_script($settings['id1'], $settings['id2'], $array);
    }

    function loadAutosuggestScript() {
        $loadFile = true;

        if (isset($_GET['live-picker']) && $_GET['live-picker'] === 'true') {
            if (isset($_GET['nonce']) && wp_verify_nonce($_GET['nonce'], 'autosuggest')) {
                $loadFile = false;
            }
        }

        if ($loadFile) {
            return get_option('_magicpage_as_input_id', []);
        }
    }

}

new MagicPageLocationsAutosuggest();
