<?php

if (!defined('ABSPATH')) {
    exit;
}

add_action('magic_page_cron_service', 'magic_page_cron_action');

function magic_page_cron_action() {
    if (is_magic_page_cron_support_basic()) {
        magic_page_perform_cron();
    }
}

function magic_page_perform_cron() {
    magic_page_cron_hadle_regular_posts();
    magic_page_cron_hadle_magicpage();
    magic_page_autoinstall_locations();
}

// rewrite regular page spintax
function magic_page_cron_hadle_regular_posts() {
    global $wpdb;

    if (magic_page_get_spintax_enabled('regularposts') && get_option('_magic_page_spintax_autorewrite')) {
        $lastRun = get_option('magic_page_last_cron_task_regular_pages');

        if ($lastRun) {
            $nextRun = (get_option('_magic_page_spintax_autorewrite_interval') * 24 * 60 * 60) + $lastRun;

            if ($nextRun > time()) {
                return false;
            }
        }

        $neededCount = magic_page_cron_get_regular_posts_count();

        if ($neededCount > 0) {
            $coverage = get_magic_page_autorefresh_content_coverage_posts() / 100;

            $elements = $wpdb->get_results("SELECT b.ID, a.post_id, a.meta_value
                                        FROM  $wpdb->posts b
                                        INNER JOIN $wpdb->postmeta a ON a.post_id = b.ID
                                        WHERE a.meta_key = '_magic_page_spintax_cache'
                                        ORDER BY RAND() LIMIT 0, $neededCount");

            magic_page_rewrite_spintax_cache($elements, $coverage, false);
        }
    }
}

function magic_page_cron_get_regular_posts_count() {
    global $wpdb;

    $countAllQuery = $wpdb->get_results("SELECT b.ID, a.post_id, COUNT(*) as CNT
                                         FROM  $wpdb->posts b
                                         INNER JOIN $wpdb->postmeta a ON a.post_id = b.ID
                                         WHERE a.meta_key LIKE '%_magic_page_spintax_cache%'");

    $count = $countAllQuery[0]->CNT;

    $howManyPercent = get_magic_page_autorefresh_content_amount_posts() / 100;
    $neededCount = round($howManyPercent * $count);

    update_option('magic_page_last_cron_task_regular_pages', time());

    return $neededCount;
}

function magic_page_cron_get_magicpageposts_count() {
    global $wpdb;

    $countAllQuery = $wpdb->get_results("SELECT COUNT(option_name) AS CNT FROM $wpdb->options WHERE option_name LIKE '%_magic_page_spintax_cache%'");
    $count = $countAllQuery[0]->CNT;

    $howManyPercent = get_magic_page_autorefresh_content_locations() / 100;
    $neededCount = round($howManyPercent * $count);

    update_option('magic_page_last_cron_task', time());

    return $neededCount;
}

// rewrite magic page spintax
function magic_page_cron_hadle_magicpage() {
    global $wpdb;

    if (magic_page_get_spintax_enabled('mp') && get_option('_magic_page_autorefresh_content_status')) {
        $lastRun = get_option('magic_page_last_cron_task');

        if ($lastRun) {
            $nextRun = (get_option('_magic_page_autorefresh_content_interval') * 24 * 60 * 60) + $lastRun;

            if ($nextRun > time()) {
                return false;
            }
        }

        $neededCount = magic_page_cron_get_magicpageposts_count();

        if ($neededCount > 0) {
            $coverage = get_magic_page_autorefresh_content_coverage() / 100;

            $elements = $wpdb->get_results(
                    "SELECT * FROM $wpdb->options WHERE option_name LIKE '%_magic_page_spintax_cache%' ORDER BY RAND() LIMIT 0, $neededCount"
            );

            magic_page_rewrite_spintax_cache($elements, $coverage, true);
        }
    }
}

function magic_page_get_array_elements_to_delete($entryArr, $coverage) {
    return get_unique_random_numbers_within_range(0, count($entryArr) - 1, round($coverage * (count($entryArr) - 1)));
}

function magic_page_rewrite_spintax_cache($elements, $coverage, $option) {
    if (is_array($elements) and!empty($elements)) {
        foreach ($elements as $element) {
            if ($option) {
                $entryArr = unserialize($element->option_value);
            } else {
                $entryArr = unserialize($element->meta_value);
            }

            if (is_array($entryArr) && $entryArr) {
                $arrayElementsToDelete = magic_page_get_array_elements_to_delete($entryArr, $coverage);

                if ($arrayElementsToDelete) {
                    foreach ($arrayElementsToDelete as $id => $_u) {
                        $i = 0;
                        foreach ($entryArr as $_id => $entry) {
                            if ($_u === $i) {
                                unset($entryArr[$_id]);
                            }
                            $i++;
                        }
                    }
                }

                if ($option) {
                    update_option($element->option_name, $entryArr);
                } else {
                    update_post_meta($element->ID, '_magic_page_spintax_cache', $entryArr);
                }
            }
        }
    }
}

function magic_page_autoinstall_locations() {
    if (magic_page_is_autoinstalled_enabled()) {
        $post = get_magic_page();
        if ($post) {
            $locations = wp_get_post_terms($post->ID, 'location');

            $installed = 0;

            $autodelayDays = get_option('_magic_page_autoinstall_days', '30');
            $allLocationsCount = get_all_locations_count();

            $locationsToInstall = ceil($allLocationsCount / $autodelayDays);
            $nearbyLocationsPerLocation = ceil($locationsToInstall / count($locations));
 

            foreach ($locations as $location) {
                if ($installed <= 100) {
                    $count = getInstallableNearbyLocationsCount($location);

                    if ($count > 0) {
                        $installableLocations = getInstallableNearbyLocations($location, $nearbyLocationsPerLocation);

                        foreach ($installableLocations as $locationToInstall) {
                            update_term_meta($locationToInstall->term_id, 'autoinstalled', time());

                            $installed++;
                            if ($installed >= $locationsToInstall) {
                                break;
                            }
                        }

                        if ($installed >= $locationsToInstall) {
                            break;
                        }
                    }
                }
            }

            if ($installed) {
                magic_page_delete_cache();
            }

            magic_page_update_locations_count($post);
        }
    }
}
