<?php
if (!defined('ABSPATH')) {
    exit;
}

class MagicPageFootprint {

    public function __construct() {
        add_action('wp_ajax_magicpage_footprint_minify_save_data', array($this, 'save'));
        add_action('wp_ajax_magicpage_footprint_minify_check_folder_permissions', array($this, 'check_folder_permissions'));

        add_action('admin_notices', array($this, 'admin_notice'));

        add_filter('body_class', array($this, 'classes'));
        add_filter('post_class', array($this, 'classes'));
    }

    public function admin_notice() {
        if (!is_magic_page_footprint_minifed() && is_magic_page_installed()) {
            $shown = get_option('_magic_page_admin_notice_minifier', 0);
            if ($shown < 3) {
                $url = admin_url('edit.php?post_type=magicpage&page=magic-page-settings&minifier=true');

                echo('<div class="notice minify-error-notice notice-error"><p>We recommend you hide your secret weapon from your competition and Google by making Magic Page Plugin'
                . ' invisible on your site. <a href="' . $url . '">ACT NOW!</a> </p></div>');

                $shown++;
                update_option('_magic_page_admin_notice_minifier', $shown);
            }
        }
    }

    public function check_folder_permissions() {
        if (!is_admin()) {
            wp_send_json_error();
        }

        footprintMinifyFolderPermissions();

        exit();
    }

    public function saveGooglemapSettings($newplugin_folder) {
        $settings = array();

        $settings['js_filename'] = rtrim(sanitize_file_name($_POST['js_filename']), '.js');
        $settings['js_folder'] = trim(sanitize_file_name($_POST['js_folder']));

        $settings['js_classname'] = trim(sanitize_html_class($_POST['js_classname']));
        $settings['js_settings'] = trim(sanitize_key($_POST['js_settings']));
        $settings['js_latitude'] = trim(sanitize_key($_POST['js_latitude']));
        $settings['js_longitude'] = trim(sanitize_key($_POST['js_longitude']));
        $settings['js_title'] = trim(sanitize_key($_POST['js_title']));

        foreach ($settings as $id => $value) {
            if (!$value) {
                if (in_array($id, array('js_filename', 'js_folder'))) {
                    $settings[$id] =  generateRandomString(6,15,true, true);
                } else {
                    $settings[$id] =  generateRandomString();
                }
            } else {
                $settings[$id] = preg_replace('/^[0-9]+/', '', $value);
                if (strlen($settings[$id]) < 5) {
                    $data = array('note' => 'Please enter at least 5 characters, no leading numbers allowed or leave blank', 'field' => $id);
                    wp_send_json_error($data);
                }
            }
        }

        $settings['js_obfuscation'] = boolval($_POST['js_obfuscation']);
        $settings['js_version'] = MAGIC_PAGE_VERSION;

        $upload_dir = wp_upload_dir();
        $js_fullfolder = $upload_dir['basedir'] . '/' . $settings['js_folder'];

        if (is_dir($js_fullfolder)) {
            $data = array('note' => 'Please change this option to avoid possibe issues', 'field' => 'js_folder');
            wp_send_json_error($data);
        }

        $folderCreated = wp_mkdir_p($js_fullfolder);
        if (!$folderCreated) {
            $data = array('note' => 'The folder could not be created', 'field' => 'js_folder');
            wp_send_json_error($data);
        }

        // if there's an old folder delete it
        $saved = get_magic_page_googlemap_parameters();
        if (is_magic_page_footprint_minifed() && $saved && isset($saved['js_fullfolder']) && $saved['js_fullfolder']) {
            if (is_dir($saved['js_fullfolder'])) {
                $this->removeDirectory($saved['js_fullfolder']);
            }
        }

        update_option('_magic_page_minify_js', $settings);

        global $magicPageApi;
        $apiResponse = $magicPageApi->minify_footprint($newplugin_folder);


        if ($apiResponse && $apiResponse['status'] === 'success') {
            return array('jsurl' => $apiResponse['jsUrl'], 'js_fullfolder' => $js_fullfolder . '/' . $settings['js_filename'] . '.js');
        }

        return array();
    }

    public function returnErrorResponseJsOptions($data) {
        delete_option('_magic_page_minify_js');
        delete_option('_magic_page_minified');
        wp_send_json_error($data);
    }

    public function save() {
        if (!is_admin()) {
            wp_send_json_error();
        }

        global  $wpdb;

        $newplugin_folder = str_replace('--', '-', trim(sanitize_title_with_dashes($_POST['plugin_folder'])));
        $newpost_type_name = sanitize_text_field($_POST['post_type_name']);

        if (strlen($newplugin_folder) < 5) {
            $data = array('note' => 'Please enter at least 5 characters', 'field' => 'plugin_folder');
            $this->returnErrorResponseJsOptions($data);
        }

        if (strlen($newpost_type_name) < 5) {
            $data = array('note' => 'Please enter at least 5 characters', 'field' => 'post_type_name');
            $this->returnErrorResponseJsOptions($data);
        }

        if ($this->similar_option_exists($newplugin_folder)) {
            $data = array('note' => 'Please change this option to avoid possibe issues', 'field' => 'plugin_folder');
            $this->returnErrorResponseJsOptions($data);
        }

        if (is_dir(WP_PLUGIN_DIR . '/' . $newplugin_folder)) {
            $data = array('note' => 'Please change this option to avoid possibe issues', 'field' => 'plugin_folder');
            $this->returnErrorResponseJsOptions($data);
        }

        $savedGmap = $this->saveGooglemapSettings($newplugin_folder);

        $folder = get_magic_page_folder();
        $string = basename($folder) . '/magic-page.php';
        $newString = $newplugin_folder . '/magic-page.php';

        $results = $wpdb->get_results("SELECT * FROM $wpdb->options WHERE option_value LIKE '%$string%'");
        if ($results) {
            foreach ($results as $res) {
                $option = get_option($res->option_name);

                $update = $this->recursive_unserialize_replace($string, $newString, $option);
                update_option($res->option_name, $update);
            }
        }

        update_option('_magic_page_post_type_name', $newpost_type_name);
        update_option('_magic_page_post_type_slug', sanitize_title_with_dashes($newpost_type_name));

        $parentFolder = dirname(MAGIC_PAGE_DIR);
        $newFolder = $parentFolder . '/' . $newplugin_folder . '/';

        delete_option('puc_cron_check_updates-' . basename($folder));
        delete_option('external_updates-' . basename($folder));

        update_option('_magic_page_plugin_folder', $newplugin_folder);
        update_option('_magic_page_minified', true);

        rename(MAGIC_PAGE_DIR, $newFolder);

        if ($savedGmap && $savedGmap['jsurl'] && $savedGmap['js_fullfolder']) {
            $raw = file_get_contents($savedGmap['jsurl']);
            file_put_contents($savedGmap['js_fullfolder'], $raw);
        }

        delete_wpseo_cache();

        wp_send_json_success();
    }

    public function similar_option_exists($option) {
        global $wpdb;

        $results = $wpdb->get_results("SELECT * FROM $wpdb->options WHERE option_value LIKE '%$option%'");
        if ($results) {
            return true;
        }
    }

    public function classes($classes) {
        if (is_magic_page_footprint_minifed()) {
            $mpClass = false;

            $out = array();
            foreach ($classes as $id => $class) {
                if (strpos($class, 'magic') === false && strpos($class, 'location') === false) {
                    $out[] = $class;
                } else {
                    if (!$mpClass) {
                        $out[] = get_option('_magic_page_post_type_slug');
                    }

                    $mpClass = true;
                }
            }

            return $out;
        }

        return $classes;
    }

    public function recursive_unserialize_replace($from = '', $to = '', $data = '', $serialized = false) {
        try {
            if (is_string($data) && ( $unserialized = @unserialize($data) ) !== false) {
                $data = $this->recursive_unserialize_replace($from, $to, $unserialized, true);
            } elseif (is_array($data)) {
                $_tmp = array();
                foreach ($data as $key => $value) {
                    $_tmp[$key] = $this->recursive_unserialize_replace($from, $to, $value, false);
                }

                $data = $_tmp;
                unset($_tmp);
            } elseif (is_object($data)) {
                $_tmp = $data;
                $props = get_object_vars($data);
                foreach ($props as $key => $value) {
                    $_tmp->$key = $this->recursive_unserialize_replace($from, $to, $value, false);
                }

                $data = $_tmp;
                unset($_tmp);
            } else {
                if (is_string($data)) {
                    $data = str_replace($from, $to, $data);
                }
            }

            if ($serialized) {
                return serialize($data);
            }
        } catch (Exception $error) {
            
        }

        return $data;
    }

    public function removeDirectory($path) {
        $files = glob(preg_replace('/(\*|\?|\[)/', '[$1]', $path) . '/{,.}*', GLOB_BRACE);
        foreach ($files as $file) {
            if ($file == $path . '/.' || $file == $path . '/..') {
                continue;
            }
            is_dir($file) ? removeDirectory($file) : unlink($file);
        }
        rmdir($path);
        return;
    }


}

new MagicPageFootprint();

function footprintMinifyFolderPermissions() {
    ?>
    <div class="hidden protectnote" id="minifyStep1">
        <h4>Folder Permissions</h4>

        <span class="minifyItem">Plugin Folder:</span>   <?php if (!is_magic_page_folder_writable()) { ?>
            Your folder is not writable. Please CHMOD 777 plugin folder (<?php echo(get_magic_page_folder()); ?> ) <?php } else {
        ?> Your folder is writable. <?php
        }
        ?> <br />
        <span class="minifyItem">WP Content Folder:</span>  <?php if (!is_wp_content_folder_writable()) { ?>
            Your wp-content folder is not writable. Please CHMOD 777  (<?php echo(get_magic_page_file()); ?>)  <?php } else {
        ?> Your wp-content folder is writable. <?php
        }
        ?> 
        <br />

        <br /><br />
        <?php if (!is_magic_page_folder_writable() || !is_wp_content_folder_writable()) { ?> 
            <span id="recheckFolderPermissions" class="button button-secondary">Recheck Folder Permissions</span>
        <?php } else { ?><span id="continueMinifyStep2" class="button button-secondary">Continue</span>  <?php } ?> 
    </div>
    <?php
}

function get_magic_page_googlemap_parameters() {
     

    $settings = get_option('_magic_page_minify_js');

    $upload_dir = wp_upload_dir();

    if ($settings) {
        $settings['js_fullurl'] = $upload_dir['baseurl'] . '/' . $settings['js_folder'] . '/' . $settings['js_filename'] . '.js';
        $settings['js_fullfolder'] = $upload_dir['basedir'] . '/' . $settings['js_folder'] . '/';

        return $settings;
    } else {
        return array(
            'js_fullurl' => MAGIC_PAGE_URL . 'js/public.js',
            'js_latitude' => 'lat',
            'js_longitude' => 'lon',
            'js_title' => 'title',
            'js_classname' => 'google_map',
            'js_settings' => 'google_map_settings'
        );
    }
}

function is_magic_page_footprint_minifed() {
    return get_option('_magic_page_minified', false);
}
