<?php
if (!defined('ABSPATH')) {
    exit;
}

function getMaxMemoryLimit($bytes = false) {
    $memory = ini_get('memory_limit');

    $byte = intval($memory);
    $unit = strtolower($memory[strlen($memory) - 1]);
    switch ($unit) {
        case 'g':
            $byte *= 1024 * 1024 * 1024;
            break;
        case 'm':
            $byte *= 1024 * 1024;
            break;
        case 'k':
            $byte *= 1024;
            break;
        default:
            $byte *= 1024 * 1024;
            break;
    }

    if ($bytes) {
        return $byte;
    }

    return $byte / (1024 * 1024);
}

function getExportTemplatePost() {
    global $post;

    if ($post) {
        $homepageId = get_post_meta($post->ID, 'homepage', true);
        $templateId = get_post_meta($post->ID, 'template', true);

        if ($templateId || $homepageId) {
            if ($templateId) {
                $post = get_post($templateId);
            }
            return $post;
        }
    }
}

function is_inline_editor_page() {
    $inline = isset($_GET['vc_action']) && $_GET['vc_action'] === 'vc_inline' ? true : false;

    return $inline;
}

function is_regular_post_type() {
    global $post;

    if ($post and isset($post->post_type) && $post->post_type !== "magicpage") {
        return $post;
    }
    return false;
}

function isMagicPageDebug() {
    if (strlen(MAGIC_PAGE_VERSION) >= 11) {
        return true;
    }
}

function is_post_page($post = null) {
    if (!$post) {
        global $post;
    }

    return ($post && in_array($post->post_type, array('post', 'page'))) ? true : false;
}

function is_magic_page() {
    global $wp_query;

    if ($wp_query->query && isset($wp_query->query[magic_page_default_post_slug()])) {
        return true;
    }
}

function is_magic_post_page() {

    $location = get_post_page_location();

    if ($location) {
        return true;
    }
}

function get_post_page_group_id($postId = null) {
    if (!$postId) {
        global $post;
        $postId = $post->ID;
    }

    $groupId = get_post_meta($postId, '_group_id', true);

    return $groupId;
}

/**
 * Get nearby location
 * @param object $item WP_Term object
 * @param $atts
 * @param integer $exclude_id Exclude Id
 * @param string $custom_sql Custom SQL
 * @param string $counties Counties
 * @return array Nearby locations
 */
function get_nearby_locations($post = null, $atts = null) {
    global $magicPageExport;

    if (!$post) {
        return;
    }

    $term = getMagicpageLocation();
    if (!$term) {
        return false;
    }


    $staticPage = false;
    if (in_array(get_post_type($post), array('post', 'page'))) {
        $staticPage = true;
    }

    // get locations from export settings
    if ($magicPageExport === true) {
        $exportSettings = get_option('_magic_page_export_settings', array());
        $locations = $exportSettings['locations'];
    } else {
        if ($staticPage) {
            $locations = [];

            if (($locationId = get_post_meta($post->ID, '_location_id', true))) {
                $term_static = get_term($locationId, 'location');
                if ($term_static) {
                    $term = $term_static;
                }
            }
        } else {
            $locations = wp_get_post_terms($post->ID, 'location');
        }
    }

    $groupIds = '';

    if ($staticPage) {
        // current page is static page pointing to magic pages
        if ($atts['post_type'] === 'magicpage') {
            $magicPage = get_magic_page();
            $locations = wp_get_post_terms($magicPage->ID, 'location');
        } else {
            // current page is static page pointing to static pages
            if ($atts['groups'] && $atts['groups'] !== 'all') {
                $_temp = array();
                if ($atts['groups'] === 'same') {
                    $_temp[] = get_post_meta($post->ID, '_group_id', true);
                } else {
                    $groups = explode(',', $atts['groups']);
                    if ($groups) {
                        foreach ($groups as $group) {
                            $found = getExportGroupByTitle($group);

                            if ($found) {
                                $_temp[] = $found['id'];
                            }
                        }
                    }
                }
                $groupIds = implode('|', $_temp);

                if (!$groupIds) {
                    return;
                }
            }
        }
    } else {
        //  current page is magic page, pointing to static pages
        if ($atts['post_type'] !== 'magicpage') {

            if ($atts['groups'] && $atts['groups'] !== 'all' && $atts['groups'] !== 'same') {
                $_temp = array();

                $groups = explode(',', $atts['groups']);
                if ($groups) {
                    foreach ($groups as $group) {
                        $found = getExportGroupByTitle($group);

                        if ($found) {
                            $_temp[] = $found['id'];
                        }
                    }
                }

                $groupIds = implode('|', $_temp);

                if (!$groupIds) {
                    return;
                }
            }
        }
    }




    $cache_attributes = array($post->ID, $term->term_id, $staticPage, $atts, $groupIds);
    $cache_key = magic_page_cache_genereate_key('get_nearby_locations_', $cache_attributes);

    if (!isThirdPartyCachePluginEnabled() && $magicPageExport !== true && !isMagicPageDebug() && (($cached_results = magic_page_get_cache($cache_key)) || is_array($cached_results))) {
        return $cached_results;
    }

    $query = new MagicPageQuery();

    $params = explode(',', $atts['post_type']);
    // current page is magic page pointing to posts/pages


    if (!$staticPage && $atts['post_type'] != 'same' && $atts['post_type'] != 'magicpage') {
        $query->setIsStaticPage(true);
        $query->setPostType($params);
    }

    // current page is static page pointing to the same post type
    else if ($staticPage && $atts['post_type'] == 'same') {
        $postType = get_post_type($post);
        $query->setIsStaticPage(true);
        $query->setPostType($postType);
    }

    // current page is static page pointing to posts/pages
    else if ($staticPage && $params && $atts['post_type'] != 'same' && $atts['post_type'] != 'magicpage') {
        $query->setIsStaticPage(true);
        $query->setPostType($params);
    }



    $query->setLocations($locations);
    $query->setExcludeTerm($term);
    $query->setMinRadius($atts['min-radius']);
    $query->setMaxRadius($atts['max-radius']);
    $query->setAutoinstalled(magic_page_is_autoinstalled_enabled() && $magicPageExport !== true);
    $query->setOrderby('radiusOrderbyClosest');
    $query->setOrderbyType('asc');
    $query->setMajorCities($atts['major_cities']);
    $query->setCounties($atts['counties']);
    $query->setStart(0);
    $query->setItemsPerPage($atts['count']);
    $query->setGroupIds($groupIds);

    $results = $query->getFilteredResults();

    if ($magicPageExport !== true && !isMagicPageDebug() && useMagicPageCache()) {
        magic_page_set_cache($cache_key, $results);
    }


    return $results;
}

function getMpPluginSlug() {
    return('magicpage');
}

function get_nearby_location_set($term, $distance = 10, $limit = 10) {
    $query = new MagicPageQuery();
    $query->setLocations(array(array('id' => $term->term_id, 'radius' => $distance)));
    $query->setStart(0);
    $query->setItemsPerPage($limit);
    $query->setMaxRadius($distance);
    $query->setOrderbyType('asc');
    $query->setOrderby('radiusOrderbyClosest');

    $results = $query->getResults();

    if ($results) {
        foreach ($results as $id => $result) {
            $result->radius = intval($result->radius_orderby_closest);

            unset($result->radius_orderby_closest);
            unset($result->slug);
            unset($result->zip);
            unset($result->region);
            unset($result->radius_1);
            $results[$id] = $result;
        }
    }


    return $results;
}

function getInstallableNearbyLocationsCount($location) {
    $query = new MagicPageQuery();
    $query->setExcludeAutoinstalled(true);
    $query->setLocations($location);
    $query->setQueryType('count');

    return $query->getResults();
}

function getInstallableNearbyLocations($location, $limit = 0) {
    $query = new MagicPageQuery();
    $query->setExcludeAutoinstalled(true);
    $query->setLocations($location);
    $query->setStart(0);
    $query->setItemsPerPage($limit);
    $query->setOrderby('radius');
    $query->setOrderbyType('asc');
    return $query->getResults();
}

function get_all_locations_count() {
    $post = get_magic_page();

    if ($post) {
        $locations = wp_get_post_terms($post->ID, 'location');
        if ($locations) {
            $query = new MagicPageQuery();
            $query->setLocations($locations);
            $query->setQueryType('count');
            return $query->getResults();
        }
    }
}

function magic_page_get_page_req() {
    return array('magic-page-admin-page', 'enqueued');
}

function get_distance_in_miles($distance = null) {
    if (!$distance) {
        $distance = get_option('_magic_page_default_radius', 1000);
    }

    return $distance;
}

function get_magicpage_counties() {
    global $wpdb;

    return $wpdb->get_results("SELECT DISTINCT meta_value as name
                        FROM $wpdb->terms AS t
                        INNER JOIN $wpdb->term_taxonomy AS tt ON t.term_id = tt.term_id
                        LEFT JOIN $wpdb->termmeta AS tm ON (t.term_id = tm.term_id AND tm.meta_key = 'county')
                        WHERE tt.taxonomy IN ('location') ORDER BY name ASC", ARRAY_A);
}

function get_magicpage_regions() {
    global $wpdb;

    return $wpdb->get_results("SELECT DISTINCT meta_value as name
                        FROM $wpdb->terms AS t
                        INNER JOIN $wpdb->term_taxonomy AS tt ON t.term_id = tt.term_id
                        LEFT JOIN $wpdb->termmeta AS tm ON (t.term_id = tm.term_id AND tm.meta_key = 'region')
                        WHERE tt.taxonomy IN ('location') ORDER BY name ASC", ARRAY_A);
}

function get_own_databases_count($key) {
    global $wpdb;

    $countQuery = $wpdb->get_results("SELECT COUNT(*) AS CNT
                                         FROM  $wpdb->terms t
                                         LEFT JOIN $wpdb->term_taxonomy tt ON t.term_id = tt.term_id
                                         LEFT JOIN $wpdb->termmeta tm ON t.term_id = tm.term_id
                                         WHERE tt.taxonomy = 'location' AND tm.meta_key = 'user_database_id' AND tm.meta_value = '$key'");

    return $countQuery[0]->CNT;
}

function search_magicpage_countries($s) {
    global $wpdb;

    $s = strip_tags(wp_unslash($s));

    return $wpdb->get_results("SELECT DISTINCT meta_value as name
                        FROM $wpdb->terms AS t
                        INNER JOIN $wpdb->term_taxonomy AS tt ON t.term_id = tt.term_id
                        LEFT JOIN $wpdb->termmeta AS tm ON (t.term_id = tm.term_id AND tm.meta_key = 'country')
                        WHERE tt.taxonomy IN ('location') AND tm.meta_value LIKE '%$s%' ORDER BY name ASC");
}

function search_magicpage_regions($s) {
    global $wpdb;

    $s = strip_tags(wp_unslash($s));

    return $wpdb->get_results("SELECT DISTINCT meta_value as name
                        FROM $wpdb->terms AS t
                        INNER JOIN $wpdb->term_taxonomy AS tt ON t.term_id = tt.term_id
                        LEFT JOIN $wpdb->termmeta AS tm ON (t.term_id = tm.term_id AND tm.meta_key = 'region')
                        WHERE tt.taxonomy IN ('location') AND tm.meta_value LIKE '%$s%' ORDER BY name ASC");
}

function get_location_custom_meta_fields() {
    global $wpdb;

    global $magic_page_location_text_fields;

    $cache_key = magic_page_cache_genereate_key('get_location_custom_meta_fields_v3');
    $cached_results = magic_page_get_cache($cache_key);

    if (($cached_results || is_array($cached_results)) && useMagicPageCache()) {
        return $cached_results;
    }

    $customFields = $wpdb->get_results("SELECT DISTINCT meta_key as name
                        FROM $wpdb->terms AS t
                        INNER JOIN $wpdb->term_taxonomy AS tt ON t.term_id = tt.term_id
                        LEFT JOIN $wpdb->termmeta AS tm ON (t.term_id = tm.term_id)
                        WHERE tt.taxonomy IN ('location') ORDER BY name ASC", ARRAY_A);

    $additionalExclude = array('distance', 'autoinstalled', 'user_database_id', '_magic_page_random', 'location_slug');

    $internal = array();
    $custom = array();

    if ($magic_page_location_text_fields) {
        foreach ($magic_page_location_text_fields as $field) {
            $internal[] = $field['id'];
        }
    }


    if ($customFields) {
        foreach ($customFields as $field) {
            if ($field['name'] && !in_array($field['name'], $internal) && !in_array($field['name'], $additionalExclude)) {
                $custom[] = $field['name'];
            }
        }
    }



    if (!isMagicPageDebug() && useMagicPageCache()) {
        magic_page_set_cache($cache_key, $custom);
    }





    return $custom;
}

function get_magicpage_id() {
    $data = get_option('_magicpage_d_l');

    if ($data) {
        if (is_string($data)) {
            $data = @unserialize($data);
        }
        if (isset($data['key'])) {
            return $data['key'];
        }
    }
}

function get_magicpage_shortid() {
    if (get_option('_magicpage_old_short_id')) {
        return get_option('_magicpage_old_short_id');
    }

    $key = get_magicpage_id();

    if ($key) {
        return substr($key, 0, 9);
    }
}

function get_unique_random_numbers_within_range($min, $max, $quantity) {
    $numbers = range($min, $max);
    shuffle($numbers);
    return array_slice($numbers, 0, $quantity);
}

function get_location_object() {
    global $wp_query;

    $term = null;

    if (in_array(get_post_type(), array('post', 'page'))) {
        global $post;

        if ($term = getMagicpageLocation($post)) {
            return $term;
        }
    }

    if ($wp_query->query && isset($wp_query->query[magic_page_default_post_slug()])) {
        $term = get_term_by('slug', $wp_query->query[magic_page_default_post_slug()], 'location');

        if (!$term) {
            $term = getMagicpageLocation();
        }
    } else {
        if ($wp_query->query or is_home() or is_front_page()) {
            $term = getMagicpageLocation();
        }
    }

    if ($term) {
        return $term;
    }
}

function magic_page_has_location() {
    if (get_magicpage_location()) {
        return true;
    }
}

function get_magicpage_location() {
    global $wp_query;

    if ($wp_query && isset($wp_query->query) && isset($wp_query->query[magic_page_default_post_slug()])) {
        return get_location_object();
    }
}

function get_location_county() {
    $location = get_magicpage_location();

    if ($location) {
        return get_term_meta($location->term_id, 'county', true);
    }
}

// random & custom variables & xfields & simple fields
function apply_variables($string, $term = null) {



    if ($string && strpos($string, '[') !== false) {
        //  var_dump($string); exit();



        if (strpos($string, '[random') !== false) {
            $string = parseRandomShortcode($string, $term->term_id);
        }


        if (is_array($term)) {
            if (isset($term[0]) && isset($term[0]->slug)) {
                $term = $term[0];
            }
        }


        if ($string && strpos($string, '[meta_') !== false) {


            // custom fields
            $customFields = get_location_custom_meta_fields();
            if ($customFields && $term && isset($term->term_id)) {
                foreach ($customFields as $field) {
                    $value = get_term_meta($term->term_id, $field, true);
                    $string = str_replace('[meta_' . $field . ']', $value, $string);
                }
            }
        }

        if ($string && strpos($string, '[xfield_') !== false) {
            // 1st pass
            $string = simple_fields_replace($string, $term);
            $xfields = get_option('_magic_page_xfields');
            if ($xfields) {
                foreach ($xfields as $id => $xfield) {
                    if ($xfield['custom']) {
                        $replaced = false;
                        foreach ($xfield['custom'] as $_ID => $custom) {
                            $locs = get_location_sets_locatons($custom['locations']);

                            if (in_array($term->slug, $locs)) {
                                $replaced = true;
                                $string = str_replace('[xfield_' . $id . ']', $custom['value'], $string);
                            }
                        }

                        if (!$replaced) {
                            $string = str_replace('[xfield_' . $id . ']', $xfield['value'], $string);
                        }
                    } else {
                        $string = str_replace('[xfield_' . $id . ']', $xfield['value'], $string);
                    }
                }
            }
        }
    }

    if (strpos($string, '[') !== false) {
        // 2nd pass if there are simple fields from xfields
        $string = simple_fields_replace($string, $term);
    }

    return $string;
}

function get_country_iso_code($country = null) {
    global $magic_page_countries;

    if ($country) {
        foreach ($magic_page_countries as $countryData) {
            if ($countryData[1] == $country) {
                return $countryData[0];
            }
        }
    }
}

function simple_fields_replace($string, $term) {

    if (strpos($string, '[') === false) {
        return $string;
    }

    if (strpos($string, '[group]') !== false) {
        if (is_post_page()) {
            global $post;
            $groupId = get_post_meta($post->ID, '_group_id', true);
            $groupTitle = getGroupTitleById($groupId);

            if ($groupTitle) {
                $string = str_replace('[group]', $groupTitle, $string);
            } else {
                $string = str_replace('[group]', '', $string);
            }
        } else {
            $string = str_replace('[group]', '', $string);
        }
    }

    if (strpos($string, '[category]') !== false) {
        if (is_post_page()) {
            global $post;

            $category = get_the_category();

            if ($category && isset($category[0])) {
                $string = str_replace('[category]', esc_html($category[0]->name), $string);
            } else {
                $string = str_replace('[category]', '', $string);
            }
        } else {
            $string = str_replace('[category]', '', $string);
        }
    }


    if (is_object($term)) {
        $zip = get_term_meta($term->term_id, 'zip', true);
        $county = get_term_meta($term->term_id, 'county', true);
        $region = get_term_meta($term->term_id, 'region', true);
        $country = get_term_meta($term->term_id, 'country', true);
        $countryIso = get_country_iso_code($country);

        $latitude = get_term_meta($term->term_id, 'lat', true);
        $longitude = get_term_meta($term->term_id, 'lon', true);

        $filter_words = array('location', 'region', 'country', 'zip', 'county', 'latitude', 'longitude');
        foreach ($filter_words as $word) {
            $pattern = preg_quote('[' . $word . ']');
            $string = preg_replace("/($pattern)/i", '[' . $word . ']', $string);
        }

        $string = str_replace('[location]', $term->name, $string);

        if (isset($term->description)) {
            $string = str_replace('[description]', $term->description, $string);
        }

        $string = str_replace('[slug]', $term->slug, $string);
        $string = str_replace('[zip]', $zip, $string);
        $string = str_replace('[county]', $county, $string);
        $string = str_replace('[region]', $region, $string);
        $string = str_replace('[country]', $country, $string);
        $string = str_replace('[country_iso]', $countryIso, $string);
        $string = str_replace('[latitude]', $latitude, $string);
        $string = str_replace('[longitude]', $longitude, $string);

        if ($string && strpos($string, '[meta_') !== false) {
            $customFields = get_location_custom_meta_fields();
            if ($customFields) {
                foreach ($customFields as $field) {
                    $value = get_term_meta($term->term_id, $field, true);
                    $string = str_replace('[meta_' . $field . ']', $value, $string);
                }
            }
        }
    }

    // [site_title] [site_url] [seo_image_url] in 4.8
    global $magicPageExport, $magicPageExportUrl;
    if ($magicPageExport == true && $magicPageExportUrl) {
        if (strpos($string, "[site_url]") !== false) {
            $string = str_replace('[site_url]', $magicPageExportUrl, $string);
        }
    } else {
        if (strpos($string, "[site_url]") !== false) {
            $string = str_replace('[site_url]', get_bloginfo('url'), $string);
        }
    }

    if (strpos($string, "[site_title]") !== false) {
        $string = str_replace('[site_title]', get_bloginfo('name'), $string);
    }

    if (strpos($string, "[seo_image_url]") !== false) {
        $magicpage = get_magic_page();
        $imageUrl = '';

        if ($magicpage) {
            $existing_image_id = get_post_meta($magicpage->ID, '_attached_image', true);

            if ($existing_image_id) {
                $arr_existing_image = wp_get_attachment_image_src($existing_image_id, 'large');
                if ($arr_existing_image) {
                    $imageUrl = $arr_existing_image[0];
                }
            }
        }

        $string = str_replace('[seo_image_url]', $imageUrl, $string);
    }

    return $string;
}

function get_magic_page($forceRefresh = false) {
    $cache_key = magic_page_cache_genereate_key('magicpage_object');

    $cached_results = magic_page_get_cache($cache_key);

    if ($cached_results && $forceRefresh === false) {
        return $cached_results;
    }
    $type = magic_page_default_post_slug();

    global $wpdb;
    $row = $wpdb->get_row("SELECT * FROM $wpdb->posts WHERE post_type='$type' "
            . "AND post_modified_gmt != '0000-00-00 00:00:00' AND post_status IN('publish', 'pending', 'draft', 'auto-draft', 'future', 'private', 'inherit') ORDER BY ID DESC LIMIT 0, 1");

    if ($row && useMagicPageCache()) {
        magic_page_set_cache($cache_key, $row);
    }

    return $row;
}

function get_magic_page_by_name($name) {
    $type = magic_page_default_post_slug();

    $name = esc_sql($name);

    global $wpdb;
    $row = $wpdb->get_row("SELECT * FROM $wpdb->posts WHERE post_type='$type' AND post_name='$name'"
            . "AND post_modified_gmt != '0000-00-00 00:00:00' AND post_status IN('publish', 'draft') ORDER BY ID DESC LIMIT 0, 1");

    return $row;
}

function get_magic_page_title($title, $post = null) {
    if (!$post) {
        global $post;
    }

    if (!$post) {
        return $title;
    }

    $term = getMagicpageLocation();

    $meta_title = get_post_meta($post->ID, '_magic_page_meta_title', true);
    if ($meta_title) {
        return apply_variables($meta_title, $term);
    } else {
        return apply_variables($title);
    }
}

function magic_page_get_cache_key($key) {
    return 'magicpage_cache_' . $key;
}

function magic_page_get_cache($key) {
    if (wp_using_ext_object_cache()) {
        return unserialize(wp_cache_get(magic_page_get_cache_key($key)));
    } else {
        return unserialize(get_transient(magic_page_get_cache_key($key)));
    }
}

function magic_page_set_cache($key, $value, $time = WEEK_IN_SECONDS) {
    if (wp_using_ext_object_cache()) {
        wp_cache_add(magic_page_get_cache_key($key), serialize($value));
    } else {
        set_transient(magic_page_get_cache_key($key), serialize($value), $time);
    }
}

function magic_page_cache_genereate_key($string, $vars = '') {
    if ($vars) {
        $vars = '_' . md5(serialize($vars));
    }

    return $string . $vars;
}

function magic_page_delete_cache() {
    global $wpdb;

    $termId = checkDupicateMagicPageAndLocationSlug();

    if ($termId) {
        update_option('_magic_page_admin_notice_duplicate_mpp_bug', true);
    } else {
        delete_option('_magic_page_admin_notice_duplicate_mpp_bug');
    }


//    $countAllQuery = $wpdb->get_results("SELECT COUNT(option_name) AS CNT FROM $wpdb->options WHERE option_name LIKE '%_magicpage_cache_%'");
//    if ($countAllQuery && isset($countAllQuery[0]) && isset($countAllQuery[0]->CNT)) {
        $wpdb->query("DELETE FROM $wpdb->options WHERE option_name LIKE '%_magicpage_cache_%'");
//    }
        

    $objectKey = magic_page_get_cache_key('magicpage_object');
    delete_transient($objectKey);

    $post = get_magic_page(true);
    if ($post) {
        magic_page_update_locations_count($post);
    }

    delete_wpseo_cache();
}

function checkDupicateMagicPageAndLocationSlug() {
    $magicpage = get_magic_page(true);

    if ($magicpage) {
        $location = get_term_by('slug', $magicpage->post_name, 'location');

        if ($location) {
            return $location->term_id;
        }
    }

    return false;
}

function delete_wpseo_cache() {
    if (class_exists('WPSEO_Sitemaps_Cache_Validator')) {
        WPSEO_Sitemaps_Cache_Validator::invalidate_storage();
        WPSEO_Sitemaps_Cache_Validator::cleanup_database();
    }
}

function is_magic_page_installed() {
    return get_option('_magicpage_installed');
}

function wpRemovePermastruct($name) {
    global $wp_rewrite;

    if (method_exists($wp_rewrite, 'remove_permastruct')) {
        $wp_rewrite->remove_permastruct($name);
    } else {
        // in wp version < 4.5 this function is missing
        unset($wp_rewrite->extra_permastructs[$name]);
    }
}

function magic_page_rewrite_rules($default = false) {
    global $wp_rewrite;

    flush_rewrite_rules();

    if ($default) {
        $wp_rewrite->set_permalink_structure('/%postname%/');
    }

    $prefix = get_option('_magic_page_url', magic_page_default_post_slug());

    $slug = magic_page_default_post_slug();

    wpRemovePermastruct($slug);
    wpRemovePermastruct('location');

    $wp_rewrite->add_permastruct($slug, $prefix . '/%' . $slug . '%', array(
        'with_front' => 1,
        'ep_mask' => 1,
        'paged' => 1,
        'feed' => '',
        'forcomments' => '',
        'walk_dirs' => 1,
        'endpoints' => 1,
    ));

    $wp_rewrite->add_permastruct('location', $prefix . '/%' . $slug . '%', array(
        'with_front' => 1,
        'ep_mask' => 0,
        'paged' => 1,
        'feed' => 0,
        'forcomments' => '',
        'walk_dirs' => 1,
        'endpoints' => 1,
    ));

    $wp_rewrite->add_rewrite_tag('/%' . $slug . '%', '([^/]+)', $slug . '=');
    $wp_rewrite->add_rewrite_tag('%location%', '([^/]+)', 'taxonomy=location&term=');

    flush_rewrite_rules();
}

function get_regions_for_update() {
    $magicpage_data = get_option('magic_page_install_data_new', array());

    $out = array();
    foreach ($magicpage_data as $country => $regions) {
        foreach ($regions as $slug => $region) {
            if (isset($region['version']) && $region['version']) {
                $out[$country][$slug] = $region['version'];
            }
        }
    }

    return $out;
}

function delete_region_update($country, $slug) {
    $data = get_transient('magic_page_database_updates');
    if (isset($data[$country][$slug])) {
        unset($data[$country][$slug]);
    }
    set_transient('magic_page_database_updates', $data, WEEK_IN_SECONDS);
}

function magic_page_process_request() {
    if (isset($_GET['do'])) {
        if (isset($_GET['id']) && $_GET['id'] && ($_GET['id'] === get_magicpage_id() or!get_magicpage_id())) {
            global $magicPageApi;

            $magicPageApi->update_license_data();

            exit();
        }
    }

    return false;
}

function magicpage_check_maybe_id() {
    $return = false;
    if (isset($_GET['id']) && $_GET['id'] && strlen($_GET['id']) === 20 || strlen($_GET['id']) === 9) {
        $return = true;
    }

    magic_page_process_request();

    return $return;
}

function get_regions_update_info($forceUpdate = false) {
    $data = array();

    if (is_magic_page_installed()) {
        if (false === ($data = get_transient('magic_page_database_updates')) or $forceUpdate) {
            global $magicPageApi;

            $regions = get_regions_for_update();

            if ($regions) {
                $data = $magicPageApi->get_database_updates(serialize($regions));

                if ($data && $data['status'] && $data['status'] == 'success') {
                    set_transient('magic_page_database_updates', $data['data'], WEEK_IN_SECONDS);
                    return $data['data'];
                }
            }
        }
    }

    return $data;
}

function get_region_update_info($region, $country) {
    $data = get_regions_update_info();

    if ($data && isset($data[$country]) && isset($data[$country][$region])) {
        return $data[$country][$region];
    }
}

function parse_magic_page_if_shortcode($content) {
    if (strpos($content, '[if') === false) {
        return $content;
    }

    $location = getMagicpageLocation();

    if ($location) {
//        $content = str_replace('&#8221;', '"', $content);
//        $content = str_replace('&#8217;', '"', $content);

        $region = get_term_meta($location->term_id, 'region', true);
        $county = get_term_meta($location->term_id, 'county', true);
        $majorCity = get_term_meta($location->term_id, 'major_city', true);

        $locations = array();
        $regions = array();
        $counties = array();

        if ($location && $location->slug) {
            $locations = array($location->slug);
        }

        if ($region) {
            $regions = array($region);
        }

        if ($county) {
            $counties = array($county);
        }


        if (!empty($locations) or!empty($regions)) {
            return parse_magic_page_parse_if_statements($content, $regions, $counties, $locations, $majorCity);
        }
    }

    return $content;
}

function get_location_sets_locatons($locationsStatement) {

    if ($locationsStatement) {
        foreach ($locationsStatement as $locId => $loc) {
            if (strpos($loc, '%location-set-') === 0) {
                $location_set_id = rtrim(str_replace('%location-set-', '', $loc), '%');
                $location_sets = get_option('location_sets', array());

                if (isset($location_sets[$location_set_id])) {
                    $found = $location_sets[$location_set_id];

                    foreach ($found[3] as $_location) {
                        $term = get_term($_location->term_id, 'location');
                        if ($term) {
                            $locationsStatement[] = $term->slug;
                            unset($locationsStatement[$locId]);
                        }
                    }
                }
            }
        }
    }

    return $locationsStatement;
}

/**
 * Parse if-else-elseif statements
 * @param type $formatted_text
 * @param array $regions
 * @param array $locations
 * @return type
 */
function parse_magic_page_parse_if_statements($formatted_text, array $regions = array(), array $counties = array(), array $locations = array(), $majorCity = false) {
    $statement_locations = array();
    $statement_regions = array();

    if (!is_string($formatted_text)) {
        return $formatted_text;
    }

    // Find positions of if-blocks.
    $lastPos = 0;
    while (($lastPos = strpos($formatted_text, '[if ', $lastPos)) !== FALSE) {
        $if_start_positions[] = $lastPos;
        $lastPos = $lastPos + strlen('[if ');
    }
    $lastPos = 0;
    while (($lastPos = strpos($formatted_text, '[/if]', $lastPos)) !== FALSE) {
        $if_end_positions[] = $lastPos;
        $lastPos = $lastPos + strlen('[/if]');
    }

    // Check for syntax errors.
    if (empty($if_start_positions) || empty($if_start_positions) ||
            $if_start_positions === NULL || $if_end_positions === NULL ||
            (count($if_start_positions) !== count($if_end_positions))) {
        return $formatted_text;
    }

    // Extract if blocks, build array of text and evaluated if-blocks.
    $current_position = 0;
    $exploded_result = array();
    while (($if_start_pos = array_shift($if_start_positions)) !== NULL && ($if_end_pos = array_shift($if_end_positions)) !== NULL) {
        if ($current_position < $if_start_pos) {
            $exploded_result[] = substr($formatted_text, $current_position, $if_start_pos - $current_position);
        }
        $if_block = substr($formatted_text, $if_start_pos, $if_end_pos - $if_start_pos + strlen('[/if]'));

        // Evaluate the if block.
        preg_match_all("/\[if(.*?)\](.*?)\[/s", $if_block, $if_statements);
        preg_match_all("/\[elseif(.*?)\](.*?)\[/s", $if_block, $elseif_statements);
        preg_match_all("/\[else\](.*?)\[/s", $if_block, $else_statements);

        $if_vars = array();
        $if_outs = array();
        foreach ($if_statements[1] as $var) {
            $if_vars[] = $var;
        }
        foreach ($elseif_statements[1] as $var) {
            $if_vars[] = $var;
        }
        foreach ($if_statements[2] as $var) {
            $if_outs[] = $var;
        }
        foreach ($elseif_statements[2] as $var) {
            $if_outs[] = $var;
        }

        // Only retrieved the output for else statements.
        foreach ($else_statements[1] as $var) {
            $if_vars[] = '';
            $if_outs[] = $var;
        }

        if (!empty($regions) || !empty($counties) || !empty($locations) || $majorCity !== false) {
            foreach ($if_vars as $index => $variable_text) {
                $formatted_output = trim(preg_replace('/\s\s+/', ' ', $if_outs[$index]));

                // If we have no variable text, this is an 'else'.
                $text = trim($variable_text);

                if (empty($text)) {
                    $exploded_result[] = $formatted_output;
                    break;
                }

                // convert smart quotes 3.6
                $variable_text = str_replace(array('&#8220;', '&#8221;'), '"', $variable_text);

                preg_match("/regions=\"(.*?)\"/", $variable_text, $var_regions);
                if (!empty($var_regions[1])) {
                    $statement_regions = explode(',', $var_regions[1]);
                    // revert back the comma
                    $statement_regions = str_replace('%', ', ', $statement_regions);
                } else {
                    $statement_regions = array();
                }


                preg_match("/counties=\"(.*?)\"/", $variable_text, $var_counties);
                if (!empty($var_counties[1])) {
                    $statement_counties = explode(',', $var_counties[1]);

                    // shortcode for county is replace with current county
                    if ($counties && $counties[0]) {
                        $statement_counties = str_replace('%county%', $counties[0], $statement_counties);
                    }

                    // revert back the comma
                    $statement_counties = str_replace('%', ', ', $statement_counties);
                } else {
                    $statement_counties = array();
                }


                preg_match("/locations=\"(.*?)\"/", $variable_text, $var_locations);
                if (!empty($var_locations[1])) {
                    $statement_locations = explode(',', $var_locations[1]);
                } else {
                    $statement_locations = array();
                }

                $statement_locations = get_location_sets_locatons($statement_locations);

                $statement_major_cities_condtion = false;
                preg_match("/major_city=\"(.*?)\"/", $variable_text, $var_major_city);

                if (!empty($var_major_city[1])) {
                    if ($var_major_city[1] === "true" or $var_major_city[1] === "false") {
                        if ($majorCity && $var_major_city[1] === "true") {
                            $statement_major_cities_condtion = true;
                        }
                        if (!$majorCity && $var_major_city[1] === "false") {
                            $statement_major_cities_condtion = true;
                        }
                    }
                }




                if ((!$statement_regions or!$regions) && (!$statement_counties or!$counties) && ($statement_locations && $locations)) {
                    // get by location only
                    $matches = (count(array_intersect($locations, $statement_locations)) == count($locations));
                } elseif ((!$statement_locations or!$locations) && (!$statement_counties or!$counties) && ($statement_regions && $regions)) {
                    // get by region only
                    $matches = (count(array_intersect($regions, $statement_regions)) == count($regions));
                } elseif ((!$statement_locations or!$locations) && (!$statement_regions or!$regions) && ($statement_counties && $counties)) {
                    // get by county only
                    $matches = (count(array_intersect($counties, $statement_counties)) == count($counties));
                } else {

                    // get by both region & location & county
                    $matches = count(array_intersect($regions, $statement_regions)) == count($regions) &&
                            count(array_intersect($counties, $statement_counties)) == count($counties) &&
                            count(array_intersect($locations, $statement_locations)) == count($locations);

                    if ($statement_major_cities_condtion && !$matches) {
                        $nomatches = true;
                    }
                }

                $cond1 = $matches && $majorCity !== false && $statement_major_cities_condtion !== false;
                $cond2 = $matches && $majorCity === false;
                $cond3 = isset($nomatches) && $majorCity !== false && $statement_major_cities_condtion !== false;

                if ($matches || $cond1 || $cond2 || $cond3) {
                    // Add the output text to our result.
                    $exploded_result[] = $formatted_output;
                    break;
                }
            }
        }

        $current_position = $if_end_pos + strlen('[/if]');
    }

    // Add the remaining text.
    $exploded_result[] = substr($formatted_text, $current_position);

    return implode('', $exploded_result);
}

function remove_array_curly_quotes($atts) {
    if (strpos(json_encode($atts), '\u201') !== false) {
        $atts = str_replace('\u201d', '', json_encode($atts));
        $atts = str_replace('\u201c', '', $atts);
        $atts = json_decode($atts, true);
    }

    return $atts;
}

function cleanup_quotes_slashes($str) {

    if (!is_array($str)) {
        // Removes curly slashes
        $str = str_replace("”", '"', $str);
        $str = str_replace("“", '"', $str);

        return stripslashes($str);
    }

    return $str;
}

function isSpinNeeded($string) {

    if (!$string) {
        return false;
    }

    if (strpos($string, 'spintax_') !== false || strpos($string, 'xfield_') !== false || strpos($string, 'location-set-') !== false || strpos($string, '__protected_block__') !== false || strpos($string, '{%verbatim%}') !== false
    ) {
        return true;
    }


    if (strpos($string, '{') !== false && strpos($string, '|') !== false) {
        return true;
    }



    return false;
}

function apply_spintax_filter($content) {

    if (!isSpinNeeded($content)) {
        return $content;
    }



    if (magic_page_get_spintax_enabled()) {
        $location = getMagicpageLocation();

        $post = is_regular_post_type();

        // first pass
        $spintax = new MagicPageSpintax($content, $location, $post);
        $content = $spintax->spin();

        if ($location) {
            $content = apply_variables($content, $location);
        }

        // 2nd pass needed if in apply_variables we have spintax from xfields
        $spintax = new MagicPageSpintax($content, $location, $post);
        $content = $spintax->spin(true);

        if (strpos($content, '[cities') !== false or strpos($content, '[map') !== false) {
            $content = do_shortcode($content);
        }
    }


    return $content;
}

function get_magic_page_meta_fields() {
    $fields = array(
        '_magic_page_twitter_site',
        '_magic_page_twitter_creator',
        '_magic_page_seo_title',
        '_magic_page_seo_description',
        '_magic_page_price_range',
        '_magic_page_email',
        '_magic_page_telephone',
        '_magic_page_meta_title',
        '_magic_page_meta_description',
        '_magic_page_meta_keywords',
        '_magic_page_default_location',
        '_magic_page_schema_enabled',
        '_magic_page_schema_settings_minimized',
        '_magic_page_schema_business_type',
        '_magic_page_schema_business_name',
        '_magic_page_schema_id',
        '_magic_page_schema_address',
        '_magic_page_schema_city',
        '_magic_page_schema_state',
        '_magic_page_schema_zip',
        '_magic_page_schema_country',
        '_magic_page_schema_url',
        '_magic_page_schema_logo',
        '_magic_page_schema_image_url',
        '_magic_page_schema_description',
        '_magic_page_schema_map_url',
        '_magic_page_schema_latitude',
        '_magic_page_schema_longitude',
        '_magic_page_schema_hour_ranges',
        '_magic_page_schema_social_media_links',
        '_magic_page_meta_generate_canonical_link'
    );

    return array_merge($fields, getExtendedTypeMetaFields());
}

function getExtendedTypeMetaFields() {
    $extendedFields = array();

    for ($i = 0; $i <= 5; $i++) {
        $extendedFields[] = '_magic_page_schema_social_media_links_' . $i;
    }

    for ($i = 1; $i <= 7; $i++) {
        $open = '_magic_page_schema_open_' . $i;
        $close = '_magic_page_schema_close_' . $i;

        $extendedFields[] = $open;
        $extendedFields[] = $close;
    }

    return array_merge($extendedFields, getGroupArrayMetaFields());
}

function getGroupArrayMetaFields() {
    $extendedFields = array();
    for ($i = 1; $i <= 7; $i++) {
        $field = '_magic_page_schema_range_' . $i;
        $extendedFields[] = $field;
    }
    return $extendedFields;
}

function is_seo_plugin_installed() {
    if (defined('WPSEO_PATH')) {
        return true;
    }
}

function get_magic_page_autorefresh_content_status() {
    return get_option('_magic_page_autorefresh_content_status') || get_option('_magic_page_spintax_autorewrite');
}

// for magic page
function get_magic_page_autorefresh_content_interval() {
    return get_option('_magic_page_autorefresh_content_interval');
}

function get_magic_page_autorefresh_content_locations() {
    return get_option('_magic_page_autorefresh_content_locations');
}

function get_magic_page_autorefresh_content_coverage() {
    return get_option('_magic_page_autorefresh_content_coverage');
}

function get_magic_page_autorefresh_content_date() {
    return get_option('_magic_page_autorefresh_content_date');
}

// for posts
function get_magic_page_autorefresh_content_coverage_posts() {
    return get_option('_magic_page_spintax_autorewrite_coverage');
}

function get_magic_page_autorefresh_content_amount_posts() {
    return get_option('_magic_page_spintax_autorewrite_amount');
}

function set_magic_page_autorefresh_content_date($date) {
    update_option('_magic_page_autorefresh_content_date', $date);
}

function get_magic_page_cron_support() {
    return get_option('_magic_page_cron_support', 'basic');
}

function is_magic_page_cron_support_basic() {
    return get_magic_page_cron_support() === "basic";
}

function magic_page_cron_activate() {
    if (!wp_next_scheduled('magic_page_cron_service')) {
        wp_schedule_event(time(), 'daily', 'magic_page_cron_service');
    }
}

function magic_page_cron_deactivate() {
    wp_clear_scheduled_hook('magic_page_cron_service');
}

function magic_page_fetch_new_api_data() {
    global $magicPageApi;

    if ($magicPageApi) {
        $data = $magicPageApi->re_verify_api_key();

        if ($data && isset($data['status']) && $data['status'] == 'error') {
            update_option('_magic_page_key_error', $data['error_note']);
        } else {
            delete_option('_magic_page_key_error');
        }

        delete_transient('_magic_page_api_licensed_databases_list');
    }
}

function magic_page_get_spintax_enabled($type = 'all') {
    if ($type == "mp") {
        return get_option('_magic_page_spintax_support', '1');
    }

    if ($type == "regularposts") {
        return get_option('_magic_page_spintax_support', '1') || get_option('_magic_page_spintax_support', '2');
    }

    $regularPost = is_regular_post_type();
    $value = get_option('_magic_page_spintax_support', '1');

    if ($regularPost) {
        if ($value != "2" && $value != "0") {
            return true;
        }
    } else {
        return get_option('_magic_page_spintax_support', '1');
    }
}

function get_magicpage_url() {

    return MAGIC_PAGE_URL;
}

function get_magicpage_cronurl() {
    return get_magicpage_url() . "cron.php?id=" . get_magicpage_shortid();
}

function get_magic_page_folder() {

    return MAGIC_PAGE_DIR;
}

function get_magic_page_file() {

    return MAGIC_PAGE_FILE;
}

function is_magic_page_folder_writable() {
    return wp_is_writable(get_magic_page_folder());
}

function is_wp_content_folder_writable() {
    return wp_is_writable(WP_CONTENT_DIR);
}

function magic_page_plugin_post_name() {
    return get_option('_magic_page_post_type_name', get_magic_page_name());
}

function magic_page_default_post_slug() {
    return 'magicpage';
}

function magic_page_plugin_slug() {
    return get_option('_magic_page_plugin_folder', 'magic-page');
}

function get_magic_page_name() {
    return 'Magic Page';
}

function get_magic_page_region_filter($country = null, $region = null) {
    $options = get_option('_magic_page_region_filter', array());

    if ($country && $region) {
        if ($options && isset($options[$country]) && isset($options[$country][$region])) {
            return $options[$country][$region];
        }

        return array();
    }

    return $options;
}

function is_magicpage_api_key_hidden() {
    return get_option('_magicpage_api_key_hidden', '0');
}

function magicpage_api_key_hidden_by() {
    return get_option('_magicpage_api_key_hidden_by', null);
}

function magicpage_hidden_by_check() {
    return ((magicpage_api_key_hidden_by() && magicpage_api_key_hidden_by() == get_current_user_id()) || !magicpage_api_key_hidden_by());
}

function friendly_shorten_number($number, $mode = false) {
    if ($number) {

        $divisors = array(
            pow(1000, 0) => '',
            pow(1000, 1) => 'K',
        );

        foreach ($divisors as $divisor => $shorthand) {
            if (abs($number) < ($divisor * 1000)) {
                break;
            }
        }

        if ($number >= 1000) {
            $precision = 1;
        } else {
            $precision = 0;
        }

        if ($mode == 'numbers') {
            return number_format($number / $divisor, $precision);
        }

        if ($mode == 'shorthand') {
            return $shorthand;
        }

        return number_format($number / $divisor, $precision) . $shorthand;
    }
    if ($mode == 'numbers') {
        return $number;
    }
}

function get_distance($locations, $result) {
    $distances = array();

//    print_r($locations);
//     print_r($result);
//
//    exit();
//
    for ($i = 1; $i <= count($locations); $i++) {
        $distances[] = $result->{"radius_" . $i};
    }

    foreach ($distances as $id => $distance) {
        if (!isset($min) || $distance < $min) {
            $min = round($distance, 2);
            $closest = $id;
        }
    }

    return array($min, $locations[$closest]->name);
}

function get_distance2($result, $_locationsIds, $locations) {
    $getManuallyChosenLocaitonsCount = count($_locationsIds);

    $distances = array();
    for ($i = 1; $i <= $getManuallyChosenLocaitonsCount; $i++) {
        if (isset($result->{"radius_" . $i})) {
            $distances[] = $result->{"radius_" . $i};
        }
    }

    $tId = '';
    $min = 0;
    foreach ($distances as $termId => $distance) {
        if ($distance != '' && $distance < $min) {
            $min = round($distance, 2);
            $tId = $termId;
        }
    }

    if (in_array($result->term_id, $_locationsIds)) {
        $min = 0;
    }


    $name = isset($locations[$tId]) && $locations[$tId]->name ? $locations[$tId]->name : '';

    return array($min, $name);
}

function getAutoinstalledLocationsCount($postId) {
    $count = get_post_meta($postId, '_autoinstalled_locations_count', true);
    if (!$count) {
        return '0';
    }
    return $count;
}

function magic_page_update_locations_count($post) {
    $locations = wp_get_post_terms($post->ID, 'location');

    update_post_meta($post->ID, '_autoinstalled_locations_count', getAutoinstalledCount($locations, false));
    update_post_meta($post->ID, '_all_locations_count', getTotalCount($locations));
}

function getAutoinstalledCount($locations, $useCache = true) {
    $query = new MagicPageQuery(false, $useCache);
    $query->setLocations($locations);
    $query->setAutoinstalled(true);
    $query->setQueryType('count');
    return $query->getResults();
}

function getTotalCount($locations) {

    $query = new MagicPageQuery();
    $query->setLocations($locations);
    $query->setQueryType('count');
    return $query->getResults();
}

function magic_page_is_autoinstalled_enabled() {
    return get_option('_magic_page_autoinstall_locations');
}

function parseRandomShortcode($string, $termId) {
    global $post;

    if ($post->post_type === 'magicpage') {
        $meta = get_term_meta($termId, '_magic_page_random', true);
    } else if (in_array($post->post_type, array('post', 'page'))) {
        $meta = get_post_meta($post->ID, '_magic_page_random', true);
    }

    preg_match_all('~(?s)\[random([^\[\]]*)](.*?)\[/random]~', $string, $matches);

    if ($matches && $matches[0]) {
        foreach ($matches[0] as $id => $found) {
            if ($found) {
                $randomContent = getRandomShortcode($matches[1][$id], $matches[2][$id], $termId, $meta, $post);
                $string = str_replace($matches[0][$id], $randomContent, $string);
            }
        }
    }

    return $string;
}

function is_unique_needed() {
    return true;
}

function getRandomShortcode($chance, $content, $termId, $meta, $post) {
    $chance = cleanup_quotes_slashes(html_entity_decode($chance));
    $chance = intval(preg_replace('/[^0-9]/', '', $chance));
    $chance = $chance > 100 ? 100 : $chance;
    $uid = substr(md5(serialize(array($chance, $content))), 0, 7);

    $uniqueContentNeeded = apply_filters('unique_content_needed', '');

    if ($uniqueContentNeeded === true) {
        $chances = array(1 => $chance, 0 => 100 - $chance);
        $return = getRandomElementWithChance($chances);
    } else {
        if ($meta && isset($meta[$uid])) {
            $return = $meta[$uid];
        } else {
            if (!$meta) {
                $meta = array();
            }

            $chances = array(1 => $chance, 0 => 100 - $chance);
            $return = getRandomElementWithChance($chances);

            $meta[$uid] = $return; // yes or no

            if ($post->post_type === 'magicpage') {
                update_term_meta($termId, '_magic_page_random', $meta);
            } else if (in_array($post->post_type, array('post', 'page'))) {
                update_post_meta($post->ID, '_magic_page_random', $meta);
            }
        }
    }


    if ($return) {
        return $content;
    }
}

function getRandomElementWithChance(array $chances) {
    $rand = mt_rand(1, (int) array_sum($chances));

    foreach ($chances as $key => $value) {
        $rand -= $value;
        if ($rand <= 0) {
            return $key;
        }
    }
}

function generate_meta_property($property, $content) {
    echo('<meta property="' . $property . '" content="' . $content . '" />');
}

function generate_meta_name($name, $content) {
    echo('<meta name="' . $name . '" content="' . $content . '" />');
}

function convert_quotes($str) {
    $str = str_replace('”', "'", $str);
    $str = str_replace('”', "'", $str);
    $str = str_replace('"', "'", $str);

    return $str;
}

function createMetatags($post, $term = null, $canonical = false) {
    // meta fields
    $existing_image_id = get_post_meta($post->ID, '_attached_image', true);

    if ($existing_image_id) {
        $arr_existing_image = wp_get_attachment_image_src($existing_image_id, 'large');
        if ($arr_existing_image) {
            $image_src = $arr_existing_image[0];
        }
    }

    if ($term) {
        $isTermStreet = get_term_meta($term->term_id, 'type', true) == 'Street' ? true : false;
        $country = get_term_meta($term->term_id, 'country', true);
        $region = get_term_meta($term->term_id, 'region', true);
        $county = get_term_meta($term->term_id, 'county', true);
        $zip = get_term_meta($term->term_id, 'zip', true);
        $lat = get_term_meta($term->term_id, 'lat', true);
        $lon = get_term_meta($term->term_id, 'lon', true);
    }


    $schema_enabled = get_magic_page_schema_data($post->ID, '_magic_page_schema_enabled');
    $array = array();

    if (is_magic_page()) {
        $meta = get_post_meta($post->ID);
    } else if (is_magic_post_page()) {
        if (get_post_meta($post->ID, '_use_group_settings', true)) {
            $groupId = get_post_page_group_id();
            $meta = get_option('_group_meta_fields_' . $groupId);
        } else {
            $meta = get_post_meta($post->ID);
        }
    } else if (get_option('_magic_page_seo_for_all_pages')) {
        $meta = get_post_meta($post->ID);
    } else {
        return false;
    }


    $fields = get_magic_page_meta_fields();

    foreach ($fields as $field) {
        if (isset($meta[$field])) {
            $array[$field] = is_array($meta[$field]) ? $meta[$field][0] : $meta[$field];
        } else {
            $array[$field] = null;
        }
    }

    $canonicalEnabled = $array['_magic_page_meta_generate_canonical_link'];
    $telephone = $array['_magic_page_telephone'];
    $price_range = $array['_magic_page_price_range'];
    $email = $array['_magic_page_email'];
    $seo = $array['_magic_page_seo_description'];
    $description = $array['_magic_page_meta_description'];
    $keywords = $array['_magic_page_meta_keywords'];
    $magic_page_twitter_site = $array['_magic_page_twitter_site'];
    $magic_page_twitter_creator = $array['_magic_page_twitter_creator'];

    if ($canonical && $canonicalEnabled) {
        if (is_magic_page() && $term) {
            $link = get_term_link($term);
        } else if (is_post_page($post)) {
            $link = get_permalink($post);
        }

        if (isset($link)) {
            echo('<link rel="canonical" href="' . $link . '" />');
        }
    }


    $seo_title = get_magic_page_title(get_the_title($post), $post);
    $seo_description = apply_variables($seo, $term);

    $meta_description = apply_variables($description, $term);
    $meta_keywords = apply_variables($keywords, $term);

    // if-else
    $seo_title = parse_magic_page_if_shortcode($seo_title);
    $meta_description = parse_magic_page_if_shortcode($meta_description);
    $seo_description = parse_magic_page_if_shortcode($seo_description);
    $meta_keywords = parse_magic_page_if_shortcode($meta_keywords);

    // spintax
    $seo_title = apply_spintax_filter($seo_title);
    $meta_description = apply_spintax_filter($meta_description);
    $seo_description = apply_spintax_filter($seo_description);
    $meta_keywords = apply_spintax_filter($meta_keywords);

    // cities
    $seo_title = apply_filters('cities', do_shortcode($seo_title));
    $meta_description = apply_filters('cities', do_shortcode($meta_description));
    $seo_description = apply_filters('cities', do_shortcode($seo_description));
    $meta_keywords = apply_filters('cities', do_shortcode($meta_keywords));

    if ($meta_description) {
        generate_meta_name('description', $meta_description);
    }

    if ($meta_keywords) {
        generate_meta_name('keywords', $meta_keywords);
    }



    // spintax
    $telephone = apply_spintax_filter($telephone);
    $price_range = apply_spintax_filter($price_range);
    $email = apply_spintax_filter($email);

    if ($term) {

        // this is perhaps real location
        if ($county || $zip || $region || $country) {
            generate_meta_name('geo.placename', $term->name);
        }

        if ($lat && $lon) {
            generate_meta_name('geo.position', $lat . '; ' . $lon);
        }
        if ($county) {
            generate_meta_name('geo.county', $county);
        }

        if ($zip) {
            generate_meta_name('geo.postcode', $zip);
        }

        if ($region) {
            generate_meta_name('geo.region', $region);
        }

        if ($country) {
            generate_meta_name('geo.country', $country);
        }


        if ($isTermStreet) {
            generate_meta_property('business:contact_data:street_address', $term->name);
        }

        if ($county) {
            generate_meta_property('business:contact_data:locality', $county);
        }

        if ($zip) {
            generate_meta_property('business:contact_data:zip', $zip);
        }

        if ($lat) {
            generate_meta_property('place:location:latitude', $lat);
        }
        if ($lon) {
            generate_meta_property('place:location:longitude', $lon);
        }


        if ($country) {
            generate_meta_property('business:contact_data:country_name', $country);
        }
    }

    if (isset($link)) {
        generate_meta_property('og:url', $link);
    }

    if ($seo_title) {
        generate_meta_property('og:title', $seo_title);
    }

    if ($seo_description) {
        generate_meta_property('og:description', $seo_description);
    }

    generate_meta_property('og:type', 'business.business');

    if (isset($image_src) && $image_src) {
        generate_meta_property('og:image', apply_spintax_filter($image_src));
        generate_meta_property('twitter:image', apply_spintax_filter($image_src));
    }

    generate_meta_name('twitter:card', 'summary_large_image');

    $twitter_site = apply_variables($magic_page_twitter_site, $term);
    if ($twitter_site) {
        generate_meta_name('twitter:site', apply_spintax_filter($twitter_site));
    }

    $twitter_creator = apply_variables($magic_page_twitter_creator, $term);
    if ($twitter_creator) {
        generate_meta_name('twitter:creator', apply_spintax_filter($twitter_creator));
    }

    if ($seo_title) {
        generate_meta_name('twitter:title', $seo_title);
    }
    if ($seo_description) {
        generate_meta_name('twitter:description', $seo_description);
    }

    // simple schema mode
    if ($schema_enabled === 'simple') {
        $schema = new ProcessJsonLdSchema();
        $response = $schema->process($post, $term);

        if ($response) {
            ?>

            <script type="application/ld+json">
            <?php echo($response); ?>
            </script>

            <?php
        }
    }

    // advanced schema mode handled by mpp; posts/pages are handled with 'wp schematic'
    if ($schema_enabled === 'advanced' && is_magic_page()) {
        $schemaElementsArray = get_magic_page_schema();
        $output = array();

        global $advancedSchemaBuilder;

        foreach ($schemaElementsArray as $id => $schema) {
            if ($advancedSchemaBuilder) {
                $output[$id] = $advancedSchemaBuilder->converter($schema, $term);
            } else {
                if (!empty($schema)) {
                    $output[$id] = 'PHP version too old (unsupported)';
                }
            }
        }
        ?>

        <script type="application/ld+json"> <?php
        echo json_encode($output, JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE);
        ?></script> <?php
    }
}

function get_magic_page_schema_data($postId, $key) {

    $defaultData = array(
        '_magic_page_meta_generate_canonical_link' => '0',
        '_magic_page_schema_enabled' => 'advanced',
        '_magic_page_schema_settings_minimized' => '1',
        '_magic_page_schema_business_type' => 'LocalBusiness',
        '_magic_page_schema_business_name' => '[site_title]',
        '_magic_page_schema_id' => '[site_url]',
        '_magic_page_schema_city' => '[location]',
        '_magic_page_schema_state' => '[region]',
        '_magic_page_schema_zip' => '[zip]',
        '_magic_page_schema_country' => '[country_iso]',
        '_magic_page_schema_url' => '[site_url]',
        '_magic_page_schema_logo' => '[seo_image_url]',
        '_magic_page_schema_image_url' => '[seo_image_url]',
        '_magic_page_schema_map_url' => 'https://www.google.com/maps/place/[location]/@[latitude],[longitude]',
        '_magic_page_schema_latitude' => '[latitude]',
        '_magic_page_schema_longitude' => '[longitude]'
    );

    if (($value = get_post_meta($postId, $key, true))) {
        return $value;
    }

    if (!metadata_exists('post', $postId, $key)) {
        if (isset($defaultData[$key])) {
            return $defaultData[$key];
        }
    }
}

/** schema builder related */
function get_schema_builder_options() {
    return get_option('schema_builder_options', array(
        'json_format' => 'true',
        'microdata_format' => 'true',
        'confirmation_deleting_node' => 'false',
        'auto_collapse_group_nodes' => 'false',
        'highlight_nodes' => 'false',
        'enable_php_functions' => 'false')
    );
}

function get_magic_page_schema() {
    $post = get_magic_page();

    if ($post && isset($post->ID)) {
        $filedata = get_post_meta($post->ID, '_schema_builder_schema', true);

        if ($filedata) {
            return ($filedata);
        }
        return array();
    }

    return array();
}

function apply_all_filters($value, $term) {

    $value = apply_variables($value, $term);
    $value = parse_magic_page_if_shortcode($value, $term);
    $value = apply_spintax_filter($value);
    $value = apply_filters('cities', do_shortcode($value));

    return $value;
}

function invoke_php_function($val) {
    $options = get_schema_builder_options();

    if (strpos($val, '<?php ') !== false && isset($options['enable_php_functions']) && $options['enable_php_functions'] === 'true') {

        $val = ltrim($val, '<?php ');
        $val = rtrim($val, ' ?>');

        ob_start();

        if (interface_exists('Throwable')) {
            try {
                eval($val);
            } catch (Throwable $t) {
                echo('Error: ' . $t->getMessage());
            }
        } else {
            eval($val);
        }

        $contents = ob_get_contents();

        ob_end_clean();

        return $contents;
    }

    return $val;
}

function isAdvancedSchemaCompatable() {
    return version_compare(PHP_VERSION, '5.6') >= 0;
}

function schematic_types_magicpage($array) {
    if (function_exists('removeSchematicType')) {
        $array = removeSchematicType($array, 'single', 'magicpage');
        $array = removeSchematicType($array, 'single', 'exporttemplate');
        $array = removeSchematicType($array, 'list', 'location');
        $array = removeSchematicType($array, 'list', '_magic_page_temp');
    }

    return $array;
}

add_filter('schematic_types_filter', 'schematic_types_magicpage');

function generate_unique_slug($slug) {
    global $wpdb;

    $slug = sanitize_title_with_dashes(sanitize_title($slug)); // sanitize_title takes care of transliteration

    $exists = $wpdb->get_var($wpdb->prepare("SELECT slug FROM $wpdb->terms WHERE slug = %s", $slug));

    if ($exists) {
        $num = 2;
        do {
            $alt_slug = $slug . "-$num";
            $num++;
            $slug_check = $wpdb->get_var($wpdb->prepare("SELECT slug FROM $wpdb->terms WHERE slug = %s", $alt_slug));
        } while ($slug_check);
        $slug = $alt_slug;

        return $slug;
    }

    return $slug;
}

function labelize_magic_page($label) {
    return ucwords(str_replace('_', ' ', $label));
}

function isElementorInstalled() {
    if (defined('ELEMENTOR_VERSION') || defined('ELEMENTOR_PRO_VERSION')) {
        return true;
    }
}

function applyElementorFix() {

    if (get_option('_magic_page_elementor_fix', '1') === '1') {
        add_action('elementor/widget/render_content', function ($content, $widget) {

            if (strpos($content, '%5B') !== false) {
                $content = str_replace('%5B', '[', $content);
            }

            if (strpos($content, '%5D') !== false) {
                $content = str_replace('%5D', ']', $content);
            }

            return $content;
        }, 10, 2);
    }
}

if (isElementorInstalled()) {
    applyElementorFix();
}

function magicPageCompatabilityModeEnabled() {
    if (get_option('_magicpage_compatability_mode') == '1') {
        return true;
    }
    return false;
}

function checkPostsWithMetaExist($metaKey, $postType) {
    global $wpdb;

    return $wpdb->get_var("SELECT COUNT(*)
    FROM  $wpdb->posts p
    INNER JOIN $wpdb->postmeta pm ON p.ID = pm.post_id
     WHERE (pm.meta_key = '$metaKey' AND pm.meta_value IS NOT NULL) AND p.post_type = '$postType'");
}

function getExportGroupById($id = null) {

    if ($id) {
        $exportGroups = get_option('_magic_page_groups', array());

        if (isset($exportGroups[$id])) {
            return $exportGroups[$id];
        }
    }
}

function getGroupTitleById($id = null) {
    $group = getExportGroupById($id);
    if ($group && $group['title']) {
        return $group['title'];
    }
}

function getExportGroupByTitle($title) {
    $exportGroups = get_option('_magic_page_groups', array());

    if ($exportGroups) {
        foreach ($exportGroups as $id => $exportGroup) {
            if ($exportGroup['title'] === $title) {
                return $exportGroups[$id];
            }
        }
    }
}

function getLocationByAttribute($attribute) {
    // find by title
    if ($location = get_term_by('name', $attribute, 'location')) {
        return $location;
    }
    // find by slug
    if ($location = get_term_by('slug', $attribute, 'location')) {
        return $location;
    }
    // find by term id
    if ($location = get_term_by('ID', $attribute, 'location')) {
        return $location;
    }

    // find by zip
    $args = array(
        'hide_empty' => false,
        'meta_query' => array(
            array(
                'key' => 'zip',
                'value' => $attribute,
                'compare' => '='
            )
        ),
        'taxonomy' => 'location',
    );
    $terms = get_terms($args);

    if ($terms) {
        return $terms[0];
    }
}

function shouldLoadCheck() {
    global $pagenow, $typenow;

    if (empty($typenow) && !empty($_GET['post'])) {
        $post = get_post($_GET['post']);

        if ($post) {
            $typenow = $post->post_type;
        }
    }


    if (($pagenow == 'post-new.php' or $pagenow == 'post.php') && $typenow == magic_page_default_post_slug()
        || ($pagenow == 'post-new.php' or $pagenow == 'post.php') && in_array($typenow, array('post', 'page'))) {
        
        return true;
    }

        
  

    return false;
}

function mpp_admin_notice($args) {
    ?>
    <div class="notice notice-success is-dismissible">
        <p><?php echo($args['notice']) ?></p>
    </div>
    <?php
    update_option($args['id'], $args['value'], false);
}

//////////////
// shorcut for schema plugin
function get_magic_page_term() {
    return getMagicpageLocation();
}

function getMagicpageLocation($requiresDefaultSelected = true) {
    global $wp_query, $post, $magic_page_term_object_exporting;

    // for posts/pages
    if (in_array(get_post_type(), array('post', 'page')) && $post) {
        // manually chosen
        if (($location = get_post_page_location())) {
            return $location;
        } else {
            // magic page default
            if (($term = getMagicPageDefaultTerm($requiresDefaultSelected))) {
                return $term;
            }
        }

        return false;
    }

    // exporting magic page
    if ($magic_page_term_object_exporting) {
        return $magic_page_term_object_exporting;
    }


    if ($wp_query) {
        if ($wp_query->get('magic_page_term')) {
            // from request for magic pages
            return $wp_query->get('magic_page_term');
        } else if (isset($wp_query->query['magicpage']) && $wp_query->query['magicpage'] && $wp_query->query['post_type'] === 'magicpage') {
            return get_term_by('slug', $wp_query->query['magicpage'], 'location');
        }
    }


    return getMagicPageDefaultTerm();
}

function getMagicPageDefaultTerm($requiresDefaultSelected = true) {
    $post = get_magic_page();

    if ($post) {

        $terms = wp_get_post_terms($post->ID, 'location');

        $defaultId = get_post_meta($post->ID, '_magic_page_default_location', true);

        if ($defaultId) {
            foreach ($terms as $term) {
                if ($term->term_id == $defaultId) {
                    return $term;
                }
            }
        }

        if (!$requiresDefaultSelected || !$terms) {
            return;
        }

        return $terms[0];
    }
}

function get_post_page_location() {
    global $post;

    if ($post && $post->ID)
        if (($locationId = get_post_meta($post->ID, '_location_id', true))) {
            return get_term($locationId, 'location');
        }
}

function createExportGroup($groupTitle) {
    $title = sanitize_text_field($groupTitle);
    $exportGroupId = sanitize_title($title);
    $exportGroups = get_option('_magic_page_groups', array());

    if ($exportGroupId && !isset($exportGroups[$exportGroupId])) {
        $exportGroups[$exportGroupId] = array(
            'id' => $exportGroupId,
            'title' => $title,
            'dateCreated' => new DateTime()
        );

        update_option('_magic_page_groups', $exportGroups);
    }

    return $exportGroupId;
}

function enablePostPageAdvancedSchemaBuilder($name, $label) {

    $enabled = get_option('_schematic_enabled_pages');
    $toEnable = [
        ['groupId' => 'single', 'objectType' => $name, 'label' => $label],
    ];

    $out = array_merge($enabled, $toEnable);
    update_option('_schematic_enabled_pages', $out);
}

function checkAdvancedSchemaBuilderEnabled($for, $type = 'single') {
    $enabledPages = get_option('_schematic_enabled_pages');
    if ($enabledPages) {
        foreach ($enabledPages as $page) {
            if ($page['groupId'] == $type && $page['objectType'] === $for) {
                return true;
            }
        }
    }
    return false;
}

function getPostTypeNamedByLabel($id) {
    $types = get_post_types(array(
        'public' => true,
            ), 'objects');

    foreach ($types as $type) {
        if ($type->labels->name == $id) {
            return $type->name;
        }
    }
}

function getMagicPageEditor() {
    return get_option('_magic_page_editor_style', 'classic');
}

function getMagicPageSitemapOptions() {
    $default = [];
    $default['Homepage'] = ['state' => 1, 'priority' => 1, 'changefreq' => 'daily'];

    $types = get_post_types(array(
        'public' => true,
            ), 'names');

    $exclude = ['attachment', 'exporttemplate'];

    foreach ($types as $type) {
        if (in_array($type, $exclude)) {
            unset($types[$type]);
        }
    }

    foreach ($types as $type) {
        $default[ucfirst($type)] = ['state' => 1, 'priority' => 1, 'changefreq' => 'daily'];
    }

    $options = get_option('_magic_page_xml_sitemap_options');

    if (!$options) {
        return $default;
    }

    return $options;
}

function getMagicPageSitemapOptionsHtml() {
    $options = getMagicPageSitemapOptions();
    $sitemapEnabled = get_option('_magic_page_xml_sitemap');
    ?><div class="sitemapEnabled" <?php if (!$sitemapEnabled) { ?>style="display:none" <?php } ?> id="rows-sitemap"> <?php
    foreach ($options as $option => $settings) {
        ?><div class="row-sitemap-label">
                <input name="_magic_page_xml_sitemap_options[<?php echo(strtolower($option)) ?>][state]" value="1" type="checkbox" <?php if (isset($settings['state']) && $settings['state']) { ?>checked="checked" <?php } ?> /><?php echo(ucfirst($option)) ?> <br />
            </div>
            <div class="row-sitemap">
                <label>Priority
                    <input name="_magic_page_xml_sitemap_options[<?php echo(strtolower($option)) ?>][priority]" value="<?php echo($settings['priority']) ?>" style="width:50px" type="number" placeholder="Priority" min="0.0" max="1" step="0.1" />
                </label>

                <label>Change Frequency
                    <select name="_magic_page_xml_sitemap_options[<?php echo(strtolower($option)) ?>][changefreq]">
                        <option <?php selected($settings['changefreq'], 'daily'); ?> value="daily">Daily </option>
                        <option <?php selected($settings['changefreq'], 'weekly'); ?> value="weekly">Weekly </option>
                        <option <?php selected($settings['changefreq'], 'monthly'); ?> value="monthly">Monthly</option>
                        <option <?php selected($settings['changefreq'], 'yearly'); ?> value="yearly">Yearly</option>
                    </select> 
                </label>
            </div>

            <?php
        }
        ?></div> <?php
}

if (!function_exists('option_exists')) {

    function option_exists($arg) {

        global $wpdb;
        $prefix = $wpdb->prefix;
        $db_options = $prefix . 'options';
        $sql_query = 'SELECT * FROM ' . $db_options . ' WHERE option_name LIKE "' . $arg . '"';

        $results = $wpdb->get_results($sql_query, OBJECT);

        if (count($results) === 0) {
            return false;
        } else {
            return true;
        }
    }

}

function generateRandomString($min = 6, $max = 15, $dashes = false, $numbers = false) {
    $characters = '0123456789abcdefghijklmnopqrstuvwxyz';
    $alphabetical = 'abcdefghijklmnopqrstuvwxyz';
    $randstring = '';

    if ($dashes) {
        $characters .= '-';
    }

    $to = rand($min, $max);

    for ($i = 0; $i < $to; $i++) {
        if ($i == 0 or!$numbers) { // no leading numbers
            $randstring .= $alphabetical[rand(0, strlen($alphabetical) - 1)];
        } else {
            $randstring .= $characters[rand(0, strlen($characters) - 1)];
        }
    }

    return $randstring;
}

function isThirdPartyCachePluginEnabled() {
    if (function_exists('rocket_clean_domain') || defined('W3TC')) {
        return true;
    }
}

function getMagicPageCacheEnabled() {

    if (!option_exists('_magic_page_cache')) {
        add_option('_magic_page_cache', '1');
    }

    return get_option('_magic_page_cache') === '1' ? true : false;
}

function useMagicPageCache() {
    if (!isThirdPartyCachePluginEnabled() && getMagicPageCacheEnabled()) {
        return true;
    }
}
