<?php

if (!defined('ABSPATH')) {
    exit;
}

class MagicPagePublic {

    public $term = '';

    public function __construct() {
        add_filter('pre_get_posts', array($this, 'get_posts'), 0);
        add_filter('template_redirect', array($this, 'template_redirect'));

        add_filter('wp_enqueue_scripts', array($this, 'load_scripts'));
        add_action('wp_head', array($this, 'head'));
        add_action('parse_query', array($this, 'parse'));
        add_filter('document_title_parts', array($this, 'title'));
        add_filter('wp_kses_allowed_html', array($this, 'allow_iframes_filter'));
        add_filter('get_post_metadata', array($this, 'custom_fields'), 10, 4);
        add_filter('wp_footer', array($this, 'check'));
    }

    function allow_iframes_filter($allowedposttags) {
        $allowedposttags['iframe'] = array(
            'align' => true,
            'width' => true,
            'height' => true,
            'frameborder' => true,
            'name' => true,
            'src' => true,
            'id' => true,
            'class' => true,
            'style' => true,
            'scrolling' => true,
            'marginwidth' => true,
            'marginheight' => true,
        );

        return $allowedposttags;
    }

    public function title($title_parts) {
        if (isset($title_parts['title'])) {
            $title_parts['title'] = get_magic_page_title($title_parts['title']);
        }
        return $title_parts;
    }

    public function checkMagicPageInsideRadius() {
        global $post, $wp_query;

        if (is_single() && get_post_type($post) === magic_page_default_post_slug()) {
            if (isset($wp_query->query['post_type']) && $wp_query->query['post_type'] == magic_page_default_post_slug()) {

                $_get_term = get_term_by('slug', $wp_query->query[magic_page_default_post_slug()], 'location');

                if ($_get_term) {
                    $locations = wp_get_post_terms($post->ID, 'location');
                    $query = new MagicPageQuery();
                    $query->setLocations($locations);
                    $query->setTerm($_get_term);
                    $query->setAutoinstalled(magic_page_is_autoinstalled_enabled());
                    $query->setQueryType('count');
                    $found = $query->getResults();

                    // inside the radius
                    if ($found) {
                        return $_get_term;
                    } else {
                        return false;
                    }
                } else {

                    if ($post->post_name === $wp_query->query[magic_page_default_post_slug()]) {
                        $term = getMagicPageDefaultTerm($post);

                        if ($term) {
                            return $term;
                        }
                    }

                    return false;
                }
            }
        } else {
            return true;
        }
    }

    public function get_posts($query) {


        if (isset($query->query['post_type']) && $query->query['post_type'] == magic_page_default_post_slug()) {

            $post = get_magic_page();

            if ($post) {
                // because it's actually location
                $query->set(magic_page_default_post_slug(), $post->post_name);
                $query->set('name', $post->post_name);
            }
        }

        return $query;
    }

    public function check() {

        if (get_option('_magic_page_key_error') && is_magic_page_installed()) {
            if (is_singular(array('magicpage', 'post', 'page'))) {
                global $post;
                if ($post->post_type === 'magicpage' || get_post_meta($post->ID, '_location_id', true) || get_post_meta($post->ID, '_group_id', true)) {
                    global $m64String;
                    echo(base64_decode($m64String));
                }
            }
        }
    }

    public function template_redirect() {
        global $wp_query;

        if (!$wp_query->get('magic_page_term')) {
            $term = $this->checkMagicPageInsideRadius();

            if (is_object($term)) {
                $wp_query->set('magic_page_term', $term);
                // allow users to view the page, fixes elementor error when trying to edit mp without location
            } else if ($term === false && !current_user_can('administrator')) {
                wp_redirect(get_option('_magic_page_outside_radius_redirect', get_home_url()));
            }
        }
    }

    public function head() {
        global $post;

        if (!empty($post)) {

            $term = getMagicpageLocation();

            createMetatags($post, $term, true);
        }
    }

    public function load_scripts() {
        global $wp_query, $post;

        $term = $wp_query->get('magic_page_term');

        if (!$post || !$term) {
            return false;
        }


        if (strpos($post->post_content, '[map') !== false) {


            $maps_api_key = get_option('_magic_page_google_maps_api_key');
            if ($maps_api_key) {

                $key = '?key=' . $maps_api_key;

                $term_meta = get_term_meta($term->term_id);
                $title = get_the_title($post->ID);

                if (!isset($term_meta['lat'][0])) {
                    wp_die('The location ' . $term->name . ' does not have latitude setting.');
                }

                if (!isset($term_meta['lon'][0])) {
                    wp_die('The location ' . $term->name . ' does not have longitude setting.');
                }

                $settings = get_magic_page_googlemap_parameters();

                global $wp_version;

                wp_register_script('magic-page', $settings['js_fullurl'], array('jquery'), $wp_version);
                wp_enqueue_script('googlemaps', 'https://maps.googleapis.com/maps/api/js' . $key, array('magic-page'), false);
                wp_localize_script('googlemaps', $settings['js_settings'], array(
                    $settings['js_latitude'] => $term_meta['lat'][0],
                    $settings['js_longitude'] => $term_meta['lon'][0],
                    $settings['js_title'] => $title
                ));
            }
        }
    }

    public function custom_fields($metadata, $object_id, $meta_key, $single) {
        if (in_array($meta_key, get_magic_page_meta_fields())) {

            $post = get_post($object_id);

            if (in_array($post->post_type, array('post', 'page'))) {

                if (get_post_meta($object_id, '_use_group_settings', true)) {

                    $groupId = get_post_meta($object_id, '_group_id', true);

                    if ($groupId) {
                        $options = get_option('_group_meta_fields_' . $groupId);

                        if (isset($options[$meta_key])) {
                            return $options[$meta_key];
                        }

                        return '';
                    }
                }
            }
        }

        return $metadata;
    }

    public function parse() {
        global $wp_query;

        // @todo
        // if (isset($wp_query->query) && isset($wp_query->query['post_type']) && $wp_query->query['post_type'] == magic_page_default_post_slug()) {
        remove_action('wp_head', 'rel_canonical');
        remove_action('wp_head', 'wp_shortlink_wp_head');
        //  }
    }

}

new MagicPagePublic();
