<?php

if (!defined('ABSPATH')) {
    exit;
}

class MagicPageQuery {

    protected $start;
    protected $itemsPerPage;
    protected $locations;
    protected $regions;
    protected $debug;
    protected $sql;
    protected $autoinstalled;
    protected $excludeAutoinstalled;
    protected $search;
    protected $queryType;
    protected $orderby;
    protected $orderbyType;
    protected $currentPage;
    protected $selects;
    protected $fields;
    protected $joins;
    protected $conditions;
    protected $term;
    protected $excludeTerm;
    protected $cacheTtl = 43200;
    protected $useCache = true;
    protected $offset = false;
    protected $radiusArray = array();
    protected $minRadius;
    protected $maxRadius;
    protected $majorCities;
    protected $counties;
    protected $isStaticPage = false;
    protected $groupIds = '';
    protected $postType = '';
    protected $latLonMissing = false;
    protected $tempQuery;
    protected $tempQuery2;
    protected $filter1;
    protected $filter2; // pick all locations not installed
    protected $excludeTerms;

    public function __construct($debug = false, $useCache = true) {
        $this->debug = $debug;
        $this->useCache = $useCache;
    }

    protected function getExcludeTerms() {
        return $this->excludeTerms;
    }

    public function setExcludeTerms($excludeTerms) {
        $this->excludeTerms = $excludeTerms;
    }

    public function setFilter1($filter1) {
        $this->filter1 = $filter1;
    }

    protected function getFilter1() {
        return $this->filter1;
    }

    public function setFilter2($filter2) {
        $this->filter2 = $filter2;
    }

    protected function getFilter2() {
        return $this->filter2;
    }

    public function getPostType() {
        return $this->postType;
    }

    public function setPostType($postType) {
        $this->postType = $postType;
    }

    protected function getPostTypeSql() {
        if (($postType = $this->getPostType()) && $this->getIsStaticPage()) {
            if (!$this->getGroupIds()) {
                $this->setTempQuery2();
            }

            if (is_array($postType)) {
                if (!$this->getFilter2()) {
                    $string = "'" . implode("', '", $postType) . "'";
                    $this->conditions .= " AND pm23.post_type IN($string) ";
                } else {
                    //
                }
            } else {
                $this->conditions .= " AND pm23.post_type ='$postType' ";
            }

            if (!$this->getFilter2()) {
                $this->conditions .= " AND pm23.post_status = 'publish'  ";
            } else {
                // @todo having
                $id = 0;
                $conditions = '';
                if ($this->getLocations()) {

                    foreach ($this->getLocations() as $item) {
                        $id++;

                        if (!is_object($item) || !isset($item->term_id)) {
                            $radius = $item['radius'];
                            $item = get_term_by('id', $item['id'], 'location');
                        } else {
                            $radius = get_term_meta($item->term_id, 'distance', true);
                        }

                        if ($radius) {
                            $lat = get_term_meta($item->term_id, 'lat', true);
                            $lon = get_term_meta($item->term_id, 'lon', true);

                            if ($lat && $lon) {

                                $conditions .= " radius_$id > $radius AND ";
                            }
                        }
                    }
                }


                $this->conditions .= " HAVING($conditions  pm61.post_id IS NULL )";
            }
        }
    }

    public function setGroupIds($groupIds) {
        $this->groupIds = $groupIds;
    }

    protected function getGroupIds() {
        return $this->groupIds;
    }

    public function setIsStaticPage($isStaticPage) {
        $this->isStaticPage = $isStaticPage;
    }

    protected function getIsStaticPage() {
        return $this->isStaticPage;
    }

    public function setCounties($counties) {
        $this->counties = $counties;
    }

    protected function getCounties() {
        return $this->counties;
    }

    protected function getGroupIdsSql() {
        global $wpdb;

        if ($this->getPostType() && $this->getIsStaticPage()) {
            $this->fields .= ' pm22.*,';

            $this->setTempQuery();
            $this->joins .= " LEFT JOIN $wpdb->postmeta AS pm22 ON (pm22.post_id = pm61.post_id AND pm22.meta_key='_group_id') ";

            $this->setTempQuery2();

            if (($groupIds = $this->getGroupIds())) {
                $this->conditions .= " AND pm22.meta_value REGEXP '$groupIds' ";
            }
        }

        if ($this->getFilter1()) {
            $this->conditions .= " GROUP BY $wpdb->terms.term_id ";
        }
    }

    public function setTempQuery2() {

        $this->setTempQuery();

        if (!$this->tempQuery2) {
            global $wpdb;
            $this->fields .= ' pm61.*, pm23.ID, pm23.post_status, pm23.post_type,';
            $this->joins .= " LEFT JOIN $wpdb->posts AS pm23 ON (pm23.ID = pm61.post_id) ";

            $this->tempQuery2 = true;
        }
    }

    protected function getCountiesSql() {
        if ($this->getCounties()) {
            global $wpdb;

            $counties = explode(',', $this->getCounties());

            if (in_array('%county%', $counties) && $this->getExcludeTerm()) {
                $currentCounty = get_term_meta($this->getExcludeTerm()->term_id, 'county', true);

                $key = array_search('%county%', $counties);
                unset($counties[$key]);

                if ($currentCounty) {
                    $counties[] = $currentCounty;
                }
            }

            $res = "'" . implode("', '", $counties) . "'";

            $this->fields .= "pm8.meta_value as county, ";
            $this->joins .= " LEFT JOIN $wpdb->termmeta AS pm8 ON ($wpdb->terms.term_id = pm8.term_id AND pm8.meta_key = 'county') ";
            $this->conditions .= " AND pm8.meta_value IN($res) ";
        }
    }

    public function setMajorCities($majorCities) {
        $this->majorCities = $majorCities;
    }

    protected function getMajorCities() {
        return $this->majorCities;
    }

    protected function getMajorCitiesSql() {
        if ($this->getMajorCities() === "true" || $this->getMajorCities() === "false") {
            global $wpdb;
            $this->fields .= "pm7.meta_value as major_city, ";
            $this->joins .= " LEFT JOIN $wpdb->termmeta AS pm7 ON ($wpdb->terms.term_id = pm7.term_id AND pm7.meta_key = 'major_city') ";
        }
        if ($this->getMajorCities() === "true") {
            $this->conditions .= ' AND pm7.meta_value IS NOT NULL ';
        } elseif ($this->getMajorCities() === "false") {
            $this->conditions .= ' AND pm7.meta_value IS NULL ';
        }
    }

    protected function setTempQuery() {
        global $wpdb;

        if (!$this->tempQuery) {
            $this->joins .= " LEFT JOIN $wpdb->postmeta AS pm61 ON ($wpdb->terms.term_id = pm61.meta_value AND pm61.meta_key='_location_id') ";
            $this->tempQuery = true;
        }
    }

    public function setMinRadius($minRadius) {
        $this->minRadius = intval($minRadius);
    }

    protected function getMinRadius() {
        return $this->minRadius;
    }

    public function setMaxRadius($maxRadius) {
        $this->maxRadius = intval($maxRadius);
    }

    protected function getMaxRadius() {
        return $this->maxRadius;
    }

    public function setStart($start = null) {
        $this->start = intval($start);
    }

    protected function getItemsPerPage() {
        return $this->itemsPerPage;
    }

    public function setItemsPerPage($itemsPerPage) {
        $this->itemsPerPage = intval($itemsPerPage);
    }

    protected function getCurrentPage() {
        return $this->currentPage;
    }

    public function setLocations($locations) {
        if (!($locations instanceof \WP_Error)) {

            if (!is_array($locations)) {
                $locations = array($locations);
            }

            $this->locations = $locations;
        }
    }

    protected function getLocations() {
        return $this->locations;
    }

    protected function getStart() {
        // pagination type
        if (!$this->offset) {
            if ($this->getCurrentPage()) {
                return $this->getItemsPerPage() * $this->getCurrentPage() - $this->getItemsPerPage();
            }
        }

        // regular offset
        return $this->start;
    }

    public function setOffset($start) {
        $this->setStart($start);
        $this->offset = true;
    }

    protected function getSql() {
        return $this->sql;
    }

    public function setCurrentPage($currentPage = null) {
        $this->currentPage = intval($currentPage);
    }

    public function setRegions($regions) {
        $this->regions = $regions;
    }

    protected function getRegions() {
        return $this->regions;
    }

    public function setAutoinstalled($autoinstalled) {


        $this->autoinstalled = $autoinstalled;
    }

    protected function getAutoinstalled() {
        return $this->autoinstalled;
    }

    protected function getExcludeAutoinstalled() {
        return $this->excludeAutoinstalled;
    }

    public function setExcludeAutoinstalled($excludeAutoinstalled) {
        $this->excludeAutoinstalled = $excludeAutoinstalled;
    }

    public function setQueryType($queryType) {
        $this->queryType = $queryType;
    }

    protected function getQueryType() {
        return $this->queryType;
    }

    public function setOrderby($orderby) {
        $this->orderby = $orderby;
    }

    protected function getOrderby() {
        return $this->orderby;
    }

    public function setOrderbyType($orderbyType) {
        $this->orderbyType = $orderbyType;
    }

    protected function getOrderbyType() {
        return $this->orderbyType;
    }

    public function setSearch($search) {
        $this->search = $search;
    }

    protected function getSearch() {
        return $this->search;
    }

    protected function getTerm() {
        return $this->term;
    }

    public function setTerm($term) {
        $this->term = $term;
    }

    protected function getExcludeTerm() {
        return $this->excludeTerm;
    }

    public function setExcludeTerm($excludeTerm) {
        $this->excludeTerm = $excludeTerm;
    }

    public function generateRadiusComparison($currentIndex, $maxIndex) {
        $string = '';

        for ($i = 1; $i <= $maxIndex; $i++) {
            if ($i !== $currentIndex && in_array($i, $this->radiusArray)) {
                $string .= "radius_$currentIndex < radius_$i and ";
            }
        }

        return rtrim($string, ' and ');
    }

    protected function getLocationsSql() {

        if (($this->getLocations() || $this->getExcludeTerm())) {
            $id = 0;

            global $wpdb;
            $fields = '';
            $conditions = 'HAVING(';

            $sql_5_array = array();
            $sql_4_array = [];

            if ($this->getLocations()) {

                foreach ($this->getLocations() as $item) {
                    $id++;

                    if (!is_object($item) || !isset($item->term_id)) {
                        $radius = $item['radius'];
                        $item = get_term_by('id', $item['id'], 'location');
                    } else {
                        $radius = get_term_meta($item->term_id, 'distance', true);
                    }

                    if ($radius && !$this->getFilter1()) {
                        $lat = get_term_meta($item->term_id, 'lat', true);
                        $lon = get_term_meta($item->term_id, 'lon', true);

                        if ($lat && $lon) {
                            if (count($this->radiusArray) <= 8) { // too many chosen
                                $this->radiusArray[] = $id;
                                $fields .= "3959 * acos(cos(radians($lat)) * cos(radians(pm1.meta_value)) * cos(radians(pm2.meta_value) - radians($lon)) + sin(radians($lat)) * sin(radians(pm1.meta_value))) as radius_$id, ";

                                if ($this->isMagicPagePouintingToStaticPages()) {
                                    if ($this->getExcludeTerm()) {
                                        //$conditions .= " OR (radius_$id <= $radius AND $wpdb->terms.term_id NOT IN(%REPLACE_IDS%))";
                                        $conditions .= " OR (radius_$id <= $radius)";
                                    } else {
                                        $conditions .= " OR (radius_$id <= $radius)";
                                    }
                                }
                            }
                            $sql_4_array[] = $item->term_id;
                        }
                    }

                    $sql_5_array[] = $item->term_id;
                }
            }

            $minRadiusSqlStart = '';
            $minRadiusSqlEnd = '';

            $item = $this->getItem();

            if ($this->getOrderby() === 'radiusOrderbyClosest' && $item && !$this->getFilter1()) {

                $lat = get_term_meta($item->term_id, 'lat', true);
                $lon = get_term_meta($item->term_id, 'lon', true);

                if ($lat && $lon) {
                    $fields .= "3959 * acos(cos(radians($lat)) * cos(radians(pm1.meta_value)) * cos(radians(pm2.meta_value) - radians($lon)) + sin(radians($lat)) * sin(radians(pm1.meta_value))) as radius_orderby_closest, ";

                    if (($minRadius = $this->getMinRadius())) {
                        $minRadiusSqlStart = "radius_orderby_closest >= $minRadius AND ";
                    }

                    if (($maxRadius = $this->getMaxRadius())) {
                        $minRadiusSqlStart .= "radius_orderby_closest <= $maxRadius AND ";
                    }

//
//                    if ($minRadius || $maxRadius) {
//                        $minRadiusSqlStart = $minRadiusSqlStart . '(';
//                        $minRadiusSqlEnd = ')';
//                    }
                } else {
                    $this->latLonMissing = true;
                }
            }

            $replace = '';
            if (($sql_5_array && $this->isMagicPagePouintingToStaticPages()) || $this->getFilter1()) {
                $conditions .= " OR ($wpdb->terms.term_id IN(%REPLACE_IDS%))";
                $replace = implode(', ', $sql_5_array);
                $this->chosenLocationsIds = $sql_5_array;
                $conditions = str_replace('%REPLACE_IDS%', $replace, $conditions);
                $conditions = str_replace('HAVING( OR ', 'HAVING(' . $minRadiusSqlStart, $conditions);
            } else {
                $conditions = 'HAVING(' . $minRadiusSqlStart . ')';
            }

            if ($sql_4_array) {
                $this->chosenLocationsIdsWithRadius = $sql_4_array;
            }


//
            $this->fields .= $fields;
            $this->conditions .= rtrim($conditions, ' OR ') . ')' . $minRadiusSqlEnd;
            $this->conditions = str_replace('HAVING()', '', $this->conditions);
            $this->conditions = str_replace('AND )', '', $this->conditions);
        }
    }

    /**
     * Current Page is Magic Page pointing to static pages
     * @return boolean
     */
    protected function isMagicPagePouintingToStaticPages() {
        if ($this->getPostType() !== 'magicpage' && !$this->getIsStaticPage()) {
            return true;
        }
    }

    protected function getItem() {
        $item = '';

        if ($this->getExcludeTerm()) {
            $item = $this->getExcludeTerm();
        } else {
            if (count($this->getLocations()) == 1) {
                $loc = $this->getLocations()[0];

                if (!is_object($loc) || !isset($loc->term_id)) {
                    $item = get_term_by('id', $loc['id'], 'location');
                }
            }
        }

        return $item;
    }

    protected function getSearchSql() {
        global $wpdb;
        if ($this->getSearch()) {
            if ($this->getSearch()) {

                if ($this->getFilter1()) {
                    $this->conditions .= " AND ($wpdb->terms.name LIKE '%$this->search%') ";
                } else {
                    $this->conditions .= " AND (lat LIKE '%$this->search%' OR $wpdb->terms.name LIKE '%$this->search%') ";
                }
            }
        }
    }

    protected function getRegionsSql() {
        if ($this->getRegions()) {
            $array = array_unique(array_map('trim', $this->getRegions()));
            $string = "'" . implode("','", $array) . "'";
            $this->conditions .= " AND (pm4.meta_value IN ($string)) ";
        }
    }

    protected function getAutoinstalledSql() {
        if ($this->getAutoinstalled() || $this->getOrderby() === 'autoinstalled' || $this->getExcludeAutoinstalled()) {
            global $wpdb;
            $this->fields .= "pm5.meta_value as autoinstalled, ";
            $this->joins .= "LEFT JOIN $wpdb->termmeta AS pm5 ON ($wpdb->terms.term_id = pm5.term_id AND pm5.meta_key = 'autoinstalled')";
        }
        if ($this->getAutoinstalled()) {
            $this->conditions .= 'AND pm5.meta_key IS NOT NULL ';
        } elseif ($this->getExcludeAutoinstalled()) {
            $this->conditions .= 'AND pm5.meta_key IS NULL ';
        }
    }

    protected function getCountQuerySql() {
        if ($this->getQueryType() === 'count') {
            $this->selects = 'SELECT COUNT(*) AS CNT FROM(';
        }
    }

    protected function getOrderbySql() {
        $orderby = $this->getOrderby();
        $orderbysql = '';

        if ($orderby === 'radiusOrderbyClosest' && $this->getItem() && !$this->latLonMissing) {
            $orderbysql = 'ORDER BY radius_orderby_closest';
        } else {
            if ($orderby === 'radius') {
                $maxIndex = count($this->chosenLocationsIds);

                if ($maxIndex > 1) {
                    $orderbyLocationIdsSql = '';
                    if ($this->chosenLocationsIds) {
                        global $wpdb;
                        foreach ($this->chosenLocationsIds as $id) {
                            $orderbyLocationIdsSql .= "$wpdb->terms.term_id = $id DESC, ";
                        }
                    }

                    if ($this->radiusArray && count($this->chosenLocationsIdsWithRadius) >= 2) {
                        $orderbysql .= "ORDER BY $orderbyLocationIdsSql case";

                        for ($i = 1; $i <= $maxIndex; $i++) {
                            if (in_array($i, $this->radiusArray)) {
                                $makeString = $this->generateRadiusComparison($i, $maxIndex);
                                $orderbysql .= " when $makeString then radius_$i";
                            }
                        }
                        $orderbysql .= ' END';
                    }
                } else {

                    if ($this->radiusArray) {
                        $orderbysql = 'ORDER BY radius_1';
                    }
                }
            } elseif (in_array($orderby, array('name', 'term_id'))) {
                global $wpdb;
                $orderbysql = "ORDER BY $wpdb->terms.$orderby";
            } elseif (in_array($orderby, array('zip', 'lat', 'lon', 'region', 'autoinstalled'))) {
                $orderbysql = "ORDER BY $orderby";
            }
        }

        if ($this->getOrderbyType() && $orderbysql) {
            $this->conditions .= ' ' . $orderbysql . ' ' . $this->getOrderbyType();
        }
    }

    protected function getMainSql() {
        global $wpdb;

        $this->fields = rtrim($this->fields, ', ');

        if (!$this->getFilter1()) {
            $fields = "$wpdb->terms.term_id, $wpdb->terms.name, $wpdb->terms.slug, pm1.meta_value as lat, pm2.meta_value as lon, pm3.meta_value as zip, pm4.meta_value as region,";
        } else {
            $fields = "$wpdb->terms.term_id, $wpdb->terms.name, $wpdb->terms.slug, ";
        }

        if (!$this->fields) {
            $fields = rtrim($fields, ',');
        }

        $this->conditions = ltrim($this->conditions, ')');

        $filter1 = '';
        if (!$this->getFilter1()) {
            $filter1 = " LEFT JOIN $wpdb->termmeta AS pm1 ON ($wpdb->terms.term_id = pm1.term_id AND pm1.meta_key = 'lat')
            LEFT JOIN $wpdb->termmeta AS pm2 ON ($wpdb->terms.term_id = pm2.term_id AND pm2.meta_key = 'lon')
            LEFT JOIN $wpdb->termmeta AS pm3 ON ($wpdb->terms.term_id = pm3.term_id AND pm3.meta_key = 'zip')
            LEFT JOIN $wpdb->termmeta AS pm4 ON ($wpdb->terms.term_id = pm4.term_id AND pm4.meta_key = 'region') ";
        }


        $this->sql .= "$this->selects SELECT $fields $this->fields
       FROM $wpdb->terms
            $filter1
            LEFT JOIN $wpdb->term_taxonomy AS pm6 ON ($wpdb->terms.term_id = pm6.term_id AND pm6.taxonomy = 'location')
            $this->joins
            WHERE pm6.taxonomy IS NOT NULL
            $this->conditions";
    }

    protected function getLimitSql() {
        if ($this->getItemsPerPage()) {
            $start = $this->getStart();
            $end = $this->getItemsPerPage();
            $this->conditions .= " LIMIT  $start, $end";
        }
    }

    protected function getEndSql() {
        if ($this->getQueryType() === 'count') {
            $this->conditions .= ') as t_alias';
        }
    }

    protected function getKey() {
        return '_magicpage_cache_query_' . md5(serialize($this->sql));
    }

    protected function getTermSql() {
        if ($this->getTerm()) {
            global $wpdb;
            $this->conditions .= " AND $wpdb->terms.term_id = " . $this->getTerm()->term_id;
        }
    }

    protected function getExcludeTermSql() {
        if ($this->getExcludeTerm()) {
            global $wpdb;
            $this->conditions .= " AND $wpdb->terms.term_id != " . $this->getExcludeTerm()->term_id;
        } else if ($this->getExcludeTerms()) {
            global $wpdb;

            $locationIds = [];
            foreach ($this->getExcludeTerms() as $location) {
                $locationIds[] = $location->term_id;
            }

            $locationIds = implode(',', $locationIds);

            $this->conditions .= " AND $wpdb->terms.term_id NOT IN($locationIds)";
        }
    }

    public function getResults() {
        global $wpdb;

        if (!$this->locations) {
            if ($this->getQueryType() === 'count') {
                if ($this->debug) {
                    echo($wpdb->last_query);
                    exit();
                }
                return '0';
            } else {
                ///
            }
        }

        $this->getCountQuerySql();
        $this->getAutoinstalledSql();
        $this->getLocationsSql();
        $this->getSearchSql();

        $this->getPostTypeSql();

        $this->getRegionsSql();
        $this->getCountiesSql();
        $this->getTermSql();
        $this->getExcludeTermSql();

        $this->getMajorCitiesSql();
        $this->getGroupIdsSql();

        $this->getOrderbySql();

        $this->getLimitSql();
        $this->getEndSql();
        $this->getMainSql();

        if (!$this->debug && get_transient($this->getKey()) && $this->useCache && !isMagicPageDebug()) {
            return get_transient($this->getKey());
        }

        $results = $wpdb->get_results($this->sql);

        if ($this->debug) {
            echo($wpdb->last_query);
            exit();
        }

        if ($results && $this->getQueryType() === 'count') {
            $results = $results[0]->CNT;
        }

        if (!$this->debug && $this->useCache && !isMagicPageDebug()) {
            set_transient($this->getKey(), $results, $this->cacheTtl);
        }


        return $results;
    }

    /*
     * Allows no more than 50% manually selected locations
     */

    public function getFilteredResults() {
        $out = [];
        $locationIds = array();
        $manuallyPickedLocationsIds = array();

        foreach ($this->getLocations() as $loc) {
            $radius = '';
            $id = '';

            if (is_array($loc) && isset($loc['id']) && isset($loc['radius'])) {
                $radius = $loc['radius'];
                $id = $loc['id'];
            } else if (is_object($loc)) {

                $radius = get_term_meta($loc->term_id, 'distance', true);
                $id = $loc->term_id;
            }

            // manually picked without radius
            if (!$radius && $id) {
                $manuallyPickedLocationsIds[] = $id;
            }
        }

        // no manually picked locations
        if (!$manuallyPickedLocationsIds) {
            return $this->getResults();
        }


        $itemsPerPage = $this->getItemsPerPage();
        $allowedManuallySelectedLocations = ceil($itemsPerPage / 2);
        $remainingLocations = $itemsPerPage - $allowedManuallySelectedLocations;

        $this->setItemsPerPage($itemsPerPage + count($manuallyPickedLocationsIds)); // so we have extra locations
        $results = $this->getResults();

        if ($results) {
            foreach ($results as $result) {
                if (in_array($result->term_id, $manuallyPickedLocationsIds)) {
                    if ($allowedManuallySelectedLocations > 0 && !in_array($result->term_id, $locationIds)) {
                        $out[] = $result;
                        $locationIds[] = $result->term_id;
                        $allowedManuallySelectedLocations--;
                    }
                } elseif (!in_array($result->term_id, $manuallyPickedLocationsIds)) {
                    if ($remainingLocations > 0 && !in_array($result->term_id, $locationIds)) {
                        $out[] = $result;
                        $locationIds[] = $result->term_id;
                        $remainingLocations--;
                    }
                }
            }

            // fill missing slots with manually selected if results are not enough
            if (count($out) < $itemsPerPage) {
                foreach ($results as $result) {
                    if (count($out) < $itemsPerPage) {
                        if (!in_array($result->term_id, $locationIds)) {
                            $out[] = $result;
                            $locationIds[] = $result->term_id;
                        }
                    }
                }
            }
        }


        return $out;
    }

}
