<?php

if (!defined('ABSPATH')) {
    exit;
}

use samdark\sitemap\Sitemap;

class MagicPageXmlSitemap {

    public function __construct() {
        add_action('parse_request', function ($wp) {

            if (preg_match('#^sitemap/(.*)/?#', $wp->request, $matches)) {
                if ($matches[1] === 'sitemap.xml') {
                    $this->create();
                }
            }
        });
    }

    protected function create() {
        $allowedPostTypes = getMagicPageSitemapOptions();

        $enabled = get_option('_magic_page_xml_sitemap');
        if (!$enabled) {
            wp_die('Sitemap feature is disabled');
        }

        $dir = wp_get_upload_dir();
        $magic_page = get_magic_page();

        if ($allowedPostTypes) {

            $sitemapItems = [];

            foreach ($allowedPostTypes as $type => $options) {
                if (!isset($options['state'])) {
                    continue;
                }

                if ($type === 'magicpage') {
                    $magic_page = get_magic_page();
                    if ($magic_page) {
                        $locations = wp_get_post_terms($magic_page->ID, 'location');
                        $query = new MagicPageQuery();
                        $query->setLocations($locations);
                        $query->setAutoinstalled(magic_page_is_autoinstalled_enabled());
                        $query->setQueryType('result');
                        $results = $query->getResults();

                        if ($results) {
                            foreach ($results as $result) {
                                $href = get_term_link(intval($result->term_id), 'location');

                                $sitemapItems[] = [
                                    'href' => $href,
                                    'time' => '',
                                    'rate' => $options['changefreq'],
                                    'priority' => $options['priority']
                                ];
                            }
                        }
                    }
                } else {

                    if ($type === 'homepage') {
                        $sitemapItems[] = [
                            'href' => get_home_url(),
                            'rate' => $options['changefreq'],
                            'priority' => $options['priority']];
                    } else {
                        $args = array(
                            'post_type' => $type,
                            'post_status' => 'publish',
                            'numberposts' => -1,
                            'orderby' => 'modified'
                        );
                        $posts = get_posts($args);
                        if ($posts) {
                            foreach ($posts as $post) {
                                $href = get_permalink($post);
                                $sitemapItems[] = [
                                    'href' => $href,
                                    'time' => $post->post_modified,
                                    'rate' => $options['changefreq'],
                                    'priority' => $options['priority']];
                            }
                        }
                    }
                }
            }
        }


        if ($sitemapItems) {
            $sitemap = new Sitemap($dir['basedir'] . '/sitemap.xml');
            foreach ($sitemapItems as $item) {
                $datetime = new DateTime();
                $now = $datetime->format('Y-m-d H:i:s');
                $time = isset($item['time']) && $item['time'] ? strtotime($item['time']) : strtotime($now);
                $sitemap->addItem($item['href'], $time, $options['changefreq'], $options['priority']);
            }



            $sitemap->write();
            header('Content-type: text/xml');

            $contents = file_get_contents($dir['basedir'] . '/sitemap.xml');

            echo($contents);
            exit();
        }
    }

}

new MagicPageXmlSitemap();
