<?php

if (!defined('ABSPATH')) {
    exit;
}

class MagicPageSpintax {

    private $location;
    private $protectedBlocks = false;
    private $cache = array();
    private $cacheKey = null;
    private $string;
    private $exp = '/\{(((?>[^\{\}]+)|(?R))*)\}/x';
    private $cacheUpdated = false;
    private $spintaxExpressions = array();
    private $hashes = array();
    private $post = array();

    public function __construct($string = null, $location = null, $regularPost = null) {
        $this->cacheKey = '_magic_page_spintax_cache';

        $this->string = $string;

        if ($regularPost) {
            $this->post = $regularPost;

            $this->cache = get_post_meta($this->post->ID, $this->cacheKey, true);

            if (!$this->cache) {
                $this->cache = array();
            }

            /// $this->spintaxExpressions = array();
        } else {

            if (is_object($location) && !empty($location)) {
                $location = $location->term_id;
                $this->location = $location;
                $this->cacheKey .= '_' . $this->location;
            }

            $uniqueContentNeeded = apply_filters('unique_content_needed', '');
            if ($uniqueContentNeeded !== true) {
                $this->cache = get_option($this->cacheKey, array());
            } else {
                $this->cache = array();
            }
        }

        $this->spintaxExpressions = get_option('_magic_page_spintax_expressions', array());
    }

    
    
    /**
     * @todo unused
     * @global type $wpdb
     * @param type $post
     * @return boolean
     */
    public function deleteUnusedHashCache($post) {
        if ($this->post) {
            return false;
        }

        // in magicpage title 
        $this->getHashesFromString($post->post_title);

        // in magicpage content 
        $content = $post->post_content;

        $this->getHashesFromString($content);


        // in metafeilds
        $fields = get_magic_page_meta_fields();


        foreach ($fields as $field) {
            $content = get_post_meta($post->ID, $field, true);
            $this->getHashesFromString($content);
        }

        //  in xfields
        $xfields = get_option('_magic_page_xfields');

        if ($xfields) {
            foreach ($xfields as $xfield) {
                $this->getHashesFromString($xfield['value']);
            }
        }

        $this->hashes = array_unique($this->hashes);

        $existingHashes = get_option('_magic_page_spintax_hashes', array());

        if (empty($existingHashes) && empty($this->spintaxExpressions)) {
            $this->deleteAllHashCache();
        }

        $deletedKeys = array_diff($existingHashes, $this->hashes);

        if ($deletedKeys) {
            global $wpdb;

            $like = implode('|', $deletedKeys);
            $results = $wpdb->get_results("SELECT * FROM $wpdb->options WHERE option_value REGEXP '$like'");

            if ($results) {
                foreach ($results as $result) {
                    $values = unserialize($result->option_value);

                    foreach ($deletedKeys as $deletedKey) {
                        if (isset($values[$deletedKey])) {
                            unset($values[$deletedKey]);
                        }
                    }

                    if ($values) {
                        $wpdb->update(
                                $wpdb->options, array('option_value' => serialize($values)), array('option_id' => $result->option_id), array('%s'), array('%d')
                        );
                    } else {
                        $this->deleteHashRow($result->option_id);
                    }
                }
            }
        }


        update_option('_magic_page_spintax_hashes', $this->hashes, 'no');
    }

    public function deleteAllHashCache() {
        global $wpdb;
        $results = $wpdb->get_results("SELECT * FROM $wpdb->options WHERE option_name LIKE '_magic_page_spintax_cache%'");
        if ($results) {
            foreach ($results as $result) {
                $this->deleteHashRow($result->option_id);
            }
        }
    }

    public function deleteHashRow($option_id) {
        global $wpdb;
        $wpdb->delete($wpdb->options, array('option_id' => $option_id));
    }

    public function getHashesFromString($string) {

        $string = $this->removeVerbatimBlocks($string);

        // normal syntax
        $this->expressions = false;
        $string = $this->processHashes($string);
    }

    public function processHashes($string) {
        return preg_replace_callback($this->exp, array($this, 'replaceHashes'), $string);
    }

    public function replaceHashes($string) {
        if (!$this->expressions && strpos($string[0], '{spintax_') === false) {
            $string = $this->processHashes($string[1]);

            return $this->getRandomNormalExpressionsHashes($string);
        }
    }

    public function getRandomNormalExpressionsHashes($string) {
        if ($string) {
            $this->hashes[] = $this->getHash($string);
        }
    }

    public function spin($lastPass = false) {
        $string = $this->string;


        if (!isSpinNeeded($string)) {
            return $string;
        }

        $string = $this->removeVerbatimBlocks($string);

        // normal syntax
        $this->expressions = false;
        $string = $this->process($string);


        //if (!$this->post) {
        // with expressions
        $this->expressions = true;
        $string = $this->process($string);
        //}

        $string = $this->returnVerbatimProtectedBlocks($string, $lastPass);

        $this->updateCache();

        return $string;
    }

    public function process($string) {
        return preg_replace_callback($this->exp, array($this, 'replace'), $string);
    }

    public function replace($string) {

        if (!isSpinNeeded($string[0])) {
            return $string[0];
        }


        // normal syntax
        if (!$this->expressions && strpos($string[0], '{spintax_') === false) {
            $string = $this->process($string[1]);

            return $this->getRandomNormalExpressions($string);
        }

        //  with expressions
        if (!$this->expressions && strpos($string[0], '{spintax_') !== false) {
            return $this->getRandomAdvancedExpressions($string);
        }
    }

    public function getRandomNormalExpressions($string) {
        $hash = $this->getHash($string);

        $parts = explode('|', $string);

        return $this->getCachedIndex($hash, $parts);
    }

    public function getRandomAdvancedExpressions($spintax) {
        $label = str_replace('spintax_', '', $spintax[1]);

        if (!empty($this->spintaxExpressions)) {
            foreach ($this->spintaxExpressions as $hash => $spintax) {
                if ($spintax['label'] === $label) {
                    if (isset($spintax['options']) and $spintax['options']) {
                        $parts = array_values($spintax['options']);

                        $string = $this->getCachedIndex($hash, $parts);

                        // apply regular synax spinning if needed
                        $this->expressions = false;
                        return $this->process($string);
                    }

                    $this->expressions = true;
                }
            }
        }
    }

    public function getCachedIndex($hash, $parts) {
        $uniqueContentNeeded = apply_filters('unique_content_needed', '');

        if (isset($this->cache[$hash]) && $uniqueContentNeeded !== true) {
            $savedIndex = $this->cache[$hash];
            if (isset($parts[$savedIndex])) {
                return $parts[$savedIndex];
            }
        }

        $index = array_rand($parts);
        $randomIndex = $parts[$index];

        $this->cache[$hash] = $index;

        $this->cacheUpdated = true;

        return $randomIndex;
    }

    public function updateCache() {
        $uniqueContentNeeded = apply_filters('unique_content_needed', '');

        if ($uniqueContentNeeded !== true) {
            if ($this->post) {
                update_post_meta($this->post->ID, $this->cacheKey, $this->cache);
            } else {
                if ($this->cacheUpdated) {
                    update_option($this->cacheKey, $this->cache, 'no');
                }
            }
        }
    }

    public function getHash($text) {
        return substr(md5($text), 0, 6);
    }

    public function removeVerbatimBlocks($string) {
        if (is_array($string)) {
            return $string;
        }

        $pattern = '/({%verbatim%})(.*?)({%endverbatim%})/si';
        preg_match_all($pattern, $string, $matches);

        if ($matches[0]) {

            $originalStatements = $matches[0];
            $this->protectedBlocks = array();

            foreach ($originalStatements as $id => $statement) {
                $string = str_replace($statement, '__protected_block__' . $id, $string);
                $this->protectedBlocks['__protected_block__' . $id] = $statement;
            }
        }

        return $string;
    }

    public function returnVerbatimProtectedBlocks($string, $lastPass) {
        if (is_array($string)) {
            return $string;
        }

        if ($this->protectedBlocks) {
            foreach ($this->protectedBlocks as $id => $replacement) {
                $string = str_replace($id, $replacement, $string);
            }

            // keep verbatim untill final pass
            if (!$lastPass) {
                $string = '{%verbatim%}' . $string . '{%endverbatim%}';
            }

            $string = preg_replace('/({%verbatim%})(.*?)({%endverbatim%})/si', '$2', $string);
        }
        return $string;
    }

}
