<?php

if (!defined('ABSPATH')) {
    exit;
}

class ProcessJsonLdSchema {

    protected $quotes = "\"";
    protected $quotesComma = "\",";
    protected $jsonString = '';
    protected $hoursString = '';
    protected $hasSameAsLinks = false;
    protected $string = '';
    protected $response = '';
    protected $post;
    protected $term;

    public function process($post = null, $term) {
        $this->post = $post;
        $this->term = $term;

        if (!$post || !$this->getProperty('_magic_page_schema_enabled')) {
            return;
        }

        $this->hoursString = "[";

        for ($i = 0; $i <= 5; $i++) {
            $item = $this->getProperty('_magic_page_schema_social_media_links_' . $i);

            if ($item) {
                $this->string .= "\"" . $item . "\",";
                $this->hasSameAsLinks = true;
            }
        }

        if ($this->hasSameAsLinks) {
            $this->sameAsComma = ",";
        } else {
            $this->sameAsComma = "";
        }

        for ($i = 1; $i <= 7; $i++) {
            $rangeDays = array();
            $item = $this->getProperty('_magic_page_schema_range_' . $i);

            if (is_array($item) && !empty($item)) {
                foreach ($item as $day) {
                    array_push($rangeDays, $day);
                }
            } else {
                if (!is_array($item) && $item) {
                    array_push($rangeDays, $item);
                }
            }
            if ($rangeDays) {
                $this->addRange($rangeDays, $i);
            }
        }

        $this->hoursString .= "]";

        $this->response .= "{" . PHP_EOL;
        $this->response .= "  \"@context\": \"http://schema.org\"," . PHP_EOL;
        $this->response .= "  \"@type\": \"" . $this->getProperty('_magic_page_schema_business_type') . "\"," . PHP_EOL;
        $this->response .= "  \"@id\": " . $this->quotes . $this->getProperty('_magic_page_schema_id') . $this->quotesComma . PHP_EOL;
        $this->response .= "  \"url\": " . $this->quotes . $this->getProperty('_magic_page_schema_url') . $this->quotesComma . PHP_EOL;
        $this->response .= "  \"logo\": " . $this->quotes . $this->getProperty('_magic_page_schema_logo') . $this->quotesComma . PHP_EOL;
        $this->response .= "  \"image\": " . $this->quotes . $this->getProperty('_magic_page_schema_image_url') . $this->quotesComma . PHP_EOL;
        $this->response .= "  \"priceRange\": " . $this->quotes . $this->getProperty('_magic_page_price_range') . $this->quotesComma . PHP_EOL;
        $this->response .= "  \"hasMap\": " . $this->quotes . $this->getProperty('_magic_page_schema_map_url') . $this->quotesComma . PHP_EOL;
        $this->response .= "  \"email\": " . $this->quotes . $this->getProperty('_magic_page_email') . $this->quotesComma . PHP_EOL;
        $this->response .= "  \"address\": {" . PHP_EOL;
        $this->response .= "      \"@type\": \"PostalAddress\"," . PHP_EOL;
        $this->response .= "      \"streetAddress\": " . $this->quotes . $this->getProperty('_magic_page_schema_address') . $this->quotesComma . PHP_EOL;
        $this->response .= "      \"addressLocality\": " . $this->quotes . $this->getProperty('_magic_page_schema_city') . $this->quotesComma . PHP_EOL;
        $this->response .= "      \"addressRegion\": " . $this->quotes . $this->getProperty('_magic_page_schema_state') . $this->quotesComma . PHP_EOL;
        $this->response .= "      \"postalCode\": " . $this->quotes . $this->getProperty('_magic_page_schema_zip') . $this->quotesComma . PHP_EOL;
        $this->response .= "      \"addressCountry\": " . $this->quotes . $this->getProperty('_magic_page_schema_country') . $this->quotes . PHP_EOL;
        $this->response .= "    }," . PHP_EOL;
        $this->response .= "  \"description\": " . $this->quotes . $this->getProperty('_magic_page_schema_description') . $this->quotesComma . PHP_EOL;
        $this->response .= "  \"name\": " . $this->quotes . $this->getProperty('_magic_page_schema_business_name') . $this->quotesComma . PHP_EOL;
        $this->response .= "  \"telephone\": " . $this->quotes . $this->getProperty('_magic_page_telephone') . $this->quotesComma . PHP_EOL;
        $this->response .= "  \"openingHoursSpecification\":" . $this->hoursString . "," . PHP_EOL;
        $this->response .= "  \"geo\": {" . PHP_EOL;
        $this->response .= "      \"@type\": \"GeoCoordinates\"," . PHP_EOL;
        $this->response .= "      \"latitude\": " . $this->quotes . $this->getProperty('_magic_page_schema_latitude') . $this->quotesComma . PHP_EOL;
        $this->response .= "      \"longitude\": " . $this->quotes . $this->getProperty('_magic_page_schema_longitude') . $this->quotes . PHP_EOL;
        $this->response .= "    }" . $this->sameAsComma . PHP_EOL;

        if ($this->hasSameAsLinks) {
            $this->response .= "  \"sameAs\" : [" . rtrim($this->string, ',') . "]" . PHP_EOL;
        }

        $this->response .= "}" . PHP_EOL;

        return $this->response;
    }

    protected function getProperty($property) {
        $value = get_magic_page_schema_data($this->post->ID, $property);

        if ($value && is_string($value)) {
            $value = apply_variables($value, $this->term);

            $value = parse_magic_page_if_shortcode($value, $this->term);
            $value = apply_spintax_filter($value);
            $value = apply_filters('cities', do_shortcode($value));
            return $value;
        }

        return $value;
    }

    protected function addRange($range, $id) {
        $rangeString = "";
        if ($id > 1) {
            $rangeString .= ",";
        }
        $rangeString .= "{";
        $rangeString .= '"@type":"OpeningHoursSpecification",';
        $rangeString .= '"dayOfWeek":[';
        $rangeString .= '"' . implode('","', $range) . '"';
        $rangeString .= '],';
        $this->hoursString .= $rangeString;
        $this->addTime($id, count($range));
    }

    protected function addTime($id) {
        $this->hoursString .= '"opens": "' . $this->getProperty('_magic_page_schema_open_' . $id) . '", "closes": "' . $this->getProperty('_magic_page_schema_close_' . $id) . '"}';
    }

}
