<?php

if (!defined('ABSPATH')) {
    exit;
}

class MagicPageShortcodesAndFilters {

    public $no_get_term = false;

    public function __construct() {
        if (!is_magic_page_installed()) {
            return false;
        }

        add_action('save_post', array($this, 'save_post'), 11, 2);
        add_filter('the_title', array($this, 'filters'), 10, 2);
        add_filter('document_title_parts', array($this, 'wp_title'), 10);

        add_filter('the_content', array($this, 'filters'), 99);
        add_filter('the_excerpt', array($this, 'filters'), 99);

        add_shortcode('map', array($this, 'map'));
        add_shortcode('location', array($this, 'location'));
        add_shortcode('description', array($this, 'description'));

        add_shortcode('zip', array($this, 'zip'));
        add_shortcode('country', array($this, 'country'));
        add_shortcode('county', array($this, 'county'));
        add_shortcode('region', array($this, 'region'));

        add_shortcode('cities', array($this, 'get_cities'));
        add_shortcode('magiclink', array($this, 'magiclink'));

        add_shortcode('latitude', array($this, 'latitude'));
        add_shortcode('longitude', array($this, 'longitude'));
        add_shortcode('googlemaps_api_key', array($this, 'googlemaps_api_key'));

        // we can't use add_shortcode directly because wp_query is not available
        add_action('wp', array($this, 'initXfields'));
        add_action('wp', array($this, 'customLocationFields'));

        // seo compatability
        add_filter('option_blogname', array($this, 'option_'));
        add_filter('option_blogdescription', array($this, 'option_'));

        // wpseo
        add_filter('wpseo_title', array($this, 'filters'));
        add_filter('wpseo_metadesc', array($this, 'filters'));
        add_filter('wpseo_opengraph_title', array($this, 'filters'));
        add_filter('wpseo_opengraph_desc', array($this, 'filters'));

        add_filter('wpseo_canonical', array($this, 'canonical'));
        add_filter('get_the_excerpt', array($this, 'get_the_excerpt'));

        // thrive architect
        add_filter('tve_thrive_shortcodes', array($this, 'filters'));

        // compatability breadcrumbs in some themes
        add_filter('breadcrumb_trail', array($this, 'filters'));

        // all in one seo
        add_filter('aioseop_description', array($this, 'seo_filter'));
        add_filter('aioseop_title', array($this, 'seo_filter'));

        // header & footer
        add_action('wp_head', array($this, 'head'));
        add_action('wp_footer', array($this, 'footer'));

        // widgets

        add_filter('widget_text_content', array($this, 'filters'));
        add_filter('widget_block_content', array($this, 'filters'));

        add_filter('widget_block_content', 'do_shortcode');
        add_filter('widget_text_content', 'do_shortcode');
    }

    public function customLocationFields() {
        $customFields = get_location_custom_meta_fields();

        if ($customFields) {
            foreach ($customFields as $field) {
                add_shortcode('meta_' . $field, function ($attributes, $content, $field) {

                    $term = get_location_object();
                    if ($term) {
                        return get_term_meta($term->term_id, str_replace('meta_', '', $field), true);
                    }
                });
            }
        }
    }

    public function magiclink($atts) {
        global $post, $magicPageExport;

        $atts = remove_array_curly_quotes($atts);

        $atts = shortcode_atts(array(
            'location' => '',
            'post_type' => 'same', // same|post,page|magicpages
            'groups' => 'same', // same|all|group1,group2
            'label' => '%location%',
            'remove_if_same_page' => 'false',
            'new_window' => 'false'), $atts);

        if (!$atts['location'] || !$post) {
            return;
        }

        $location = getLocationByAttribute($atts['location']);

        if (!$location) {
            return;
        }

        $args = array();
        $args['post_type'] = get_post_type($post);

        $newWindow = $atts['new_window'] == 'true' ? ' target="_blank" ' : '';

        $params = !is_array($atts['post_type']) ? explode(',', $atts['post_type']) : $atts['post_type'];

        $isMagicPage = $atts['post_type'] === 'magicpage' || ($args['post_type'] === 'magicpage' && $atts['post_type'] === 'same');

        // not exporting, is not magic page, same post type
        if ($atts['post_type'] !== 'same' && $params && !$magicPageExport && !$isMagicPage) {
            $args['post_type'] = $params;

            if ($atts['remove_if_same_page'] == 'true' && is_magic_post_page()) {
                if (($term = getMagicpageLocation())) {
                    if ($term->term_id == $location->term_id) {
                        return;
                    }
                }
            }
        } else if ($magicPageExport || $isMagicPage) {

            $magicPage = get_magic_page();

            if ($magicPageExport) {
                $exportSettings = get_option('_magic_page_export_settings', array());
                $locs = $exportSettings['locations'];
            } else {
                $locs = wp_get_post_terms($magicPage->ID, 'location');
            }


            if ($locs) {
                $query = new MagicPageQuery();

                $query->setLocations($locs);

                $query->setTerm($location);
                $query->setAutoinstalled(magic_page_is_autoinstalled_enabled() && $magicPageExport !== true);
                $found = $query->getResults();

                if ($found && $found[0]) {
                    $term = get_term($found[0]->term_id, 'location');

                    if ($term) {
                        if ($atts['remove_if_same_page'] == 'true') {
                            if (($current = getMagicpageLocation())) {
                                if ($term->term_id == $current->term_id) {
                                    return;
                                }
                            }
                        }

                        $href = get_term_link($term, 'location');
                        $title = $this->applyTitleParams($atts, $location, $magicPage);

                        return '<a ' . $newWindow . ' href="' . $href . '">' . $title . '</a>';
                    }
                }
            }

            return false;
        }


        // posts, pages
        $args['meta_query'] = array();
        $args['posts_per_page'] = '1';
        $args['post_status'] = 'publish';

        $exportGroups = $atts['groups'];

        $meta_query = array(
            'relation' => 'AND',
            array(
                'key' => '_location_id',
                'value' => $location->term_id,
                'compare' => '='
            )
        );

        if ($exportGroups === 'same') {
            $group = get_post_meta($post->ID, '_group_id', true);

            if ($group) {
                $temp = array(
                    array(
                        'key' => '_group_id',
                        'value' => $group,
                        'compare' => '=',
                    )
                );

                $meta_query = array_merge($meta_query, $temp);
            }
        } else if ($exportGroups !== 'all') {
            $values_to_search = explode(',', $exportGroups);
            if ($values_to_search && !empty($values_to_search)) {
                $array = array('relation' => 'OR');

                $success = false;

                foreach ($values_to_search as $value) {
                    $group = getExportGroupByTitle($value);

                    if ($group) {
                        $success = true;
                        $array[] = array(
                            'key' => '_group_id',
                            'value' => $group['id'],
                            'compare' => '=',
                        );
                    }
                }
                if ($success) {
                    $meta_query = array_merge($meta_query, array($array));
                } else {
                    // no groups found so exit
                    return false;
                }
            }
        }

        $args['meta_query'] = $meta_query;

        $query = new WP_Query($args);
        $posts = $query->get_posts();

        if ($posts) {
            $href = get_permalink($posts[0]);

            $title = $this->applyTitleParams($atts, $location, $posts[0]);

            return '<a ' . $newWindow . ' href="' . $href . '">' . $title . '</a>';
        }
    }

    protected function applyTitleParams($atts, $location, $post) {
        if (strpos($atts['label'], '%location%') !== false) {
            $atts['label'] = str_replace('%location%', $location->name, $atts['label']);
        }

        if (strpos($atts['label'], '%post_title%') !== false) {
            $title = get_the_title($post);
            $atts['label'] = str_replace('%post_title%', $title, $atts['label']);
        }

        if (strpos($atts['label'], '%group%') !== false) {
            $groupId = get_post_meta($post->ID, '_group_id', true);

            $groupTitle = '';
            if ($groupId) {
                $groupTitle = getGroupTitleById($groupId);
            }

            $atts['label'] = str_replace('%group%', $groupTitle, $atts['label']);
        }

        return $atts['label'];
    }

    public function seo_filter($string) {
        return apply_spintax_filter($string);
    }

    public function option_($string) {
        if (!is_admin($string)) {
            return $this->filters($string);
        }

        return $string;
    }

    function save_post($post_id, $post) {

        if (!current_user_can('edit_post', $post_id)) {
            return false;
        }



        if (!wp_is_post_revision($post_id) and $post->post_status != 'auto-draft' and!defined('DOING_AJAX')) {

            if (!isset($_POST['post_title']) or!$_POST['post_title'] or!is_string($_POST['post_title'])) {
                return false;
            }

            $saved = get_post_meta($post_id, '_spintax_slug_saved', true);
            if ($saved) {
                return false;
            }

            if (strpos($_POST['post_title'], '{') === false) {
                return false;
            }

            $title = apply_spintax_filter($_POST['post_title']);
            $permalink = get_sample_permalink($post_id, $title, $title);

            if (strpos($permalink, '{') === false) {
                return false;
            }

            if ($permalink && $permalink[1]) {
                global $wpdb;

                update_post_meta($post_id, '_spintax_slug_saved', true);

                $wpdb->query($wpdb->prepare("UPDATE $wpdb->posts SET post_name = %s WHERE ID = %d", $permalink[1], $post_id));
            }
        }
    }

    public function head() {
        $head = '';

        if (is_magic_page()) {
            $head = get_option('_magicpage_headercode');
        } else if (is_magic_post_page()) {
            global $post;
            $groupId = get_post_meta($post->ID, '_group_id', true);

            $head = get_option('_group_headercode_' . $groupId);
        }

        if ($head) {
            echo($this->filters(html_entity_decode($head)));
        }
    }

    public function footer() {
        $foot = '';

        if (is_magic_page()) {
            $foot = get_option('_magicpage_footercode');
        } else if (is_magic_post_page()) {
            global $post;
            $groupId = get_post_meta($post->ID, '_group_id', true);

            $foot = get_option('_group_footercode_' . $groupId);
        }

        if ($foot) {
            echo($this->filters(html_entity_decode($foot)));
        }
    }

    public function wp_title($title) {
        if (isset($title['title']) && $title['title']) {
//          //  if (is_singular(magic_page_default_post_slug())) {
//                $title['title'] = $this->if_shortcode($title['title']);
//           // }
//            $title['title'] = apply_spintax_filter($title['title']);

            $title['title'] = $this->filters($title['title']);
        }

        return $title;
    }

    function get_the_excerpt($exceprt) {
        global $post;

        if ($post && is_object($post) && isset($post->post_type) && $post->post_type === magic_page_default_post_slug()) {
            $excerpt = strip_tags($this->filters($post->post_content));
            $excerpt_length = apply_filters('excerpt_length', 55);

            $exceprt = wp_trim_words($excerpt, $excerpt_length);

            return $exceprt;
        }
        return $exceprt;
    }

    public function filters($string) {



        if ((strpos($string, '[') !== false && strpos($string, ']') !== false ) ||
                (strpos($string, '{') !== false && strpos($string, '}') !== false)
        ) {

            global $post;

            if (!$post) {
                return $string;
            }


            $term = getMagicpageLocation($post->ID);

            $string = apply_variables($string, $term);
            $string = apply_spintax_filter($string);
            $string = $this->if_shortcode($string);
        }

        return $string;
    }

    public function canonical($link) {
        if (is_magic_page()) {
            $term = get_location_object();

            if ($term) {
                $link = get_term_link($term);
            }
        }

        return $link;
    }

    public function initXfields() {
        $xfields = get_option('_magic_page_xfields');

        if (!$xfields) {
            return false;
        }


        $location = getMagicpageLocation();

        foreach ($xfields as $id => $xfield) {
            add_shortcode('xfield_' . $id, function () use ($xfield, $location) {

                if ($location && $xfield['custom']) {
                    foreach ($xfield['custom'] as $_ID => $custom) {
                        //
                        foreach ($custom['locations'] as $loc) {
                            if (strpos($loc, '%location-set-') === 0) {
                                $location_set_id = rtrim(str_replace('%location-set-', '', $loc), '%');

                                $location_sets = get_option('location_sets', array());

                                if (isset($location_sets[$location_set_id])) {
                                    $found = $location_sets[$location_set_id];

                                    foreach ($found[3] as $_location) {



                                        if ($location && $location->term_id == $_location->term_id) {
                                            return $custom['value'];
                                        }
                                    }
                                }
                            }
                        }
                        //
                        if ($location && in_array($location->slug, $custom['locations'])) {
                            return $custom['value'];
                        }
                    }
                }

                return $xfield['value'];
            });
        }
    }

//    function the_title($string) {
//        $term = getMagicpageLocation();
//
//        if ($term) {
//            $string = apply_variables($string, $term);
//        } else {
//            $magicpage = get_magic_page();
//
//            $terms = wp_get_post_terms($magicpage->ID, 'location');
//            if (is_array($terms) && isset($terms[0])) {
//                return apply_variables($string, $terms[0]);
//            }
//        }
//
//
//        return $string;
//    }

    public function location() {
        $term = get_location_object();

        if ($term) {
            return $term->name;
        }
    }

    public function description() {
        $term = get_location_object();

        if ($term) {

            return apply_variables($term->description, $term);
        }
    }

    public function zip() {
        $term = get_location_object();

        if ($term) {
            return get_term_meta($term->term_id, 'zip', true);
        }
    }

    public function county() {
        $term = get_location_object();

        if ($term) {
            return get_term_meta($term->term_id, 'county', true);
        }
    }

    public function region() {
        $term = get_location_object();

        if ($term) {
            return get_term_meta($term->term_id, 'region', true);
        }
    }

    public function country() {
        $term = get_location_object();

        if ($term) {
            return get_term_meta($term->term_id, 'country', true);
        }
    }

    public function latitude() {
        $term = get_location_object();

        if ($term) {
            return get_term_meta($term->term_id, 'lat', true);
        }
    }

    public function longitude() {
        $term = get_location_object();

        if ($term) {
            return get_term_meta($term->term_id, 'lon', true);
        }
    }

    public function map($atts) {
        $settings = get_magic_page_googlemap_parameters();

        $defaults = shortcode_atts(
                array(
                    'width' => '100%',
                    'height' => "500px",
                    'zoom' => '12',
                    'type' => 'roadmap',
                    'min_height' => '500px'
                ), $atts);

        return '<div class="' . $settings['js_classname'] . '" data-type="' . $defaults['type'] . '"  data-zoom="' . $defaults['zoom'] . '" '
                . 'style="display: none; width:' . $defaults['width'] . '; min-height:' . $defaults['height'] . '"></div>';
    }

    public function magicpage($atts) {
        $post = get_magic_page();

        if ($post) {
            return $this->get_locations($atts, $post);
        }
    }

    public function get_cities($atts) {

        global $post;

//        if (!is_preview() && is_admin() && !$magicPageExport) {
//            return;
//        }


        $atts = remove_array_curly_quotes($atts);

        $atts = shortcode_atts(array(
//            // endof 6.8
//            'template' => '',
//            'pagination' => '',
//            'location' => '',
//            'locations' => '',
//            // endof 6.8
            'type' => 'csv',
            'count' => '10',
            'counties' => '',
            'major_cities' => '',
            'max-radius' => '',
            'min-radius' => '',
            'label' => "%location%",
            // for posts/pages
            'groups' => 'same',
            'post_type' => 'magicpage', // same|post,page,magicpage
            'categories' => array()), $atts);

        $nearby = get_nearby_locations($post, $atts);

        $out = '';

        if (($nearby)) {
      
            if ($atts['type'] == 'array') {
                $out = array();

                foreach ($nearby as $city) {

                    if (isset($city->post_id)) {
                        $_post = get_post($city->post_id);
                        $out[] = $this->applyTitleParams($atts, $city, $_post);
                    } else {
                        $out[] = $this->applyTitleParams($atts, $city, $post);
                    }
                }

                return serialize($out);
            }

            if ($atts['type'] == 'csv' or $atts['type'] == 'csvt') {
                foreach ($nearby as $city) {

                    if (isset($city->post_id)) {
                        $_post = get_post($city->post_id);
                        $title = $this->applyTitleParams($atts, $city, $_post);
                        $link = get_permalink($city->post_id);
                    } else {
                        $title = $this->applyTitleParams($atts, $city, $post);
                        $link = get_term_link(intval($city->term_id), 'location');
                    }

                    if ($atts['type'] == 'csv') {
                        if (!is_object($link)) {
                            $out .= '<a href="' . $link . '">' . $title . '</a>, ';
                        }
                    } else {
                        $out .= "$title, ";
                    }
                }
                $out = rtrim($out, ', ');
            } else if ($atts['type'] == 'ul') {


                $out .= '<ul>';
                foreach ($nearby as $city) {
                    if (isset($city->post_id)) {
                        $_post = get_post($city->post_id);
                        $title = $this->applyTitleParams($atts, $city, $_post);
                        $link = get_permalink($city->post_id);
                    } else {
                        $title = $this->applyTitleParams($atts, $city, $post);
                        $link = get_term_link(intval($city->term_id), 'location');
                    }

                    if (!is_object($link)) {
                        $out .= '<li><a href="' . $link . '">' . $title . '</a></li>';
                    }
                }

                $out .= '</ul>';
            } else if ($atts['type'] == 'ult') {
                $out .= '<ul>';
                foreach ($nearby as $city) {
                    if (isset($city->post_id)) {
                        $_post = get_post($city->post_id);
                        $title = $this->applyTitleParams($atts, $city, $_post);
                    } else {
                        $title = $this->applyTitleParams($atts, $city, $post);
                    }

                    $out .= '<li>' . $title . '</li>';
                }

                $out .= '</ul>';
            }
        }

        return $out;
    }

    public function googlemaps_api_key() {
        return get_option('_magic_page_google_maps_api_key');
    }

    public function if_shortcode($content) {
        return parse_magic_page_if_shortcode($content);
    }

}

new MagicPageShortcodesAndFilters();
