<?php

/*
 * Copyright 2019 TopWebStudio 
 * All rights reserved.
 * For license information refer to your license agreement
 */
if (!function_exists('boolval')) {

    function boolval($var) {
        return !!$var;
    }

}

if (!function_exists('getImageSource')) {

    function getImageSource($content) {
        preg_match_all('/<img.*?>/', $content, $matches);
        if ($matches[0]) {
            return $matches[0];
        }
        return array();
    }

}


if (!function_exists('minifyCss')) {

    function minifyCss($css) {
        // some of the following functions to minimize the css-output are directly taken
        // from the awesome CSS JS Booster: https://github.com/Schepp/CSS-JS-Booster
        // all credits to Christian Schaefer: http://twitter.com/derSchepp
        // remove comments
        $css = preg_replace('!/\*[^*]*\*+([^/][^*]*\*+)*/!', '', $css);

        // backup values within single or double quotes
        preg_match_all('/(\'[^\']*?\'|"[^"]*?")/ims', $css, $hit, PREG_PATTERN_ORDER);
        for ($i = 0; $i < count($hit[1]); $i++) {
            $css = str_replace($hit[1][$i], '##########' . $i . '##########', $css);
        }

        // remove traling semicolon of selector's last property
        $css = preg_replace('/;[\s\r\n\t]*?}[\s\r\n\t]*/ims', "}\r\n", $css);
        // remove any whitespace between semicolon and property-name
        $css = preg_replace('/;[\s\r\n\t]*?([\r\n]?[^\s\r\n\t])/ims', ';$1', $css);
        // remove any whitespace surrounding property-colon
        $css = preg_replace('/[\s\r\n\t]*:[\s\r\n\t]*?([^\s\r\n\t])/ims', ':$1', $css);
        // remove any whitespace surrounding selector-comma
        $css = preg_replace('/[\s\r\n\t]*,[\s\r\n\t]*?([^\s\r\n\t])/ims', ',$1', $css);
        // remove any whitespace surrounding opening parenthesis
        $css = preg_replace('/[\s\r\n\t]*{[\s\r\n\t]*?([^\s\r\n\t])/ims', '{$1', $css);
        // remove any whitespace between numbers and units
        $css = preg_replace('/([\d\.]+)[\s\r\n\t]+(px|em|pt|%)/ims', '$1$2', $css);
        // shorten zero-values
        $css = preg_replace('/([^\d\.]0)(px|em|pt|%)/ims', '$1', $css);
        // constrain multiple whitespaces
        $css = preg_replace('/\p{Zs}+/ims', ' ', $css);
        // remove newlines
        $css = str_replace(array("\r\n", "\r", "\n"), '', $css);

        // Restore backupped values within single or double quotes
        for ($i = 0; $i < count($hit[1]); $i++) {
            $css = str_replace('##########' . $i . '##########', $hit[1][$i], $css);
        }

        return $css;
    }

}
if (!function_exists('getRandomElement')) {

    function getRandomElement($array) {
        return $array[mt_rand(0, count($array) - 1)];
    }

}

if (!function_exists('cleanUrl')) {

    function cleanUrl($url) {
        $url = str_replace("http://", "", $url);
        $url = str_replace("https://", "", $url);

        if (strtolower(substr($url, 0, 4)) == 'www.') {
            $url = substr($url, 4);
        }

        $url = explode('/', $url);
        $url = reset($url);
        $url = explode(':', $url);
        return reset($url);
    }

}
if (!function_exists('htaccessCreate')) {

    function htaccessCreate($subdir = '', $redirect = false) {
        $text = 'RewriteEngine On

RewriteBase /

RewriteCond %{THE_REQUEST} ^GET\ (.*)\.html\ HTTP
RewriteRule (.*)\.html$ ' . $subdir . '$1 [R=301]

RewriteRule (.*)/index$ ' . $subdir . '$1/ [R=301]
RewriteCond %{REQUEST_FILENAME} !-d
RewriteCond %{REQUEST_URI} /$
RewriteRule (.*)/ ' . $subdir . '$1 [R=301]

RewriteCond %{REQUEST_FILENAME}.html -f
RewriteCond %{REQUEST_URI} !/$
RewriteRule (.*) ' . $subdir . '$1\.html [L]';

        if ($redirect) {
            $text .= PHP_EOL.PHP_EOL.'RewriteCond %{REQUEST_FILENAME} !-f'.PHP_EOL;

            if ($subdir) {
                $text .= 'RewriteRule . ' . $subdir . ' [L,R=301]';
            } else {
                $text .= 'RewriteRule . / [L,R=301]';
            }
        }
        return $text;
    }

}
