<?php

/*
 * Copyright 2019 TopWebStudio 
 * All rights reserved.
 * For license information refer to your license agreement
 */

namespace TopWebStudio\PhpUtilities;

class ImageParser {

    protected $contents;
    protected $uploadFolder;
    protected $uploadUrl;
    protected $embed;

    /**
     * @param html $contents html
     * @param string $uploadFolder The image upload folder
     * @param string $uploadUrl The new image upload url
     * @param bool $embed Whether to use base64 embed
     */
    public function __construct($contents, $uploadFolder, $uploadUrl, $embed = false) {
        $this->content = $contents;
        $this->uploadFolder = $uploadFolder;
        $this->uploadUrl = $uploadUrl;
        $this->embed = $embed;
    }

    public function parse() {
        foreach (getImageSource($this->content) as $image) {
            preg_match('@src=(\"|\')([^(\"|\')]+?)(\"|\')@', $image, $match);
            if ($match) {
                $data = $this->getUploadData($match[2]);
                $newImageHtml = str_replace($match[2], $data, $image);
                $this->replace($image, $newImageHtml);

                preg_match('@srcset=(\"|\')([^(\"|\')]+?)(\"|\')@', $image, $srcset);
                if ($srcset) {
                    preg_match_all('/http[^ ]*/', $srcset[2], $matchSrcSet);
                    if ($matchSrcSet[0]) {
                        foreach ($matchSrcSet[0] as $src) {
                            $data = $this->getUploadData($src);
                            $this->replace($src, $data);
                        }
                    }
                }
            }
        }

        return $this->content;
    }

    protected function replace($search, $replace) {
        $this->content = str_replace($search, $replace, $this->content);
    }

    protected function getUploadData($imageSrcUrl) {
        $imageSrcFilename = basename($imageSrcUrl);
        $newFilename = $this->uploadFolder . $imageSrcFilename;

        if ($this->embed) {
            $newSrcUrl = $this->getImageDataFromUrl($imageSrcUrl);
        } else {
            $newSrcUrl = $this->uploadUrl . $imageSrcFilename;
            if (!file_exists($newFilename)) {
                @file_put_contents($newFilename, @file_get_contents($imageSrcUrl));
            }
        }

        return $newSrcUrl;
    }

    protected function getImageDataFromUrl($url) {
        $urlParts = pathinfo($url);
        $extension = $urlParts['extension'];
        $content = file_get_contents($url);
        return 'data:image/' . $extension . ';base64,' . base64_encode($content);
    }

}
